<< capdRedHom`

(** Compute persistence homology from a point cloud. Arguments:
- a list of points in n-dimensional euclidean space
- max complex dimension
- max distance
 **)

Print[RedHomPointCloudPersistentHomology[{{.1, .2, .3}, {.0, .0, .0}}, 1, .1]];





(** Random points example with visualization **)
Needs["HierarchicalClustering`"];


numberOfPoints = 100;
points = Table[{RandomReal[1], RandomReal[1], RandomReal[1]}, {i, numberOfPoints}];
distanceMatrix = DistanceMatrix[points];
Distance[p_, q_] := distanceMatrix[[p]][[q]];
SimplexCoordinate[simplex_] := Map[Function[p, points[[p]]], simplex];

maxDistance = 0.3;
maxDimension = 2;
(** compute persistence intervals **)
persistenceIntervals =
  RedHomPointCloudPersistentHomology[points, maxDimension,
   maxDistance];
(** compute simplicial Rips complex, only for visualization **)
complex =  RipsComplex[Distance, Length[points], maxDistance, maxDimension];


distance = 0.1;
dimension = maxDimension;
(** create filtration based on the precomputed Rips complex **)
filteredSimplices =
  ComplexFiltration[Distance, complex, distance, dimension];
(** For the filtration we can compute regular homology **)
filteredComplex = RedHomCreateSimplicialComplex[Union @@ filteredSimplices];
betti = RedHomBettiNumbers[filteredComplex];
Print[betti];

plotDimension = 2;
plot = PlotSimplicialComplex[filteredSimplices, plotDimension,  SimplexCoordinate];
Show[plot]
