import capdRedHom
import sys
import ast

def read_dat(filename):
    with open(filename, 'r') as input:
        for line in input:
            simplex = map(int, line.split())
            yield simplex

def read_pcub(filename):
    with open(filename, 'r') as input:
        for line in input:
            cube = ast.literal_eval(line)
            yield cube

def complex_type(filename):
    if filename.endswith('.dat'):
        return 'simplicial'
    elif filename.endswith('.pcub_py'):
        return 'cubical'
    else:
        raise RuntimeError("Supported file types: {}".format(['.dat', '.pcub_py']))

def read_complex(complex_filename):
    type = complex_type(complex_filename)

    if type == 'simplicial':
        facets = list(read_dat(complex_filename))
        complex = capdRedHom.SimplicialComplex(facets)
    else:
        cubes = list(read_pcub(complex_filename))
        complex = capdRedHom.CubicalComplex(cubes=cubes)

    return complex

if __name__ == '__main__':
    complex = read_complex(sys.argv[1])
    algorithm = capdRedHom.BettiNumbersOverZ(complex)
    betti = algorithm()
    print betti
