import sys, os
import gzip
import capdRedHom

def read_image(filename):
    if filename.endswith('.gz'):
        fileobj = gzip.open(filename, 'rb')
        filename = filename[:-3]
    else:
        fileobj = open(filename, 'rb')

    if filename.endswith('fits'):
        import pyfits
        data_cube, header_data_cube = pyfits.getdata(fileobj, 0, header=True)
        data = data_cube
    else:
        try:
            from skimage.io import imread
            image = imread(fileobj).astype(float)
            data = image
        except ImportError as ex:
            import PIL.Image
            image = PIL.Image.open(fileobj)
            w, h = image.size
            image_pixels = image.load()
            data = [ [image_pixels[i, j] for j in xrange(w)] for i in xrange(h)]

    return data


def compute_persistence_homology(data):
    algorithm = capdRedHom.persistence.ImagePersistentHomology(data)
    algorithm()
    full_diagram = algorithm.diagram()
    dim_1_diagram = algorithm.diagram(1)
    dim_10_diagram = algorithm.diagram(10) # empty dimension
    return full_diagram, dim_1_diagram, dim_10_diagram


def main(argv=sys.argv):
    image_file = argv[1]
    image_file_bn = os.path.basename(image_file)

    image = read_image(image_file)
    diagram, dim_1_diagram, dim_10_diagram = compute_persistence_homology(image)

    finite, infinite = (len(diagram.finite_intervals), len(diagram.infinite_intervals))
    dim_1_finite, dim_1_infinite = (len(dim_1_diagram.finite_intervals), len(dim_1_diagram.infinite_intervals))
    dim_10_finite, dim_10_infinite = (len(dim_10_diagram.finite_intervals), len(dim_10_diagram.infinite_intervals))

    print "Persistence diagram contains {} finite intervals and {} infinite interals".format(finite, infinite)
    print "Persistence diagram in dimension 1 contains {} finite intervals and {} infinite interals".format(dim_1_finite, dim_1_infinite)
    print "Persistence diagram in dimension 10 contains {} finite intervals and {} infinite interals".format(dim_10_finite, dim_10_infinite)

    with open(image_file_bn + '.intervals', 'w') as out:
        for interval in diagram:
            out.write('{} {}\n'.format(*interval))

    try:
        import matplotlib
        matplotlib.use("Agg")
        import matplotlib.pyplot as plt
        plot_diagram = capdRedHom.persistence.PlotDiagram(diagram)
        plot_diagram(plt)
        plt.savefig(image_file_bn + '.png')
        plt.clf()
    except ImportError:
        pass


if __name__ == '__main__':
    main(sys.argv)
