#!/usr/bin/env sage -python

import sage.all as sage
import sys
import time
import ast

def read_dat(filename):
    with open(filename, 'r') as input:
        for line in input:
            try:
                data = ast.literal_eval(line)
            except:
                data = line

            if isinstance(data, str):
                simplex = data.split()
            elif isinstance(data, int):
                simplex = [data]
            else:
                simplex = data
            yield simplex

def read_cub(filename):
    with open(filename, 'r') as input:
        for line in input:
            try:
                data = ast.literal_eval(line)
            except:
                data = line

            if isinstance(data, str):
                cube = data.split()
            else:
                cube = data
            cube = map(int, cube)
            yield map(lambda x: (x, x+1), cube)

def complex_type(filename):
    if filename.endswith('.dat'):
        return 'simplicial'
    elif filename.endswith('.cub') or filename.endswith('.fcub'):
        return 'cubical'
    else:
        raise RuntimeError("Supported file types: {}".format(['.dat', '.cub', '.fcub']))


def read_complex(complex_filename):
    type = complex_type(complex_filename)

    if type == 'simplicial':
        facets = list(read_dat(complex_filename))
        complex = sage.SimplicialComplex(facets)
    else:
        cubes = list(read_cub(complex_filename))
        complex = sage.CubicalComplex(cubes)

    return complex

def betti_to_vector(betti):
    betti_vec = [0]*(max(betti.keys()) + 1)
    for i, b in betti.iteritems():
        betti_vec[i] = b
    return betti_vec

if __name__ == '__main__':
    complex = read_complex(sys.argv[1])
    start = time.time()
    print "Betti numbers: ", ", ".join(map(str, betti_to_vector(complex.betti())))
    end = time.time()

    print end - start
