#!/bin/bash

set -e

source ${srcdir}/tests_env.sh

if !  python -c 'import capdRedHom'; then
    echo "No capdRedHom module"
    exit 77 # skipped
fi


for complex_fn in "torus.pcub_py" "Torus_0.simplices.dat"; do
    python ${srcdir}/homology.py ${srcdir}/../apiRedHom/inputs/${complex_fn}
done

echo "Computing persistence of hardcoded image"
python ${srcdir}/image_persistence_2.py

echo "Computing persistence of images"
if  python -c 'from skimage.io import imread' || python -c 'import PIL.Image'; then

    for image_fn in "img1.png" "img1.png.gz" "img2.png" "img2.png.gz"; do
        python ${srcdir}/image_persistence.py ${srcdir}/../apiRedHom/inputs/${image_fn}
        echo "Check if persistence diagram file exists..."
        test -s ${image_fn}.png || echo "FAIL"
        echo "Check if persistence intervals file exists..."
        test -s ${image_fn}.intervals || echo "FAIL"
    done

    dist_0=$(python ${srcdir}/bottleneck_distance.py 1 img1.png.intervals img1.png.intervals)
    dist_1=$(python ${srcdir}/bottleneck_distance.py 1 img1.png.intervals img2.png.intervals)

    echo $dist_0 $dist_1

    dist_0=$(python ${srcdir}/wasserstein_distance.py 1 img1.png.intervals img1.png.intervals)
    dist_1=$(python ${srcdir}/wasserstein_distance.py 1 img1.png.intervals img2.png.intervals)

    echo $dist_0 $dist_1
else
    echo "No image library (skimage or PIL)"
    exit 77 # skipped
fi
