/////////////////////////////////////////////////////////////////////////////
/// @file capdRedHom_homology
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-08-03
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include <capd/apiRedHom/Complex.h>


#include <capd/apiRedHom/Algorithms.h>
#include <boost/assign.hpp>

#include <algorithm>
#include <iostream>
#include <fstream>
#include <stdexcept>
#include <sstream>
#include <string>
#include <capd/apiRedHom/ComplexHelper.h>

using namespace capd::apiRedHom;

int main(int argc, char** argv)
{

  try {
    if (argc != 2) {
      std::stringstream msg;
      msg  << "usage: " << argv[0] << " <input_file>";
      throw std::runtime_error(msg.str());
      return 1;
    }

    const std::string inputFile = argv[1];

    if (inputFile.size() < 4 || std::string(inputFile.end() - 4, inputFile.end()) != std::string(".dat")) {
      throw std::runtime_error("Only dat files suported");
    }

    std::ifstream input(inputFile.c_str());
    boost::shared_ptr<Complex> complex = ComplexHelper::create(input, ComplexHelper::INPUT_TYPE_DAT);
    HomologyOverZ alg(*complex);
    Signature signature = alg();

    std::cout << signature.str << std::endl;

  } catch (const std::exception& ex) {
    std::cerr << "Error: " << ex.what() << std::endl;
    return 1;
  }

  return 0;
}
