#include <capd/apiRedHom/c_interface.h>

#include <stdio.h>

// gcc -I$HOME/work/capd/capdRedHom/include example.c -o example -L. -lcapd


void cubicalComplexExample()
{

  printf("\n\nCubica sphere\n");

  RedHom_ComplexInfo sphereComplex;
  RedHom_CreateCubicalComplex(&sphereComplex, 3, (int[]){4, 4, 4});

  // sphere
  RedHom_AddCubeToComplex(&sphereComplex, (int[]){1, 1, 1}, (int[]){1, 1, 0});
  RedHom_AddCubeToComplex(&sphereComplex, (int[]){1, 1, 1}, (int[]){1, 0, 1});
  RedHom_AddCubeToComplex(&sphereComplex, (int[]){1, 1, 1}, (int[]){0, 1, 1});
  RedHom_AddCubeToComplex(&sphereComplex, (int[]){1, 1, 2}, (int[]){1, 1, 0});
  RedHom_AddCubeToComplex(&sphereComplex, (int[]){1, 2, 1}, (int[]){1, 0, 1});
  RedHom_AddCubeToComplex(&sphereComplex, (int[]){2, 1, 1}, (int[]){0, 1, 1});

  printf("Size: %d\nFilling boundaries\n", RedHom_ComplexCardinality(&sphereComplex));

  RedHom_CubicalComplexFillWithBoundaries(&sphereComplex);

  printf("Size: %d\n", RedHom_ComplexCardinality(&sphereComplex));


  RedHom_ComplexInfo antipodalComplex;
  RedHom_CreateCubicalComplex(&antipodalComplex, 3, (int[]){4, 4, 4});

  // antipodal points
  RedHom_AddCubeToComplex(&antipodalComplex, (int[]){1, 1, 1}, (int[]){0, 0, 0});
  RedHom_AddCubeToComplex(&antipodalComplex, (int[]){2, 2, 2}, (int[]){0, 0, 0});

  int betti[RedHom_ComplexDim(&sphereComplex) + 1];
  //int bettiDim = RedHom_RedHomOrchestratorOverZ(&sphereComplex, betti);
  int bettiDim = RedHom_RelativeBettiNumbersOverZOfComplex(&sphereComplex, &antipodalComplex, betti, 0, 0, 0, 0);

  for (int i = 0; i < bettiDim; ++i) {
    printf("%d\n", betti[i]);
  }


  RedHom_DeleteComplex(&sphereComplex);
  RedHom_DeleteComplex(&antipodalComplex);
}

void generalComplexExample()
{
  printf("\n\nGeneral sphere\n");

  RedHom_ComplexInfo sphereComplex;

  // simplicial tetrahedron
  int dims[] = {0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2};
  unsigned int cofaces[] =   {4,   7,  8,  6, 7,  9,  5, 8, 9, 4, 5, 6, 10, 12, 10, 11, 11, 12, 12, 13, 10, 13, 11, 13};
  unsigned int faces[] =     {0,   0,  0,  1, 1,  1,  2, 2, 2, 3, 3, 3,  4,  4,  5,  5,  6,  6,  7,  7,  8,  8,  9,  9};
  int coincidenceIndices[] = {-1, -1, -1, -1, 1, -1, -1, 1, 1, 1, 1, 1,  1,  1, -1,  1, -1, -1,  1,  1,  1, -1,  1,  1};
  int entries = 24;
  int size = 14;

  RedHom_CreateGeneralizedSComplex(&sphereComplex, size, dims, entries, cofaces, faces, coincidenceIndices);

  int betti[RedHom_ComplexDim(&sphereComplex) + 1];
  int bettiDim = RedHom_RedHomOrchestratorOverZ(&sphereComplex, betti, 0, 0, 0);


  for (int i = 0; i < bettiDim; ++i) {
    printf("%d\n", betti[i]);
  }

  RedHom_DeleteComplex(&sphereComplex);
}

int main()
{
   RedHom_ComplexInfo Complex;
   int dimsLen[] = {20,20,20,20,20};
   RedHom_CreateCubicalComplex(&Complex, 5, dimsLen);
   int cube[] = {1,1,1,1,1};
   int len[] = {1,1,1,1,1};
   RedHom_AddCubeToComplex(&Complex, cube,len);

  //  cubicalComplexExample();
  //  generalComplexExample();

  return 0;
}
