#include <capd/apiRedHom/Complex.h>
#include <capd/apiRedHom/Algorithms.h>
#include <capd/apiRedHom/ComplexHelper.h>
#include <boost/assign.hpp>

#include <iostream>
#include <algorithm>

// g++ -I$HOME/work/capd/capdRedHom/include example.cpp -o example -L$HOME/work/capd/capdMake/libcapd/.libs -lcapd
int main()
{
  using boost::assign::list_of;
  using namespace capd::apiRedHom;


  std::cout << "\n\n Cubical\n\n";

  {
    // full cubse circle 'O'
    std::vector<CubicalComplex::FullCube> cubes =
      {{0, 0}, {0, 1}, {0, 2}, {1, 0}, {1, 2}, {2, 0}, {2, 1}, {2, 2}};

    boost::shared_ptr<CubicalComplex> complex = CubicalHelper::CreateFromFullCubes(cubes);
    HomologyOverZ homology(*complex);
    std::cout << "Circle homology:\n" << homology().str << std::endl;
  }

  // cubical circle, intervals C++11
  std::vector<CubicalComplex::CellCode> sphereCubes =
  {
    {{1, true}, {1, true}, {1, false}},
    {{1, true}, {1, false}, {1, true}},
    {{1, false}, {1, true}, {1, true}},
    {{1, true}, {1, true}, {2, false}},
    {{1, true}, {2, false}, {1, true}},
    {{2, false}, {1, true}, {1, true}}
  };
  std::vector<size_t> sphereSize(3, 10);

  {
    std::cout << "Regular homology:\n\n";
    CubicalComplex sphereComplex(sphereSize, sphereCubes);

    std::cout << "Size: " << sphereComplex.cardinality() << std::endl << "Filling boundaries" << std::endl;
    sphereComplex.fillWithBoundaries();
    std::cout << "Size: " << sphereComplex.cardinality() << std::endl;

    RedHomOrchestratorOverZ alg(sphereComplex);
    std::vector<int> betti = alg();
    std::copy(betti.begin(), betti.end(), std::ostream_iterator<int>(std::cout, "\n"));
  }


  { // intervals, old-school
    std::vector<CubicalComplex::CellCode> antipodalCubes = list_of
      (list_of<CubicalComplex::Interval>(1, false)(1, false)(1, false).convert_to_container<CubicalComplex::CellCode>())
      (list_of<CubicalComplex::Interval>(2, false)(2, false)(2, false).convert_to_container<CubicalComplex::CellCode>())
      ;
    std::vector<size_t> antipodalSize(3, 10);

    std::cout << "Relative homology:\n\n";

    CubicalComplex sphereComplex(sphereSize, sphereCubes);
    sphereComplex.fillWithBoundaries();

    CubicalComplex antipodalComplex(antipodalSize, antipodalCubes);
    antipodalComplex.fillWithBoundaries();

    RelativeBettiNumbersOverZ alg(sphereComplex, antipodalComplex, false);
    std::vector<int> betti = alg();
    std::copy(betti.begin(), betti.end(), std::ostream_iterator<int>(std::cout, "\n"));
  }


  return 0;
}
