/////////////////////////////////////////////////////////////////////////////
/// @file SmithFormMain.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-11
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.


#include <capd/apiRedHom/Complex.h>
#include <capd/apiRedHom/Algorithms.h>
#include <capd/auxil/ApplicationDesc.h>
#include <capd/auxil/Logger.h>

#include <boost/program_options.hpp>

#include <iostream>
#include <string>
#include <fstream>
#include <capd/apiRedHom/ComplexHelper.h>

using namespace capd;
using namespace capd::apiRedHom;

namespace {
  const capd::auxil::ApplicationDesc applicationDesc("SmithFormMain.cpp",
					"Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>",
					"2014-06-11",
					"");
}

int main(int argc, char *argv[])
{

  try {
    INIT_CAPD_LOGGER;
    CAPD_INFO("Entering application:\n" << applicationDesc);

    namespace po = boost::program_options;

    po::options_description desc("Allowed options");
    po::positional_options_description positionalDesc;

    desc.add_options()
      ("help,h", "produce help message");

    std::string inputFilename;

    //desc.add_options()
    //   ("f,files", po::value<std::vector<std::string> >(&filenames)->multitoken(), "list of files")
      //("i,input", po::value<std::string>(&inputFilename)->required(), "input complex file name - for now only dat");

    positionalDesc.add("input_file", 1);
    desc.add_options()("input_file", po::value<std::string>(&inputFilename)->required(), "Input file (simplices only!)");

    po::variables_map vm;
    po::store(po::command_line_parser(argc, argv).options(desc).positional(positionalDesc).run(), vm);
    po::notify(vm);

    if (vm.count("help") > 0) {
      std::cout << desc << std::endl;
      return 0;
    }


    std::fstream input(inputFilename.c_str());
    boost::shared_ptr<Complex> complex = ComplexHelper::create(input, ComplexHelper::INPUT_TYPE_OBJ);

    BettiNumbersViaSmithFormOverZ bettiAlg(*complex);
    std::vector<BettiNumbersViaSmithFormOverZ::Scalar> betti = bettiAlg();

    std::cout << "Betti numbers: ";
    std::copy(betti.begin(), betti.end(), std::ostream_iterator<int>(std::cout, " "));
    std::cout << "\n";


    return 0;

  } catch (std::exception& ex) {
    std::cerr << "Error: " << ex.what() << std::endl;
    return 1;
  } catch (const char* ex) {
    std::cerr << "Error: " << ex << std::endl;
    return 1;
  } catch (const std::string& ex) {
    std::cerr << "Error: " << ex << std::endl;
    return 1;
  } catch(...) {
    std::cerr << "Unknown exception" << std::endl;
    return 1;
  }

  return 1;
}
