/////////////////////////////////////////////////////////////////////////////
/// @file ComplexHelper
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-01-18
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_APIREDHOM_COMPLEXHELPER_H
#define CAPD_FILE_CAPDREDHOM_APIREDHOM_COMPLEXHELPER_H

// #include
#include "Complex.h"
// there is a bug in any_range.hpp, we need to include it
#include <boost/range/concepts.hpp>
#include <boost/utility.hpp>

#include <boost/range/any_range.hpp>
#include <iosfwd>
#include <vector>


namespace capd
{
  namespace apiRedHom
  {

    class CubicalHelper
    {

    public:
      typedef CubicalComplex::FullCube FullCube;
      typedef boost::any_range<FullCube, boost::forward_traversal_tag, const FullCube&, std::ptrdiff_t> FullCubeRange;

      static boost::shared_ptr<GeneralizedSComplex> CreateGeneralizedSComplexFromFullCubes(int dim, const FullCubeRange& fullCubeRange);

      static boost::shared_ptr<CubicalComplex> CreateFromFullCubes(const FullCubeRange& fullCubeRange, int type=CubicalComplex::TYPE_DEFAULT);
      static std::vector<size_t> CubicalComplexSize(const FullCubeRange& fullCubeRange);

      static std::vector<int> BettiNumbers(const FullCubeRange& fullCubeRange, int modulo = 0); // modulo 0 means over integers

    };


    struct ComplexHelper
    {
      enum InputType { INPUT_TYPE_UNKNOWN = -1, INPUT_TYPE_CUB, INPUT_TYPE_PCUB, INPUT_TYPE_FCUB, INPUT_TYPE_BMD,
                       INPUT_TYPE_DAT, INPUT_TYPE_OBJ, INPUT_TYPE_GEN  };

      static boost::shared_ptr<Complex> create(std::istream& stream, InputType inputType);
    };


  }
}

#endif // CAPD_FILE_CAPDREDHOM_APIREDHOM_COMPLEXHELPER_H
