/////////////////////////////////////////////////////////////////////////////
/// @file c_interface.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-07
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_APIREDHOM_C_H
#define CAPD_FILE_APIREDHOM_C_H


#if defined(__cplusplus)
extern "C" {
#endif

  typedef struct
  {
    void* complexPtr;
  } RedHom_ComplexInfo;

  void RedHom_Init();
  void RedHom_CreateSimplicialComplex(RedHom_ComplexInfo* complexInfo);
  void RedHom_CreateCubicalComplex(RedHom_ComplexInfo* complexInfo, int dim, const int* size);
  void RedHom_CreateGeneralizedSComplex(RedHom_ComplexInfo* complexInfo, unsigned int size, int* dims, unsigned int kappaEntries, unsigned  int* cofaces, unsigned int* faces, int* coincidenceIndices);

  void RedHom_DeleteComplex(RedHom_ComplexInfo* complexInfo);

  int RedHom_ComplexCardinality(RedHom_ComplexInfo* complexInfo);
  int RedHom_ComplexDim(RedHom_ComplexInfo* complexInfo);

  void RedHom_AddSimplexToComplex(RedHom_ComplexInfo* complexInfo, const int* vertices, int size);
  void RedHom_AddCubeToComplex(RedHom_ComplexInfo* complexInfo, const int* coords, const int* unitInterval);
  void RedHom_CubicalComplexFillWithBoundaries(RedHom_ComplexInfo* complexInfo);

  int RedHom_BettiNumbersOverZOfComplex(RedHom_ComplexInfo* complexInfo, int* bettiVector);
  int RedHom_RelativeBettiNumbersOverZOfComplex(RedHom_ComplexInfo* complexInfo, RedHom_ComplexInfo* subcomplexInfo, int* bettiVector, int keepClosed, int reductions, int coreductions, int morse);
  int RedHom_Coreduction(RedHom_ComplexInfo* complexInfo);
  int RedHom_RedHomOrchestratorOverZ(RedHom_ComplexInfo* complexInfo, int* bettiVector, int reductions, int coreductions, int morse);

  //  int RedHom_FundGroup(RedHom_ComplexInfo* complexInfo, int** buffer);
  int RedHom_FundamentalGroup(RedHom_ComplexInfo* complexInfo, int** buffer, int vectorFieldType);

  void RedHom_ZigZag(RedHom_ComplexInfo* complexInfo, int dim);
  unsigned long RedHom_CountCriticalCells(RedHom_ComplexInfo* complexInfo, int method);

#if defined(__cplusplus)
}
#endif


#endif // CAPD_FILE_APIREDHOM_C_H
