/// @addtogroup matrixAlgorithms
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file EuclBitSet_NeighbSliceIterator.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2010 by the CAPD Group.
//
// This file constitutes a part of the CAPD library,
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.ii.uj.edu.pl/ for details.

#if !defined(_EUCLBITSETT_NEIGHBSLICEITERATOR_H_)
#define _EUCLBITSETT_NEIGHBSLICEITERATOR_H_

#include "capd/basicalg/powerThree.h"


/*=======================================================================*/
// NeighbSliceIterator is a EuclBitSetT iterator
// intended for scan of a slice of the neighborhood of a given point

template <typename P_BitSet,int dim>
class EuclBitSetT<P_BitSet,dim>::NeighbSliceIterator : public EuclBitSetT<P_BitSet,dim>::BitIterator{
  friend class EuclBitSetT<P_BitSet,dim>;
protected:
  unsigned int locCoord[dim]; // locCoord takes values in {0,1,2} only, but for efficiency int seems to be better
  unsigned int sliceDir; //
  unsigned int sliceCoord; //
public:
  static const unsigned int embDim=dim;

  NeighbSliceIterator(
    const EuclBitSetT& A_s,
    const EuclBitSetT<P_BitSet,dim>::BitIterator& A_it,
    int A_sliceDir,
    int A_sliceCoord
  ):EuclBitSetT<P_BitSet,dim>::BitIterator(A_s,A_it),
    sliceDir(A_sliceDir),
    sliceCoord(A_sliceCoord)
  {
    for(int i=0;i<dim;++i) locCoord[i]=0;
    locCoord[sliceDir]=sliceCoord;
  }
  void operator++(){
//std::cout << "operator++" << locCoord[0] << " " << locCoord[1] << " " << locCoord[2] << std::endl;
    for(uint i=0;i<dim;++i){
//std::cout << "operator++ i=" << i << std::endl;
      if(i==sliceDir) continue;
//std::cout << "operator++ i!=sliceDir=" << sliceDir << std::endl;
      if(++locCoord[i]<3){
//std::cout << "operator++ locCoord increased to " << locCoord[0] << " " << locCoord[1] << " " << locCoord[2] << std::endl;
//std::cout << "operator++ increasing at " << sliceDir << " by " << (this->baseEuclBitSet()->neighbStride[i]) << " + " << ((i>sliceDir)*2*this->baseEuclBitSet()->bitStride[sliceDir]) << "," << this->baseEuclBitSet()->bitStride[sliceDir] << std::endl;
//std::cout << "operator++ increasing by " << (this->baseEuclBitSet()->neighbStride[i]+(i>sliceDir)*2*this->baseEuclBitSet()->bitStride[sliceDir]) << std::endl;
//        (*this)+=this->baseEuclBitSet()->neighbStride[i]+(i>sliceDir)*2*this->baseEuclBitSet()->bitStride[sliceDir];
        (*this)+=this->baseEuclBitSet()->neighbSliceStride[sliceDir][i];
//std::cout << "operator++ Leaving at " << *this << std::endl;
        return;
      }else{
        locCoord[i]=0;
      }
    }
    this->BitIterator::operator++();
//std::cout << "operator++ Leaving(2) at " << *this << std::endl;
  }
  int getLocCoord(int i){
    return locCoord[i];
  }
};
#endif //_EUCLBITSETT_NEIGHBSLICEITERATOR_H_


/*=======================================================================*/
/// @}

