/////////////////////////////////////////////////////////////////////////////
/// @file AcyclicShaving
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-16
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_ACYCLICSHAVING_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_ACYCLICSHAVING_H

// #include
#include "AcyclicNeighbourhood.h"
#include <capd/complex/ComplexTraits.h>


namespace capd
{
  namespace complex
  {


    template<typename Complex, size_t DIM>
    class AcyclicShaving
    {

    public:
      typedef typename ComplexTraits<Complex>::Cell Cell;
      typedef typename Cell::Id Id;

      AcyclicShaving(Complex& complex, size_t dim=DIM): _complex(complex), _dim(dim),
                                                        _acyclicNeighbourhood(_complex)
      {}

      size_t operator()()
      {
        size_t reducedCounter = 0;
        bool reduced = true;

        while (reduced) {
          reduced = false;

          for (auto cell : _complex.iterators(1).dimCells(_dim)) {
            if ((*this)(cell)) {
              reduced = true;
              ++reducedCounter;
            }
          }
        }

        return reducedCounter;
      }

      bool operator()(Cell& cell)
      {
        if (_acyclicNeighbourhood(cell)) {
          cell.setColor(2);
          return true;
        }
        return false;
      }

      virtual ~AcyclicShaving() {}

    private:
      Complex& _complex;
      size_t _dim;
      AcyclicNeighbourhood<Complex, DIM> _acyclicNeighbourhood;
    };

    template<size_t DIM, typename Complex>
    size_t DoAcyclicShaving(Complex& complex)
    {
      AcyclicShaving<Complex, DIM> algo(complex, DIM);
      return algo();
    }

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_ACYCLICSHAVING_H
