/////////////////////////////////////////////////////////////////////////////
/// @file ArrayCubicalHomology
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-18
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_ARRAYCUBICALHOMOLOGY_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_ARRAYCUBICALHOMOLOGY_H

// #include

#include <capd/complex/BettiNumbers.h>
#include "CubicalComplexFromArray.h"

namespace capd
{
  namespace complex
  {

    template<typename Data, size_t DIM, typename Scalar, typename Int>
    class ArrayCubicalHomology
    {

    public:
      typedef boost::multi_array<Data, DIM> Array;

      explicit ArrayCubicalHomology(const Array& array):
        _array(array)
      {}

      std::vector<Int> operator()() const
      {
        auto complex = BuildCubicalComplexFromArray(_array);
        return (*this)(*complex);
      }

    private:
      template<typename Complex>
      std::vector<Int> operator()(Complex& complex) const
      {
        BettiNumbers<Complex, Scalar, Int> betti(complex);
        return betti();
      }

      const Array& _array;
    };


    template<typename Scalar, typename Int, typename Data, size_t DIM>
    std::vector<Int> ComputeArrayCubicalHomology(const boost::multi_array<Data, DIM>& array)
    {
      ArrayCubicalHomology<Data, DIM, Scalar, Int> algo(array);
      return algo();
    }

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_ARRAYCUBICALHOMOLOGY_H
