/////////////////////////////////////////////////////////////////////////////
/// @file BdNeighbours
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-06
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_BDNEIGHBOURS_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_BDNEIGHBOURS_H

// #include
#include "ComplexTraits.h"
#include "IteratedBoundary.h"
#include "IteratedCoboundary.h"
#include <capd/homAux/MultiValuedTransformIterator.h>

#include <boost/iterator/filter_iterator.hpp>

#include <vector>
#include <algorithm>




namespace capd
{
  namespace complex
  {

    template<typename Complex, size_t DEPTH>
    struct BdNeighbours
    {
      typedef IteratedBoundary<Complex, DEPTH> Boundary;
      typedef IteratedCoboundary<Complex, DEPTH> Coboundary;
      typedef typename ComplexTraits<Complex>::Cell Cell;
      typedef typename Cell::Id Id;

      struct DiffId
      {
        Id id;
        DiffId(Id id=Id(0)): id(id) {}

        template<typename CellT> //Cell ConstCell
        bool operator()(const CellT& cell) const
        {
          return id != cell.getId();
        }
      };

      typedef homAux::MultiValuedTransformRange<Coboundary, typename Boundary::result_type::iterator> Range;
      typedef boost::filter_iterator<DiffId, typename Range::iterator> iterator;
      typedef boost::filter_iterator<DiffId, typename Range::const_iterator> const_iterator;

      explicit BdNeighbours(Complex& complex, const Cell& arg, size_t depth=DEPTH):
        _complex(complex), _depth(depth), _bd(_complex, _depth), _argBd(_bd(arg)),
        _result(_argBd.begin(), _argBd.end(), Coboundary(_complex, _depth)),
        _diffId(arg.getId())
      {}

      iterator begin() { return iterator(_diffId, _result.begin(), _result.end()); }
      const_iterator begin() const { return const_iterator(_diffId, _result.begin(), _result.end()); }
      iterator end() { return iterator(_diffId, _result.end(), _result.end()); }
      const_iterator end() const { return const_iterator(_diffId, _result.end(), _result.end()); }

    private:
      Complex& _complex;
      size_t _depth;
      Boundary _bd;
      typename Boundary::result_type _argBd;
      Range _result;
      DiffId _diffId;
    };

    template<size_t DEPTH, typename Complex>
    BdNeighbours<Complex, DEPTH> makeBdNeighbours(Complex& complex, const typename BdNeighbours<Complex, DEPTH>::Cell& cell)
    {
      return BdNeighbours<Complex, DEPTH>(complex, cell, DEPTH);
    }

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_BDNEIGHBOURS_H
