/////////////////////////////////////////////////////////////////////////////
/// @file BuildFiltration
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-09-20
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_BUILDFILTRATION_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_BUILDFILTRATION_H

// #include
#include <vector>
#include <utility>
#include <stddef.h>

namespace capd
{
  namespace complex
  {

    template<typename Complex>
    class BuildFiltration
    {

    public:
      typedef typename Complex::Id Id;
      typedef typename Complex::Dim Dim;
      typedef typename Complex::Cell Cell;

      typedef std::vector<size_t> Bd;
      typedef std::pair<Dim, Bd> FiltrationElement;
      typedef std::vector<FiltrationElement> Filtration;

      explicit BuildFiltration(Complex& complex):
        _complex(complex) {}

      virtual ~BuildFiltration() {}

      void operator()();
      const Filtration& filtration() const { return _filtration; }
      Cell operator[](size_t filtrationPos) { return _complex[_positionToId[filtrationPos]]; }

    private:
      Complex& _complex;
      Filtration _filtration;
      std::vector<Id> _positionToId;
      std::vector<size_t> _idToPosition;
    };


  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_BUILDFILTRATION_H
