/////////////////////////////////////////////////////////////////////////////
/// @file BuildFiltration
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-09-20
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_BUILDFILTRATION_HPP
#define CAPD_FILE_CAPDREDHOM_COMPLEX_BUILDFILTRATION_HPP

#include <capd/complex/BuildFiltration.h>
#include <boost/foreach.hpp>
#include <algorithm>
#include <map>
#include <boost/assert.hpp>
#include <capd/auxil/Logger.h>

namespace capd
{
  namespace complex
  {

    //BuildFiltration::BuildFiltration() {}

    //BuildFiltration::~BuildFiltration() {}

    template<typename Complex>
    void BuildFiltration<Complex>::operator()()
    {
      typedef std::pair<Dim, Id> CellData;
      std::vector<CellData> cellsData;


      typedef typename Complex::ColoredIterators::AllCells AllCells;
      typedef typename AllCells::iterator AllIt;
      AllCells allCells = _complex.iterators(1).allCells();

      for (AllIt it = allCells.begin(), end = allCells.end(); it != end; ++it) {
        cellsData.push_back(CellData(it->getDim(), it->getId()));
      }

      std::sort(cellsData.begin(), cellsData.end());

      _filtration.resize(cellsData.size());
      _positionToId.resize(cellsData.size());
      _idToPosition.resize(_complex.size());

      for (size_t idx = 0; idx < cellsData.size(); ++idx) {
        _positionToId[idx] = cellsData[idx].second;
        // BOOST_ASSERT(cellsData[idx].second >= 0);
        // BOOST_ASSERT(cellsData[idx].second < _idToPosition.size());
        _idToPosition[cellsData[idx].second] = idx;
      }


      for (AllIt it = allCells.begin(), end = allCells.end(); it != end; ++it) {
        typedef typename Complex::ColoredIterators::BdCells BdCells;
        typedef typename BdCells::iterator BdIt;
        BdCells bdCells = _complex.iterators(1).bdCells(*it);
        FiltrationElement& element = _filtration[_idToPosition[it->getId()]];

        BOOST_ASSERT(element.second.size() == 0);
        element.first = it->getDim();

        for (BdIt bdit = bdCells.begin(), bdend = bdCells.end(); bdit != bdend; ++bdit) {
          // BOOST_ASSERT(bdit->getId() < _idToPosition.size());
          // BOOST_ASSERT(bdit->getId() >= 0);
          element.second.push_back(_idToPosition[bdit->getId()]);
        }

      }

    }
  }
}


#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_BUILDFILTRATION_HPP
