/////////////////////////////////////////////////////////////////////////////
/// @file BuildMorseComplex
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-04-15
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_BUILDMORSECOMPLEX_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_BUILDMORSECOMPLEX_H

// #include
#include "BuildMorseGraph.h"
#include "HasseDiagramCellComplex.h"

namespace capd
{
  namespace complex
  {

    template<typename DiscreteVectorField, typename Color>
    class BuildMorseComplex
    {

    public:
      typedef capd::complex::BuildMorseGraph<DiscreteVectorField> BuildMorseGraph;
      typedef capd::complex::HasseDiagramCellComplex<typename BuildMorseGraph::Graph, Color> MorseComplex;

      BuildMorseComplex(const DiscreteVectorField& dvf):
        _dvf(dvf)
      {}

      MorseComplex operator()()
      {
        BuildMorseGraph bmg(_dvf);
        bmg();
        return BuildHasseDiagramCellComplex<Color>(bmg.graph(), false);
      }

    private:
      BuildMorseComplex(const BuildMorseComplex&);
      BuildMorseComplex& operator=(const BuildMorseComplex&);

      const DiscreteVectorField& _dvf;
    };


    template<typename Color, typename DiscreteVectorField>
    typename BuildMorseComplex<DiscreteVectorField, Color>::MorseComplex
    ConvertToMorseComplex(const DiscreteVectorField& dvf)
    {
      BuildMorseComplex<DiscreteVectorField, Color> bmc(dvf);
      return bmc();
    }

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_BUILDMORSECOMPLEX_H
