/////////////////////////////////////////////////////////////////////////////
/// @file CellsByDim
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-24
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_CELLSBYDIM_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_CELLSBYDIM_H

// #include
#include <capd/complex/ComplexFunctors.h>
#include <capd/homAux/MultiValuedTransformIterator.h>
#include <boost/iterator/counting_iterator.hpp>
#include <boost/iterator/reverse_iterator.hpp>

namespace capd
{
  namespace complex
  {


    template<typename Complex>
    struct CellsByDim:
      homAux::MultiValuedTransformRange<DimCells<Complex>,
                                        boost::counting_iterator<typename Complex::Dim> >
    {
      explicit CellsByDim(Complex& complex):
        homAux::MultiValuedTransformRange<DimCells<Complex>,
                                          boost::counting_iterator<typename Complex::Dim> >
        (boost::counting_iterator<typename Complex::Dim>(0),
         boost::counting_iterator<typename Complex::Dim>(complex.getDim() + 1),
         DimCells<Complex>(complex))
      {}
    };

    template<typename Complex>
    struct CellsByDimReversed:
      homAux::MultiValuedTransformRange<DimCellsReversed<Complex>,
                                        boost::counting_iterator<typename Complex::Dim> >
    {
      explicit CellsByDimReversed(Complex& complex):
        homAux::MultiValuedTransformRange<DimCellsReversed<Complex>,
                                          boost::counting_iterator<typename Complex::Dim> >
        (boost::counting_iterator<typename Complex::Dim>(0),
         boost::counting_iterator<typename Complex::Dim>(complex.getDim() + 1),
         DimCellsReversed<Complex>(complex))
      {}
    };

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_CELLSBYDIM_H
