/////////////////////////////////////////////////////////////////////////////
/// @file ChainMap.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-17
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_CHAINMAP_H
#define CAPD_FILE_CHAINMAP_H

#include "Chain.h"

#include <boost/foreach.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>

namespace capd
{
namespace complex
{

  template<typename DomHomologyModel, typename ImgHomologyModel, typename Function, typename Scalar = int>
  class ChainMap
  {
  public:
    typedef typename DomHomologyModel::Chain DomChain;
    typedef typename ImgHomologyModel::Chain ImgChain;

    typedef typename DomHomologyModel::Complex::Id DomId;
    typedef typename ImgHomologyModel::Complex::Id ImgId;
    typedef typename DomHomologyModel::Complex::CellCode DomCellCode;
    typedef typename ImgHomologyModel::Complex::CellCode ImgCellCode;


    ChainMap(const DomHomologyModel& domHomologyModel, const ImgHomologyModel& imgHomologyModel, const Function& function):
      _domHomologyModel(domHomologyModel),
      _imgHomologyModel(imgHomologyModel),
      _function(function)
    {}

    ImgChain operator()(const DomChain& domChain)
    {
      ImgChain result;
      (*this)(domChain, result);
      return result;
    }

    void operator()(const DomChain& domChain, ImgChain& result)
    {
      namespace bl = boost::lambda;


      BOOST_FOREACH(const typename DomChain::value_type& dom, domChain) {
	_function(_domHomologyModel.complex()[dom.first].code(), dom.second, bl::bind(&ChainMap::processDomCell, this, bl::_1, bl::_2,  boost::ref(result)));
      }
    }

  private:
    void processDomCell(const ImgCellCode& imgCell, const Scalar& s, ImgChain& result)
    {
      result += typename ImgChain::value_type(_imgHomologyModel.complex()[imgCell].getId(), s);
    }

    const DomHomologyModel& _domHomologyModel;
    const ImgHomologyModel& _imgHomologyModel;
    Function _function;
  };


  template<typename DomHomologyModel, typename ImgHomologyModel, typename Function>
  ChainMap<DomHomologyModel, ImgHomologyModel, Function> makeChainMap(const DomHomologyModel& domHomologyModel,
							  const ImgHomologyModel& imgHomologyModel,
							  Function function)
   {
     return ChainMap<DomHomologyModel, ImgHomologyModel, Function>(domHomologyModel, imgHomologyModel, function);
  }


}
}

#endif // CAPD_FILE_CHAINMAP_H
