/////////////////////////////////////////////////////////////////////////////
/// @file ComplexTraits
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-09
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_COMPLEXTRAITS_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_COMPLEXTRAITS_H

// #include
#include "Tags.h"
#include <boost/mpl/contains.hpp>
#include <boost/mpl/bool.hpp>
#include <boost/mpl/integral_c.hpp>
#include <limits>
#include <boost/mpl/greater.hpp>
#include <boost/utility/enable_if.hpp>


namespace capd
{
  namespace complex
  {

    template<typename Complex>
    struct ComplexTraits
    {
      typedef typename Complex::Cell Cell;
      typedef typename Complex::Iterators Iterators;
      typedef typename Complex::ColoredIterators ColoredIterators;
    };

    template<typename Complex>
    struct ComplexTraits<const Complex>
    {
      typedef typename Complex::ConstCell Cell;
      typedef typename Complex::ConstIterators Iterators;
      typedef typename Complex::ColoredConstIterators ColoredIterators;
    };

    template<typename Complex>
    struct IsRegularCubical: boost::mpl::contains<typename Complex::Tags, RegularCubicalTag>
    {};

    template<typename Complex, typename Enabled=void>
    struct NumberOfColors: boost::mpl::integral_c<size_t, std::numeric_limits<typename Complex::Color>::max()>
    {};

    template<typename Complex>
    struct NumberOfColors<Complex,
                          typename boost::enable_if_c<Complex::ColorValues >= 0>::type>:
    boost::mpl::integral_c<size_t, Complex::ColorValues>
    {};

    template<typename Complex>
    struct IsColorable: boost::mpl::greater<NumberOfColors<Complex>,
                                            boost::mpl::integral_c<size_t, 1> >
    {};

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_COMPLEXTRAITS_H
