/////////////////////////////////////////////////////////////////////////////
/// @file ComputeHomologyOrchestrator.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-05-26
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_COMPUTEHOMOLOGYORCHESTRATOR_H
#define CAPD_FILE_COMPUTEHOMOLOGYORCHESTRATOR_H

#include <vector>
#include <capd/homAlgebra/HomologySignature.h>
#include <capd/auxil/Logger.h>


namespace capd
{
namespace complex
{

  template<typename ComplexT, typename ScalarT, typename IntT>
  class ComputeHomologyOrchestrator
  {

  public:
    typedef ComplexT Complex;
    typedef ScalarT Scalar;
    typedef IntT Int;
    typedef std::vector<IntT> result_type;

    ComputeHomologyOrchestrator(Complex& complex, bool reductions, bool coreductions, bool morse, bool useAlgebraicReductions):
      _complex(complex),
      _reductions(reductions),
      _coreductions(coreductions),
      _morse(morse),
      _useAlgebraicReductions(useAlgebraicReductions),
      _squeezeFactor(1)
    {}


    result_type operator()(); //TODO Should return Homology signature
    HomologySignature<ScalarT, IntT> signature() const { return _signature; }

    void squeezeFactor(double s) { _squeezeFactor = s; }

    ~ComputeHomologyOrchestrator()
    {}

  private:
    Complex& _complex;
    bool _reductions;
    bool _coreductions;
    bool _morse;
    bool _useAlgebraicReductions;
    double _squeezeFactor;
    HomologySignature<ScalarT, IntT> _signature;
    CAPD_CLASS_LOGGER;
  };


}
}

#endif // CAPD_FILE_COMPUTEHOMOLOGYORCHESTRATOR_H
