/////////////////////////////////////////////////////////////////////////////
/// @file ComputeHomologySignature
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-09-20
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_COMPUTEHOMOLOGYSIGNATURE_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_COMPUTEHOMOLOGYSIGNATURE_H

// #include
#include <capd/homAlgebra/HomologySignature.h>
#include <capd/auxil/Logger.h>

class Z2;


namespace capd
{
  namespace complex
  {

    template<typename ComplexT, typename ScalarT, typename IntT>
    class ComputeHomologySignature
    {

    public:
      typedef ComplexT Complex;
      typedef ScalarT Scalar;
      typedef IntT Int;
      typedef HomologySignature<ScalarT, IntT> result_type;

      explicit ComputeHomologySignature(Complex& complex, bool useAlgebraicReductions):
        _complex(complex),
        _useAlgebraicReductions(useAlgebraicReductions)
      {}

      virtual ~ComputeHomologySignature() {}

      result_type operator()();

    private:
      Complex& _complex;
      const bool _useAlgebraicReductions;
      CAPD_CLASS_LOGGER;
    };



   template<typename ComplexT, typename IntT>
   class ComputeHomologySignature<ComplexT, Z2, IntT>
   {

   public:
     typedef ComplexT Complex;
     typedef Z2 Scalar;
     typedef IntT Int;
     typedef HomologySignature<Scalar, Int> result_type;

     explicit ComputeHomologySignature(Complex& complex, bool useAlgebraicReductions):
       _complex(complex),
       _useAlgebraicReductions(useAlgebraicReductions)
     {}

     virtual ~ComputeHomologySignature() {}

     result_type operator()();

   private:
     Complex& _complex;
     const bool _useAlgebraicReductions;
     CAPD_CLASS_LOGGER;
   };




  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_COMPUTEHOMOLOGYSIGNATURE_H
