/////////////////////////////////////////////////////////////////////////////
/// @file CoreductionPairProjectionChainModel.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-30
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_COREDUCTIONPAIRPROJECTIONCHAINMODEL_H
#define CAPD_FILE_COREDUCTIONPAIRPROJECTIONCHAINMODEL_H

#include "Chain.h"
#include "SReductionPair.h"

#include <boost/optional.hpp>

namespace capd
{
namespace complex
{

  template<typename Complex, typename Scalar, typename CoreductionPairs>
  class CoreductionPairProjectionChainModel
  {

  public:
    typedef void result_type;

    typedef typename Complex::Id Id;
    typedef typename Complex::Dim Dim;
    typedef capd::complex::Chain<Complex, Scalar> Chain;
    typedef SReductionPair<Complex> ReductionPair;

    explicit CoreductionPairProjectionChainModel(const Complex& complex,
						 const CoreductionPairs& pairs):
      _complex(complex), _pairs(pairs)
    {}

    void operator()(Chain& chain)
    {
      const Dim chainDim = _complex[chain.begin()->first].getDim();

      if (chainDim != 0) {
	projectNonZeroDim(chain, chainDim);
      } else {
	projectZeroDim(chain);
      }
    }

  private:

    void projectNonZeroDim(Chain& chain, Dim chainDim)
    {
      typedef typename Chain::value_type ChV;
      BOOST_FOREACH(const ReductionPair& pair, _pairs) {
	if (chainDim == pair.dim) {
	  chain.insert(ChV(pair.coface, Scalar(0)));
	} else {
	  chain.insert(ChV(pair.face, Scalar(0)));
	}
      }

      chain.compact();
    }

    void projectZeroDim(Chain& chain)
    {
      typedef typename Chain::value_type ChV;
      Chain result;

      BOOST_FOREACH(const ChV& chv, chain) {
	boost::optional<Id> lastExctracted;

	BOOST_FOREACH(const ReductionPair& pair, _pairs) {
	  if (pair.dim == 0) {
	    lastExctracted = pair.coface;
	  }

	  if (pair.face == chv.first) {
	    break;
	  }
	}

	if (lastExctracted) {
	  result += ChV(*lastExctracted, chv.second);
	}
      }

      result.compact();
      chain = result;
    }


    Id projectZeroDim(Id id)
    {
    }

    const Complex& _complex;
    const CoreductionPairs& _pairs;
  };

  template<typename Scalar, typename Complex, typename CoreductionPairs>
  CoreductionPairProjectionChainModel<Complex, Scalar, CoreductionPairs>
  makeCoreductionPairProjectionChainModel(const Complex& complex, const CoreductionPairs& pairs)
  {
    return CoreductionPairProjectionChainModel<Complex, Scalar, CoreductionPairs>(complex, pairs);
  }

}
}

#endif // CAPD_FILE_COREDUCTIONPAIRPROJECTIONCHAINMODEL_H
