/////////////////////////////////////////////////////////////////////////////
/// @file CubicalComplexFromArray
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-18
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_CUBICALCOMPLEXFROMARRAY_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_CUBICALCOMPLEXFROMARRAY_H

// #include
#include <capd/complex/CubCellComplex.h>
#include <boost/multi_array.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/make_shared.hpp>

namespace capd
{
  namespace complex
  {

    template<typename Data, size_t DIM>
    class CubicalComplexFromArray
    {

    public:
      typedef boost::multi_array<Data, DIM> Array;
      typedef boost::array<typename Array::index, DIM> Index;

      typedef typename DenseCubCellComplex<DIM, Data>::type Complex;
      typedef typename Complex::FullCube FullCube;

      typedef boost::shared_ptr<Complex> result_type;

      explicit CubicalComplexFromArray(const Array& array):
        _array(array)
      {}

      result_type operator()() const
      {
        result_type complex = boost::make_shared<Complex>(std::vector<size_t>(_array.shape(), _array.shape() + _array.num_dimensions()));

        const Data* element = _array.data();
        typename Complex::FullCube fullCube = complex->fullCube();

        for (size_t i = 0; i < _array.num_elements(); i++) {
            getIndex(element, fullCube);
            auto cell = complex->insert((*complex)(fullCube));
            cell.setColor(*element);
            complex->fillWithBoundary(cell, cell.getColor());
            ++element;
        }

        //complex->fillWithBoundaries();
        return complex;
      }

    private:
      void getIndex(const Data* requestedElement, FullCube& fullCube) const
      {
        for (size_t direction = 0; direction < Array::dimensionality; ++direction) {
          size_t offset = requestedElement - _array.origin();
          fullCube[direction] = offset / _array.strides()[direction] % _array.shape()[direction] +  _array.index_bases()[direction];
        }
      }

      const Array& _array;
    };


    template<typename Data, size_t DIM>
    typename CubicalComplexFromArray<Data, DIM>::result_type BuildCubicalComplexFromArray(const boost::multi_array<Data, DIM>& array)
    {
      CubicalComplexFromArray<Data, DIM> algo(array);
      return algo();
    }

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_CUBICALCOMPLEXFROMARRAY_H
