/////////////////////////////////////////////////////////////////////////////
/// @file CubicalComplexFullCubesBounds
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-19
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_CUBICALCOMPLEXFULLCUBESBOUNDS_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_CUBICALCOMPLEXFULLCUBESBOUNDS_H

// #include
#include <limits>

namespace capd
{
  namespace complex
  {

    template<typename Coordinate>
    struct CubicalComplexFullCubesBounds
    {
      //typedef typename Cell::Code::value_type::first Coordinate;
      typedef std::pair<Coordinate, Coordinate> Bound;
      typedef bool result_type;

      CubicalComplexFullCubesBounds()
      {}

      template<typename Complex>
      explicit CubicalComplexFullCubesBounds(const Complex& complex):
        _bounds(complex.getDim(), EmptyBound())
      {
        adjust(complex);
      }

      template<typename Cell>
      bool operator()(const Cell& cell) const
      {
        bool inBounds = true;
        auto code = cell.code();

        for (size_t idx = 0; inBounds && idx < _bounds.size(); ++idx) {
          inBounds &= (_bounds[idx].first <= code[idx].first
                       &&
                       (code[idx].first + code[idx].second) <= _bounds[idx].second);
        }

        return inBounds;
      }

      template<typename Complex>
      bool adjust(const Complex& complex)
      {
        const size_t dim = complex.getDim();
        BOOST_ASSERT(dim == _bounds.size());

        std::vector<Bound> newBounds(dim, EmptyBound());

        for (auto cell : complex.iterators(1).dimCells(dim)) {
          auto code = cell.code();
          for (size_t idx = 0; idx < dim; ++idx) {
            BOOST_ASSERT(code[idx].second == true);

            if (newBounds[idx].first > code[idx].first) {
              newBounds[idx].first = code[idx].first;
            }

            if (newBounds[idx].second < code[idx].first + code[idx].second) {
              newBounds[idx].second = code[idx].first + code[idx].second;
            }
          }
        }

        bool progress = false;
        for (size_t idx = 0; ! progress && idx < dim; ++idx) {
          progress |= ( _bounds[idx].first < newBounds[idx].first)
            || (newBounds[idx].second < _bounds[idx].second);
        }

        _bounds = newBounds;
        return progress;
      }

    private:

      static Bound EmptyBound()
      {
        return Bound(std::numeric_limits<Coordinate>::max(),
                     std::numeric_limits<Coordinate>::min());
      }

      std::vector<Bound> _bounds;
    };

    template<typename Complex>
    CubicalComplexFullCubesBounds<typename Complex::CellCode::value_type::first_type>
    makeCubicalComplexFullCubesBounds(const Complex& complex)
    {
      return CubicalComplexFullCubesBounds<typename Complex::CellCode::value_type::first_type>(complex);
    }

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_CUBICALCOMPLEXFULLCUBESBOUNDS_H
