#ifndef _CAPD_REDHOM_COMPLEX_CUB_DEFAULT_REDUCE_STRATEGY_HPP_
#define _CAPD_REDHOM_COMPLEX_CUB_DEFAULT_REDUCE_STRATEGY_HPP_


#include <boost/optional.hpp>
#include <boost/foreach.hpp>
#include <utility>
#include <capd/auxil/Logger.h>


namespace capd
{
    namespace complex
    {


template<typename SComplexT>
class DefaultReduceStrategyTraits {
public:

    template<typename ArgT>
    struct GetReductionPair : public std::unary_function<const ArgT&,
                boost::optional<typename SComplexT::Cell> > {};

    template<typename ArgT>
    struct GetCoreductionPair : public std::unary_function<const ArgT&,
                boost::optional<typename SComplexT::Cell> > {};

    struct Extract
    {
        typedef boost::optional<typename SComplexT::Cell >  result_type;
    };

};

template<typename SComplexT>
class DefaultReduceStrategyBase {

public:
  typedef SComplexT SComplex;
  typedef DefaultReduceStrategyTraits<SComplex> Traits;
  typedef typename SComplex::Cell Cell;


  DefaultReduceStrategyBase(SComplex& _complex): complex(_complex), dummyCell2(),  dummyCell3(),
                                                 extracted(false),
                                                 _closedComplex(_complex.isClosed())
  {}

  SComplex& getComplex() const {
	 return complex;
  }

  template<typename ImplT>
  static bool reduced(const ImplT& cell) {
  	 return cell.getColor() == 2;
  }

  template<typename ImplT1, typename ImplT2>
  static void coreduce(ImplT1& a, ImplT2& b)  {
  	 a.setColor(2);
  	 b.setColor(2);
  }

  template<typename ImplT1, typename ImplT2>
  static void reduce(ImplT1& a, ImplT2& b)  {
  	 a.setColor(2);
  	 b.setColor(2);
  }

  template<typename ImplT>
  static void reduce(ImplT& cell) {
	 cell.setColor(2);
  }

  typename Traits::Extract::result_type extract() {
    if (_closedComplex) {
      typename SComplex::ColoredIterators::DimCells dimCells = complex.iterators(1).dimCells(0);
      typename SComplex::ColoredIterators::DimCells::iterator end = dimCells.end(),
        it = dimCells.begin();

      if (it != end) {
        return typename Traits::Extract::result_type::value_type(*it);
      } else {
        CAPD_DEBUG_TMPL("No more 0-dim cells, nothing to extract.");
      }
    } else {
      CAPD_DEBUG_TMPL("Complex not closed, cannot extract.");
    }
    return typename Traits::Extract::result_type();
  }

  template<typename Inserter>
  bool forceCoreductionPair(Inserter inserter) {
    typedef typename SComplex::ColoredIterators::AllCells AllCells;
    AllCells allCells = complex.iterators(1).allCells();
    bool result = false;

    for (typename AllCells::iterator it = allCells.begin(), end = allCells.end();
	 it != end; ++it) {
      if (getCoreductionPair(*it)) {
	inserter(*it);
	result = true;
      }
    }
    return result;
  }

  template<typename ArgT>
  typename Traits::template GetCoreductionPair<ArgT>::result_type
  getCoreductionPair(const ArgT& cell)
  {
    int times = 0;
    BOOST_FOREACH(typename SComplex::ColoredIterators::BdCells::iterator::value_type v,
                  complex.iterators(1).bdCells(cell)) {
      if (times == 0) {
        dummyCell3 = v;
      }
      ++times;
      if (times == 2) {
        break;
      }
    }

    if (times == 1) {
      return typename Traits::template GetCoreductionPair<ArgT>::result_type(dummyCell3);
    }
    return typename Traits::template GetCoreductionPair<ArgT>::result_type();
  }

  template<typename ArgT>
  typename Traits::template GetCoreductionPair<ArgT>::result_type
  getReductionPair(const ArgT& cell)
  {
  	 int times = 0;
  	 BOOST_FOREACH(typename SComplex::ColoredIterators::CbdCells::iterator::value_type v,
  						complex.iterators(1).cbdCells(cell)) {
  		if (times == 0) {
  		  dummyCell2 = v;
  		}
  		++times;
  		if (times == 2) {
  		  break;
  		}
  	 }

  	 if (times == 1) {
  		return typename Traits::template GetReductionPair<ArgT >::result_type(dummyCell2);
  	 }
  	 return typename Traits::template GetReductionPair<ArgT>::result_type();
  }

protected:
  SComplex& complex;
  Cell dummyCell2, dummyCell3;
  bool extracted;
  bool _closedComplex;

  CAPD_CLASS_LOGGER;
};

template<typename SComplexT>
class DefaultReduceStrategy: public DefaultReduceStrategyBase<SComplexT> {
public:
  DefaultReduceStrategy(SComplexT& _complex): DefaultReduceStrategyBase<SComplexT>(_complex) {}

};

  }
}

#include "DefaultReduceStrategy_CubSComplex.hpp"

#endif // _CAPD_REDHOM_COMPLEX_CUB_DEFAULT_REDUCE_STRATEGY_HPP_
