#ifndef _CAPD_REDHOM_COMPLEX_CUB_DEFAULT_REDUCE_STRATEGY_CUBSCOMPLEX_HPP_
#define _CAPD_REDHOM_COMPLEX_CUB_DEFAULT_REDUCE_STRATEGY_CUBSCOMPLEX_HPP_

#include "CubSComplex.h"
#include "DefaultReduceStrategy.hpp"

namespace capd
{
    namespace complex
    {


template<int DIM>
class DefaultReduceStrategy<capd::complex::CubSComplex<DIM> >: public DefaultReduceStrategyBase<capd::complex::CubSComplex<DIM> >
{
protected:
    typename CubSComplex<DIM>::DynamicCell dynamicCell;
    using DefaultReduceStrategyBase<CubSComplex<DIM> >::complex;
public:
    typedef typename DefaultReduceStrategyBase<CubSComplex<DIM> >::Traits Traits;

    DefaultReduceStrategy(CubSComplex<DIM>& complex): DefaultReduceStrategyBase<CubSComplex<DIM> >(complex), dynamicCell(complex) {}

    template<typename ArgT>
    typename Traits::template GetCoreductionPair<ArgT>::result_type
    getCoreductionPair(const ArgT& cell)
    {
        if (complex.getUniqueFace(cell, dynamicCell))
        {
            return typename Traits::template GetCoreductionPair<ArgT>::result_type(dynamicCell.getImpl());
        }
        else
        {
            return typename Traits::template GetCoreductionPair<ArgT>::result_type();
        }
    }

    template<typename ArgT>
    typename Traits::template GetReductionPair<ArgT>::result_type
    getReductionPair(const ArgT& cell)
    {
        if (complex.getUniqueCoFace(cell, dynamicCell))
        {
            return typename Traits::template GetReductionPair<ArgT>::result_type(dynamicCell.getImpl());
        }
        else
        {
            return typename Traits::template GetReductionPair<ArgT>::result_type();
        }
    }

    size_t getMaxDim()
    {
        return complex.getDim();
    }
};


  }
}

#include "DefaultReduceStrategyTraits_CubSComplex.hpp"

#endif // _CAPD_REDHOM_COMPLEX_CUB_DEFAULT_REDUCE_STRATEGY_CUBSCOMPLEX_HPP_
