/////////////////////////////////////////////////////////////////////////////
/// @file EulerCharacteristic
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-08-18
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_EULERCHARACTERISTIC_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_EULERCHARACTERISTIC_H

// #include

namespace capd
{
  namespace complex
  {

    template<typename Complex>
    class EulerCharacteristic
    {

    public:
      EulerCharacteristic(Complex& complex): _complex(complex) {}

      long operator()()
      {
        long result = 0;

        typedef typename Complex::Iterators::AllCells::iterator AllIt;
        typename Complex::Iterators::AllCells allCells = _complex.iterators().allCells();
        for (AllIt it = allCells.begin(), end = allCells.end();
             it != end; ++it) {
          result += ( it->getDim() % 2 == 0 ? 1 : -1 );
        }

        return result;
      }
    private:
      Complex& _complex;
    };


  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_EULERCHARACTERISTIC_H
