/////////////////////////////////////////////////////////////////////////////
/// @file Excision.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-05-23
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_EXCISION_H
#define CAPD_FILE_EXCISION_H

#include <boost/optional.hpp>
#include <boost/lambda/lambda.hpp>
namespace capd
{
namespace complex
{

  template<typename SComplexT>
  class Excision
  {

  public:
    typedef SComplexT SComplex;
    typedef typename SComplex::CellCode CellCode;
    typedef typename SComplex::Cell Cell;
    typedef typename SComplex::Id Id;


    Excision(SComplex& complex): _complex(complex)
    {
      _complex.setClosed(false);
    }

    void operator()(const typename SComplex::CellCode& cellCode)
    {
      boost::optional<typename SComplex::Cell> cellInComplex = _complex.find(cellCode);
      if (cellInComplex) {
	cellInComplex->setColor(2);
      }
    }

    void operator()(SComplex& subspace)
    {
      typedef typename SComplex::ColoredIterators::AllCells AllCells;
      typedef typename AllCells::iterator AllIt;

      AllCells allCells = subspace.iterators(1).allCells();
      for (AllIt it = allCells.begin(), end = allCells.end(); it != end; ++it) {
	(*this)(it->code());
      }
    }

  private:
    SComplex& _complex;
  };


}
}

#endif // CAPD_FILE_EXCISION_H
