/////////////////////////////////////////////////////////////////////////////
/// @file FilteredIteratorFamily
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-13
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_FILTEREDITERATORFAMILY_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_FILTEREDITERATORFAMILY_H

// #include
#include <boost/iterator/filter_iterator.hpp>
#include <boost/range/iterator_range.hpp>
#include <boost/typeof/typeof.hpp>

namespace capd
{
  namespace complex
  {

    template<typename IteratorFamily, typename Filter>
    class FilteredIteratorFamily
    {
    public:
      typedef typename IteratorFamily::Complex Complex;
      typedef typename Complex::Cell Cell;
      typedef typename Complex::ConstCell ConstCell;
    private:

      typedef boost::filter_iterator<Filter, typename IteratorFamily::AllCells::iterator> AllIterator;
      typedef boost::filter_iterator<Filter, typename IteratorFamily::DimCells::iterator> DimIterator;
      typedef boost::filter_iterator<Filter, typename IteratorFamily::BdCells::iterator> BdIterator;
      typedef boost::filter_iterator<Filter, typename IteratorFamily::CbdCells::iterator> CbdIterator;

    public:

      typedef boost::iterator_range<AllIterator> AllCells;
      typedef boost::iterator_range<DimIterator> DimCells;
      typedef boost::iterator_range<BdIterator> BdCells;
      typedef boost::iterator_range<CbdIterator> CbdCells;

      FilteredIteratorFamily()
      {}

      explicit FilteredIteratorFamily(IteratorFamily iteratorFamily, Filter filter=Filter()):
        _iteratorFamily(iteratorFamily), _filter(filter)
      {}

      AllCells allCells()
      {
        BOOST_AUTO(cells, _iteratorFamily.allCells());
        return AllCells(AllIterator(_filter, cells.begin(), cells.end()),
                        AllIterator(_filter, cells.end(), cells.end()));
      }

      DimCells dimCells(const typename Complex::Dim& dim)
      {
        BOOST_AUTO(cells, _iteratorFamily.dimCells(dim));
        return DimCells(DimIterator(_filter, cells.begin(), cells.end()),
                        DimIterator(_filter, cells.end(), cells.end()));
      }

      BdCells bdCells(const ConstCell& cell)
      {
        BOOST_AUTO(cells, _iteratorFamily.bdCells(cell));
        return BdCells(BdIterator(_filter, cells.begin(), cells.end()),
                       BdIterator(_filter, cells.end(), cells.end()));
      }

      CbdCells cbdCells(const ConstCell& cell)
      {
        BOOST_AUTO(cells, _iteratorFamily.cbdCells(cell));
        return CbdCells(CbdIterator(_filter, cells.begin(), cells.end()),
                        CbdIterator(_filter, cells.end(), cells.end()));
      }

    private:
      IteratorFamily _iteratorFamily;
      Filter _filter;
    };


  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_FILTEREDITERATORFAMILY_H
