/////////////////////////////////////////////////////////////////////////////
/// @file FundamentalGroupOrderedBd
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-04-15
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_FUNDAMENTALGROUPORDEREDBD_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_FUNDAMENTALGROUPORDEREDBD_H


// #include
#include <stdexcept>
#include <capd/complex/ComplexTraits.h>
#include <boost/utility/enable_if.hpp>
#include <capd/auxil/ignoreUnused.h>
#include <boost/assert.hpp>

namespace capd
{
  namespace complex
  {

    // This is a refactorization of void HomologyHelpers<Traits>::Reorder2Boundary() written by Piotr Brendel

    template<typename Complex, typename Id, typename Scalar>
    void FundamentalGroupOrderedBd(std::list<std::pair<Id, Scalar> >& boundary,
                                   typename boost::disable_if<IsRegularCubical<Complex> >::type* p=0)
    {
      capd::auxil::ignoreUnused(boundary);
      capd::auxil::ignoreUnused(p);
      // in general case does nothing
    }

    template<typename Complex, typename Id, typename Scalar>
    void FundamentalGroupOrderedBd(std::list<std::pair<Id, Scalar> >& boundary,
                                   typename boost::enable_if<IsRegularCubical<Complex> >::type* p=0)
    {
      capd::auxil::ignoreUnused(p);
      typedef std::pair<Id, int> Cell;
      typedef std::list<Cell> CellsList;

      // reordering (1, 3, 4, 2) -> (1, 2, 3, 4)
      BOOST_ASSERT(boundary.size() == 4);
      // remember last cell and remove ir
      typename CellsList::iterator it = boundary.end();
      it--;
      Cell cell = *it;
      boundary.erase(it);
      // skip first cell and insert remembered cell at second position
      it = boundary.begin();
      it++;
      boundary.insert(it, cell);
    }


  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_FUNDAMENTALGROUPORDEREDBD_H
