/////////////////////////////////////////////////////////////////////////////
/// @file GeometricBoundary
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-10
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_GEOMETRICBOUNDARY_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_GEOMETRICBOUNDARY_H

// #include
#include "IteratedBoundary.h"
#include "ComplexTraits.h"
#include <vector>
#include <algorithm>
#include <boost/lambda/bind.hpp>

namespace capd
{
  namespace complex
  {

    template<typename Complex>
    struct GeometricBoundary
    {
      typedef std::vector<typename ComplexTraits<Complex>::Cell> result_type;
      typedef typename ComplexTraits<Complex>::Cell argument_type;

      GeometricBoundary(Complex& complex):
        _complex(complex)
      {}

      result_type operator()(const argument_type& cell)
      {
        result_type result;
        result.push_back(cell);
        auto iteratedBd = makeIteratedBoundary<1>(_complex);

        for (size_t nextIdx = 0; nextIdx < result.size(); ++nextIdx) {
          auto bd = iteratedBd(result[nextIdx]);
          result.insert(result.end(), bd.begin(), bd.end());
        }

        namespace bl = boost::lambda;
        typedef typename result_type::value_type V;
        std::sort(result.begin(), result.end());
        typename result_type::iterator it = std::unique(result.begin(), result.end(),
                                                        bl::bind(&V::getId, bl::_1) == bl::bind(&V::getId, bl::_2));
        result.resize(std::distance(result.begin(),it));
        return result;
      }

    private:
      Complex& _complex;
    };

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_GEOMETRICBOUNDARY_H
