/////////////////////////////////////////////////////////////////////////////
/// @file HasseDiagramCellComplex
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-04-03
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_HASSEDIAGRAMCELLCOMPLEX_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_HASSEDIAGRAMCELLCOMPLEX_H

// #include
#include "CellComplex.h"
#include "HasseDiagram.h"
#include "Tags.h"
#include <iosfwd>

namespace capd
{
  namespace complex
  {


    template<typename HasseDiagramT, typename ColorT>
    struct HasseDiagramMemoryModel
    {
      typedef HasseDiagramTag Tag;
      typedef HasseDiagramT HasseDiagram;
      typedef typename HasseDiagram::Index Id;
      typedef typename HasseDiagram::Dim Dim;
      typedef typename HasseDiagram::Scalar Scalar;;
      typedef ColorT Color;

      typedef HasseDiagram EnumerateCellCodes;

      template<typename Value>
      struct ValueById
      {
        typedef std::vector<Value> type;

        static type Create(size_t size)
        {
          return type(size);
        }

      };

    };

    template<typename HasseDiagramT, typename ColorT>
    class HasseDiagramCellComplex: public CellComplex<HasseDiagramMemoryModel<HasseDiagramT, ColorT> >
    {

    public:
      typedef HasseDiagramT HasseDiagram;
      typedef HasseDiagramMemoryModel<HasseDiagram, ColorT> MemoryModel;
      typedef CellComplex<MemoryModel> Base;


      HasseDiagramCellComplex(HasseDiagram& diagram, bool reference):
        Base(&diagram, reference)
      {
        insertVertices();
      }

      typedef boost::mpl::vector<typename MemoryModel::Tag> Tags;

      typedef CellComplex<MemoryModel> BaseCellComplex;
      typedef typename BaseCellComplex::Cell Cell;
      typedef typename BaseCellComplex::ConstCell ConstCell;
      typedef typename BaseCellComplex::CellCode CellCode;
      typedef typename BaseCellComplex::Dim Dim;

      typename HasseDiagram::Scalar coincidenceIndex(const ConstCell& cellA, const ConstCell& cellB) const
      {
        return this->enumerateCellCodes().coincidence(cellA.getId(), cellB.getId());
      }

    private:

      void insertVertices()
      {
        for (auto vertex : this->enumerateCellCodes().all()) {
          this->insert(CellCode(vertex));
        }
      }

      //      HasseDiagramCellComplex(const HasseDiagramCellComplex&);
      HasseDiagramCellComplex& operator=(const HasseDiagramCellComplex&);
    };

    template<typename ColorT, typename HasseDiagram>
    HasseDiagramCellComplex<HasseDiagram, ColorT>
    BuildHasseDiagramCellComplex(HasseDiagram& diagram, bool reference)
    {
      return HasseDiagramCellComplex<HasseDiagram, ColorT>(diagram, reference);
    }

    template<typename ScalarT, typename Complex>
    HasseDiagramCellComplex<HasseDiagram<typename Complex::Dim, ScalarT>, typename Complex::Color>
    ConvertToHasseDiagramCellComplex(/*const*/ Complex& complex) // should be const, but legacy implementations are broken
    {
      typedef capd::complex::HasseDiagram<typename Complex::Dim, ScalarT> HasseDiagram;
      HasseDiagram diagram(complex.size());

      for (auto cell : complex.iterators().allCells()) {
        diagram.set(cell.getId(), cell.getDim());
      }

      for (auto cell : complex.iterators().allCells()) {
        for (auto bdCell : complex.iterators().bdCells(cell)) {
          diagram.connect(cell.getId(), bdCell.getId(), complex.coincidenceIndex(cell, bdCell));
        }
      }

      auto result = BuildHasseDiagramCellComplex<typename Complex::Color>(diagram, false);
      result.resetColors(0);

      for (auto cell : complex.iterators().allCells()) {
        result[cell.getId()].setColor(cell.getColor());
      }

      return result;
    }

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_HASSEDIAGRAMCELLCOMPLEX_H
