/////////////////////////////////////////////////////////////////////////////
/// @file IteratedBoundary
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-11
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_ITERATEDBOUNDARY_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_ITERATEDBOUNDARY_H

// #include
#include "ComplexFunctors.h"
#include <capd/homAux/IteratedMultiValuedFunctor.h>


namespace capd
{
  namespace complex
  {

    template<typename Complex, size_t DEPTH>
    struct IteratedBoundary
    {
      typedef homAux::IteratedMultiValuedFunctor<DEPTH, BdCells<Complex> > IteratedBd;
      typedef typename IteratedBd::result_type result_type;
      typedef typename IteratedBd::argument_type argument_type;

      IteratedBoundary() {}

      IteratedBoundary(Complex& complex, size_t depth=DEPTH):
        _iteratedBd(depth, BdCells<Complex>(complex))
      {}

      result_type operator()(const argument_type& arg)
      {
        return _iteratedBd(arg);
      }

    private:
      IteratedBd _iteratedBd;
    };

    template<size_t DEPTH, typename Complex>
    IteratedBoundary<Complex, DEPTH> makeIteratedBoundary(Complex& complex)
    {
      return IteratedBoundary<Complex, DEPTH>(complex);
    }

    template<typename Complex>
    IteratedBoundary<Complex, 0> makeIteratedBoundary(Complex& complex, size_t depth)
    {
      return IteratedBoundary<Complex, 0>(complex, depth);
    }

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_ITERATEDBOUNDARY_H
