/////////////////////////////////////////////////////////////////////////////
/// @file IteratedCoboundary
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-11
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_ITERATEDCOBOUNDARY_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_ITERATEDCOBOUNDARY_H

// #include
#include "ComplexFunctors.h"
#include <capd/homAux/IteratedMultiValuedFunctor.h>

namespace capd
{
  namespace complex
  {

    template<typename Complex, size_t DEPTH>
    struct IteratedCoboundary
    {
      typedef homAux::IteratedMultiValuedFunctor<DEPTH, CbdCells<Complex> > IteratedCbd;
      typedef typename IteratedCbd::result_type result_type;
      typedef typename IteratedCbd::argument_type argument_type;

      IteratedCoboundary() {}

      IteratedCoboundary(Complex& complex, size_t depth=DEPTH):
        _iteratedCbd(depth, CbdCells<Complex>(complex))
      {}

      result_type operator()(const argument_type& arg)
      {
        return _iteratedCbd(arg);
      }

    private:
      IteratedCbd _iteratedCbd;
    };

    template<size_t DEPTH, typename Complex>
    IteratedCoboundary<Complex, DEPTH> makeIteratedCoboundary(Complex& complex)
    {
      return IteratedCoboundary<Complex, DEPTH>(complex);
    }

    template<typename Complex>
    IteratedCoboundary<Complex, 0> makeIteratedCoboundary(Complex& complex, size_t depth)
    {
      return IteratedCoboundary<Complex, 0>(complex, depth);
    }

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_ITERATEDCOBOUNDARY_H
