/////////////////////////////////////////////////////////////////////////////
/// @file RelativeHomology.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-05-26
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_RELATIVEHOMOLOGY_H
#define CAPD_FILE_RELATIVEHOMOLOGY_H

#include "Excision.h"

#include <boost/lambda/lambda.hpp>
#include <vector>

namespace capd
{
namespace complex
{
  template<typename ComplexT, typename ScalarT, typename IntT>
  class RelativeHomology
  {

  public:
    typedef ComplexT Complex;
    typedef ScalarT Scalar;
    typedef IntT Int;
    typedef std::vector<IntT> result_type;

    RelativeHomology(Complex& complex, Complex& subcomplex, bool keepClosed):
      _complex(complex), _subcomplex(subcomplex), _keepClosed(keepClosed),
      _reductions(true),
      _coreductions(true),
      _morse(true)
    {}

    ~RelativeHomology()
    {}

    void reductions(bool v) { _reductions = v; }
    void coreductions(bool v) { _coreductions = v; }
    void morse(bool v) { _morse = v; }

    result_type operator()();

  private:
    Complex& _complex;
    Complex& _subcomplex;
    bool _keepClosed;
    bool _reductions;
    bool _coreductions;
    bool _morse;
  };


}
}

#endif // CAPD_FILE_RELATIVEHOMOLOGY_H
