/////////////////////////////////////////////////////////////////////////////
/// @file RelativeHomology
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-23
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_RELATIVEHOMOLOGY_HPP
#define CAPD_FILE_CAPDREDHOM_COMPLEX_RELATIVEHOMOLOGY_HPP

#include <capd/complex/RelativeHomology.h>
#include <capd/complex/ComputeHomologyOrchestrator.hpp>

namespace capd
{
  namespace complex
  {

    template<typename ComplexT, typename ScalarT, typename IntT>
    std::vector<IntT> RelativeHomology<ComplexT, ScalarT, IntT>::operator()()
    {
      const bool isClosed = _complex.isClosed();
      Excision<Complex> excision(_complex);
      excision(_subcomplex);

      if (isClosed && _keepClosed) {
        _complex.setClosed(true);
      }

      ComputeHomologyOrchestrator<Complex, Scalar, Int> compute(_complex, _reductions, _coreductions, _morse, true);
      return compute();
    }
  }
}


#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_RELATIVEHOMOLOGY_HPP
