/////////////////////////////////////////////////////////////////////////////
/// @file SComplexToFreeChainComplex.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-03
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_SCOMPLEXTOFREECHAINCOMPLEX_H
#define CAPD_FILE_SCOMPLEXTOFREECHAINCOMPLEX_H


#include <capd/homAlgebra/ChainContainer.h>
#include <capd/auxil/CRef.h>

#include <boost/iterator/transform_iterator.hpp>
#include <boost/bind.hpp>
#include <boost/utility/enable_if.hpp>


#include <capd/homAlgebra/FreeModule.h>
#include <capd/redAlg/ReducibleFreeChainComplex.h>
#include <capd/homAlgebra/FreeChainComplex.h>


namespace capd
{
  namespace complex
  {


    namespace
    {
      template <typename FreeModuleT, bool reducible>
      struct SelectFreeChainComplex;

      template <typename FreeModuleT>
      struct SelectFreeChainComplex<FreeModuleT, true>
      {
        typedef ReducibleFreeChainComplex<FreeModuleT> type;

        template<typename SComplex, typename Iterator>
        CRef<type> operator()(Iterator begin, Iterator end, const SComplex& complex, bool storeReducedPairs);
      };


      template <typename FreeModuleT>
      struct SelectFreeChainComplex<FreeModuleT, false>
      {
        typedef FreeChainComplex<FreeModuleT> type;

        template<typename SComplex, typename Iterator>
        CRef<type> operator()(Iterator begin, Iterator end, const SComplex& complex, bool storeReducedPairs);
      };
    }


    template<typename SComplexT, typename ScalarT, typename IntT, typename Reducible>
    class SComplexToFreeChainComplex {
    public:
      typedef SComplexT SComplex;
      typedef FreeModule<typename SComplex::Id, capd::vectalg::Matrix<ScalarT,0,0>, IntT> FreeModuleT;
      typedef SelectFreeChainComplex<FreeModuleT, Reducible::value> SelectFreeChainComplexT;
      typedef typename SelectFreeChainComplexT::type FreeChainComplexT;
      typedef CRef<FreeChainComplexT> result_type;

    public:

      // TODO use const SComplex
      SComplexToFreeChainComplex(const SComplex& complex, bool storeReducedPairs = false): _complex(complex),
                                                                                           _storeReducedPairs(storeReducedPairs) {}

      result_type operator()();

    private:
      const SComplex& _complex;
      bool _storeReducedPairs;
    };


  }
}

#endif // CAPD_FILE_SCOMPLEXTOFREECHAINCOMPLEX_H
