/////////////////////////////////////////////////////////////////////////////
/// @file SComplexToFreeChainComplex
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-23
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/complex/SComplexToFreeChainComplex.h>
#if !defined(CAPD_FILE_CAPDREDHOM_COMPLEX_SCOMPLEXTOFREECHAINCOMPLEX_HPP) && !defined(USE_PRECOMPILED_CAPDREDHOM_COMPLEX_SCOMPLEXTOFREECHAINCOMPLEX_HPP)
#define CAPD_FILE_CAPDREDHOM_COMPLEX_SCOMPLEXTOFREECHAINCOMPLEX_HPP

#include <capd/redAlg/ReducibleFreeChainComplex.hpp>
#include <capd/homAlgebra/FreeChainComplex.hpp>
#include <capd/vectalg/Container.hpp>
#include <capd/auxil/Logger.h>

namespace capd
{
  namespace complex
  {

    namespace
    {
      template<typename SComplex, typename ScalarT>
      class GeneratorDataFromSComplex {
        SComplex& complex;
        typedef typename SComplex::Id Id;
        typedef typename SComplex::Cell Cell;

      public:

        GeneratorDataFromSComplex(SComplex& _complex):
          complex(_complex) {}

        int ownDim(const Id& generator) const { return complex[generator].getDim(); }

        int embDim(const Id& /*generator*/) const { return complex.getDim(); }

        void boundary(const Id& generator, ChainContainer<Id, ScalarT >& boundaryChain) const
        {
          Cell cell = complex[generator];
          typename SComplex::ColoredIterators::BdCells bdCells = complex.iterators(1).bdCells(cell);

          for (typename SComplex::ColoredIterators::BdCells::iterator it = bdCells.begin(),
                 end = bdCells.end(); it != end; ++it) {

            boundaryChain.insert(std::make_pair(it->getId(), complex.coincidenceIndex(cell, *it) ));
          }
        }

      };


      template <typename FreeModuleT>
      template<typename SComplex, typename Iterator>
      CRef<typename SelectFreeChainComplex<FreeModuleT, true>::type>
      SelectFreeChainComplex<FreeModuleT, true>::operator()(Iterator begin, Iterator end, const SComplex& complex, bool storeReducedPairs)
      {
        SComplex& tmpComplex = const_cast<SComplex&>(complex); //TODO const_cast
        return CRef<type>(new type(begin, end,
                                   GeneratorDataFromSComplex<SComplex, typename FreeModuleT::MatrixType::ScalarType>(tmpComplex),
                                   storeReducedPairs));
      }

      template <typename FreeModuleT>
      template<typename SComplex, typename Iterator>
      CRef<typename SelectFreeChainComplex<FreeModuleT, false>::type>
      SelectFreeChainComplex<FreeModuleT, false>::operator()(Iterator begin, Iterator end, const SComplex& complex, bool /*storeReducedPairs*/)
      {
        SComplex& tmpComplex = const_cast<SComplex&>(complex); //TODO const_cast
        return CRef<type>(new type(begin, end,
                                   GeneratorDataFromSComplex<SComplex, typename FreeModuleT::MatrixType::ScalarType>(tmpComplex)));
      }
    }


    template<typename SComplexT, typename ScalarT, typename IntT, typename Reducible>
    CRef<typename SComplexToFreeChainComplex<SComplexT, ScalarT, IntT, Reducible>::FreeChainComplexT>
    SComplexToFreeChainComplex<SComplexT, ScalarT, IntT, Reducible>::operator()()
    {
      SComplex& tmpComplex = const_cast<SComplex&>(_complex); //TODO const_cast

      namespace b = boost;
      typedef typename SComplex::ColoredIterators::AllCells AllCells;
      typedef typename AllCells::iterator::value_type Cell;
      AllCells allCells = tmpComplex.iterators(1).allCells();

      return SelectFreeChainComplexT()(b::make_transform_iterator(allCells.begin(), b::bind(&Cell::getId, _1)),
                                       b::make_transform_iterator(allCells.end(), b::bind(&Cell::getId, _1)), _complex, _storeReducedPairs);
    }

  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_SCOMPLEXTOFREECHAINCOMPLEX_HPP
