/////////////////////////////////////////////////////////////////////////////
/// @file SReductionPair.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-31
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_SREDUCTIONPAIR_H
#define CAPD_FILE_SREDUCTIONPAIR_H


namespace capd
{
namespace complex
{

  template<typename Complex>
  struct SReductionPair
  {
    typedef typename Complex::Id Id;
    typedef typename Complex::Dim Dim;

    SReductionPair(Id face, Id coface, Dim dim):
      face(face), coface(coface), dim(dim)
    {}

    bool operator<(const SReductionPair& p2) const
    {
      const SReductionPair& p1 = *this;
      return p1.face < p2.face || (p1.face == p2.face && p1.coface < p2.coface);
    }

    /*const*/ Id face;
    /*const*/ Id coface;
    /*const*/ Dim dim;
  };

  template<typename Complex>
  SReductionPair<Complex>
  makeSReductionPair(const Complex& /*complex*/, const typename Complex::Cell& face,
		     const typename Complex::Cell& coface)
  {
    return SReductionPair<Complex>(face.getId(), coface.getId(),
			  coface.getDim());
  }

  template<typename Complex>
  SReductionPair<Complex>
  makeSReductionPair(const Complex& /*complex*/, const typename Complex::Cell& extracted)
  {
    return SReductionPair<Complex>(extracted.getId(), extracted.getId(),
			  extracted.getDim());
  }

}
}

#endif // CAPD_FILE_SREDUCTIONPAIR_H
