#ifndef _SCOMPLEX_ALGS_SHAVE_HPP
#define _SCOMPLEX_ALGS_SHAVE_HPP

#include "DefaultReduceStrategy.hpp"
#include "DefaultReduceStrategy_CubSComplex.hpp"

#include <boost/shared_ptr.hpp>

template<typename StrategyT>
class ShaveAlgorithm {

public:
  typedef StrategyT Strategy;
  typedef typename Strategy::SComplex SComplex;
  typedef typename SComplex::Cell Cell;
  typedef std::pair<Cell, Cell> ReductionPair;

  ShaveAlgorithm(Strategy* _strategy): strategy(_strategy), storeReducedCells(false) {}

  ~ShaveAlgorithm() {
    delete strategy;
  }

  size_t operator()();

  void setStoreReducedCells(bool storeReducedCells)
  {
    this->storeReducedCells = storeReducedCells;
  }

  const std::deque<ReductionPair>& getReducedCells()
  {
    return reducedCells;
  }

private:
  template<typename ImplT1, typename ImplT2>
  void storeReducedPair(const ImplT1& a, const ImplT2& b)
  {
    if (storeReducedCells)
      reducedCells.push_back(std::make_pair(Cell(b), Cell(a)));
  }

  Strategy* strategy;
  bool storeReducedCells;
  std::deque<ReductionPair> reducedCells;

};

template<typename SComplex>
class ShaveAlgorithmFactory {

public:
  typedef ShaveAlgorithm<capd::complex::DefaultReduceStrategy<SComplex> > DefaultAlgorithm;

  static boost::shared_ptr<DefaultAlgorithm> createDefault(SComplex& s) {
    return boost::shared_ptr<DefaultAlgorithm>(new DefaultAlgorithm(new capd::complex::DefaultReduceStrategy<SComplex>(s)));
  }
};

template<typename StrategyT>
inline size_t ShaveAlgorithm<StrategyT>::operator()(){

  size_t reduced = 0;

  for(typename SComplex::Dim dim = strategy->getComplex().getDim(), currDim = 0; currDim <= dim; ++currDim){
	 typedef typename SComplex::ColoredIterators::DimCells::iterator DimIt;

	 typename SComplex::ColoredIterators::DimCells dimCells = strategy->getComplex().iterators(1).dimCells(dim - currDim);
	 for (DimIt it = dimCells.begin(),
	 		  end = dimCells.end();
	 		it != end; ) {
		typename StrategyT::Traits::template GetReductionPair<typename DimIt::value_type>::result_type reductionPair =
		  strategy->getReductionPair(*it);
		if (reductionPair) {
		  strategy->reduce(*reductionPair);
		  Cell v = *it;
		  ++it;
		  strategy->reduce(v);
		  reduced += 2;
        storeReducedPair(v, *reductionPair);
		} else {
		  ++it;
		}
	 }
  }

  return reduced;
}

#endif
