//#ifndef COLOREDITERATORS_HPP
// #define COLOREDITERATORS_HPP

struct ColoredIterators
{
  typedef SimplexSComplex Complex;

  const int color;
  SimplexSComplex &comp;

  ColoredIterators(SimplexSComplex &cmplx, int c) : color(c), comp(cmplx)
  {
  }

  struct DimCells
  {
    typedef Simplex::colored_iterator iterator;
    typedef iterator::value_type value_type;

    SimplexSComplex* comp;
    int color;
    int dim;

    explicit DimCells(SimplexSComplex &cmplx, int col, int d) : comp(&cmplx), color(col), dim(d) {}

    iterator begin()
    {
      return comp->dim_begin(dim, color);
    }

    iterator end()
    {
      return comp->dim_end(dim, color);
    }
  };

  DimCells dimCells(int dim)
  {
    return DimCells(comp, color, dim);
  }

  struct BdCells
  {
    typedef Simplex::border_iterator iterator;
    typedef Simplex::const_border_iterator const_iterator;
    typedef iterator::value_type value_type;

    SimplexSComplex* comp;
    Simplex const* cell; // ?? really const ??
    int color;

    explicit BdCells(SimplexSComplex &cmplx, const Simplex &c, int col) : comp(&cmplx), cell(&c), color(col) {}

    iterator begin()
    {
      BOOST_ASSERT(cell);

      return const_cast<Simplex&>(*cell).border_begin(color);
    }

    iterator end()
    {
      BOOST_ASSERT(cell);

      return const_cast<Simplex&>(*cell).border_end(color);
    }

    const_iterator begin() const
    {
      BOOST_ASSERT(cell);

      return cell->border_begin(color);
    }

    const_iterator end() const
    {
      BOOST_ASSERT(cell);

      return cell->border_end(color);
    }
  };

  template<typename ImplT>
  BdCells bdCells(const CellProxy<ImplT> &c)
  {
    return BdCells(comp, *c.getImpl(), color);
  }

  struct AllCells
  {
    typedef Simplex::colored_iterator iterator;
    typedef Simplex::const_colored_iterator const_iterator;
    typedef iterator::value_type value_type;

    SimplexSComplex* comp;
    int color;

    explicit AllCells(SimplexSComplex &cmplx, int col) : comp(&cmplx), color(col) {}

    iterator begin()
    {
      return comp->all_begin(color);
    }

    iterator end()
    {
      return comp->all_end(color);
    }

    const_iterator begin() const
    {
      return comp->all_begin(color);
    }

    const_iterator end() const
    {
      return comp->all_end(color);
    }

  };

  AllCells allCells()
  {
    return AllCells(comp, color);
  }

  struct CbdCells
  {
    typedef Simplex::coborder_iterator iterator;
    typedef Simplex::const_coborder_iterator const_iterator;
    typedef iterator::value_type value_type;

    SimplexSComplex* comp;
    Simplex const* cell; // ?? really const ??
    int color;

    explicit CbdCells(SimplexSComplex &cmplx, const Simplex &c, int col) : comp(&cmplx), cell(&c), color(col) {}

    iterator begin()
    {
      BOOST_ASSERT(cell);

      return const_cast<Simplex&>(*cell).coborder_begin(color);
    }

    iterator end()
    {
      BOOST_ASSERT(cell);

      return const_cast<Simplex&>(*cell).coborder_end(color);
    }

    const_iterator begin() const
    {
      BOOST_ASSERT(cell);

      return cell->coborder_begin(color);
    }

    const_iterator end() const
    {
      BOOST_ASSERT(cell);

      return cell->coborder_end(color);
    }
  };

  template<typename ImplT>
  CbdCells cbdCells(const CellProxy<ImplT> &c)
  {
    return CbdCells(comp, *c.getImpl(), color);
  }
};

struct Iterators
{
  typedef SimplexSComplex Complex;
    const int color;
    SimplexSComplex* comp;

    Iterators(SimplexSComplex &cmplx, int c = -1) : color(c), comp(&cmplx)
    {
    }

    struct DimCells
    {
      typedef Simplex::colored_iterator iterator;
      typedef iterator::value_type value_type;

        SimplexSComplex* comp;
        int color;
        int dim;

        explicit DimCells(SimplexSComplex &cmplx, int col, int d) : comp(&cmplx), color(col), dim(d) {}

        iterator begin()
        {
            return comp->dim_begin(dim, color);
        }

        iterator end()
        {
            return comp->dim_end(dim, color);
        }
    };

    DimCells dimCells(int dim)
    {
      return DimCells(*comp, color, dim);
    }

    struct BdCells
    {
      typedef Simplex::border_iterator iterator;
      typedef Simplex::const_border_iterator const_iterator;
      typedef iterator::value_type value_type;

        SimplexSComplex* comp;
        Simplex const *cell; // ?? really const ??
        int color;

      BdCells(): comp(0), cell(0), color(0) {}
        explicit BdCells(SimplexSComplex &cmplx, const Simplex &c, int col) : comp(&cmplx), cell(&c), color(col) {}

        iterator begin()
        {
          BOOST_ASSERT(cell);
            return const_cast<Simplex&>(*cell).border_begin(color);
        }

        iterator end()
        {
          BOOST_ASSERT(cell);
          return const_cast<Simplex&>(*cell).border_end(color);
        }

        const_iterator begin() const
        {
          BOOST_ASSERT(cell);

            return cell->border_begin(color);
        }

        const_iterator end() const
        {
          BOOST_ASSERT(cell);

          return cell->border_end(color);
        }
    };

    template<typename ImplT>
    BdCells bdCells(const CellProxy<ImplT> &c)
    {
        return BdCells(*comp, *c.getImpl(), color);
    }

    struct AllCells
    {
      typedef Simplex::colored_iterator iterator;
      typedef Simplex::const_colored_iterator const_iterator;
      typedef iterator::value_type value_type;

        SimplexSComplex* comp;
        int color;

        explicit AllCells(SimplexSComplex &cmplx, int col) : comp(&cmplx), color(col) {}

        iterator begin()
        {
          return comp->all_begin(color);
        }

        iterator end()
        {
            return comp->all_end(color);
        }

          const_iterator begin() const
        {
          return comp->all_begin(color);
        }

      const_iterator end() const
        {
            return comp->all_end(color);
        }

    };

    AllCells allCells()
    {
      return AllCells(*comp, color);
    }

    struct CbdCells
    {
      typedef Simplex::coborder_iterator iterator;
      typedef Simplex::const_coborder_iterator const_iterator;
      typedef iterator::value_type value_type;

        SimplexSComplex* comp;
        Simplex const* cell; // ?? really const ??
        int color;

      CbdCells(): comp(0), cell(0), color(0) {}

        explicit CbdCells(SimplexSComplex &cmplx, const Simplex &c, int col) : comp(&cmplx), cell(&c), color(col) {}

        iterator begin()
        {
          BOOST_ASSERT(cell);

            return const_cast<Simplex&>(*cell).coborder_begin(color);
        }

        iterator end()
        {
          BOOST_ASSERT(cell);

            return const_cast<Simplex&>(*cell).coborder_end(color);
        }

        const_iterator begin() const
        {
          BOOST_ASSERT(cell);

          return cell->coborder_begin(color);
        }

        const_iterator end() const
        {
          BOOST_ASSERT(cell);

            return cell->coborder_end(color);
        }
    };

    template<typename ImplT>
    CbdCells cbdCells(const CellProxy<ImplT> &c)
    {
        return CbdCells(*comp, *c.getImpl(), color);
    }
};


// #endif
