/////////////////////////////////////////////////////////////////////////////
/// @file SqueezeIds
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-01-19
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_SQUEEZEIDS_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_SQUEEZEIDS_H

// #include
#include <unordered_map>
#include <boost/iterator/transform_iterator.hpp>
#include <vector>
#include <capd/auxil/Logger.h>
#include <boost/assert.hpp>

namespace capd
{
  namespace complex
  {


    struct squeeze_by_unorderedmap_tag {};

    template<typename Id, typename squeeze_tag=squeeze_by_unorderedmap_tag>
    class SqueezeIds;


    template<typename Id>
    class SqueezeIds<Id, squeeze_by_unorderedmap_tag>
    {
      struct MakeZeroValue
      {
        typedef std::pair<Id, Id> result_type;
        result_type operator()(Id id) const
        {
          return result_type(id, 0);
        }
      };

      typedef std::unordered_map<Id, Id> Map;
    public:
      typedef typename Map::iterator iterator;
      typedef typename Map::const_iterator const_iterator;

      SqueezeIds() {}

      template<typename Iterator>
      SqueezeIds(Iterator begin, Iterator end):
        _map(boost::make_transform_iterator(begin, MakeZeroValue()),
             boost::make_transform_iterator(end, MakeZeroValue()))
      {
        _reverse.resize(_map.size());
        size_t count = 0;
        for (typename Map::iterator it = _map.begin(); it != _map.end(); ++it) {
          _reverse.at(count) = it->first;
          it->second = count++;
        }
      }

      Id operator[](const Id& id) const
      {
        BOOST_ASSERT(contains(id));
        return _map.find(id)->second;
      }

      bool contains(const Id& id) const
      {
        return _map.find(id) != _map.end();
      }

      Id reverse(const Id& id) const
      {
        BOOST_ASSERT(0 <= id && (size_t)id < _reverse.size());
        return _reverse[id];
      }

      size_t size() const
      {
        return _map.size();
      }


      iterator begin() { return _map.begin(); }
      iterator end() { return _map.end(); }
      const_iterator begin() const { return _map.begin(); }
      const_iterator end() const { return _map.end(); }

      virtual ~SqueezeIds() {}

    private:
      Map _map;
      std::vector<Id> _reverse;
      CAPD_CLASS_LOGGER;
    };


  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_SQUEEZEIDS_H
