/////////////////////////////////////////////////////////////////////////////
/// @file TheCubicalAcyclicity
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-23
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_THECUBICALACYCLICITY_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_THECUBICALACYCLICITY_H

// #include
#include <boost/multi_array.hpp>
#include <bitset>
#include "ArrayCubicalHomology.h"
#include <capd/auxil/Logger.h>

namespace capd
{
  namespace complex
  {

    template<size_t DIM>
    class TheCubicalAcyclicity
    {

    public:
      static TheCubicalAcyclicity& getInstance()
      {
        static TheCubicalAcyclicity instance;
        return instance;
      }

      bool operator()(const boost::multi_array<size_t, DIM>& neighberhood)
      {
        const size_t position = configurationPosition(neighberhood);

        if (_cacheComputed[position]) {
          const bool value = _cacheValue[position];
          return value;
        } else {
          std::vector<int> betti = ComputeArrayCubicalHomology<int, int>(neighberhood);
          const bool value = (betti.size() == 1 && betti[0] == 1);
          _cacheComputed.set(position, true);
          _cacheValue.set(position, value);
          return value;
        }
      }

    private:
      TheCubicalAcyclicity()
      {
      }
      virtual ~TheCubicalAcyclicity() {}


      template<size_t A, size_t N>
      struct Pow
      {
        static const size_t value = A * Pow<A, N - 1>::value;
      };

      template<size_t A>
      struct Pow<A, 0>
      {
        static const size_t value = 1;
      };

      size_t configurationPosition(const boost::multi_array<size_t, DIM>& neighberhood)
      {
        std::bitset<CONFIG_BITS> config;
        BOOST_ASSERT(config.size() == neighberhood.num_elements());

        for (size_t idx = 0; idx < neighberhood.num_elements(); ++idx) {
          if (*(neighberhood.origin() + idx) == 1) {
            config[idx] = 1;
          }
        }

        return config.to_ulong();
      }

      TheCubicalAcyclicity(const TheCubicalAcyclicity&);
      TheCubicalAcyclicity& operator=(const TheCubicalAcyclicity&);


      static const size_t CONFIG_BITS = Pow<3, DIM>::value;
      static const size_t CONFIG_NUMBER = Pow<2, CONFIG_BITS>::value;

      std::bitset<CONFIG_NUMBER> _cacheValue;
      std::bitset<CONFIG_NUMBER> _cacheComputed;
      CAPD_CLASS_LOGGER;
    };


  }
}

#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_THECUBICALACYCLICITY_H
