/////////////////////////////////////////////////////////////////////////////
/// @file TypesInstantiation
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-02-27
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_COMPLEX_TYPESINSTANTIATION_H
#define CAPD_FILE_CAPDREDHOM_COMPLEX_TYPESINSTANTIATION_H

// #include
#include <boost/mpl/begin.hpp>
#include <boost/mpl/end.hpp>
#include <boost/mpl/vector.hpp>
#include <boost/mpl/deref.hpp>
#include <capd/auxil/ignoreUnused.h>
#include <boost/utility.hpp>
#include <boost/any.hpp>

namespace capd
{
  namespace complex
  {

    struct CallInstantiation
    {
      template<typename Type>
      static boost::any call()
      {
        typedef typename boost::result_of<Type()>::type result_type;
        result_type (Type::*x)() = &Type::operator();
        capd::auxil::ignoreUnused(x);
        return x;
      }
    };

    template<typename Types, typename Instantiation>
    struct TypesInstantiation
    {

      template <typename Begin, typename End>
      struct ApplyEach {

        explicit ApplyEach()
        {
          typedef typename boost::mpl::deref<Begin>::type Type;
          // keepIt so compiler won't optimize dead code
          keepIt = Instantiation::template call<Type>();
        }

        ApplyEach<typename boost::mpl::next<Begin>::type, End> next;
        boost::any keepIt;
      };

      template <typename End>
      struct ApplyEach<End, End>
      {};

      ApplyEach<typename boost::mpl::begin<Types>::type,
                typename boost::mpl::end<Types>::type > first;
    };

  }
}


#endif // CAPD_FILE_CAPDREDHOM_COMPLEX_TYPESINSTANTIATION_H
