/// @addtogroup homologicalAlgebra
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file acyclicConfigs.hpp
///
/// @author Marian Mrozek, Pawel Pilarczyk
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file constitutes a part of the CAPD library,
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details.

#if !defined(_ACYCLICCONFIGS_HPP)
#define _ACYCLICCONFIGS_HPP

#include <iostream>
#include <fstream>
#include <stdexcept>
#include <cstring>
#include <cstdlib>
#include "capd/cubSet/acyclicConfigs.h"
#include "capd/bzip2/bzlib.h"
#include "capd/cubSet/acyclcfg.h"

// This table stores the acyclicity information concerning all the neighborhood configurations in dim 3
// Bit set to 1 means the configuraiton is acyclic
// The contents of this very large table must be either read from an external file in the current directory for every
// call of teh executable or the executable must be preprocessed with setAcyclicConfigsD3
// to import the table directly into the executable
//unsigned char acyclicConfigsD3[acyclicConfigsSize]=ACYCLIC_CONFIGS_D3MARKER;

/// The acyclic configurations table compressed with bzip2.
extern unsigned char bzip2acyclicConfigsD3 [];
extern unsigned int bzip2acyclicConfigsD3size;



// A previous version of the function: Reading the table from a file.
#define ACYCLIC_CONFIGS_D3MARKER "ChomP_acyclicConfigsD3"

void readAcyclicConfigs();
void readAcyclicConfigs_previous();


#endif //_ACYCLICCONFIGS_HPP

/// @}
