/////////////////////////////////////////////////////////////////////////////
/// @file PositionInSequence
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-04-15
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_HOMAUX_POSITIONINSEQUENCE_H
#define CAPD_FILE_CAPDREDHOM_HOMAUX_POSITIONINSEQUENCE_H

// #include
#include <boost/container/flat_map.hpp>
#include <boost/range/combine.hpp>
#include <boost/range/irange.hpp>
#include <boost/range/size.hpp>
#include <functional>
#include <boost/bind.hpp>

namespace capd
{
  namespace homAux
  {

    template<typename Range, typename Comparator=std::less<typename Range::value_type> >
    class PositionInSequence
    {

      typedef boost::container::flat_map<typename Range::value_type, size_t> Map;

    public:
      typedef typename Map::value_type value_type;
      typedef typename Map::key_type key_type;
      typedef typename Map::mapped_type mapped_type;

      explicit PositionInSequence(const Range& range, Comparator comparator=Comparator()):
        _range(range),
        _comparator(comparator)
      {}


      void operator()()
      {
        std::vector<value_type> values;
        values.reserve(boost::size(_range));

        auto combined = boost::combine(_range,
                                       boost::irange((size_t)0, (size_t) boost::size(_range)));

        for (auto x : combined) {
          values.push_back(value_type(boost::get<0>(x), boost::get<1>(x)));
        }
        std::sort(values.begin(), values.end(), boost::bind(&PositionInSequence::compare, this, _1, _2));
        _map = Map(boost::container::ordered_unique_range, values.begin(), values.end());
      }

      size_t operator[](const key_type& key) const { return _map.find(key)->second; }

    private:
      PositionInSequence& operator=(const PositionInSequence&);

      bool compare(const value_type& a, const value_type& b) const
      {
        return _comparator(a.first, b.first);
      }

      const Range& _range;
      Comparator _comparator;
      Map _map;
    };

    template<typename Range, typename Comparator=std::less<typename Range::value_type> >
    PositionInSequence<Range, Comparator>
    ComputePositionInSequence(const Range& range, Comparator cmp=Comparator())
    {
      PositionInSequence<Range, Comparator> pos(range, cmp);
      pos();
      return pos;
    }


  }
}

#endif // CAPD_FILE_CAPDREDHOM_HOMAUX_POSITIONINSEQUENCE_H
