/////////////////////////////////////////////////////////////////////////////
/// @file UniversalVariable
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-04-03
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_HOMAUX_UNIVERSALVARIABLE_H
#define CAPD_FILE_CAPDREDHOM_HOMAUX_UNIVERSALVARIABLE_H

// #include
#include <boost/any.hpp>
#include <stdexcept>
#include <boost/ref.hpp>
#include <boost/variant.hpp>
#include <boost/variant/static_visitor.hpp>
#include <boost/variant/apply_visitor.hpp>
#include <boost/shared_ptr.hpp>
#include <functional>


namespace capd
{
  namespace homAux
  {

    template<typename TypeT>
    class UniversalVariable
    {
    public:
      typedef TypeT Type;
      typedef boost::variant<Type,
                             boost::reference_wrapper<Type>,
                             boost::shared_ptr<Type> > Storage;

      template<typename Result>
      struct GetRef: boost::static_visitor<Result>
      {
        template<typename T>
        Result operator()(T& x) const { return x; }

        template<typename T>
        Result operator()(boost::reference_wrapper<T>& x) const { return x.get(); }

        template<typename T>
        Result operator()(const boost::reference_wrapper<T>& x) const { return x.get(); }

        template<typename T>
        Result operator()(boost::shared_ptr<T>& x) const { return *x; }

        template<typename T>
        Result operator()(const boost::shared_ptr<T>& x) const { return *x; }

      };


      UniversalVariable(const Type& var):
        _storage(var)
      {}

      UniversalVariable(Type& var, bool reference):
        _storage(Construct(var, reference))
      {}

      UniversalVariable(Type* var, bool reference):
        _storage(Construct(var, reference))
      {}

      Type const* operator ->() const
      {
        return &boost::apply_visitor(GetRef<const Type&>(), _storage);
      }

      Type* operator ->()
      {
        return &boost::apply_visitor(GetRef<Type&>(), _storage);
      }

      Type const& operator *() const
      {
        return boost::apply_visitor(GetRef<const Type&>(), _storage);
      }

      Type& operator *()
      {
        return boost::apply_visitor(GetRef<Type&>(), _storage);
      }


    private:
      static Storage Construct(Type* var, bool reference)
      {
        if (!reference) {
          return Storage(*var);
        } else {
          return Storage(boost::ref(*var));
        }
      }

      static Storage Construct(Type& var, bool reference)
      {
        if (!reference) {
          return Storage(var);
        } else {
          return Storage(boost::ref(var));
        }
      }

      Storage _storage;

      UniversalVariable(const UniversalVariable&);
      UniversalVariable& operator=(const UniversalVariable&);
    };


  }
}

#endif // CAPD_FILE_CAPDREDHOM_HOMAUX_UNIVERSALVARIABLE_H
