/////////////////////////////////////////////////////////////////////////////
/// @file BuildFiltration
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-30
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_PERSISTENCE_BUILDFILTRATION_H
#define CAPD_FILE_CAPDREDHOM_PERSISTENCE_BUILDFILTRATION_H

// #include
#include <vector>
#include <boost/tuple/tuple.hpp>


namespace capd
{
  namespace persistence
  {

    template<typename FilteredComplexT>
    class BuildFiltration
    {
    public:
      typedef FilteredComplexT FilteredComplex;
      typedef typename FilteredComplex::Dim Dim;
      typedef typename FilteredComplex::Id Id;

      typedef boost::tuple<double, Dim, Id> CellInfo;

      explicit BuildFiltration(FilteredComplex& filteredComplex):
        _filteredComplex(filteredComplex)
      {}

      void operator()();
      size_t size() const { return _cellInfos.size(); }
      std::vector<size_t> operator[](size_t position) const;
      double weight(size_t position) const { return _cellInfos[position].template get<0>(); }
      size_t posiotion(Id cellId) const { return _cellIdToPosition[cellId]; }
      Id cellId(size_t posiotion) const { return _cellInfos[posiotion].template get<2>(); }
      Dim cellDim(size_t posiotion) const { return _cellInfos[posiotion].template get<1>(); }

    private:
      FilteredComplex& _filteredComplex;
      std::vector<CellInfo> _cellInfos;
      std::vector<Id> _cellIdToPosition;
    };
  }
}

#endif // CAPD_FILE_CAPDREDHOM_PERSISTENCE_BUILDFILTRATION_H
