/////////////////////////////////////////////////////////////////////////////
/// @file BuildFiltration
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-09-17
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#ifndef CAPD_FILE_CAPDREDHOM_PERSISTENCE_BUILDFILTRATION_HPP
#define CAPD_FILE_CAPDREDHOM_PERSISTENCE_BUILDFILTRATION_HPP

#include <capd/persistence/BuildFiltration.h>
#include <boost/tuple/tuple_comparison.hpp>
#include <boost/typeof/typeof.hpp>
#include <algorithm>
#include <boost/lambda/bind.hpp>
#include <capd/auxil/Logger.h>

namespace capd
{
  namespace persistence
  {

    //BuildFiltration::BuildFiltration() {}

    //BuildFiltration::~BuildFiltration() {}

    template<typename FilteredComplexT>
    void BuildFiltration<FilteredComplexT>::operator()()
    {
      for (Id idx = 0, size = _filteredComplex.size();
           idx < size; ++idx) {
        if (!_filteredComplex.exists(idx)) {
          continue;
        }

        typename FilteredComplex::Cell cell = _filteredComplex[idx];
        _cellInfos.push_back(CellInfo(cell.color(), cell.dim(), cell.id()));
      }

      std::sort(_cellInfos.begin(), _cellInfos.end());

      _cellIdToPosition.resize(_filteredComplex.size());
      for (size_t idx = 0; idx < _cellInfos.size(); ++idx) {
        _cellIdToPosition[_cellInfos[idx].template get<2>()] = idx;
      }
    }

    template<typename FilteredComplexT>
    std::vector<size_t> BuildFiltration<FilteredComplexT>::operator[](size_t position) const
    {
      namespace bl = boost::lambda;

      typedef typename FilteredComplex::Iterators::BdCells BdCells;
      BdCells bd =
        _filteredComplex.iterators().bdCells(_filteredComplex[_cellInfos[position].template get<2>()]);

      std::vector<size_t> result;

      // std::transform(bd.begin(), bd.end(), result.begin(),
      //  		     bl::constant(_cellIdToPosition)[bl::_1]);

      for (typename BdCells::const_iterator it = bd.begin(), end = bd.end();
           it != end; ++it) {
        result.push_back(_cellIdToPosition[it->id()]);
      }

      return result;
    }

  }
}


#endif // CAPD_FILE_CAPDREDHOM_PERSISTENCE_BUILDFILTRATION_HPP
