/////////////////////////////////////////////////////////////////////////////
/// @file ComputePersistentHomology
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-30
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_PERSISTENCE_COMPUTEPERSISTENCEHOMOLOGY_H
#define CAPD_FILE_CAPDREDHOM_PERSISTENCE_COMPUTEPERSISTENCEHOMOLOGY_H

// #include
#include "PersistentHomologyTypes.h"
#include "PersistentSignature.h"

namespace capd
{
  namespace persistence
  {

    template<typename FilteredComplexT>
    class ComputePersistentHomology
    {

    public:
      typedef FilteredComplexT FilteredComplex;
      typedef capd::persistence::Pairs Pairs;
      typedef capd::persistence::Cycles Cycles;

      typedef capd::persistence::DiagramInterval DiagramInterval;
      typedef capd::persistence::Diagram Diagram;


      struct DiagramIntervalComparator
      {
        bool operator()(const DiagramInterval& a, const DiagramInterval& b) const
        {
          return (a.second - a.first) < (b.second - b.first);
        }
      };

      static bool enabled();

      ComputePersistentHomology(FilteredComplex& filteredComplex):
        _filteredComplex(filteredComplex)
      {}

      virtual ~ComputePersistentHomology() {}

      void operator()();

      Pairs pairs(typename FilteredComplex::Dim dim=-1) const;
      Cycles cycles(typename FilteredComplex::Dim dim=-1) const;
      Diagram diagram(typename FilteredComplex::Dim dim=-1) const;
      typename FilteredComplex::Dim dim() const { return _filteredComplex.dim(); }

      PersistentSignature signature() const;

    private:
      FilteredComplex& _filteredComplex;
      Pairs _pairs;
      Cycles _cycles;
    };

  }
}

#endif // CAPD_FILE_CAPDREDHOM_PERSISTENCE_COMPUTEPERSISTENCEHOMOLOGY_H
