/////////////////////////////////////////////////////////////////////////////
/// @file DegreeLimitedNeighbourhood
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-02
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#ifndef CAPD_FILE_CAPDREDHOM_PERSISTENCE_DEGREELIMITEDNEIGHBOURHOOD_HPP
#define CAPD_FILE_CAPDREDHOM_PERSISTENCE_DEGREELIMITEDNEIGHBOURHOOD_HPP

#include <capd/persistence/DegreeLimitedNeighbourhood.h>

#include <boost/tuple/tuple.hpp>
#include <boost/tuple/tuple_comparison.hpp>
#include <algorithm>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>

namespace capd
{
  namespace persistence
  {

    namespace bl = boost::lambda;

    template<typename PointCloudT>
    void DegreeLimitedNeighbourhood<PointCloudT>::operator()()
    {
      typedef boost::tuple<double, size_t> DistanceElement;
      std::vector<DistanceElement> distances;
      distances.reserve(_cloud.size());

      for (size_t idxI = 0, size = _cloud.size();
           idxI < size; ++idxI) {
        distances.clear();

        for (size_t idxJ = idxI + 1; idxJ < size; ++idxJ) {
          const double dist = _cloud.distance(idxI, idxJ);

          if (_eps <= 0 || (_eps > 0 && dist <= _eps )) {
            distances.push_back(DistanceElement(dist, idxJ));
          }
        }

        if (_limit > 0 && distances.size() > _limit) {
          nth_element(distances.begin(), distances.begin() + _limit, distances.end());
          distances.resize(_limit);
        }

        for (std::vector<DistanceElement>::const_iterator it = distances.begin(),
              end = distances.end(); it != end; ++it) {
          _cloud.setNeighbour(idxI, it->get<1>(), it->get<0>());
        }
      }

    }


  }
}


#endif // CAPD_FILE_CAPDREDHOM_PERSISTENCE_DEGREELIMITEDNEIGHBOURHOOD_HPP
