/////////////////////////////////////////////////////////////////////////////
/// @file DistanceMatrixPersistentHomology
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-10-21
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_PERSISTENCE_DISTANCEMATRIXPERSISTENCEHOMOLOGY_H
#define CAPD_FILE_CAPDREDHOM_PERSISTENCE_DISTANCEMATRIXPERSISTENCEHOMOLOGY_H

#include "PersistentHomologyTypes.h"
#include <capd/vectalg/Matrix.h>
#include <boost/function.hpp>


namespace capd
{
  namespace persistence
  {

    class DistanceMatrixPersistentHomology
    {

    public:
      typedef capd::vectalg::Matrix<double, 0, 0> DistanceMatrix;

      DistanceMatrixPersistentHomology(const DistanceMatrix& distance, size_t dim, size_t degreeLimit):
        _distance(distance), _dim(dim), _degreeLimit(degreeLimit),
        _pairs(dim), _cycles(dim), _diagrams(dim)
      {}

      virtual ~DistanceMatrixPersistentHomology() {}
      static bool Enabled();

      const Pairs& pairs(int dim = -1) const { return dim == -1 ? _pairsAll : _pairs[dim]; }
      const Cycles& cycles(int dim = -1) const { return dim == -1 ? _cyclesAll : _cycles[dim]; }
      const Diagram& diagram(int dim = -1) const { return dim == -1 ? _diagramsAll : _diagrams[dim]; }

      void operator()();

    private:
      DistanceMatrix _distance;
      const size_t _dim;
      const size_t _degreeLimit;
      std::vector<Pairs> _pairs;
      std::vector<Cycles> _cycles;
      std::vector<Diagram> _diagrams;
      Pairs _pairsAll;
      Cycles _cyclesAll;
      Diagram _diagramsAll;
    };


  }
}

#endif // CAPD_FILE_CAPDREDHOM_PERSISTENCE_DISTANCEMATRIXPERSISTENCEHOMOLOGY_H
