/////////////////////////////////////////////////////////////////////////////
/// @file DistanceMatrixPointCloud
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-10-21
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_PERSISTENCE_DISTANCEMATRIXPOINTCLOUD_H
#define CAPD_FILE_CAPDREDHOM_PERSISTENCE_DISTANCEMATRIXPOINTCLOUD_H

// #include
#include "AbstractPointCloud.h"

#include <capd/vectalg/Matrix.h>
#include <boost/assert.hpp>


namespace boost
{

  struct vecS;
  struct undirectedS;
  struct directedS;
  struct listS;
  struct no_property;

  template <class OutEdgeListS , // = vecS, // a Sequence or an AssociativeContainer
            class VertexListS , // = vecS, // a Sequence or a RandomAccessContainer
            class DirectedS , // = directedS,
            class VertexProperty , // = no_property,
            class EdgeProperty , // = no_property,
            class GraphProperty , // = no_property,
            class EdgeListS> // = listS>
  class adjacency_list;
}

namespace capd
{
  namespace persistence
  {

    struct DistanceMatrixMetric
    {
      typedef double Distance;
      typedef size_t Point;
      typedef vectalg::Matrix<double, 0, 0> DistanceMatrix;

      explicit DistanceMatrixMetric(const DistanceMatrix& distance):
        _distance(distance)
      {
        BOOST_ASSERT(_distance.numberOfRows() == _distance.numberOfColumns());
      }

      Distance operator()(const Point& point1, const Point& point2) const
      {
        return _distance[point1][point2];
      }

      size_t size() const { return _distance.numberOfRows(); }

      const DistanceMatrix& _distance;
    };

    class DistanceMatrixPointCloud: public AbstractPointCloud
    {

      typedef boost::adjacency_list<boost::vecS, boost::vecS,
                                    boost::undirectedS,
                                    boost::no_property,
                                    boost::no_property,
                                    boost::no_property,
                                    boost::listS> Graph;

    public:
      typedef DistanceMatrixMetric Metric;
      typedef Metric::Point Point;
      typedef Metric::Distance Distance;


      explicit DistanceMatrixPointCloud(const Metric::DistanceMatrix& distance):
        _metric(distance),
        _graph(NULL)
      {
        init();
      }

      virtual ~DistanceMatrixPointCloud();

      virtual size_t size() const
      {
        return _metric.size();
      }

      Point operator[](size_t idx) const { return Point(idx); }

      void setNeighbour(Id idxI, Id idxJ, double weight);
      virtual void neighbours(Id idxI, std::vector<Id>& result) const;
      using AbstractPointCloud::neighbours;

      virtual size_t edges();
      Distance distance(Id p1, Id p2) const;


    private:
      void init();

      Metric _metric;
      Graph* _graph; // do not want to use boost graph here, so the interface is
    };


  }
}

#endif // CAPD_FILE_CAPDREDHOM_PERSISTENCE_DISTANCEMATRIXPOINTCLOUD_H
