/////////////////////////////////////////////////////////////////////////////
/// @file PersistentSignature
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-04-09
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_PERSISTENCE_PERSISTENTSIGNATURE_H
#define CAPD_FILE_CAPDREDHOM_PERSISTENCE_PERSISTENTSIGNATURE_H

// #include
#include <vector>
#include "PersistentHomologyTypes.h"

namespace capd
{
  namespace persistence
  {

    class PersistentSignature
    {

    public:
      PersistentSignature():
        _maxDim(0),
        _cyclesByDim(_maxDim + 2),
        _pairsByDim(_maxDim + 2),
        _diagramsByDim(_maxDim + 2)
      {}

      template<typename ComputePersistentHomology>
      explicit PersistentSignature(const ComputePersistentHomology& compute):
        _maxDim(compute.dim()),
        _cyclesByDim(_maxDim + 2),
        _pairsByDim(_maxDim + 2),
        _diagramsByDim(_maxDim + 2)
      {
        for (size_t dim = 0; dim <= _maxDim; ++dim) {
          _cyclesByDim[dim] = compute.cycles(dim);
          _pairsByDim[dim] = compute.pairs(dim);
          _diagramsByDim[dim] = compute.diagram(dim);
        }
        _cyclesByDim[_maxDim + 1] = compute.cycles();
        _pairsByDim[_maxDim + 1] = compute.pairs();
        _diagramsByDim[_maxDim + 1] = compute.diagram();
      }

      const Cycles& cycles(size_t dim) const
      {
        return dim > _maxDim ? _emptyCycles :  _cyclesByDim[dim];
      }

      const Cycles& cycles() const
      {
        return _cyclesByDim[_maxDim + 1];
      }

      const Pairs& pairs(size_t dim) const
      {
        return dim > _maxDim ? _emptyPairs : _pairsByDim[dim];
      }

      const Pairs& pairs() const
      {
        return _pairsByDim[_maxDim + 1];
      }

      const Diagram& diagram(size_t dim) const
      {
        return dim > _maxDim ? _emptyDiagram : _diagramsByDim[dim];
      }

      const Diagram& diagram() const
      {
        return _diagramsByDim[_maxDim + 1];
      }

    private:
      size_t _maxDim;
      std::vector<Cycles> _cyclesByDim;
      std::vector<Pairs> _pairsByDim;
      std::vector<Diagram> _diagramsByDim;
      Cycles _emptyCycles;
      Pairs _emptyPairs;
      Diagram _emptyDiagram;
    };


  }
}

#endif // CAPD_FILE_CAPDREDHOM_PERSISTENCE_PERSISTENTSIGNATURE_H
