/////////////////////////////////////////////////////////////////////////////
/// @file PointCloud.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-25
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_POINTCLOUD_H
#define CAPD_FILE_POINTCLOUD_H

#include "AbstractPointCloud.h"

#include <capd/vectalg/Vector.h>
#include <capd/auxil/Logger.h>

#include <utility>
#include <vector>
#include <map>




namespace boost
{

  struct vecS;
  struct undirectedS;
  struct directedS;
  struct listS;
  struct no_property;

  template <class OutEdgeListS , // = vecS, // a Sequence or an AssociativeContainer
            class VertexListS , // = vecS, // a Sequence or a RandomAccessContainer
            class DirectedS , // = directedS,
            class VertexProperty , // = no_property,
            class EdgeProperty , // = no_property,
            class GraphProperty , // = no_property,
            class EdgeListS> // = listS>
  class adjacency_list;
}

namespace capd
{
  namespace persistence
  {

    template<typename Metric, size_t _DIM=0>
    class PointCloud: public AbstractPointCloud
    {

    public:
      typedef typename Metric::Scalar Scalar;
      typedef typename Metric::Distance Distance;
      static const size_t DIM = _DIM;

      typedef capd::vectalg::Vector<Scalar, DIM> Point;
      typedef std::vector<Point> Points;

      typedef boost::adjacency_list<boost::vecS, boost::vecS, boost::undirectedS,
                                    boost::no_property,
                                    boost::no_property,
                                    boost::no_property,
                                    boost::listS> Graph;

      template<typename Iterator>
      PointCloud(Iterator begin, Iterator end, Metric metric = Metric()):
        _points(begin, end),
        _graph(NULL),
        _metric(metric)
      {
        init();
      }

      virtual ~PointCloud() { explicitDestructor(); }

      void setNeighbour(Id idxI, Id idxJ, double weight);
      size_t size() const { return _points.size(); }
      const Point& operator[](size_t idx) const { return _points[idx]; }

      void neighbours(Id idxI, std::vector<Id>& result) const;
      using AbstractPointCloud::neighbours;

      size_t edges();
      Distance distance(Id p1, Id p2) const;


    private:
      CAPD_CLASS_LOGGER;

      void init();
      void explicitDestructor(); // it is difficult to make explicit template instantion for destructors, it is easy for methods
      Points _points;
      Graph* _graph; // do not want to use boost graph here, so the interface is
      Metric _metric;
    };

  }
}

#endif // CAPD_FILE_POINTCLOUD_H
