/////////////////////////////////////////////////////////////////////////////
/// @file PointCloud.hpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-25
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_POINTCLOUD_HPP
#define CAPD_FILE_POINTCLOUD_HPP

#include "PointCloud.h"


#include <boost/graph/graph_traits.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/typeof/typeof.hpp>

namespace capd
{
  namespace persistence
  {

    template<typename ScalarT, size_t _DIM>
    void PointCloud<ScalarT, _DIM>::init()
    {
      _graph = new Graph(_points.size());
    }

    template<typename ScalarT, size_t _DIM>
    void PointCloud<ScalarT, _DIM>::explicitDestructor()
    {
      delete _graph;
      _graph = NULL;
    }


    template<typename ScalarT, size_t _DIM>
    void PointCloud<ScalarT, _DIM>::setNeighbour(Id idxI, Id idxJ, double weight)
    {
      add_edge(idxI, idxJ, *_graph);
    }

    template<typename ScalarT, size_t _DIM>
    size_t PointCloud<ScalarT, _DIM>::edges()
    {
      return num_edges(*_graph);
    }

    template<typename ScalarT, size_t _DIM>
    void PointCloud<ScalarT, _DIM>::neighbours(Id idxI, std::vector<typename PointCloud<ScalarT, _DIM>::Id>& result) const
    {
      BOOST_AUTO(adj, boost::adjacent_vertices(idxI, *_graph));
      result.assign(adj.first, adj.second);
    }

    template<typename ScalarT, size_t _DIM>
    typename PointCloud<ScalarT, _DIM>::Distance PointCloud<ScalarT, _DIM>::distance(Id p1, Id p2) const
    {
      return _metric(_points[p1], _points[p2]);
    }

  }
}


#endif // CAPD_FILE_POINTCLOUD_HPP
