/////////////////////////////////////////////////////////////////////////////
/// @file PointCloudUtils
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-01
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_PERSISTENCE_POINTCLOUDUTILS_H
#define CAPD_FILE_CAPDREDHOM_PERSISTENCE_POINTCLOUDUTILS_H

// #include
#include "PointCloud.h"
#include "SizeLimitedNeighbourhood.h"
#include "DegreeLimitedNeighbourhood.h"

#include <iterator>
#include <iosfwd>
#include <string>
#include <algorithm>
#include <sstream>

namespace capd
{
  namespace persistence
  {

    template<typename Traits, typename Iterator>
    typename Traits::PointCloudType* makeSizeLimitedPointCloud(Iterator begin, Iterator end, size_t sizeLimit, double epsLimit = 0)
    {
      typedef typename Traits::PointCloudType PointCloud;
      PointCloud* pointCloud = new PointCloud(begin, end, typename Traits::Metric());
      SizeLimitedNeighbourhood<PointCloud> neighbourhood(*pointCloud, sizeLimit, epsLimit);

      neighbourhood();

      return pointCloud;
    }

    template<typename Traits, typename Iterator>
    typename Traits::PointCloudType* makeDegreeLimitedPointCloud(Iterator begin, Iterator end, size_t degreeLimit, double epsLimit = 0)
    {
      typedef typename Traits::PointCloudType PointCloud;
      PointCloud* pointCloud = new PointCloud(begin, end, typename Traits::Metric());
      DegreeLimitedNeighbourhood<PointCloud> neighbourhood(*pointCloud, degreeLimit, epsLimit);

      neighbourhood();

      return pointCloud;
    }

    template<typename Traits>
    struct PointCloudInputStreamIterator
    {
      static const size_t DIM = Traits::DIM;
      typedef capd::vectalg::Vector<typename Traits::ScalarType, DIM> Point;

      struct PointCloudPoint
      {
        friend std::istream &operator>>(std::istream &is, PointCloudPoint &point) {
          if (DIM != 0) {
            point._point.resize(DIM);
            for (size_t dim = 0; dim < DIM; ++dim) {
              is >> point._point[dim];
            }
          } else {
            typedef typename Point::ScalarType Scalar;
            std::vector<Scalar> values;
            std::string line;
            std::getline(is, line);
            std::stringstream lineStream(line);

            std::copy(std::istream_iterator<Scalar>(lineStream),
                      std::istream_iterator<Scalar>(),
                      std::back_inserter(values));

            point._point = Point(values.begin(), values.end());
          }
          return is;
        }

        operator Point() const { return _point; }
      private:
        Point _point;
      };

      typedef std::istream_iterator<PointCloudPoint> type;
    };

  }
}

#endif // CAPD_FILE_CAPDREDHOM_PERSISTENCE_POINTCLOUDUTILS_H
