/////////////////////////////////////////////////////////////////////////////
/// @file SizeLimitedNeighbourhood.hpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-25
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_SIZELIMITEDNEIGHBOURHOOD_HPP
#define CAPD_FILE_SIZELIMITEDNEIGHBOURHOOD_HPP

#include "SizeLimitedNeighbourhood.h"

#include <boost/tuple/tuple.hpp>
#include <boost/tuple/tuple_comparison.hpp>
#include <queue>

namespace capd
{
  namespace persistence
  {


    template<typename PointCloudT>
    void SizeLimitedNeighbourhood<PointCloudT>::operator()()
    {
      typedef boost::tuple<double, size_t, size_t> DistanceElement;
      typedef std::priority_queue<DistanceElement>  Queue;
      Queue queue;

      for (size_t idxI = 0, size = _cloud.size();
           idxI < size; ++idxI) {
        for (size_t idxJ = idxI + 1; idxJ < size; ++idxJ) {
          const double dist = _cloud.distance(idxI, idxJ);

          if (_eps <= 0 || (_eps > 0 && dist <= _eps )) {
            queue.push(DistanceElement(dist, idxI, idxJ));
          }

          if (_limit > 0 && queue.size() > _limit) {
            queue.pop();
          }
        }
      }

      while (queue.size() > 0) {
        const DistanceElement& curr = queue.top();
        _cloud.setNeighbour(boost::get<1>(curr), boost::get<2>(curr), boost::get<0>(curr));
        queue.pop();
      }
    }


  }
}

#endif // CAPD_FILE_SIZELIMITEDNEIGHBOURHOOD_HPP
