/////////////////////////////////////////////////////////////////////////////
/// @file Convert
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-22
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_PROGRAM_CONVERT_H
#define CAPD_FILE_CAPDREDHOM_PROGRAM_CONVERT_H

// #include
#include <boost/python.hpp>
#include <capd/auxil/Logger.h>

namespace capd
{
  namespace apiRedHom
  {

    namespace python
    {
      template<typename Object2>
      struct Convert
      {
        typedef boost::python::object PyT;
        typedef Object2 RawT;

        static RawT fromPy(const PyT& obj)
        {
          return boost::python::extract<RawT>(obj);
        }

        static PyT toPy(const RawT& obj)
        {
          return boost::python::object(obj);
        }
      };


      template <typename Object>
      struct Convert<std::vector<Object> >
      {
        typedef boost::python::list PyT;
        typedef std::vector<Object> RawT;

        static RawT fromPy(const PyT& list)
        {
          const size_t size = len(list);
          RawT res(size);

          for(int i = 0; i < size; ++i) {
            res[i] = Convert<Object>::fromPy(boost::python::extract<typename Convert<Object>::PyT>(list[i])());
          }

          return res;
        }

        static PyT toPy(const RawT& vec)
        {
          PyT res;
          const size_t size = vec.size();

          for (typename std::vector<Object>::const_iterator it = vec.begin(),
                 end = vec.end(); it != end; ++it) {
            res.append(Convert<Object>::toPy(*it));
          }

          return res;
        }
      };

      template <typename Object1, typename Object2>
      struct Convert<std::pair<Object1, Object2> >
      {
        typedef boost::python::tuple PyT;
        typedef std::pair<Object1, Object2> RawT;

        static PyT toPy(const RawT& p)
        {
          return boost::python::make_tuple(Convert<Object1>::toPy(p.first), Convert<Object2>::toPy(p.second));
        }

      };

      template<typename T>
      typename Convert<T>::PyT ToPython(const T& obj)
      {
        return Convert<T>::toPy(obj);
      }

      template<typename T>
      typename Convert<T>::RawT FromPython(const typename Convert<T>::PyT& obj)
      {
        return Convert<T>::fromPy(obj);
      }

    }
  }
}

#endif // CAPD_FILE_CAPDREDHOM_PROGRAM_CONVERT_H
