/////////////////////////////////////////////////////////////////////////////
/// @file CubicalComplex.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-10-28
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include "CubicalComplex.h"

#include "capd/apiRedHom/Complex.h"
#include "Convert.h"

#include <boost/python.hpp>
#include <boost/bind.hpp>

namespace b = boost;
namespace bp = boost::python;


namespace capd
{
  namespace apiRedHom
  {
    namespace python
    {

      template <>
      struct Convert<apiRedHom::CubicalComplex::Interval>
      {
        typedef boost::python::tuple PyT;

        static apiRedHom::CubicalComplex::Interval fromPy(const bp::object& cellCode)
        {
          BOOST_ASSERT(len(cellCode) == 2);
          return apiRedHom::CubicalComplex::Interval(Convert<apiRedHom::CubicalComplex::Coordinate>::fromPy(cellCode[0]),
                                                     Convert<bool>::fromPy(cellCode[1]));
        }
      };
    }
  }
}

using namespace capd;
using namespace capd::apiRedHom::python;

namespace
{
  capd::apiRedHom::CubicalComplex* CreateCubicalComplex(const bp::list& size)
  {
    return new apiRedHom::CubicalComplex(FromPython<std::vector<size_t> >(size));
  }

}


CubicalComplex::CubicalComplex(const bp::list& size):
  Complex(CreateCubicalComplex(size))
{

}

apiRedHom::CubicalComplex& CubicalComplex::impl()
{
  return dynamic_cast<apiRedHom::CubicalComplex&>(this->Complex::impl());
};

const apiRedHom::CubicalComplex& CubicalComplex::impl() const
{
  return dynamic_cast<const apiRedHom::CubicalComplex&>(this->Complex::impl());
};


void CubicalComplex::insert(const CellCode& cube)
{
  impl().insert(FromPython<apiRedHom::CubicalComplex::CellCode>(cube));
}

void CubicalComplex::fillWithBoundaries()
{
  impl().fillWithBoundaries();
}


extern "C" void PyCubicalComplex_insertFullCubes(PyObject* py_obj, long* coords, int coordsNum, int coordDim)
{
  typedef apiRedHom::CubicalComplex::Dim Dim;
  apiRedHom::CubicalComplex& impl = boost::python::extract<CubicalComplex&>(py_obj)().impl();

  const Dim dim = impl.dim();
  apiRedHom::CubicalComplex::CellCode cellCode(dim, apiRedHom::CubicalComplex::Interval(0, true));

  BOOST_ASSERT(dim == coordDim);

  size_t index = 0;
  for (int coordsIdx = 0; coordsIdx < coordsNum; ++coordsIdx) {
    for (Dim d = 0; d < dim; ++d) {
      cellCode[d].first = coords[index];
      ++index;
    }

    impl.insert(cellCode);
  }
}
