/////////////////////////////////////////////////////////////////////////////
/// @file DistanceMatrixPersistentHomology
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-10-21
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include "DistanceMatrixPersistentHomology.h"
#include <capd/apiRedHom/Algorithms.h>

#include "Convert.h"

#include <boost/python.hpp>
#include <capd/auxil/Logger.h>


using namespace capd;
using namespace capd::apiRedHom::python;

DistanceMatrixPersistentHomology::DistanceMatrixPersistentHomology(const boost::python::list& distanceMatrix, size_t dim, size_t degreeLimit):
  _distanceMatrix(FromPython<apiRedHom::DistanceMatrixPersistentHomology::DistanceMatrix>(distanceMatrix)),
  _impl(new apiRedHom::DistanceMatrixPersistentHomology(_distanceMatrix, dim, degreeLimit))
{
}

void DistanceMatrixPersistentHomology::operator()()
{
  (*_impl)();
}

boost::python::list DistanceMatrixPersistentHomology::diagram(int dim) const
{
  return ToPython(_impl->diagram(dim));
}

boost::python::list DistanceMatrixPersistentHomology::cycles(int dim) const
{
  return ToPython(_impl->cycles(dim));
}

boost::python::list DistanceMatrixPersistentHomology::pairs(int dim) const
{
  return ToPython(_impl->pairs(dim));
}
