import unittest as ut
import capdRedHom

class TestCubicalComplex(ut.TestCase):

    def test_empty_1(self):
        c = capdRedHom.CubicalComplex(size=[1,1])
        self.assertEqual(2, c.dim)

    def test_cubes(self):
        c = capdRedHom.CubicalComplex(cubes=(x for x in [ [[0, 1], (0, 1)] ]))
        self.assertEqual({0: 1}, capdRedHom.BettiNumbersOverZ(c)())

    def test_cubes_size(self):
        c = capdRedHom.CubicalComplex([1, 1], cubes=(x for x in [ [[0, 1], (0, 1)] ]))
        self.assertEqual({0: 1}, capdRedHom.BettiNumbersOverZ(c)())

    def test_full_cubes(self):
        c = capdRedHom.CubicalComplex(full_cubes=(x for x in [(0, 0)]))
        self.assertEqual({0: 1}, capdRedHom.BettiNumbersOverZ(c)())

    def test_full_cubes_size(self):
        c = capdRedHom.CubicalComplex([1, 1], full_cubes=(x for x in [(0, 0)]))
        self.assertEqual({0: 1}, capdRedHom.BettiNumbersOverZ(c)())

    def test_full_cubes_size(self):
        c = capdRedHom.CubicalComplex([1, 1], full_cubes=(x for x in [(0, 0)]))
        self.assertEqual({0: 1}, capdRedHom.BettiNumbersOverZ(c)())

    def test_numpy_full_cubes(self):
        has_numpy=True
        try:
            import numpy as np
        except ImportError:
            has_numpy = False
            pass

        if not has_numpy:
            return

        data = np.ones([5, 5], np.int)
        data[2,2] = 0
        c = capdRedHom.CubicalComplex(data.shape, numpy_full_cubes=np.transpose(np.nonzero(data)).copy(order='C'))
        self.assertEqual({0: 1, 1:1}, capdRedHom.BettiNumbersOverZ(c)())
