#!/usr/bin/env python

import unittest

import sys, os
import imp

if os.uname()[0] == 'Darwin':
    imp.load_dynamic('libcapdapiRedHom_py', '.libs/libcapdapiRedHom_py.dylib')
elif os.uname()[0] == 'Linux':
    imp.load_dynamic('libcapdapiRedHom_py', '.libs/libcapdapiRedHom_py.so')
else:
    raise ImportError('What is the system uname?')

import libcapdapiRedHom_py as capd


class TestCubicalRectangle(unittest.TestCase):

    def setUp(self):
        pass

    def test_insert_points(self):
        c = capd.CubicalComplex([10,10])
        c.insert([(0, True), (0, True)])
        c.fillWithBoundaries()

        self.assertEqual((c.dim(), c.cardinality()), (2, 9))


class TestSimplicialTriangle(unittest.TestCase):

    def setUp(self):
        pass

    def test_insert_points(self):
        c = capd.SimplicialComplex()
        c.insert([0, 1, 2])

        self.assertEqual((c.dim(), c.cardinality()), (2, 7))

        betti = capd.BettiNumbersOverZ(c)
        self.assertEqual({}, betti.__dict__)
        self.assertEqual(betti(), [1])

class TestSimplicialSphere(unittest.TestCase):

    def setUp(self):
        pass

    def test_insert_points(self):
        c = capd.SimplicialComplex()
        for s in [[0, 1, 2], [0, 1, 3], [0, 2, 3], [1, 2, 3]]:
            c.insert(s)

        self.assertEqual((c.dim(), c.cardinality()), (2, 14))

        betti = capd.BettiNumbersOverZ(c)
        self.assertEqual({}, betti.__dict__)
        self.assertEqual(betti(), [1, 0, 1])

class TestDistanceMatrixPersistentHomology(unittest.TestCase):
    def test_triangle(self):
        distance_m = [[0, 1, 1], [1, 0, 1], [1, 1, 0]]
        pers = capd.DistanceMatrixPersistentHomology(distance_m, 3, 0)
        pers()
        self.assertEqual(2, len(pers.pairs()))
        self.assertEqual(1, len(pers.cycles()))
        self.assertEqual(2, len(pers.pairs(0)))
        self.assertEqual(1, len(pers.cycles(0)))
        self.assertEqual(0, len(pers.pairs(1)))
        self.assertEqual(0, len(pers.cycles(1)))

if __name__ == '__main__':
	pass
	unittest.main()
