/////////////////////////////////////////////////////////////////////////////
/// @file gap_interface.c
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-07
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include "helpers.h"

#include <capd/config-capdRedHom.h>
#include <src/compiled.h>
#include <src/integer.h>
#include <src/objects.h>
#include <capd/apiRedHom/c_interface.h>

#include <stdio.h>

#define UNUSED(x) (void)(x)

Obj Dimension;
Obj EvaluateProperty;
Obj ArrayValue;


RedHom_ComplexInfo* Complex(Obj obj)
{
  return * ((RedHom_ComplexInfo**) ADDR_OBJ(obj));
}

Obj GAP_RedHom_CreateSimplicialComplex(Obj self)
{
  UNUSED(self);
  Obj obj = NewBag(T_DATOBJ, sizeof(RedHom_ComplexInfo*));

  *ADDR_OBJ(obj) = malloc(sizeof(RedHom_ComplexInfo));
  RedHom_CreateSimplicialComplex(Complex(obj));

  return obj;
}


Obj GAP_RedHom_CreateCubicalComplex(Obj self, Obj gapSize)
{
  UNUSED(self);
  Obj obj = NewBag(T_DATOBJ, sizeof(RedHom_ComplexInfo*));
  *ADDR_OBJ(obj) = malloc(sizeof(RedHom_ComplexInfo));

  int* size;
  const int dim = ConvertGAPIntList(gapSize, &size);
  RedHom_CreateCubicalComplex(Complex(obj), dim, size);
  free(size);

  return obj;
}



Obj GAP_RedHom_CreateComplexFromPureCubicalComplex(Obj self, Obj pureCubicalComplex)
{
  UNUSED(self);
  Obj obj = NewBag(T_DATOBJ, sizeof(RedHom_ComplexInfo*));
  *ADDR_OBJ(obj) = malloc(sizeof(RedHom_ComplexInfo));

  ConvertPureCubicalComplex(pureCubicalComplex, Complex(obj));

  return obj;
}


Obj GAP_RedHom_CreateSComplexFromPureCubicalComplex(Obj self, Obj pureCubicalComplex)
{
  UNUSED(self);
  Obj obj = NewBag(T_DATOBJ, sizeof(RedHom_ComplexInfo*));
  *ADDR_OBJ(obj) = malloc(sizeof(RedHom_ComplexInfo));

  ConvertPureCubicalComplexToSComplex(pureCubicalComplex, Complex(obj));

  return obj;
}


Obj GAP_RedHom_DeleteComplex(Obj self, Obj obj)
{
  UNUSED(self);
  RedHom_DeleteComplex(Complex(obj));
  free(*ADDR_OBJ(obj));
  return INTOBJ_INT(0);
}


Obj GAP_RedHom_AddSimplexToComplex(Obj self, Obj obj, Obj gapSimplex)
{
  UNUSED(self);
  int* simplex;
  const int dim = ConvertGAPIntList(gapSimplex, &simplex);
  RedHom_AddSimplexToComplex(Complex(obj), simplex, dim);
  free(simplex);

  return obj;
}


Obj GAP_RedHom_AddFullCubeToComplex(Obj self, Obj obj, Obj gapCube)
{
  UNUSED(self);
  int* cube;
  const int dim = ConvertGAPIntList(gapCube, &cube);

  if (dim != RedHom_ComplexDim(Complex(obj))) {
    return obj;
  }

  int unitArray[dim];
  int i = 0;
  for (i = 0; i < dim; ++i) {
    unitArray[i] = 1;
  }

  RedHom_AddCubeToComplex(Complex(obj), cube, unitArray);
  free(cube);

  return obj;
}


Obj GAP_RedHom_CubicalComplexFillWithBoundaries(Obj self, Obj obj)
{
  UNUSED(self);
  RedHom_CubicalComplexFillWithBoundaries(Complex(obj));
  return obj;
}

Obj GAP_RedHom_ComplexCardinality(Obj self, Obj obj)
{
  UNUSED(self);
  return INTOBJ_INT(RedHom_ComplexCardinality(Complex(obj)));
}

Obj GAP_RedHom_ComplexDim(Obj self, Obj obj)
{
  UNUSED(self);
  return INTOBJ_INT(RedHom_ComplexDim(Complex(obj)));
}

Obj GAP_RedHom_BettiNumbersOverZOfComplex(Obj self, Obj obj)
{
  UNUSED(self);
  int betti[RedHom_ComplexDim(Complex(obj)) + 1];
  int dim = RedHom_BettiNumbersOverZOfComplex(Complex(obj), betti);

  return GAPBettiList(betti, dim);
}

Obj GAP_RedHom_Coreduction(Obj self, Obj obj)
{
  UNUSED(self);
  size_t reduced = RedHom_Coreduction(Complex(obj));

  return INTOBJ_INT(reduced);
}

/* Obj GAP_RedHom_FundGroup(Obj self, Obj obj) */
/* { */
/*   UNUSED(self); */
/*   int* buffer = 0; */
/*   int size = RedHom_FundGroup(Complex(obj), &buffer); */
/*   Obj res = GAPIntList(buffer, size); */
/*   free(buffer); */
/*   return res; */
/* } */

Obj GAP_RedHom_FundamentalGroup(Obj self, Obj obj, Obj vectorFieldType)
{
  UNUSED(self);
  int* buffer = 0;
  int size = RedHom_FundamentalGroup(Complex(obj), &buffer, INT_INTOBJ(vectorFieldType));
  Obj res = GAPIntList(buffer, size);
  free(buffer);
  return res;
}

Obj GAP_RedHom_ZigZag(Obj self, Obj obj, Obj dim)
{
  UNUSED(self);
  RedHom_ZigZag(Complex(obj), INT_INTOBJ(dim));
  return obj;
}

Obj GAP_RedHom_CountCriticalCells(Obj self, Obj obj, Obj method)
{
  UNUSED(self);
  unsigned long criticalCells = RedHom_CountCriticalCells(Complex(obj), INT_INTOBJ(method));
  return INTOBJ_INT(criticalCells);
}


Obj Call_1_Wrapper(Obj obj, Obj arg1)
{
  return CALL_1ARGS(obj, arg1);
}

Obj Call_2_Wrapper(Obj obj, Obj arg1, Obj arg2)
{
  return CALL_2ARGS(obj, arg1, arg2);
}



/**
Details of the functions to make available to GAP.
This is used in InitKernel() and InitLibrary()
*/
static StructGVarFunc GVarFuncs[] =
{

  {"GAP_RedHom_CreateSimplicialComplex",             /* GAP function name */
   0,                  /* Number of parameters */
   "",     /* String for GAP to display list of parameter names */
   GAP_RedHom_CreateSimplicialComplex,           /* C function to call */
   "gap_interface.c" /* String to display function location */
  },


  {"GAP_RedHom_CreateCubicalComplex",
   1,
   "size",
   GAP_RedHom_CreateCubicalComplex,
   "gap_interface.c"
  },


  {"GAP_RedHom_CreateComplexFromPureCubicalComplex",
   1,
   "pureCubicalComplex",
   GAP_RedHom_CreateComplexFromPureCubicalComplex,
   "gap_interface.c"
  },

  {"GAP_RedHom_CreateSComplexFromPureCubicalComplex",
   1,
   "pureCubicalComplex",
   GAP_RedHom_CreateSComplexFromPureCubicalComplex,
   "gap_interface.c"
  },

  {"GAP_RedHom_DeleteComplex",             /* GAP function name */
   1,                  /* Number of parameters */
   "complex",     /* String for GAP to display list of parameter names */
   GAP_RedHom_DeleteComplex,           /* C function to call */
   "gap_interface.c" /* String to display function location */
  },


  {"GAP_RedHom_CubicalComplexFillWithBoundaries",
   1,
   "complex",
   GAP_RedHom_CubicalComplexFillWithBoundaries,
   "gap_interface.c"
  },


  {"GAP_RedHom_AddSimplexToComplex",             /* GAP function name */
   2,                  /* Number of parameters */
   "complex, simplex",     /* String for GAP to display list of parameter names */
   GAP_RedHom_AddSimplexToComplex,           /* C function to call */
   "gap_interface.c" /* String to display function location */
  },


  {"GAP_RedHom_AddFullCubeToComplex",             /* GAP function name */
   2,                  /* Number of parameters */
   "complex, cube",     /* String for GAP to display list of parameter names */
   GAP_RedHom_AddFullCubeToComplex,           /* C function to call */
   "gap_interface.c" /* String to display function location */
  },


  {"GAP_RedHom_ComplexCardinality",             /* GAP function name */
   1,                  /* Number of parameters */
   "complex",     /* String for GAP to display list of parameter names */
   GAP_RedHom_ComplexCardinality,           /* C function to call */
   "gap_interface.c" /* String to display function location */
  },

  {"GAP_RedHom_ComplexDim",             /* GAP function name */
   1,                  /* Number of parameters */
   "complex",     /* String for GAP to display list of parameter names */
   GAP_RedHom_ComplexDim,           /* C function to call */
   "gap_interface.c" /* String to display function location */
  },

  {"GAP_RedHom_BettiNumbersOverZOfComplex",
   1,
   "complex",
   GAP_RedHom_BettiNumbersOverZOfComplex,
   "gap_interface.c"
  },


  {"GAP_RedHom_Coreduction",
   1,
   "complex",
   GAP_RedHom_Coreduction,
   "gap_interface.c"
  },

  /* {"GAP_RedHom_FundGroup", */
  /*  1, */
  /*  "complex", */
  /*  GAP_RedHom_FundGroup, */
  /*  "gap_interface.c" */
  /* }, */

  {"GAP_RedHom_FundamentalGroup",
   2,
   "complex, vectorFieldType",
   GAP_RedHom_FundamentalGroup,
   "gap_interface.c"
  },

  {"GAP_RedHom_ZigZag",
   2,
   "complex, dim",
   GAP_RedHom_ZigZag,
   "gap_interface.c"
  },

  {"GAP_RedHom_CountCriticalCells",
   2,
   "complex, method",
   GAP_RedHom_CountCriticalCells,
   "gap_interface.c"
  },


  { 0, 0, 0, 0, 0 } /* Finish with an empty entry */
};


/**
Function called by GAP after a workspace restore. This is also called by us in
InitLibrary();
**/
static Int PostRestore(StructInitInfo* module)
{
  UNUSED(module);
  Int i;
  Int gvar;
  Obj tmp;

  gvar = GVarName("CAPD");
  tmp = (Obj)VAL_GVAR(gvar);
  if(!tmp)
  {
    tmp = (Obj)NEW_PREC(0);
  }

  /* Write the names of my functions into th LinBox record */
  for(i = 0; GVarFuncs[i].name != 0; i++)
  {
    AssPRec(
      tmp,
      RNamName((Char*)GVarFuncs[i].name),
      NewFunctionC(
        GVarFuncs[i].name,
        GVarFuncs[i].nargs,
        GVarFuncs[i].args,
        GVarFuncs[i].handler) );
  }

  MakeReadWriteGVar(gvar);
  AssGVar(gvar, tmp);
  MakeReadOnlyGVar(gvar);
  return 0;
}


/**
The first function to be called when the library is loaded by the kernel.
**/
static Int InitKernel(StructInitInfo* module)
{
  UNUSED(module);
  /* init filters and functions                                          */
  InitHdlrFuncsFromTable( GVarFuncs );
  ImportFuncFromLibrary("Dimension", &Dimension);
  ImportFuncFromLibrary("EvaluateProperty", &EvaluateProperty);
  ImportFuncFromLibrary("ArrayValue", &ArrayValue);
  /* return success                                                      */
  return 0;
}


/**
The second function to be called when the library is loaded by the kernel.
**/
static Int InitLibrary(StructInitInfo* module)
{
    PostRestore( module );

    RedHom_Init();

    /* return success                                                      */
    return 0;
}


/**
Information about this library, returned when the library is loaded,
for example by Init__Dynamic(). This contains details of the library name,
and the further initialisation functions to call.
**/
static StructInitInfo module = {
#ifdef STATICMODULE
 /* type        = */ MODULE_STATIC,
#else
 /* type        = */ MODULE_DYNAMIC,
#endif
 /* name        = */ "simple CAPD interface",
 /* revision_c  = */ 0,
 /* revision_h  = */ 0,
 /* version     = */ 0,
 /* crc         = */ 0,
 /* initKernel  = */ InitKernel,
 /* initLibrary = */ InitLibrary,
 /* checkInit   = */ 0,
 /* preSave     = */ 0,
 /* postSave    = */ 0,
 /* postRestore = */ PostRestore,
 0,
 0
};


#ifndef STATICGAP
/**
Function called by GAP as soon as the library is dynamically loaded.
This returns the StructInitInfo data for this library
**/
StructInitInfo * Init__Dynamic (void)
{
 return &module;
}
#endif
