/// @addtogroup multiEngHom
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file CrbHomMD.cpp
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2005-2010 by Marian Mrozek
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.ii.uj.edu.pl/ for details.

#include <iostream>
#include <fstream>
#include <sstream>
#include <exception>
#include <new>
using namespace std;

#include "capd/homology/CrbHomMD.h"
#include "capd/auxil/handleUnexpectedTerminate.h"
#include "capd/auxil/commandLineArgs.h"

int main(int argc,char** argv){
  set_unexpected(handle_unexpected);
  set_terminate(handle_terminate);
  fcout.open("report.txt");
  fcout.turnOn();

  setupCommandLineArgs;
  declareCommandLineArg(std::string,inFile,"");
  declareCommandLineArg(std::string,engine,"CR");
  declareCommandLineArg(std::string,help,"");
  declareCommandLineArg(int,verbose,2);

  std::string executableName("CrbHomMD");
  std::string version(" 0.9 ");
  std::string copyright("(C) CAPD::RedHom Group, 2010");
  std::string logo(executableName+version+copyright);
  std::string webpage("http://redhom.ii.uj.edu.pl");

  fcout << logo  << std::endl;
  fcout << webpage  << std::endl;
  fcout << "-----------------"  << std::endl;

  int fileCount=getCommandLineFileCount();
  if(fileCount>1){
    throw "Only one file can be processed";
  }else if(fileCount==0 && help==""){
    help="intro";
  }else{
    inFile=getCommandLineFile(1);
    if(inFile=="help") help="intro";
  }


  if(help != ""){
    if(help=="input"){
//      fcout << logo  << std::endl;
      fcout << "" << executableName << " accepts files in the following formats\n";
      fcout << "   bmp - two dimensional bitmaps in the standard BMP bitmap format\n";
      fcout << "   bmd - multidimensional bitmaps in the bmd format\n";
      fcout << "   cub/txt - various text formats\n";
      fcout << "Consult http://ii.uj.edu.pl/~mrozek/software/homology.html for details\n";
      exit(0);
    }else if(help=="output"){
//      fcout << logo  << std::endl;
      fcout << "The amount of information provided may be controlled with  \n\n";
      fcout << "   verbose=n\n\n";
      fcout << "where n means the following output\n";
      fcout << "   0 - only Betti numbers\n";
      fcout << "   1 - Betti numbers and computation time \n";
      fcout << "   2 - Betti and torsion numbers (default)\n";
      fcout << "   3 - Betti and torsion numbers with some details of the computation process\n";
      fcout << "Consult http://ii.uj.edu.pl/~mrozek/software/homology.html for details\n";
      exit(0);
    }else{
//      fcout << logo  << std::endl;
      fcout << "Basic usage: \n";
      fcout << "      " << executableName << " filename\n";
      fcout << "Type: \n";
      fcout << "      " << executableName << " help=xx\n";
      fcout << "where xx may be:  input  or output\n";
      fcout << "to learn about available input/output formats\n";
      exit(0);
    }
  }

  try{
    CrbHom<0> ch;
    ch(inFile.c_str(),verbose);
  }catch(std::exception& e){
    fcout.turnOn();
    fcout << "Caught exception: " << e.what() << endl;
  }catch(std::string& s){
    fcout.turnOn();
    fcout << "Caught exception: " << s.c_str() << endl;
  }catch(const char* c){
    fcout.turnOn();
    fcout << "Caught exception: " << c << endl;
  }catch(...){
    fcout.turnOn();
    fcout << "Caught an unknown exception: " << endl;
  }

}

/// @}

