/// @addtogroup homologicalAlgebra
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file cubRedHom.cpp
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Marian Mrozek 2005-2010
//
// This file constitutes a part of the CAPD library,
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details.



#include <cstdlib>
#include "capd/auxil/commandLineArgs.h"
#include "cubRedHom.hpp"
#include "capd/auxil/handleUnexpectedTerminate.h"
#include "capd/cubSet/acyclicConfigs.hpp"
#include <capd/vectalg/Matrix.hpp>

ofstreamcout fcout;

int main(int argc,char **argv){

  Stopwatch swTot;
  set_unexpected(handle_unexpected);
  set_terminate(handle_terminate);
  fcout.open("report.txt");
  fcout.turnOn();

  setupCommandLineArgs;
  declareCommandLineArg(std::string,inFile,"");
  declareCommandLineArg(std::string,engine,"");
  declareCommandLineArg(std::string,help,"");
  declareCommandLineArg(int,mod,0);
  declareCommandLineArg(int,silent,0);
  declareCommandLineArg(std::string,outputMode,"");
  declareCommandLineArg(std::string,rescaleFactor,"");

  declareCommandLineArg(std::string,testAS,"");
  declareCommandLineArg(std::string,acAlg,"");
  declareCommandLineArg(std::string,shave,"");

  std::string executableName("cubRedHom");
  std::string version(" 0.991 ");
  std::string copyright("(C) CAPD::RedHom Group, 2010");
  std::string logo(executableName+version+copyright);
  std::string webpage("http://redhom.ii.uj.edu.pl");

//showCommandLineArgs;
  fcout << logo  << std::endl;
  fcout << webpage  << std::endl;
  fcout << "-----------------"  << std::endl;

  int fileCount=getCommandLineFileCount();
  if(fileCount>1){
    std::cout << "Only one file can be processed";
    exit(1);
  }else if(fileCount==0 && help==""){
    help="intro";
  }else{
    inFile=getCommandLineFile(1);
    if(inFile=="help") help="intro";
  }

  Zp::setMod(mod);

  if(help != ""){
      std::cout << "cubRedHom computes homology groups, i.e. Betti numbers\n";
      std::cout << "and torsion coefficients of cubical sets. Basic usage: \n";
      std::cout << "      " << executableName << " filename [mod=p] [silent=n]\n";
      std::cout << "  The optional, bracketed parameters are: \n";
      std::cout << "  mod=p --- take Z_p with p a prime as coefficient ring\n";
      std::cout << "  silent=1 --- provide silent output.\n";
      std::cout << "  File filename must contain the set\n";
      std::cout << "  in one of available formats.\n";
      std::cout << "Consult " << webpage << " for details\n\n";
      exit(0);
  }

  if(silent) fcout.turnOff();
  try{
    if(mod==1 || mod<0){
      std::string s;
      s << "Option mod=" << mod << " is not supported\n";
      throw s;
    }
    if(mod>=2) fcout << "  Using integers modulo " << mod  << std::endl;

    RedHom(inFile.c_str(),engine,_command_line_args_);
  }catch(std::exception& e){
    std::cout << "Caught exception: " << e.what() << endl;
  }catch(std::string& s){
    std::cout << "Caught exception: " << s.c_str() << endl;
  }catch(const char* c){
    std::cout << "Caught exception: " << c << endl;
  }catch(...){
    std::cout << "Caught an unknown exception: " << endl;
  }
  std::cout << "Total time (including input/output) " << swTot  << std::endl;

}

/// @}
