#ifndef _RED_HOM_OPTIONS_H
#define _RED_HOM_OPTIONS_H

#include <boost/program_options.hpp>
#include <boost/program_options/parsers.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string.hpp>

#include <sstream>
#include <string>

namespace redHom {
  namespace util {

    class RedHomOptions {
    public:
      enum FileType { FT_UNKNOWN = -1, FT_CUB, FT_PCUB, FT_FCUB, FT_BMD, FT_DAT, FT_OBJ, FT_GEN  };


    public:

      RedHomOptions(int ac, const char** av, int inputFilesCount=3);

      bool isHelp() const;
      std::string getHelp() const ;
      bool useDefaultAlgorithms() const;
      FileType getFileType() const;
      bool isCubicalFileType() const;
      bool isSimplicialFileType() const;
      bool isGeneralFileType() const;
      const std::string& getInputFile(int i) const;
      size_t getInputFilesCount() const;
      const std::vector<std::string> getInputFiles() const;
      bool isGeneralSComplex() const;
      bool isHomBasis() const;
      bool isRfcAtStart() const;
      bool isExperimentalRfcAtStart() const;
      bool isRfcAtEnd() const;
      bool isExperimentalRfcAtEnd() const;
      bool isCoreduce() const;
      bool isShave() const;
      bool isMorse() const;
      bool isKMS() const;
      bool isMatchingReduce() const;
      bool isBMatrices() const;
      bool isSaveFiltration() const;
      bool isStatisticsFile() const;
      const std::string& getStatisticsFile() const;
      int getModulo() const;
      bool isBigInt() const;

    private:

      std::string getInputFileKey(int i) const;

      boost::program_options::variables_map vm;
      boost::program_options::options_description desc;
      std::vector<std::string> inputFiles;
      std::string statisticsFile;
      bool generalSComplex;
      bool homBasis;
      bool rfcAtStart;
      bool experimentalRfcAtStart;
      bool rfcAtEnd;
      bool experimentalRfcAtEnd;
      bool noDefault;
      bool coreduce;
      bool shave;
      bool morse;
      bool kms;
      bool bmatrices;
      bool saveFiltration;
      int modulo;
      bool bigInt;
      FileType fileType;
    };

    static void validate(boost::any& v,
    const std::vector<std::string>& values,
    RedHomOptions::FileType*, int)
    {
      using namespace boost::program_options;

      validators::check_first_occurrence(v);
      const std::string& s = validators::get_single_string(values);

      if (s == "dat") {
	v = boost::any(RedHomOptions::FT_DAT);
      } else if (s == "obj") {
	v = boost::any(RedHomOptions::FT_OBJ);
      } else if (s == "cub") {
	v = boost::any(RedHomOptions::FT_CUB);
      } else if (s == "pcub") {
	v = boost::any(RedHomOptions::FT_PCUB);
      } else if (s == "fcub") {
	v = boost::any(RedHomOptions::FT_FCUB);
      } else if (s == "bmd") {
	v = boost::any(RedHomOptions::FT_BMD);
      } else {
	throw validation_error(validation_error::invalid_option_value);
      }
    }

  }
}

#endif //  _RED_HOM_OPTIONS_H
