/////////////////////////////////////////////////////////////////////////////
/// @file AlgorithmExecutor.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-11-29
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_ALGORITHMEXECUTOR_H
#define CAPD_FILE_ALGORITHMEXECUTOR_H

#include <capd/apiRedHom/AnyWrapper.h>

#include <boost/mpl/for_each.hpp>
#include <boost/assert.hpp>
namespace capd
{
  namespace apiRedHom
  {

    template <typename Algorithm, typename Callback = void, typename Enable = void>
    struct AlgorithmExecutor
    {
      explicit AlgorithmExecutor(AnyWrapper& anyWrapper, Algorithm algorithm, bool& called):
        _anyWrapper(anyWrapper),
        _algorithm(algorithm),
        _called(called)
      {}

      template<typename TypeDescriptor>
      void operator()(TypeDescriptor)
      {
        if (TypeDescriptor::TYPE_ID == _anyWrapper.type_id) {
          BOOST_ASSERT(!_called);
          _algorithm.template operator()(TypeDescriptor(), _anyWrapper);
          _called = true;
        }
      }

      AnyWrapper& _anyWrapper;
      Algorithm _algorithm;
      bool& _called;
    };

    template <typename Algorithm, typename Callback>
    struct AlgorithmExecutor<Algorithm, Callback, typename boost::disable_if<boost::is_same<Callback, void> >::type>
    {

      explicit AlgorithmExecutor(AnyWrapper& anyWrapper, Algorithm algorithm, Callback callback, bool& called):
        _anyWrapper(anyWrapper),
        _algorithm(algorithm),
        _callback(callback),
        _called(called)
      {}

      template<typename TypeDescriptor>
      void operator()(TypeDescriptor)
      {
        if (TypeDescriptor::TYPE_ID == _anyWrapper.type_id) {
          BOOST_ASSERT(!_called);
          _callback(_algorithm.template operator()(TypeDescriptor(), _anyWrapper));
          _called = true;
        }
      }

      AnyWrapper& _anyWrapper;
      Algorithm _algorithm;
      Callback _callback;
      bool& _called;
    };


    template <typename TypeDescriptors, typename Algorithm>
    void RunAlgorithm(AnyWrapper& object, Algorithm algorithm)
    {
      bool called = false;
      AlgorithmExecutor<Algorithm> exe(object, algorithm, called);
      boost::mpl::for_each<TypeDescriptors>(exe);
      BOOST_ASSERT(called);
    }

    template <typename TypeDescriptors, typename Algorithm, typename Callback>
    void RunAlgorithm(AnyWrapper& object, Algorithm algorithm, Callback callback)
    {
      bool called = false;
      AlgorithmExecutor<Algorithm, Callback> exe(object, algorithm, callback, called);
      boost::mpl::for_each<TypeDescriptors>(exe);
      BOOST_ASSERT(called);
    }

  }
}

#endif // CAPD_FILE_ALGORITHMEXECUTOR_H
