/////////////////////////////////////////////////////////////////////////////
/// @file Algorithms_BettiNumbers
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-23
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/apiRedHom/Algorithms.h>
#include <capd/apiRedHom/Complex.h>


#include <capd/complex/BettiNumbers.h>
#include <capd/complex/BettiNumbersViaSmithForm.h>

#include <AlgorithmExecutor.h>
#include <ComplexDescriptors.h>

#include <capd/vectalg/Container.hpp>
#include <boost/lambda/lambda.hpp>
#include <capd/rings/Zp.h>
#include <capd/rings/Z2.h>



using namespace capd;
using namespace capd::apiRedHom;

using namespace capd::apiRedHom;
namespace bl =  boost::lambda;
namespace b = boost;

namespace
{

  template<typename Scalar, typename Int>
  struct RunBettiNumbers
  {
    template<typename TypeDescriptor>
    std::vector<Int> operator()(TypeDescriptor, AnyWrapper& anyComplex)
    {
      typedef typename TypeDescriptor::type Complex;
      Complex& complex = *anyComplex.get<boost::shared_ptr<Complex> >();
      capd::complex::BettiNumbers<Complex, Scalar, Int> bettiNumbersImpl(complex);

      return bettiNumbersImpl();
    }
  };
}


std::vector<int> BettiNumbersOverZ::operator()()
{
  std::vector<int> result;
  RunAlgorithm<capd::complex::Complexes>(complex.anyComplex(), RunBettiNumbers<int, int>(), bl::var(result) = bl::_1);
  return result;
}

std::vector<int> BettiNumbersOverZp::operator()()
{
  const int oldP = Zp::getMod();
  Zp::setMod(p);
  std::vector<int> result;
  RunAlgorithm<capd::complex::Complexes>(complex.anyComplex(), RunBettiNumbers<Zp, int>(), bl::var(result) = bl::_1);
  Zp::setMod(oldP);
  return result;
}

std::vector<int> BettiNumbersOverZ2::operator()()
{
  std::vector<int> result;
  RunAlgorithm<capd::complex::Complexes>(complex.anyComplex(), RunBettiNumbers<Z2, int>(), bl::var(result) = bl::_1);
  return result;
}


std::vector<int> capd::apiRedHom::ComputeBettiNumbersOverZ(Complex& complex)
{
  BettiNumbersOverZ algo(complex);
  return algo();
}

std::vector<int> capd::apiRedHom::ComputeBettiNumbersOverZp(Complex& complex, int p)
{
  BettiNumbersOverZp algo(complex, p);
  return algo();
}

std::vector<int> capd::apiRedHom::ComputeBettiNumbersOverZ2(Complex& complex)
{
  BettiNumbersOverZ2 algo(complex);
  return algo();
}

std::vector<int> capd::apiRedHom::ComputeBettiNumbers(Complex& complex, int modulo)
{
  if (modulo == 0) {
    return ComputeBettiNumbersOverZ(complex);
  } else if (modulo == 2) {
    return ComputeBettiNumbersOverZ2(complex);
  } else {
    return ComputeBettiNumbersOverZp(complex, modulo);
  }
}

namespace
{
  template<typename Scalar, typename Int>
  struct RunBettiNumbersViaSmithFormOverZ
  {

    RunBettiNumbersViaSmithFormOverZ()
    {}

    template<typename TypeDescriptor>
    std::vector<Scalar> operator()(TypeDescriptor, AnyWrapper& anyComplex)
    {
      typedef typename TypeDescriptor::type Complex;
      Complex& complex = *anyComplex.get<boost::shared_ptr<Complex> >();

      capd::complex::BettiNumbersViaSmithForm<Complex, Scalar, Int> alg(complex);
      return alg();
    }
  };
}

std::vector<int> apiRedHom::BettiNumbersViaSmithFormOverZ::operator()()
{
  std::vector<int> result;
  RunAlgorithm<capd::complex::Complexes>(_complex.anyComplex(),
                                   RunBettiNumbersViaSmithFormOverZ<Scalar, int>(),
                                   bl::var(result) = bl::_1);
  return result;
}
