/////////////////////////////////////////////////////////////////////////////
/// @file Algorithms_FundGroup
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-23
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#define TURN_OFF_LEGACY_REDHOM 1

#define USE_PRECOMPILED_CAPDREDHOM_COMPLEX_COMPUTEHOMOLOGYORCHESTRATOR_HPP

#include <capd/apiRedHom/Algorithms.h>
#include <capd/apiRedHom/Complex.h>

#include <AlgorithmExecutor.h>
#include <ComplexDescriptors.h>

#include <capd/complex/BuildGreedyDiscreteVectorField.h>
#include <capd/complex/FundamentalGroup.h>

#include <capd/complex/Shave.h>

using namespace capd;
using namespace capd::apiRedHom;
using namespace capd::complex;

namespace bl = boost::lambda;

namespace
{
  // template<typename Traits>
  // struct FundGroupComplexSupplierType
  // {
  //   typedef AKQReducedSComplexSupplier<Traits> Type;
  //   //typedef NotReducedSComplexSupplier<Traits> Type;
  // };

  // template<int DIM>
  // struct FundGroupComplexSupplierType<CubicalHomology<DIM> >
  // {
  //   typedef AKQReducedSComplexSupplier<CubicalHomology<DIM> > Type;
  //   //Collapsed may produce 'complicated' groups
  //   //typedef CollapsedAKQReducedCubSComplexSupplier<CubicalHomology<3> > Type;
  // };

  // template<>
  // struct FundGroupComplexSupplierType<CubicalHomology<3> >
  // {
  //   //typedef AKQReducedSComplexSupplier<CubicalHomology<DIM> > Type;
  //   //Collapsed may produce 'complicated' groups
  //   typedef CollapsedAKQReducedCubSComplexSupplier<CubicalHomology<3> > Type;
  // };

  // template<typename ComplexTag>
  // struct FundGroupType
  // {
  //   //    typedef ::FundGroup<typename FundGroupComplexSupplierType<DefaultHomologyTraits>::Type> Type;
  // };

  // template<>
  // struct FundGroupType<capd::complex::OldCubicalTag>
  // {
  //   typedef ::FundGroup<typename FundGroupComplexSupplierType<CubicalHomology<DIM> >::Type> Type;
  // };

  // template<>
  // struct FundGroupType<capd::complex::DenseCubicalTag>
  // {
  //   typedef ::FundGroup<typename FundGroupComplexSupplierType<CubicalCellHomology<DIM> >::Type> Type;
  // };

  // template<>
  // struct FundGroupType<capd::complex::GeneralizedTag>
  // {
  //   typedef ::FundGroup<FundGroupComplexSupplierType<SComplexHomology>::Type > Type;
  // };

  // template<>
  // struct FundGroupType<capd::complex::SimplicialTag>
  // {
  //   typedef ::FundGroup<FundGroupComplexSupplierType<SimplicialHomology>::Type > Type;
  // };

  // struct RunFundGroup
  // {

  //   template<typename TypeDescriptor>
  //   std::vector<int> operator()(TypeDescriptor, AnyWrapper& anyComplex)
  //   {
  //     typedef typename TypeDescriptor::type Complex;
  //     typedef ::FundGroup<typename FundGroupComplexSupplierType<HomologyTraits<TypeDescriptor> >::Type>  FundGroup;
  //     typedef typename FundGroup::ComplexSupplier ComplexSupplier;
  //     typedef typename FundGroup::ComplexSupplierPtr ComplexSupplierPtr;

  //     boost::shared_ptr<Complex> complex = anyComplex.get<boost::shared_ptr<Complex> >();

  //     typedef ShaveAlgorithmFactory<Complex> ShaveFactory;
  //     boost::shared_ptr<typename ShaveFactory::DefaultAlgorithm> shave = ShaveFactory().createDefault(*complex);
  //     (*shave)();
  //     FundGroup fundGroup(ComplexSupplierPtr(new ComplexSupplier(complex)));

  //     return fundGroup.HapInterfaceVector();
  //   }

  // };

  struct RunFundamentalGroup
  {

    explicit RunFundamentalGroup(int vectorFieldType):
      _vectorFieldType(vectorFieldType)
    {}

    template<typename TypeDescriptor>
    std::vector<int> operator()(TypeDescriptor, AnyWrapper& anyComplex)
    {
      typedef typename TypeDescriptor::type Complex;
      boost::shared_ptr<Complex> complex = anyComplex.get<boost::shared_ptr<Complex> >();

      typedef ShaveAlgorithmFactory<Complex> ShaveFactory;
      boost::shared_ptr<typename ShaveFactory::DefaultAlgorithm> shave = ShaveFactory().createDefault(*complex);
      (*shave)();

      if (_vectorFieldType == 0) {
        typename BuildGreedyDiscreteVectorFieldViaCoreductions<Complex, int>::type vfAlgo(*complex);
        return FundamentalGroupHapInterfaceVector(vfAlgo());
      } else {
        typename BuildGreedyDiscreteVectorFieldViaReductions<Complex, int>::type vfAlgo(*complex);
        return FundamentalGroupHapInterfaceVector(vfAlgo());
      }

      return std::vector<int>();
    }

    int _vectorFieldType;
  };

}

// std::vector<int> apiRedHom::FundGroup::operator()()
// {
//   std::vector<int> result;
//   RunAlgorithm<capd::complex::Complexes>(complex.anyComplex(), RunFundGroup(), bl::var(result) = bl::_1);
//   return result;
// }

std::vector<int> apiRedHom::FundamentalGroup::operator()()
{
  std::vector<int> result;
  RunAlgorithm<capd::complex::Complexes>(complex.anyComplex(), RunFundamentalGroup(_vectorFieldType), bl::var(result) = bl::_1);
  return result;
}
