/////////////////////////////////////////////////////////////////////////////
/// @file Algorithms_Homology
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-01-18
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include <capd/apiRedHom/Algorithms.h>
#include <capd/apiRedHom/Complex.h>


#include <capd/complex/Homology.h>

#include <AlgorithmExecutor.h>
#include <ComplexDescriptors.h>

#include <capd/vectalg/Container.hpp>
#include <boost/lambda/lambda.hpp>
#include <capd/rings/Zp.h>
#include <capd/rings/Zp.h>
#include <capd/rings/Z2.h>



using namespace capd;
using namespace capd::apiRedHom;

using namespace capd::apiRedHom;
namespace bl =  boost::lambda;
namespace b = boost;

namespace
{

  template<typename Scalar, typename Int>
  struct RunHomology
  {
    template<typename TypeDescriptor>
    Signature operator()(TypeDescriptor, AnyWrapper& anyComplex)
    {
      typedef typename TypeDescriptor::type Complex;
      typedef capd::complex::Homology<Complex, Scalar, Int> HomologyImpl;
      typedef typename HomologyImpl::Signature SignatureImpl;

      Complex& complex = *anyComplex.get<boost::shared_ptr<Complex> >();
      HomologyImpl homologyImpl(complex);
      SignatureImpl signatureImpl = homologyImpl();
      Signature result;

      result.raw = signatureImpl.raw;
      result.str = signatureImpl.str;

      return result;
    }
  };

  template<typename Scalar, typename Int>
  Signature Run(Complex& complex)
  {
    Signature result;
    RunAlgorithm<capd::complex::Complexes>(complex.anyComplex(), RunHomology<Scalar, Int>(), bl::var(result) = bl::_1);
    return result;
  }
}


Signature HomologyOverZ::operator()()
{
  return Run<int, int>(complex);
}

Signature HomologyOverZp::operator()()
{
  const int oldP = Zp::getMod();
  Zp::setMod(p);
  Signature res = Run<Zp, int>(complex);
  Zp::setMod(oldP);

  return res;
}

Signature HomologyOverZ2::operator()()
{
  return Run<Z2, int>(complex);
}


Signature capd::apiRedHom::ComputeHomologyOverZ(Complex& complex)
{
  HomologyOverZ algo(complex);
  return algo();
}

Signature capd::apiRedHom::ComputeHomologyOverZp(Complex& complex, int p)
{
  HomologyOverZp algo(complex, p);
  return algo();
}

Signature capd::apiRedHom::ComputeHomologyOverZ2(Complex& complex)
{
  HomologyOverZ2 algo(complex);
  return algo();
}
