/////////////////////////////////////////////////////////////////////////////
/// @file Complex.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-11-28
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <capd/apiRedHom/Complex.h>

#include <capd/complex/SComplexKappaBuilderFromCubes.h>

#include "ComplexDescriptors.h"
#include "AlgorithmExecutor.h"
#include "Complex.hpp"

#include <boost/make_shared.hpp>
#include <boost/functional/value_factory.hpp>

using namespace capd::apiRedHom;
using namespace capd::complex;
namespace bl =  boost::lambda;
namespace b = boost;


Complex::~Complex()
{
  // empty is in implementation where we have all fwd types declared
}


namespace
{
  struct CreateComplexIterators
  {
    template<typename TypeDescriptor>
    ComplexIterators operator()(TypeDescriptor, AnyWrapper& complex)
    {
      return ComplexIterators(AnyWrapper(complex.get<boost::shared_ptr<typename TypeDescriptor::type> >()->iterators(), TypeDescriptor::TYPE_ID));
    }
  };
}

ComplexIterators Complex::iterators()
{
  ComplexIterators result;
  RunAlgorithm<capd::complex::Complexes>(this->_complex, CreateComplexIterators(), bl::var(result) = bl::_1);
  return result;
}



namespace
{
  struct CreateAllCells
  {
    template<typename TypeDescriptor>
    ComplexIterators::AllCells operator()(TypeDescriptor, AnyWrapper& iterators)
    {
      return ComplexIterators::AllCells(iterators.get<typename TypeDescriptor::type>().allCells());
    }
  };
}

// ComplexIterators::AllCells ComplexIterators::allCells()
// {
//   AllCells result;
//   RunAlgorithm<IteratorsDescriptors>(this->_iterators, CreateAllCells(), bl::var(result) = bl::_1);
//   return result;
// }
