/////////////////////////////////////////////////////////////////////////////
/// @file Complex
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-23
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#ifndef CAPD_FILE_CAPDREDHOM_APIREDHOM_COMPLEX_HPP
#define CAPD_FILE_CAPDREDHOM_APIREDHOM_COMPLEX_HPP

#include <capd/apiRedHom/Complex.h>
#include <capd/apiRedHom/AnyWrapper.h>

#include "ComplexDescriptors.h"
#include "AlgorithmExecutor.h"

#include <boost/mpl/map.hpp>
#include <boost/mpl/at.hpp>
#include <boost/mpl/pair.hpp>

namespace bl =  boost::lambda;
namespace b = boost;

namespace capd
{
  namespace apiRedHom
  {
    template<typename T>
    struct XXX { typedef T type; };

    template<typename ComplexT>
    struct SetFunctions
    {
      ComplexT& complex;

      template<typename SComplex>
      struct FindOp
      {
        template<typename T>
        b::optional<typename Complex::Cell> operator()(const b::optional<T>& o) const
        {
          return o ? b::optional<Complex::Cell>(*o) : b::optional<Complex::Cell>();
        }
      };

      template<typename TypeDescriptor>
      void operator()(TypeDescriptor, AnyWrapper& anyComplex)
      {
        typedef typename TypeDescriptor::type UnwrappedComplex;
        typedef boost::shared_ptr<UnwrappedComplex> UnwrappedComplexPtr;
        typedef typename UnwrappedComplex::Cell Cell;
        typedef typename UnwrappedComplex::CellCode CellCode;

        typedef typename XXX<Cell (UnwrappedComplex::*)(const CellCode&)>::type FindByCode;
        typedef typename XXX<boost::optional<Cell> (UnwrappedComplex::*)(const CellCode&)>::type FindByCodeOpt;

        UnwrappedComplex* unwrappedComplex = anyComplex.get<UnwrappedComplexPtr>().get();
        complex._dim = bl::bind(&UnwrappedComplex::getDim, unwrappedComplex);
        complex._cardinality = bl::bind(&UnwrappedComplex::cardinality, unwrappedComplex);

        complex._operator_array = bl::bind(FindByCode(&UnwrappedComplex::operator[]), unwrappedComplex, bl::_1);
        complex._find =  bl::bind<b::optional<Complex::Cell> >(typename SetFunctions::template FindOp<UnwrappedComplex>(),
                                                               bl::bind(FindByCodeOpt(&UnwrappedComplex::find), unwrappedComplex, bl::_1));
      }
    };
  }
}

namespace capd
{
  namespace apiRedHom
  {

    template<typename ThisT>
    void Complex::set_complex(const capd::apiRedHom::AnyWrapper &complex)
    {
      namespace m = boost::mpl;
      typedef m::map<m::pair<CubicalComplex, complex::CubicalComplexes>,
                     m::pair<SimplicialComplex, complex::SimplicialComplexes>,
                     m::pair<GeneralizedSComplex, complex::GeneralizedComplexes> > ComplexToDescriptors;
      typedef typename m::at<ComplexToDescriptors, ThisT>::type ThisDescriptors;

      this->_complex = complex;
      SetFunctions<ThisT> setFunctions = {dynamic_cast<ThisT&>(*this)};
      RunAlgorithm<ThisDescriptors>(this->_complex, setFunctions);
    }

  }
}


#endif // CAPD_FILE_CAPDREDHOM_APIREDHOM_COMPLEX_HPP
