/////////////////////////////////////////////////////////////////////////////
/// @file Complex_Cubical
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-23
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include <capd/apiRedHom/Complex.h>
#include "Complex.hpp"

#include "ComplexDescriptors.h"
#include "AlgorithmExecutor.h"
#include <boost/bind.hpp>

using namespace capd::apiRedHom;
using namespace capd::complex;
namespace bl =  boost::lambda;
namespace b = boost;

namespace capd
{
namespace apiRedHom
{

  // static and outside heder file. It allows us to use capd::complex namespace in api without header file dependency
  const int CubicalComplex::TYPE_DEFAULT = complex::CubicalComplexes::OLD;
  const int CubicalComplex::TYPE_OLD = complex::CubicalComplexes::OLD;
  const int CubicalComplex::TYPE_DENSE = complex::CubicalComplexes::DENSE;
  const int CubicalComplex::TYPE_SPARSE = complex::CubicalComplexes::SPARSE;

  struct ConvertWrapperType
  {
    template<typename TypeDescriptor>
    void operator()(TypeDescriptor, AnyWrapper& native)
    {
      typedef typename TypeDescriptor::type CubComplex;
      typedef boost::shared_ptr<CubComplex> CubComplexPtr;
      typedef boost::shared_ptr<complex::CubSComplexBase> CubComplexBasePtr;

      CubComplexPtr ptr = boost::dynamic_pointer_cast<CubComplex>(native.get<CubComplexBasePtr>());
      native = AnyWrapper(ptr, TypeDescriptor::TYPE_ID);
    }
  };

  struct SetCubicalComplex
  {
    CubicalComplex& complex;

    template<typename TypeDescriptor>
    void operator()(TypeDescriptor, AnyWrapper& native)
    {
      typedef typename TypeDescriptor::type CubComplex;
      typedef boost::shared_ptr<CubComplex> CubComplexPtr;
      CubComplex* cubComplex = native.get<CubComplexPtr>().get();
      complex._insert = bl::bind(&CubComplex::insert, cubComplex, bl::_1);

      BOOST_AUTO(getFullCubeCellCode,
                 bl::bind(static_cast<typename CubComplex::CellCode (CubComplex::*)(const typename CubComplex::FullCube&)>(&CubComplex::operator()), cubComplex, bl::_1));
      complex._insertFullCube = bl::bind(&CubComplex::insert, cubComplex, getFullCubeCellCode);

      complex._fillWithBoundaries = bl::bind<void>(&CubComplex::fillWithBoundaries, cubComplex);

      complex.set_complex<CubicalComplex>(native);
    }
  };

  struct CreateCubicalComplex
  {
    const std::vector<size_t>& size;
    const CubicalComplex::CellCodeRange& cubes;

    CubicalComplex& complex;

    template<typename TypeDescriptor>
    void operator()(TypeDescriptor t, AnyWrapper&)
    {
      typedef typename TypeDescriptor::type CubComplex;
      boost::shared_ptr<CubComplex> cubComplex(new CubComplex(size));

      //std::for_each(cubes.begin(), cubes.end(), bl::bind(&CubComplex::insert, *cubComplexPtr, _1));
      for (auto cube : cubes) {
        cubComplex->insert(cube);
      }
      SetCubicalComplex set = {complex};
      AnyWrapper native(cubComplex, TypeDescriptor::TYPE_ID);
      set(t, native);
    }
  };

}
}


CubicalComplex::CubicalComplex(const std::vector<size_t>& size,
                               const CubicalComplex::CellCodeRange& cubes, int type)
{
  CAPD_TRACE("Creating CubicalComplex, size: " << size);
  CreateCubicalComplex create = {size, cubes, *this};

  AnyWrapper dummy(NULL, complex::CubicalComplexes::id(size.size(), type));

  RunAlgorithm<complex::CubicalComplexes>(dummy, create);
}
