/////////////////////////////////////////////////////////////////////////////
/// @file Complex_Generalized
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-23
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include <capd/apiRedHom/Complex.h>
#include "Complex.hpp"

#include "ComplexDescriptors.h"
#include "AlgorithmExecutor.h"
#include <boost/mpl/front.hpp>

using namespace capd::apiRedHom;
using namespace capd::complex;
namespace bl =  boost::lambda;
namespace b = boost;

namespace capd
{
namespace apiRedHom
{
  struct CreateSComplex
  {
    const GeneralizedSComplex::Dims& dims;
    const GeneralizedSComplex::KappaMapEntries& kappaMapEntries;
    GeneralizedSComplex& complex;

    template<typename TypeDescriptor>
    void operator()(TypeDescriptor, AnyWrapper&)
    {
      typedef typename TypeDescriptor::type SComplexImpl;
      boost::shared_ptr<SComplexImpl>scomplexImpl(new SComplexImpl(3, dims,
                                                                   typename SComplexImpl::KappaMap(kappaMapEntries.begin(), kappaMapEntries.end()), 1));

      complex.set_complex<GeneralizedSComplex>(AnyWrapper(scomplexImpl, TypeDescriptor::TYPE_ID));
    }
  };

}
}


GeneralizedSComplex::GeneralizedSComplex(const GeneralizedSComplex::Dims& dims, const GeneralizedSComplex::KappaMapEntries& kappaMapEntries)
{
  CreateSComplex create = {dims, kappaMapEntries, *this};
AnyWrapper dummy(NULL, GeneralizedComplexes::id());

  RunAlgorithm<GeneralizedComplexes>(dummy, create);
}
