/////////////////////////////////////////////////////////////////////////////
/// @file Complex_Simplicial
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-23
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include <capd/apiRedHom/Complex.h>
#include "Complex.hpp"

#include <capd/complex/SimplicesReader.h>

#include "ComplexDescriptors.h"
#include "AlgorithmExecutor.h"

using namespace capd::apiRedHom;
using namespace capd::complex;
namespace bl =  boost::lambda;
namespace b = boost;

namespace capd
{
namespace apiRedHom
{
  struct CreateSimplicialComplex
  {
    const SimplicialComplex::CellCodeRange& simplices;
    SimplicialComplex& complex;

    template<typename TypeDescriptor>
    void operator()(TypeDescriptor, AnyWrapper&)
    {
      typedef typename TypeDescriptor::type SimpComplex;
      boost::shared_ptr<SimpComplex> simpComplex(new SimpComplex());


      complex._insert = bl::bind(&SimpComplex::insert, simpComplex.get(), bl::_1);
      complex.set_complex<SimplicialComplex>(AnyWrapper(simpComplex, TypeDescriptor::TYPE_ID));

      std::for_each(simplices.begin(), simplices.end(), bl::bind(complex._insert, bl::_1));
    }
  };

}
}

SimplicialComplex::SimplicialComplex(const SimplicialComplex::CellCodeRange& simplices)
{
  CreateSimplicialComplex create = {simplices, *this};
  AnyWrapper dummy(NULL, capd::complex::SimplicialComplexes::id());

RunAlgorithm<capd::complex::SimplicialComplexes>(dummy, create);
}
