/// @addtogroup bitSet
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file EuclBitSet_UL_lowDim.cpp
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Marian Mrozek 2005-2006
//
// This file constitutes a part of the CAPD library,
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details.


#include "EuclBitSetT_Helper.h"


template<>
void EuclBitSetT<BitSetT<BitmapT<unsigned long int> >,2>::writeBmp(const char* fileName,unsigned int A_type) const{
  std::ofstream file(fileName,std::ios::out|std::ios::binary);
  BmpHeader<unsigned long int,2> header;
  // Under construction!!!
  // Header must be properly setup before it is written!!!
  int bytesToWrite=length*sizeof(unsigned long int);
  // standardType is 'BM', repset2DType is 'BN'
  header.type=A_type;
  header.offset=62;
  header.size=bytesToWrite+header.offset;
  header.ihsize=40;
  header.width=actualDimZeroBitWidth;
  header.height=paddedBitWidth[1];
  header.planes=1;
  header.bits=1;
  header.compression=0;
  header.imagesize=0;
  header.xresolution=0;
  header.yresolution=0;
  header.ncolours=2;
  header.importantcolours=0;
  header.color0=0xffffff;  // define that index 0 means white
  header.color1=0;         // define that index 1 means black
  header.write(file);

  // In bmp files each byte stores bits in a reversed order, so we need to change
  char* byteBitmapPtr=reinterpret_cast<char *>(bitmap);
  char* byteBitmapEndPtr=byteBitmapPtr+bytesToWrite;
  for(char* cPtr=byteBitmapPtr; cPtr<byteBitmapEndPtr;++cPtr){
    char w=0;
    for(int i=0;i<8;++i){
      w = w | (((*cPtr >> i) & 1) << (7-i));
    }
    *cPtr=w;
  }
  file.write(byteBitmapPtr,bytesToWrite);
}

template<>
void EuclBitSetT<BitSetT<BitmapT<unsigned long int> >,2>::readBmp(std::ifstream& file){
  BmpHeader<unsigned long int,2> header;
  header.read(file);

  paddedBitWidth[0]=header.width;
  paddedBitWidth[1]=header.height;
  setupWidths();
  setupStrides();

  BitSetT<BitmapT<unsigned long int> > internalBitSet(this->length,true);
  swap(internalBitSet,*(BitSetT<BitmapT<unsigned long int> >*)(this));

  int bytesToRead=paddedBitWidth[0]*header.height/8;

  unsigned char* byteBitmapPtr=reinterpret_cast<unsigned char *>(bitmap);
  file.read(reinterpret_cast<char *>(byteBitmapPtr),bytesToRead);

  // In bmp files each byte stores bits in a reversed order, so we need to change
  unsigned char* byteBitmapEndPtr=byteBitmapPtr+bytesToRead;
  for(unsigned char* cPtr=byteBitmapPtr; cPtr<byteBitmapEndPtr;++cPtr){
    char w=0;
    for(int i=0;i<8;++i){
      w = w | (((*cPtr >> i) & 1) << (7-i));
    }
    *cPtr=w;
  }


  if(header.color1>header.color0){ // black indicated by 0 index, so we must invert
    // We need to invert (flip), because black pixels are stored as zeros
    // We cannot use the class invert method, because in the bitmap file
    // the excess bits resulting from padding to 32 bits are marked as black
    for(int i=0;i<header.height;++i){
//std::cout << "Line " << i  << std::endl;
      unsigned long int* wordPtr=bitmap+i*paddedBitWidth[0]/bitsPerWord;
      for(int j=0;j<header.width;j+=bitsPerWord){
//std::cout << "  Word " << j  << std::endl;
        *wordPtr=~(*wordPtr);
        // In the last byte,
        // if width is not a multiciplity of bitsPerWord,
        // we need to cut the excessive inverted bytes
        int validExtraBits;
        if( (validExtraBits=header.width-j)<bitsPerWord){
//std::cout << "  Word before invertion " << std::hex << *wordPtr  << std::dec << std::endl;
          *wordPtr &= ((1 << validExtraBits)-1);
//std::cout << "  Word after invertion " << std::hex << *wordPtr  << std::dec << std::endl;
        };
        ++wordPtr;
      }
    }
  }
}

template<>
void EuclBitSetT<BitSetT<BitmapT<unsigned long int> >,2>::readBmp(const char* fileName){
  std::ifstream file(fileName,std::ios::in|std::ios::binary);
  readBmp(file);
}

SETUP(1)
SETUP(2)
SETUP(3)
