/////////////////////////////////////////////////////////////////////////////
/// @file ImageCubCellComplex
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-09-17
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include <capd/complex/ImageCubCellComplex.h>
#include <boost/assign.hpp>
#include <limits>

namespace capd
{
  namespace complex
  {

    ImageCubCellComplex::Complex* ImageCubCellComplex::Create(const ImageCubCellComplex::Image& image)
    {
      const size_t rows = image.numberOfRows();
      const size_t cols = image.numberOfColumns();
      const std::vector<size_t> lengths = boost::assign::list_of(rows)(cols);

      Complex* complex = new Complex(lengths);
      complex->resetColors(std::numeric_limits<Complex::Color>::max());

      for (size_t row = 0; row < rows; ++row) {
        for (size_t col = 0; col < cols; ++col) {
          Complex::Cell cell = complex->insert((*complex)(row, col));
          cell.setColor(image[row][col]);
          complex->fillWithBoundary(cell, cell.color());
        }
      }

      return complex;
    }


  }
}
