/////////////////////////////////////////////////////////////////////////////
/// @file SimplicesReader.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-11
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <capd/complex/SimplicesReader.h>
#include <capd/auxil/Logger.h>

#include <boost/tokenizer.hpp>
#include <boost/lexical_cast.hpp>

#include <string>
#include <sstream>
#include <algorithm>

using namespace capd::complex;
namespace b = boost;

void SimplicesReader::operator()(const boost::function<void(const SimplicesReader::CellCode&)>& callback)
{
  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
  const boost::char_separator<char> sep("()[];, ");

  std::string line;
  CellCode cellCode;

  while (std::getline(_input, line)) {
    const tokenizer tokens(line, sep);
    cellCode.clear();

    for (tokenizer::const_iterator tok_iter = tokens.begin();
	 tok_iter != tokens.end(); ++tok_iter) {
      cellCode.push_back(boost::lexical_cast<int>(*tok_iter));
    }

    if (!cellCode.empty()) {
      std::sort(cellCode.begin(), cellCode.end());
      callback(cellCode);
    }
  }

}
