/////////////////////////////////////////////////////////////////////////////
/// @file DistanceMatrixPersistentHomology
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-10-21
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include <capd/persistence/DistanceMatrixPersistentHomology.h>
#include <capd/persistence/DistanceMatrixPointCloud.h>
#include <capd/persistence/PointCloudSimplicialCellSComplex.h>
#include <capd/persistence/ComputePersistentHomology.h>
#include <capd/persistence/DegreeLimitedNeighbourhood.hpp>

namespace capd
{
  namespace persistence
  {

    void DistanceMatrixPersistentHomology::operator()()
    {
      DistanceMatrixPointCloud pointCloud(_distance);
      DegreeLimitedNeighbourhood<DistanceMatrixPointCloud> neighbourhood(pointCloud, _degreeLimit, 0);
      neighbourhood();

      PointCloudSimplicialCellSComplex complex(pointCloud, _dim);

      complex();

      ComputePersistentHomology<PointCloudSimplicialCellSComplex> persistence(complex);
      persistence();

      _diagramsAll = persistence.diagram();
      _pairsAll = persistence.pairs();
      _cyclesAll = persistence.cycles();

      for (int idx = 0; idx < _dim; ++idx) {
        _diagrams[idx] = persistence.diagram(idx);
        _pairs[idx] = persistence.pairs(idx);
        _cycles[idx] = persistence.cycles(idx);
      }
    }

    bool DistanceMatrixPersistentHomology::Enabled()
    {
      return ComputePersistentHomology<PointCloudSimplicialCellSComplex>::enabled();
    }


  }
}
