/////////////////////////////////////////////////////////////////////////////
/// @file DistanceMatrixPointCloud
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-10-21
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include <capd/persistence/DistanceMatrixPointCloud.h>


#include <boost/graph/graph_traits.hpp>
#include <boost/graph/adjacency_list.hpp>

#include <algorithm>
#include <boost/typeof/typeof.hpp>

namespace capd
{
  namespace persistence
  {

    void DistanceMatrixPointCloud::init()
    {
      _graph = new Graph(this->size());
    }

    DistanceMatrixPointCloud::~DistanceMatrixPointCloud()
    {
      delete _graph;
      _graph = NULL;
    }


    void DistanceMatrixPointCloud::setNeighbour(Id idxI, Id idxJ, double weight)
    {
      add_edge(idxI, idxJ, *_graph);
    }

    size_t DistanceMatrixPointCloud::edges()
    {
      return num_edges(*_graph);
    }

    void DistanceMatrixPointCloud::neighbours(Id idxI, std::vector<DistanceMatrixPointCloud::Id>& result) const
    {
      BOOST_AUTO(adj, boost::adjacent_vertices(idxI, *_graph));
      result.assign(adj.first, adj.second);
    }

    DistanceMatrixPointCloud::Distance DistanceMatrixPointCloud::distance(Id p1, Id p2) const
    {
      return _metric(p1, p2);
    }

 }
}
