/////////////////////////////////////////////////////////////////////////////
/// @file ImagePersistentHomology
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-09-17
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include <capd/persistence/ImagePersistentHomology.h>
#include <capd/persistence/ComputePersistentHomology.h>
#include <capd/complex/ImageCubCellComplex.h>
#include <memory>
#include <capd/vectalg/Matrix.hpp>

namespace capd
{
  namespace persistence
  {

    const PersistentSignature& ImagePersistentHomology::operator()()
    {
      typedef capd::complex::ImageCubCellComplex ImageCubCellComplex;
      std::auto_ptr<ImageCubCellComplex::Complex> complex(ImageCubCellComplex::Create(_image));

      ComputePersistentHomology<ImageCubCellComplex::Complex> persistence(*complex);
      persistence();
      _signature = persistence.signature();
      return _signature;
    }

    bool ImagePersistentHomology::Enabled()
    {
      return ComputePersistentHomology<capd::complex::ImageCubCellComplex::Complex>::enabled();
    }

  }
}
