/////////////////////////////////////////////////////////////////////////////
/// @file PointCloudSimplicialCellSComplex
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-27
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include <capd/persistence/PointCloudSimplicialCellSComplex.h>
#include <capd/persistence/BuildVietorisRips.h>
#include <boost/assign.hpp>
#include <boost/typeof/typeof.hpp>
#include <boost/bind.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/iterator/counting_iterator.hpp>
#include <utility>
#include <capd/auxil/Logger.h>





namespace capd
{
  namespace persistence
  {

    PointCloudSimplicialCellSComplex::PointCloudSimplicialCellSComplex(const AbstractPointCloud& cloud, Complex::Dim maxDim):
      _cloud(cloud),
      _maxDim(maxDim)
    {
    }


    void PointCloudSimplicialCellSComplex::operator()()
    {
      namespace bl = boost::lambda;
      namespace b = boost;

      std::deque<double> filtrationValue;
      BOOST_AUTO(pushToStorage, boost::assign::push_back(_storage));
      BOOST_AUTO(pushToFiltrationValue, boost::assign::push_back(filtrationValue));


      BuildVietorisRips build(_cloud, _maxDim);
      build(
            (
             bl::bind(b::bind<void>(b::ref(pushToStorage), _1), bl::_1),
             bl::bind(b::bind<void>(b::ref(pushToFiltrationValue), _1), bl::_3)
             )
            );

      std::vector<Complex::Id> idToPosition(_storage.size()); // we need at most _storage.size(), maybe less. It depends if there are duplicates from build()?

      _complex.reset(new Complex(_storage,
                                 bl::var(idToPosition)[bl::_2] = bl::_1
                                 ));


      _weights.resize(_complex->size());
      for (Complex::Id idx = 0, size = _complex->size(); idx < size; ++idx) {
        _weights[idx] = filtrationValue[idToPosition[idx]];
      }

    }



  }
}
