/////////////////////////////////////////////////////////////////////////////
/// @file PointCloudTypes.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-25
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <capd/persistence/PointCloudTypes.h>
#include <capd/persistence/PointCloud.hpp>
#include <capd/persistence/SizeLimitedNeighbourhood.hpp>
#include <capd/persistence/DegreeLimitedNeighbourhood.hpp>

#include <capd/vectalg/Vector.hpp>

namespace capd
{
  namespace persistence
  {

#define WRAP(...) __VA_ARGS__

#define INSTANCE_POINT_CLOUD_METHODS(TYPE) \
    template void TYPE::explicitDestructor();           \
    template void TYPE::init();                         \
    template void TYPE::neighbours(TYPE::Id idxI, std::vector<TYPE::Id>& result) const ; \
    template size_t TYPE::edges();                      \
    template double TYPE::distance(TYPE::Id p1, TYPE::Id p2) const; \
    template void SizeLimitedNeighbourhood<TYPE>::operator()();     \
    template void DegreeLimitedNeighbourhood<TYPE>::operator()();     \

#define INSTANCE_POINT_CLOUD(METRIC,DIM)   \
    INSTANCE_POINT_CLOUD_METHODS(WRAP(PointCloud<METRIC, DIM>));  \


#define INSTANCE_EUCLIDEAN_POINT_CLOUD(SCALAR,DIM) \
    INSTANCE_POINT_CLOUD(WRAP(EuclideanMetric<SCALAR, DIM>), DIM);  \


    INSTANCE_EUCLIDEAN_POINT_CLOUD(double, 0);
    INSTANCE_EUCLIDEAN_POINT_CLOUD(double, 3);

  }

  namespace vectalg
  {
#define VECTOR_INSTANCE(TYPE)                                           \
    template capd::vectalg::Vector<TYPE, 0u>::Vector(std::vector<TYPE>::iterator, std::vector<TYPE>::iterator); \
    template capd::vectalg::Vector<TYPE, 0u>::Vector(std::vector<TYPE>::const_iterator, std::vector<TYPE>::const_iterator); \

    VECTOR_INSTANCE(double);
  }
}
