/////////////////////////////////////////////////////////////////////////////
/// @file ComplexHelperTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-12
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <capd/apiRedHom/Complex.h>
#include <capd/auxil/Logger.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <capd/apiRedHom/ComplexHelper.h>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd;
using namespace capd::apiRedHom;

BOOST_AUTO_TEST_SUITE(ComplexHelperTestSuite)

std::vector<CubicalHelper::FullCube> kleinCubes =  {{12, 6, 6, 6}, {11, 6, 6, 6}, {11, 6, 6, 7}, {10, 6, 6, 7}, {9, 6, 6, 7}, {8, 6, 6, 7}, {8, 6, 6, 6}, {8, 6, 6, 5}, {9, 6, 6, 5}, {10, 6, 6, 5}, {11, 6, 6, 5}, {11, 7, 6, 6}, {11, 7, 6, 7}, {11, 7, 6, 5}, {11, 8, 6, 6}, {10, 8, 6, 7}, {10, 7, 6, 7}, {9, 7, 6, 7}, {8, 7, 6, 7}, {7, 6, 6, 6}, {8, 7, 6, 5}, {9, 7, 6, 5}, {10, 7, 6, 5}, {10, 8, 6, 5}, {10, 9, 6, 6}, {10, 9, 6, 7}, {9, 8, 6, 7}, {7, 7, 6, 7}, {7, 7, 6, 6}, {7, 7, 6, 5}, {9, 8, 6, 5}, {10, 9, 6, 5}, {9, 10, 6, 6}, {9, 9, 6, 7}, {8, 8, 6, 7}, {8, 8, 6, 5}, {9, 9, 6, 5}, {8, 10, 6, 6}, {8, 10, 6, 7}, {8, 9, 6, 7}, {7, 9, 6, 7}, {7, 8, 6, 7}, {7, 8, 6, 5}, {7, 9, 6, 5}, {8, 9, 6, 5}, {8, 10, 6, 5}, {7, 11, 7, 6}, {7, 10, 6, 7}, {6, 9, 6, 7}, {6, 8, 6, 7}, {6, 8, 5, 6}, {6, 8, 6, 5}, {6, 9, 6, 5}, {7, 10, 6, 5}, {6, 11, 7, 6}, {6, 10, 6, 7}, {6, 8, 5, 7}, {6, 8, 5, 5}, {6, 10, 6, 5}, {5, 10, 7, 6}, {5, 10, 7, 7}, {5, 10, 6, 7}, {5, 9, 6, 7}, {5, 9, 6, 5}, {5, 10, 6, 5}, {5, 10, 7, 5}, {4, 10, 7, 6}, {4, 10, 7, 7}, {4, 9, 6, 7}, {5, 8, 5, 7}, {5, 8, 5, 6}, {5, 8, 5, 5}, {4, 9, 6, 5}, {4, 10, 7, 5}, {3, 9, 7, 6}, {3, 9, 7, 7}, {4, 8, 6, 7}, {4, 8, 5, 7}, {4, 8, 5, 6}, {4, 8, 5, 5}, {4, 8, 6, 5}, {3, 9, 7, 5}, {3, 8, 7, 6}, {3, 8, 7, 7}, {3, 8, 6, 7}, {4, 7, 5, 6}, {3, 8, 6, 5}, {3, 8, 7, 5}, {2, 7, 7, 6}, {3, 7, 7, 7}, {3, 7, 6, 7}, {3, 7, 5, 7}, {3, 7, 5, 6}, {3, 7, 5, 5}, {3, 7, 6, 5}, {3, 7, 7, 5}, {2, 6, 7, 6}, {2, 6, 7, 7}, {3, 6, 6, 7}, {3, 6, 5, 7}, {3, 6, 5, 6}, {3, 6, 5, 5}, {3, 6, 6, 5}, {2, 6, 7, 5}, {2, 6, 8, 6}, {2, 6, 6, 7}, {2, 6, 6, 5}, {3, 6, 7, 6}, {3, 6, 7, 7}, {2, 6, 5, 7}, {2, 6, 5, 6}, {2, 6, 5, 5}, {3, 6, 7, 5}, {3, 5, 7, 6}, {3, 5, 7, 7}, {3, 5, 6, 7}, {3, 5, 5, 7}, {2, 5, 5, 6}, {3, 5, 5, 5}, {3, 5, 6, 5}, {3, 5, 7, 5}, {4, 5, 7, 6}, {4, 4, 7, 6}, {4, 4, 7, 7}, {3, 4, 6, 7}, {3, 4, 5, 7}, {3, 4, 5, 6}, {3, 4, 5, 5}, {3, 4, 6, 5}, {4, 4, 7, 5}, {4, 4, 6, 7}, {4, 3, 6, 7}, {3, 3, 5, 7}, {3, 3, 5, 6}, {3, 3, 5, 5}, {4, 3, 6, 5}, {4, 4, 6, 5}, {5, 4, 7, 6}, {5, 4, 7, 7}, {5, 3, 6, 7}, {4, 2, 5, 7}, {4, 2, 5, 6}, {4, 2, 5, 5}, {5, 3, 6, 5}, {5, 4, 7, 5}, {6, 4, 7, 6}, {6, 3, 7, 7}, {5, 2, 6, 7}, {5, 2, 5, 7}, {5, 2, 5, 6}, {5, 2, 5, 5}, {5, 2, 6, 5}, {6, 3, 7, 5}, {6, 3, 6, 7}, {6, 2, 6, 7}, {6, 1, 5, 7}, {6, 1, 5, 6}, {6, 1, 5, 5}, {6, 2, 6, 5}, {6, 3, 6, 5}, {6, 4, 6, 7}, {6, 4, 6, 5}, {7, 3, 6, 7}, {7, 2, 6, 7}, {7, 1, 6, 7}, {7, 1, 5, 6}, {7, 1, 6, 5}, {7, 2, 6, 5}, {7, 3, 6, 5}, {7, 5, 6, 6}, {7, 4, 6, 6}, {7, 4, 6, 7}, {8, 3, 6, 7}, {8, 2, 6, 7}, {8, 2, 6, 6}, {8, 2, 6, 5}, {8, 3, 6, 5}, {7, 4, 6, 5}, {8, 4, 6, 7}, {9, 3, 6, 7}, {9, 2, 6, 7}, {9, 2, 6, 6}, {9, 2, 6, 5}, {9, 3, 6, 5}, {8, 4, 6, 5}, {8, 5, 6, 7}, {9, 4, 6, 7}, {10, 4, 6, 7}, {10, 3, 6, 7}, {10, 3, 6, 6}, {10, 3, 6, 5}, {10, 4, 6, 5}, {9, 4, 6, 5}, {8, 5, 6, 5}, {9, 5, 6, 7}, {11, 4, 6, 7}, {11, 4, 6, 6}, {11, 4, 6, 5}, {9, 5, 6, 5}, {10, 5, 6, 7}, {11, 5, 6, 7}, {11, 5, 6, 6}, {11, 5, 6, 5}, {10, 5, 6, 5}};

BOOST_AUTO_TEST_CASE( tetrahedron )
{
  std::stringstream input;
  input << "1, 0; 3 2\n   0 1 2 ";
  boost::shared_ptr<Complex> complex = ComplexHelper::create(input, ComplexHelper::INPUT_TYPE_OBJ);
  BOOST_CHECK_EQUAL(3, complex->dim());
  BOOST_CHECK_EQUAL(15, complex->cardinality());
}


BOOST_AUTO_TEST_CASE(kleinBetti)
{
  std::vector<int> betti = CubicalHelper::BettiNumbers(kleinCubes);
  std::vector<int> expectedBetti = {1, 1};
  BOOST_CHECK_EQUAL_COLLECTIONS(expectedBetti.begin(), expectedBetti.end(), betti.begin(), betti.end());
}

BOOST_AUTO_TEST_CASE(kleinBettiZ2)
{
  std::vector<int> betti = CubicalHelper::BettiNumbers(kleinCubes, 2);
  std::vector<int> expectedBetti = {1, 2, 1};
  BOOST_CHECK_EQUAL_COLLECTIONS(expectedBetti.begin(), expectedBetti.end(), betti.begin(), betti.end());
}

BOOST_AUTO_TEST_CASE(kleinBettiZp)
{
  std::vector<int> betti = CubicalHelper::BettiNumbers(kleinCubes, 1019);
  std::vector<int> expectedBetti = {1, 1};
  BOOST_CHECK_EQUAL_COLLECTIONS(expectedBetti.begin(), expectedBetti.end(), betti.begin(), betti.end());
}

BOOST_AUTO_TEST_SUITE_END()
