/////////////////////////////////////////////////////////////////////////////
/// @file Fixtures.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-04
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_FIXTURES_H
#define CAPD_FILE_FIXTURES_H

#define USE_PRECOMPILED_CAPDREDHOM_COMPLEX_CUBSCOMPLEX_HPP

#include <capd/complex/CubSComplex.h>
#include <capd/complex/SimplexSComplex.hpp>
#include <capd/complex/SComplex.h>
#include <capd/complex/SComplexDefaultTraits.h>

#include "capd/apiRedHom/Complex.h"
#include <vector>
#include <boost/shared_ptr.hpp>
#include <boost/assign.hpp>
#include <boost/mpl/list.hpp>
#include <boost/mpl/joint_view.hpp>
#include <boost/assign.hpp>
#include <capd/apiRedHom/ComplexHelper.h>
#include <capd/complex/ComplexSelector.h>
#include <capd/apiRedHom/ComplexHelper.h>


namespace capd
{
  namespace test
  {
    using namespace capd;
    using namespace capd::apiRedHom;
    using namespace boost::assign;

    struct ComplexFixture
    {
      ComplexFixture(): euler(0), cardinality(0), coreductions(0),
                        dim(0), _bettiMod(3), betti(_bettiMod[0])
      {}

      boost::shared_ptr<Complex> complex;
      int euler;
      size_t cardinality;
      size_t coreductions;
      size_t dim;

      std::vector<std::vector<size_t> > _bettiMod; // betti[0] - over int, betti[2] - mod 2
      std::vector<size_t>& betti;

      const std::vector<size_t>& bettiMod(size_t p) const
      {
        BOOST_ASSERT(p < _bettiMod.size());
        if (_bettiMod[p].empty()) {
          return betti; // if mod p not set then assume equals to p==0 (over integers)
        } else {
          return _bettiMod[p];
        }
      }

      size_t bettiInDim(size_t dim) const
      {
        return (betti.size() <= dim ? 0 : betti[dim]);
      }

    };

    struct SimplicialComplexFixture: ComplexFixture
    {
      typedef complex::SimplexSComplex ComplexType;
      typedef SimplicialComplex APIComplexType;
      ComplexType& rawComplex() { return complex->rawComplex<ComplexType>(); }
    };

    typedef capd::complex::DenseCubicalTag DEFAULT_CUBICAL_COMPLEX_TAG;

    template<int DIM, typename Tag=DEFAULT_CUBICAL_COMPLEX_TAG>
    struct CubicalComplexFixture: ComplexFixture
    {
      typedef Tag ComplexTag;
      static const int TYPE_ID = ComplexTag::ID;

      typedef typename capd::complex::ComplexSelector<ComplexTag, DIM, size_t>::type ComplexType;
      typedef CubicalComplex APIComplexType;
      ComplexType& rawComplex() { return complex->template rawComplex<ComplexType>(); }
    };


    struct GeneralizedComplexFixture: ComplexFixture
    {
      typedef complex::SComplex<complex::SComplexDefaultTraits> ComplexType;
      typedef GeneralizedSComplex APIComplexType;
      ComplexType& rawComplex() { return complex->rawComplex<ComplexType>(); }
    };

    struct SimplicialEmpty: SimplicialComplexFixture
    {
      SimplicialEmpty()
      {
        complex.reset(new SimplicialComplex());
        //betti = list_of();
        this->euler = 0;
        this->cardinality = 0;
        this->coreductions = 0;
        this->dim = 0;
      }

    };

    struct SimplicialPoint: SimplicialComplexFixture
    {
      SimplicialPoint()
      {
        std::vector<SimplicialComplex::CellCode> simplices = list_of
          (list_of(0).convert_to_container<SimplicialComplex::CellCode>())
          ;
        complex.reset(new SimplicialComplex(simplices));
        betti = list_of(1).convert_to_container<std::vector<size_t> >();
        this->euler = 1;
        this->cardinality = 1;
        this->coreductions = 1;
        this->dim = 0;
      }

    };

    struct SimplicialCircle: SimplicialComplexFixture
    {
      SimplicialCircle()
      {
        std::vector<SimplicialComplex::CellCode> simplices = list_of
          (list_of(0)(1).convert_to_container<SimplicialComplex::CellCode>())
          (list_of(1)(2).convert_to_container<SimplicialComplex::CellCode>())
          (list_of(0)(2).convert_to_container<SimplicialComplex::CellCode>())
          ;

        SimplicialComplex* complex = new SimplicialComplex(simplices);
        this->complex.reset(complex);
        betti = list_of(1)(1).convert_to_container<std::vector<size_t> >();
        this->euler = 0;
        this->cardinality = 6;
        this->coreductions = 5;
        this->dim = 1;

        a = (*complex)[list_of(0).convert_to_container<SimplicialComplex::CellCode>()];
        b = (*complex)[list_of(1).convert_to_container<SimplicialComplex::CellCode>()];
        c = (*complex)[list_of(2).convert_to_container<SimplicialComplex::CellCode>()];

        ab = (*complex)[list_of(0)(1).convert_to_container<SimplicialComplex::CellCode>()];
        ac = (*complex)[list_of(0)(2).convert_to_container<SimplicialComplex::CellCode>()];
        bc = (*complex)[list_of(1)(2).convert_to_container<SimplicialComplex::CellCode>()];
      }

      SimplicialComplex::Cell a, b, c, ab, ac, bc;
    };


    struct SimplicialTriangle: SimplicialComplexFixture
    {
      SimplicialTriangle()
      {
        std::vector<SimplicialComplex::CellCode> simplices = list_of
          (list_of(0)(1)(2).convert_to_container<SimplicialComplex::CellCode>())
          ;

        SimplicialComplex* complex = new SimplicialComplex(simplices);
        this->complex.reset(complex);
        betti = list_of(1).convert_to_container<std::vector<size_t> >();
        this->euler = 1;
        this->cardinality = 7;
        this->coreductions = 7;
        this->dim = 2;

        a = (*complex)[list_of(0).convert_to_container<SimplicialComplex::CellCode>()];
        b = (*complex)[list_of(1).convert_to_container<SimplicialComplex::CellCode>()];
        c = (*complex)[list_of(2).convert_to_container<SimplicialComplex::CellCode>()];

        ab = (*complex)[list_of(0)(1).convert_to_container<SimplicialComplex::CellCode>()];
        ac = (*complex)[list_of(0)(2).convert_to_container<SimplicialComplex::CellCode>()];
        bc = (*complex)[list_of(1)(2).convert_to_container<SimplicialComplex::CellCode>()];

        abc = (*complex)[list_of(0)(1)(2).convert_to_container<SimplicialComplex::CellCode>()];
      }

      SimplicialComplex::Cell a, b, c, ab, ac, bc, abc;
    };


    struct SimplicialRealProjectivePlane: SimplicialComplexFixture
    {
      SimplicialRealProjectivePlane()
      {
        std::vector<SimplicialComplex::CellCode> simplices = list_of
          (list_of(0)(2)(3).convert_to_container<SimplicialComplex::CellCode>())
          (list_of(0)(1)(3).convert_to_container<SimplicialComplex::CellCode>())
          (list_of(0)(1)(5).convert_to_container<SimplicialComplex::CellCode>())
          (list_of(0)(4)(5).convert_to_container<SimplicialComplex::CellCode>())
          (list_of(0)(2)(4).convert_to_container<SimplicialComplex::CellCode>())
          (list_of(1)(2)(5).convert_to_container<SimplicialComplex::CellCode>())
          (list_of(2)(3)(5).convert_to_container<SimplicialComplex::CellCode>())
          (list_of(3)(4)(5).convert_to_container<SimplicialComplex::CellCode>())
          (list_of(1)(3)(4).convert_to_container<SimplicialComplex::CellCode>())
          (list_of(1)(2)(4).convert_to_container<SimplicialComplex::CellCode>())
          ;

        SimplicialComplex* complex = new SimplicialComplex(simplices);
        this->complex.reset(complex);
        betti = list_of(1).convert_to_container<std::vector<size_t> >();
        _bettiMod[2] = list_of(1)(1)(1).convert_to_container<std::vector<size_t> >();
        this->euler = 1;
        this->cardinality = 31;
        this->coreductions = 21;
        this->dim = 2;
      }
    };

    template<typename Tag=DEFAULT_CUBICAL_COMPLEX_TAG>
    struct CubicalEmpty: CubicalComplexFixture<2, Tag>
    {
      CubicalEmpty()
      {
        std::vector<size_t> size = list_of(10)(10);
        this->complex.reset(new CubicalComplex(size, CubicalComplex::CellCodeRange(), this->TYPE_ID));
        //betti = list_of();
        this->euler = 0;
        this->cardinality = 0;
        this->coreductions = 0;
        this->dim = 2;
      }
    };

    template<typename Tag=DEFAULT_CUBICAL_COMPLEX_TAG>
    struct CubicalPoint: CubicalComplexFixture<2, Tag>
    {
      CubicalPoint()
      {
        std::vector<CubicalComplex::CellCode> cubes = list_of
          (list_of<CubicalComplex::Interval>(1, false)(1, false).convert_to_container<CubicalComplex::CellCode>())
          ;
        std::vector<size_t> size = list_of(10)(10);
        this->complex.reset(new CubicalComplex(size, cubes, this->TYPE_ID));
        dynamic_cast<CubicalComplex*>(this->complex.get())->fillWithBoundaries();
        this->betti = list_of(1).convert_to_container<std::vector<size_t> >();
        this->euler = 1;
        this->cardinality = 1;
        this->coreductions = 1;
        this->dim = 2;
      }
    };

    template<typename Tag=DEFAULT_CUBICAL_COMPLEX_TAG>
    struct CubicalRectangle: CubicalComplexFixture<2, Tag>
    {
      CubicalRectangle()
      {
        std::vector<CubicalComplex::CellCode> cubes = list_of
          (list_of<CubicalComplex::Interval>(1, true)(1, true).convert_to_container<CubicalComplex::CellCode>())
          ;
        std::vector<size_t> size = list_of(10)(10);
        this->complex.reset(new CubicalComplex(size, cubes, this->TYPE_ID));
        dynamic_cast<CubicalComplex*>(this->complex.get())->fillWithBoundaries();
        this->betti = list_of(1).convert_to_container<std::vector<size_t> >();
        this->euler = 1;
        this->cardinality = 9;
        this->coreductions = 9;
        this->dim = 2;
      }
    };

    template<typename Tag=DEFAULT_CUBICAL_COMPLEX_TAG>
    struct CubicalCircle: CubicalComplexFixture<2, Tag>
    {
      CubicalCircle()
      {
        std::vector<CubicalComplex::CellCode> cubes = list_of
          (list_of<CubicalComplex::Interval>(1, true)(1, false).convert_to_container<CubicalComplex::CellCode>())
          (list_of<CubicalComplex::Interval>(1, false)(1, true).convert_to_container<CubicalComplex::CellCode>())
          (list_of<CubicalComplex::Interval>(2, false)(1, true).convert_to_container<CubicalComplex::CellCode>())
          (list_of<CubicalComplex::Interval>(1, true)(2, false).convert_to_container<CubicalComplex::CellCode>())
          ;
        std::vector<size_t> size = list_of(10)(10);
        this->complex.reset(new CubicalComplex(size, cubes, this->TYPE_ID));
        dynamic_cast<CubicalComplex*>(this->complex.get())->fillWithBoundaries();
        this->betti = list_of(1)(1).convert_to_container<std::vector<size_t> >();
        this->euler = 0;
        this->cardinality = 8;
        this->coreductions = 7;
        this->dim = 2;
      }

    };

    template<typename Tag=DEFAULT_CUBICAL_COMPLEX_TAG>
    struct CubicalCircle2: CubicalComplexFixture<2, Tag>
    {
      CubicalCircle2()
      {
        std::vector<CubicalComplex::FullCube> cubes = list_of
          (list_of<CubicalComplex::Coordinate>(0)(0).convert_to_container<CubicalComplex::FullCube>())
          (list_of<CubicalComplex::Coordinate>(0)(1).convert_to_container<CubicalComplex::FullCube>())
          (list_of<CubicalComplex::Coordinate>(0)(2).convert_to_container<CubicalComplex::FullCube>())
          (list_of<CubicalComplex::Coordinate>(1)(0).convert_to_container<CubicalComplex::FullCube>())
          (list_of<CubicalComplex::Coordinate>(1)(2).convert_to_container<CubicalComplex::FullCube>())
          (list_of<CubicalComplex::Coordinate>(2)(0).convert_to_container<CubicalComplex::FullCube>())
          (list_of<CubicalComplex::Coordinate>(2)(1).convert_to_container<CubicalComplex::FullCube>())
          (list_of<CubicalComplex::Coordinate>(2)(2).convert_to_container<CubicalComplex::FullCube>())
          ;

        this->complex = CubicalHelper::CreateFromFullCubes(cubes, this->TYPE_ID);
        this->betti = list_of(1)(1).convert_to_container<std::vector<size_t> >();
        this->euler = 0;
        this->cardinality = 48;
        this->coreductions = 45;
        this->dim = 2;
      }
    };

    template<typename Tag=DEFAULT_CUBICAL_COMPLEX_TAG>
    struct CubicalSphere: CubicalComplexFixture<3, Tag>
    {
      CubicalSphere()
      {
        std::vector<CubicalComplex::CellCode> cubes = list_of
          (list_of<CubicalComplex::Interval>(1, true)(1, true)(1, false).convert_to_container<CubicalComplex::CellCode>())
          (list_of<CubicalComplex::Interval>(1, true)(1, false)(1, true).convert_to_container<CubicalComplex::CellCode>())
          (list_of<CubicalComplex::Interval>(1, false)(1, true)(1, true).convert_to_container<CubicalComplex::CellCode>())
          (list_of<CubicalComplex::Interval>(1, true)(1, true)(2, false).convert_to_container<CubicalComplex::CellCode>())
          (list_of<CubicalComplex::Interval>(1, true)(2, false)(1, true).convert_to_container<CubicalComplex::CellCode>())
          (list_of<CubicalComplex::Interval>(2, false)(1, true)(1, true).convert_to_container<CubicalComplex::CellCode>())
          ;
        std::vector<size_t> size = list_of(10)(10)(10);
        this->complex.reset(new CubicalComplex(size, cubes, this->TYPE_ID));
        dynamic_cast<CubicalComplex*>(this->complex.get())->fillWithBoundaries();
        this->betti = list_of(1)(0)(1).convert_to_container<std::vector<size_t> >();
        this->euler = 2;
        this->cardinality = 26;
        this->coreductions = 25;
        this->dim = 3;
      }
    };

    template<typename Tag=DEFAULT_CUBICAL_COMPLEX_TAG>
    struct CubicalAntipodalPoints: CubicalComplexFixture<3, Tag>
    {
      CubicalAntipodalPoints()
      {
        std::vector<CubicalComplex::CellCode> cubes = list_of
          (list_of<CubicalComplex::Interval>(1, false)(1, false)(1, false).convert_to_container<CubicalComplex::CellCode>())
          (list_of<CubicalComplex::Interval>(2, false)(2, false)(2, false).convert_to_container<CubicalComplex::CellCode>())
          ;
        std::vector<size_t> size = list_of(4)(4)(4);
        this->complex.reset(new CubicalComplex(size, cubes, this->TYPE_ID));
        dynamic_cast<CubicalComplex*>(this->complex.get())->fillWithBoundaries();
        this->betti = list_of(2).convert_to_container<std::vector<size_t> >();
        this->euler = 2;
        this->cardinality = 2;
        this->coreductions = 2;
        this->dim = 3;
      }

    };

    template<typename Tag=DEFAULT_CUBICAL_COMPLEX_TAG>
    struct CubicalTorus: CubicalComplexFixture<3, Tag>
    {
      CubicalTorus()
      {
        std::vector<std::vector<CubicalComplex::Coordinate> >
          cubes = {
          {0,0,0}, {1,0,0}, {2,0,0}, {3,0,0}, {4,0,0}, {5,0,0}, {6,0,0}, {0,1,0}, {1,1,0}, {2,1,0}, {3,1,0}, {4,1,0}, {5,1,0}, {6,1,0}, {0,2,0}, {1,2,0}, {2,2,0}, {3,2,0}, {4,2,0}, {5,2,0}, {6,2,0}, {0,3,0}, {1,3,0}, {2,3,0}, {4,3,0}, {5,3,0}, {6,3,0}, {0,4,0}, {1,4,0}, {2,4,0}, {3,4,0}, {4,4,0}, {5,4,0}, {6,4,0}, {0,5,0}, {1,5,0}, {2,5,0}, {3,5,0}, {4,5,0}, {5,5,0}, {6,5,0}, {0,6,0}, {1,6,0}, {2,6,0}, {3,6,0}, {4,6,0}, {5,6,0}, {6,6,0}, {0,0,1}, {1,0,1}, {2,0,1}, {3,0,1}, {4,0,1}, {5,0,1}, {6,0,1}, {0,1,1}, {6,1,1}, {0,2,1}, {2,2,1}, {3,2,1}, {4,2,1}, {6,2,1}, {0,3,1}, {2,3,1}, {4,3,1}, {6,3,1}, {0,4,1}, {2,4,1}, {3,4,1}, {4,4,1}, {6,4,1}, {0,5,1}, {6,5,1}, {0,6,1}, {1,6,1}, {2,6,1}, {3,6,1}, {4,6,1}, {5,6,1}, {6,6,1}, {0,0,2}, {1,0,2}, {2,0,2}, {3,0,2}, {4,0,2}, {5,0,2}, {6,0,2}, {0,1,2}, {1,1,2}, {2,1,2}, {3,1,2}, {4,1,2}, {5,1,2}, {6,1,2}, {0,2,2}, {1,2,2}, {2,2,2}, {3,2,2}, {4,2,2}, {5,2,2}, {6,2,2}, {0,3,2}, {1,3,2}, {2,3,2}, {4,3,2}, {5,3,2}, {6,3,2}, {0,4,2}, {1,4,2}, {2,4,2}, {3,4,2}, {4,4,2}, {5,4,2}, {6,4,2}, {0,5,2}, {1,5,2}, {2,5,2}, {3,5,2}, {4,5,2}, {5,5,2}, {6,5,2}, {0,6,2}, {1,6,2}, {2,6,2}, {3,6,2}, {4,6,2}, {5,6,2}, {6,6,2}
        };

        this->complex = CubicalHelper::CreateFromFullCubes(cubes, this->TYPE_ID);
        dynamic_cast<CubicalComplex*>(this->complex.get())->fillWithBoundaries();
        this->betti = list_of(1)(2)(1).convert_to_container<std::vector<size_t> >();
        this->euler = 0;
        this->cardinality = 1536;
        this->coreductions = 1395;
        this->dim = 3;
      }

    };

    // template<typename Tag=DEFAULT_CUBICAL_COMPLEX_TAG>
    // struct CubicalKleinBottle: CubicalComplexFixture<4, Tag>
    // {
    //   CubicalKleinBottle()
    //   {
    //     std::vector<std::vector<CubicalComplex::Coordinate> >
    //       cubes =
    //    {{12, 6, 6, 6}, {11, 6, 6, 6}, {11, 6, 6, 7}, {10, 6, 6, 7}, {9, 6, 6, 7}, {8, 6, 6, 7}, {8, 6, 6, 6}, {8, 6, 6, 5}, {9, 6, 6, 5}, {10, 6, 6, 5}, {11, 6, 6, 5}, {11, 7, 6, 6}, {11, 7, 6, 7}, {11, 7, 6, 5}, {11, 8, 6, 6}, {10, 8, 6, 7}, {10, 7, 6, 7}, {9, 7, 6, 7}, {8, 7, 6, 7}, {7, 6, 6, 6}, {8, 7, 6, 5}, {9, 7, 6, 5}, {10, 7, 6, 5}, {10, 8, 6, 5}, {10, 9, 6, 6}, {10, 9, 6, 7}, {9, 8, 6, 7}, {7, 7, 6, 7}, {7, 7, 6, 6}, {7, 7, 6, 5}, {9, 8, 6, 5}, {10, 9, 6, 5}, {9, 10, 6, 6}, {9, 9, 6, 7}, {8, 8, 6, 7}, {8, 8, 6, 5}, {9, 9, 6, 5}, {8, 10, 6, 6}, {8, 10, 6, 7}, {8, 9, 6, 7}, {7, 9, 6, 7}, {7, 8, 6, 7}, {7, 8, 6, 5}, {7, 9, 6, 5}, {8, 9, 6, 5}, {8, 10, 6, 5}, {7, 11, 7, 6}, {7, 10, 6, 7}, {6, 9, 6, 7}, {6, 8, 6, 7}, {6, 8, 5, 6}, {6, 8, 6, 5}, {6, 9, 6, 5}, {7, 10, 6, 5}, {6, 11, 7, 6}, {6, 10, 6, 7}, {6, 8, 5, 7}, {6, 8, 5, 5}, {6, 10, 6, 5}, {5, 10, 7, 6}, {5, 10, 7, 7}, {5, 10, 6, 7}, {5, 9, 6, 7}, {5, 9, 6, 5}, {5, 10, 6, 5}, {5, 10, 7, 5}, {4, 10, 7, 6}, {4, 10, 7, 7}, {4, 9, 6, 7}, {5, 8, 5, 7}, {5, 8, 5, 6}, {5, 8, 5, 5}, {4, 9, 6, 5}, {4, 10, 7, 5}, {3, 9, 7, 6}, {3, 9, 7, 7}, {4, 8, 6, 7}, {4, 8, 5, 7}, {4, 8, 5, 6}, {4, 8, 5, 5}, {4, 8, 6, 5}, {3, 9, 7, 5}, {3, 8, 7, 6}, {3, 8, 7, 7}, {3, 8, 6, 7}, {4, 7, 5, 6}, {3, 8, 6, 5}, {3, 8, 7, 5}, {2, 7, 7, 6}, {3, 7, 7, 7}, {3, 7, 6, 7}, {3, 7, 5, 7}, {3, 7, 5, 6}, {3, 7, 5, 5}, {3, 7, 6, 5}, {3, 7, 7, 5}, {2, 6, 7, 6}, {2, 6, 7, 7}, {3, 6, 6, 7}, {3, 6, 5, 7}, {3, 6, 5, 6}, {3, 6, 5, 5}, {3, 6, 6, 5}, {2, 6, 7, 5}, {2, 6, 8, 6}, {2, 6, 6, 7}, {2, 6, 6, 5}, {3, 6, 7, 6}, {3, 6, 7, 7}, {2, 6, 5, 7}, {2, 6, 5, 6}, {2, 6, 5, 5}, {3, 6, 7, 5}, {3, 5, 7, 6}, {3, 5, 7, 7}, {3, 5, 6, 7}, {3, 5, 5, 7}, {2, 5, 5, 6}, {3, 5, 5, 5}, {3, 5, 6, 5}, {3, 5, 7, 5}, {4, 5, 7, 6}, {4, 4, 7, 6}, {4, 4, 7, 7}, {3, 4, 6, 7}, {3, 4, 5, 7}, {3, 4, 5, 6}, {3, 4, 5, 5}, {3, 4, 6, 5}, {4, 4, 7, 5}, {4, 4, 6, 7}, {4, 3, 6, 7}, {3, 3, 5, 7}, {3, 3, 5, 6}, {3, 3, 5, 5}, {4, 3, 6, 5}, {4, 4, 6, 5}, {5, 4, 7, 6}, {5, 4, 7, 7}, {5, 3, 6, 7}, {4, 2, 5, 7}, {4, 2, 5, 6}, {4, 2, 5, 5}, {5, 3, 6, 5}, {5, 4, 7, 5}, {6, 4, 7, 6}, {6, 3, 7, 7}, {5, 2, 6, 7}, {5, 2, 5, 7}, {5, 2, 5, 6}, {5, 2, 5, 5}, {5, 2, 6, 5}, {6, 3, 7, 5}, {6, 3, 6, 7}, {6, 2, 6, 7}, {6, 1, 5, 7}, {6, 1, 5, 6}, {6, 1, 5, 5}, {6, 2, 6, 5}, {6, 3, 6, 5}, {6, 4, 6, 7}, {6, 4, 6, 5}, {7, 3, 6, 7}, {7, 2, 6, 7}, {7, 1, 6, 7}, {7, 1, 5, 6}, {7, 1, 6, 5}, {7, 2, 6, 5}, {7, 3, 6, 5}, {7, 5, 6, 6}, {7, 4, 6, 6}, {7, 4, 6, 7}, {8, 3, 6, 7}, {8, 2, 6, 7}, {8, 2, 6, 6}, {8, 2, 6, 5}, {8, 3, 6, 5}, {7, 4, 6, 5}, {8, 4, 6, 7}, {9, 3, 6, 7}, {9, 2, 6, 7}, {9, 2, 6, 6}, {9, 2, 6, 5}, {9, 3, 6, 5}, {8, 4, 6, 5}, {8, 5, 6, 7}, {9, 4, 6, 7}, {10, 4, 6, 7}, {10, 3, 6, 7}, {10, 3, 6, 6}, {10, 3, 6, 5}, {10, 4, 6, 5}, {9, 4, 6, 5}, {8, 5, 6, 5}, {9, 5, 6, 7}, {11, 4, 6, 7}, {11, 4, 6, 6}, {11, 4, 6, 5}, {9, 5, 6, 5}, {10, 5, 6, 7}, {11, 5, 6, 7}, {11, 5, 6, 6}, {11, 5, 6, 5}, {10, 5, 6, 5}};

    //     this->complex = CubicalHelper::CreateFromFullCubes(cubes, this->TYPE_ID);
    //     dynamic_cast<CubicalComplex*>(this->complex.get())->fillWithBoundaries();
    //     this->betti = list_of(2).convert_to_container<std::vector<size_t> >();
    //     this->euler = 2;
    //     this->cardinality = 2;
    //     this->coreductions = 2;
    //     this->dim = 4;
    //   }

    // };

    struct GeneralizedEmpty: GeneralizedComplexFixture
    {
      GeneralizedEmpty()
      {
        this->complex.reset(new GeneralizedSComplex(GeneralizedSComplex::Dims(), GeneralizedSComplex::KappaMapEntries()));
        //this->betti = list_of();
        this->euler = 0;
        this->cardinality = 0;
        this->coreductions = 0;
        this->dim = 0;
      }

    };

    struct GeneralizedPoint: GeneralizedComplexFixture
    {
      GeneralizedPoint()
      {
        GeneralizedSComplex::Dims dims = list_of(0);

        this->complex.reset(new GeneralizedSComplex(dims, GeneralizedSComplex::KappaMapEntries()));
        this->betti = list_of(1).convert_to_container<std::vector<size_t> >();
        this->euler = 1;
        this->cardinality = 1;
        this->coreductions = 1;
        this->dim = 0;
      }

    };

    struct GeneralizedCircle: GeneralizedComplexFixture
    {
      GeneralizedCircle()
      {
        GeneralizedSComplex::Dims dims = list_of(0)(0)(0)(1)(1)(1);
        std::vector<GeneralizedSComplex::KappaMapEntry> kappa = tuple_list_of(3, 0, -1)(3, 1, 1)(4, 1, -1)(4, 2, 1)(5, 2, -1)(5, 0, 1);
        this->complex.reset(new GeneralizedSComplex(dims, kappa));

        this->betti = list_of(1)(1).convert_to_container<std::vector<size_t> >();
        this->euler = 0;
        this->cardinality = 6;
        this->coreductions = 5;
        this->dim = 1;
      }
    };


    struct GeneralizedRectangle: GeneralizedComplexFixture
    {
      GeneralizedRectangle()
      {
        GeneralizedSComplex::Dims dims = list_of(0)(0)(0)(1)(1)(1)(2);
        std::vector<GeneralizedSComplex::KappaMapEntry> kappa = tuple_list_of(3, 0, -1)(3, 1, 1)(4, 1, -1)(4, 2, 1)(5, 2, -1)(5, 0, 1)(6, 3, -1)(6, 4, 1)(6, 5, -1);
        this->complex.reset(new GeneralizedSComplex(dims, kappa));

        this->betti = list_of(1).convert_to_container<std::vector<size_t> >();
        this->euler = 1;
        this->cardinality = 7;
        this->coreductions = 7;
        this->dim = 2;
      }
    };



    struct GeneralizedPointFromCubes: GeneralizedComplexFixture
    {
      GeneralizedPointFromCubes()
      {
        CubicalHelper::FullCube cube = list_of(0)(0);
        std::vector<CubicalHelper::FullCube> cubes = list_of(cube);

        complex = CubicalHelper::CreateGeneralizedSComplexFromFullCubes(2, cubes);

        this->betti = list_of(1).convert_to_container<std::vector<size_t> >();
        this->euler = 1;
        this->cardinality = 9;
        this->coreductions = 9;
        this->dim = 2;
      }
    };



    typedef boost::mpl::list<SimplicialEmpty, SimplicialPoint, SimplicialCircle,
                             SimplicialTriangle, SimplicialRealProjectivePlane> SimplicialFixtures;
    typedef boost::mpl::list<
      CubicalEmpty<complex::DenseCubicalTag>,
      CubicalPoint<complex::DenseCubicalTag>,
      CubicalCircle<complex::DenseCubicalTag>,
      CubicalCircle2<complex::DenseCubicalTag>,
      CubicalRectangle<complex::DenseCubicalTag>,
      CubicalSphere<complex::DenseCubicalTag>,
      CubicalAntipodalPoints<complex::DenseCubicalTag>,
      CubicalTorus<complex::DenseCubicalTag>,
      //      CubicalKleinBottle<complex::DenseCubicalTag>,

      CubicalEmpty<complex::OldCubicalTag>,
      CubicalPoint<complex::OldCubicalTag>,
      CubicalCircle<complex::OldCubicalTag>,
      CubicalCircle2<complex::OldCubicalTag>,
      CubicalRectangle<complex::OldCubicalTag>,
      CubicalSphere<complex::OldCubicalTag>,
      CubicalAntipodalPoints<complex::OldCubicalTag>,
      CubicalTorus<complex::OldCubicalTag>
      //      CubicalKleinBottle<complex::OldCubicalTag>

      > CubicalFixtures;
    typedef boost::mpl::list<GeneralizedEmpty,
                             GeneralizedPoint,
                             GeneralizedCircle,
                             GeneralizedRectangle,
                             GeneralizedPointFromCubes
                             > GeneralizedFixtures;

    typedef boost::mpl::joint_view<
      boost::mpl::joint_view<SimplicialFixtures,
                             CubicalFixtures>,
      GeneralizedFixtures> ComplexFixtures;

  }
}


#endif // CAPD_FILE_FIXTURES_H
