/////////////////////////////////////////////////////////////////////////////
/// @file SimplicialComplexTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-03
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include "capd/apiRedHom/Complex.h"
#include "capd/apiRedHom/Algorithms.h"

#include "CommonTools.h"
#include "Fixtures.h"

#define BOOST_TEST_DYN_LINK



#include <boost/test/unit_test.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/assign.hpp>

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd::apiRedHom;
using namespace capd::test;

BOOST_AUTO_TEST_SUITE(SimplicialComplexSuite)

BOOST_AUTO_TEST_CASE(euler_1)
{
  SimplicialComplex::CellCode simplex = list_of<SimplicialComplex::Point>(0)(1);
  std::vector<SimplicialComplex::CellCode> simplices = list_of(simplex);

  SimplicialComplex complex(simplices);

  EulerCharacteristic eCh(complex);
  BOOST_CHECK_EQUAL(1, eCh());
}


BOOST_AUTO_TEST_CASE_TEMPLATE(betti, Fixture, SimplicialFixtures)
{
  Fixture fixture;

  EulerCharacteristic eCh(*fixture.complex);
  BOOST_CHECK_EQUAL(fixture.euler, eCh());
}

BOOST_AUTO_TEST_CASE(methods)
{
  SimplicialComplex::CellCode simplex = list_of<SimplicialComplex::Point>(0)(1);
  std::vector<SimplicialComplex::CellCode> simplices = list_of(simplex);

  SimplicialComplex complex(simplices);
  complex.insert(simplex);

//  BOOST_CHECK_EQUAL(1, complex.dim());
  BOOST_CHECK_EQUAL(3, complex.cardinality());

  SimplicialComplex::CellCode simplex2 = list_of<SimplicialComplex::Point>(3);
  complex.insert(simplex2);

  BOOST_CHECK_EQUAL(4, complex.cardinality());

  BOOST_CHECK(complex.find(list_of<SimplicialComplex::Point>(0)(1).convert_to_container<SimplicialComplex::CellCode>()));
  BOOST_CHECK(!complex.find(list_of<SimplicialComplex::Point>(1)(1).convert_to_container<SimplicialComplex::CellCode>()));
  BOOST_CHECK_EQUAL(0, complex.find(list_of<SimplicialComplex::Point>(0).convert_to_container<SimplicialComplex::CellCode>()).get().dim());
  BOOST_CHECK_EQUAL(0, complex[list_of<SimplicialComplex::Point>(0).convert_to_container<SimplicialComplex::CellCode>()].dim());


}

BOOST_AUTO_TEST_SUITE_END()
