/////////////////////////////////////////////////////////////////////////////
/// @file c_interfaceTest.c
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-07
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.


#include "capd/apiRedHom/c_interface.h"
#include <assert.h>

int c_interface_simplicial_test()
{

  RedHom_ComplexInfo complexData;
  RedHom_ComplexInfo* complex = &complexData;

  RedHom_CreateSimplicialComplex(complex);

  assert(RedHom_ComplexDim(complex) == 0);

  RedHom_AddSimplexToComplex(complex, (int[]){0, 1}, 2);
  assert(RedHom_ComplexDim(complex) == 1);
  assert(RedHom_ComplexCardinality(complex) == 3);

  RedHom_AddSimplexToComplex(complex, (int[]){3, 4, 5}, 3);
  assert(RedHom_ComplexDim(complex) == 2);
  assert(RedHom_ComplexCardinality(complex) == 10);


  int betti[RedHom_ComplexDim(complex) + 1];
  RedHom_BettiNumbersOverZOfComplex(complex, betti);

  assert(betti[0] == 2);
  assert(betti[1] == 0);
  assert(betti[2] == 0);

  RedHom_DeleteComplex(complex);
  return 0;
}


RedHom_ComplexInfo CreateCubicalComplex()
{
  RedHom_ComplexInfo complexData;
  RedHom_ComplexInfo* complex = &complexData;

  RedHom_CreateCubicalComplex(complex, 2, (int[]){10, 10});
  assert(RedHom_ComplexDim(complex) == 2);

  RedHom_AddCubeToComplex(complex, (int[]){2, 2}, (int[]){1, 1});

  RedHom_CubicalComplexFillWithBoundaries(complex);
  assert(RedHom_ComplexCardinality(complex) == 9);

  return complexData;
}

int c_interface_cubical_test()
{
  RedHom_ComplexInfo complexData = CreateCubicalComplex();
  RedHom_ComplexInfo* complex = &complexData;

  int betti[RedHom_ComplexDim(complex) + 1];
  RedHom_BettiNumbersOverZOfComplex(complex, betti);

  assert(betti[0] == 1);
  assert(betti[1] == 0);
  assert(betti[2] == 0);

  RedHom_DeleteComplex(complex);
  return 0;
}

int c_interface_generalized_test()
{
  RedHom_ComplexInfo complexData;
  RedHom_ComplexInfo* complex = &complexData;

  int dims[] = {0, 0, 1};
  unsigned int cofaces[] = {2, 2};
  unsigned int faces[] = {0, 1};
  int coincidenceIndices[] = {-1, 1};
  int entries = 2;
  int size = 3;

  RedHom_CreateGeneralizedSComplex(&complexData, size, dims, entries, cofaces, faces, coincidenceIndices);

  int betti[RedHom_ComplexDim(complex) + 1];
  RedHom_BettiNumbersOverZOfComplex(complex, betti);

  assert(betti[0] == 1);
  assert(betti[1] == 0);

  RedHom_DeleteComplex(complex);
  return 0;
}


int c_interface_orchestrator()
{
  RedHom_ComplexInfo complexData = CreateCubicalComplex();
  RedHom_ComplexInfo* complex = &complexData;

  int betti[RedHom_ComplexDim(complex) + 1];
  RedHom_BettiNumbersOverZOfComplex(complex, betti);

  assert(betti[0] == 1);
  assert(betti[1] == 0);
  assert(betti[2] == 0);

  RedHom_DeleteComplex(complex);
  return 0;
}

int c_interface_relative_cubical_test()
{
  RedHom_ComplexInfo complexData = CreateCubicalComplex();
  RedHom_ComplexInfo* complex = &complexData;

  RedHom_ComplexInfo subcomplexData = CreateCubicalComplex();
  RedHom_ComplexInfo* subcomplex = &subcomplexData;

  int betti[RedHom_ComplexDim(complex) + 1];
  RedHom_RelativeBettiNumbersOverZOfComplex(complex, subcomplex, betti, 0, 0, 0, 0);

  assert(betti[0] == 0);
  assert(betti[1] == 0);
  assert(betti[2] == 0);

  RedHom_DeleteComplex(complex);
  RedHom_DeleteComplex(subcomplex);
  return 0;
}

int c_interface_test()
{

  return (c_interface_simplicial_test() == 0 &&
	  c_interface_cubical_test() == 0 &&
	  c_interface_orchestrator() == 0 &&
	  c_interface_generalized_test() == 0
	  ? 0 : 1);
}
