/////////////////////////////////////////////////////////////////////////////
/// @file AcyclicNeighbourhoodTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-18
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <TestTools.h>
#include <capd/complex/AcyclicNeighbourhood.h>
#include <capd/auxil/Logger.h>
#include <../apiRedHom/Fixtures.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>

namespace capd
{
  namespace test
  {

  }
}

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(AcyclicNeighbourhoodTestSuite)

BOOST_AUTO_TEST_CASE( init )
{

}

BOOST_AUTO_TEST_CASE(point)
{
  typedef CubicalRectangle<capd::complex::DenseCubicalTag> Fixture;
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;

  ComplexType& complex = fixture.complex->rawComplex<ComplexType>();
  AcyclicNeighbourhood<ComplexType, 2> acyclic(complex);

  BOOST_REQUIRE_EQUAL(1, boost::distance(complex.iterators(1).dimCells(2)));
  BOOST_CHECK(!acyclic(*complex.iterators(1).dimCells(2).begin()));
}

BOOST_AUTO_TEST_CASE(circle)
{
  typedef CubicalCircle2<capd::complex::DenseCubicalTag> Fixture;
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;

  ComplexType& complex = fixture.complex->rawComplex<ComplexType>();
  AcyclicNeighbourhood<ComplexType, 2> acyclic(complex);

  size_t acyclicCounter = 0;
  for (auto cell : complex.iterators(1).dimCells(2)) {
    if (acyclic(cell)) {
      ++acyclicCounter;
    }
  }

  BOOST_CHECK_EQUAL(4, acyclicCounter);
}

BOOST_AUTO_TEST_CASE(colorMattersCircle)
{
  std::vector<size_t> shape = {{3, 3}};
  boost::multi_array<size_t, 2> array(shape);

  std::fill(array.origin(), array.origin() + array.num_elements(), 1);
  array[1][1] = 2;
  auto _complex = BuildCubicalComplexFromArray(array);
  auto& complex = *_complex;

  BOOST_CHECK(! IsAcyclicNeighbourhood<2>(complex, complex[complex(0, 1)]));

  BOOST_CHECK(IsAcyclicNeighbourhood<2>(complex, complex[complex(0, 0)]));
}

BOOST_AUTO_TEST_CASE(cross)
{
  std::vector<size_t> shape = {{3, 3}};
  boost::multi_array<size_t, 2> array(shape);

  std::fill(array.origin(), array.origin() + array.num_elements(), 2);
  array[0][1] = 1;
  array[1][0] = 1;
  array[2][1] = 1;
  array[1][2] = 1;
  auto _complex = BuildCubicalComplexFromArray(array);
  auto& complex = *_complex;


  BOOST_CHECK(! IsAcyclicNeighbourhood<2>(complex, complex[complex(1, 1)]));
}

BOOST_AUTO_TEST_SUITE_END()
