/////////////////////////////////////////////////////////////////////////////
/// @file AcyclicShavingTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-17
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <TestTools.h>
#include <capd/complex/AcyclicShaving.h>
#include <capd/auxil/Logger.h>
#include <../apiRedHom/Fixtures.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <capd/complex/CubicalComplexFromArray.h>

namespace capd
{
  namespace test
  {

  }
}

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(AcyclicShavingTestSuite)

BOOST_AUTO_TEST_CASE(shaveCircle)
{
  std::vector<size_t> shape = {{3, 3}};
  boost::multi_array<size_t, 2> array(shape);

  std::fill(array.origin(), array.origin() + array.num_elements(), 1);
  array[1][1] = 2;
  auto complex = BuildCubicalComplexFromArray(array);

  size_t shaved = DoAcyclicShaving<2>(*complex);
  BOOST_CHECK_EQUAL(4, shaved);
  BOOST_CHECK_EQUAL(4, b::distance(complex->iterators(1).dimCells(2)));
}

BOOST_AUTO_TEST_CASE(shaveSphere)
{
  std::vector<size_t> shape = {{3, 3, 3}};
  boost::multi_array<size_t, 3> array(shape);

  std::fill(array.origin(), array.origin() + array.num_elements(), 1);
  array[1][1][1] = 2;
  auto complex = BuildCubicalComplexFromArray(array);

  size_t shaved = DoAcyclicShaving<3>(*complex);
  BOOST_CHECK_EQUAL(20, shaved);
  BOOST_CHECK_EQUAL(6, b::distance(complex->iterators(1).dimCells(3)));
}

BOOST_AUTO_TEST_SUITE_END()
