/////////////////////////////////////////////////////////////////////////////
/// @file AlgorithmsTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-20
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.


#include <capd/complex/BettiNumbers.h>
#include <capd/complex/BettiNumbersViaSmithForm.h>
#include <../apiRedHom/Fixtures.h>
#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd::test;
using namespace capd::complex;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(AlgorithmsSuite)

BOOST_AUTO_TEST_CASE_TEMPLATE(betti, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef BettiNumbers<ComplexType, int, int> BettiAlg;

  BettiAlg bettiAlg(fixture.complex->template rawComplex<ComplexType>());
  std::vector<typename BettiAlg::Scalar> betti = bettiAlg();
  CAPD_DEBUG(fixture.betti << " " << betti);
  BOOST_CHECK_EQUAL_COLLECTIONS(fixture.betti.begin(), fixture.betti.end(), betti.begin(), betti.end());
}

BOOST_AUTO_TEST_CASE_TEMPLATE(doubleBetti, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef BettiNumbers<ComplexType, int, int> BettiAlg;

  BettiAlg bettiAlg(fixture.complex->template rawComplex<ComplexType>());
  std::vector<typename BettiAlg::Scalar> betti = bettiAlg();

  BOOST_CHECK_EQUAL_COLLECTIONS(fixture.betti.begin(), fixture.betti.end(), betti.begin(), betti.end());
}


BOOST_AUTO_TEST_CASE_TEMPLATE(bettiViaSmith, Fixture, ComplexFixtures)
{
  Fixture fixture;

  if (fixture.cardinality > 100) {
    return;
  }

  typedef typename Fixture::ComplexType ComplexType;
  typedef BettiNumbersViaSmithForm<ComplexType, int, int> BettiAlg;

  BettiAlg bettiAlg(fixture.complex->template rawComplex<ComplexType>());
  std::vector<typename BettiAlg::Scalar> betti = bettiAlg();

  BOOST_CHECK_EQUAL_COLLECTIONS(fixture.betti.begin(), fixture.betti.end(), betti.begin(), betti.end());
}


BOOST_AUTO_TEST_SUITE_END()
