/////////////////////////////////////////////////////////////////////////////
/// @file BuildFiltrationTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-09-20
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/complex/BuildFiltration.hpp>
#include <capd/auxil/Logger.h>

#include <../apiRedHom/Fixtures.h>
#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <capd/persistence/PHATInterface.h>

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::complex;
using namespace capd::test;

BOOST_AUTO_TEST_SUITE(BuildFiltrationTestSuite)

BOOST_AUTO_TEST_CASE_TEMPLATE(build, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();

  BuildFiltration<ComplexType> buildFiltration(complex);
  buildFiltration();

  BOOST_CHECK_EQUAL(complex.cardinality(), buildFiltration.filtration().size());
}


BOOST_AUTO_TEST_CASE_TEMPLATE(checkFiltrationBetti, Fixture, ComplexFixtures)
{
  using capd::persistence::PHATInterface;

  if (!PHATInterface::enabled()) {
    return;
  }

  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();

  BuildFiltration<ComplexType> buildFiltration(complex);
  buildFiltration();
  typename BuildFiltration<ComplexType>::Filtration filtration = buildFiltration.filtration();

  PHATInterface phat;
  PHATInterface::Pairs pairs;
  PHATInterface::Cycles cycles;
  std::vector<PHATInterface::FiltrationElement> phatFiltration(filtration.size());

  //TODO convert on the fly with transform iterator ?
  for (size_t idx = 0; idx < filtration.size(); ++idx) {
    phatFiltration[idx].first = filtration[idx].first;
    phatFiltration[idx].second.assign(filtration[idx].second.begin(), filtration[idx].second.end());
  }

  phat(PHATInterface::Filtration(phatFiltration.begin(), phatFiltration.end()), phatFiltration.size(), pairs, cycles);

  size_t bettiSum = 0;
  for (size_t idx = 0; idx < fixture.bettiMod(2).size(); ++idx) {
    bettiSum += fixture.bettiMod(2)[idx];
  }

  BOOST_CHECK_EQUAL(bettiSum, cycles.size());
}

BOOST_AUTO_TEST_SUITE_END()
