/////////////////////////////////////////////////////////////////////////////
/// @file CellsByDimTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-24
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <TestTools.h>
#include <capd/complex/CellsByDim.h>
#include <capd/auxil/Logger.h>
#include <../apiRedHom/Fixtures.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <boost/range/adaptors.hpp>

namespace capd
{
  namespace test
  {

  }
}

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(CellsByDimTestSuite)

BOOST_AUTO_TEST_CASE_TEMPLATE(check, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();
  CellsByDim<ComplexType> cellsByDim(complex);

  BOOST_CHECK_EQUAL(std::distance(complex.iterators().allCells().begin(),
                                    complex.iterators().allCells().end()),
                    boost::distance(cellsByDim));

  typename ComplexType::Dim dim = 0;
  for (auto x : cellsByDim) {
    BOOST_CHECK(dim <= x.getDim());
    dim = x.getDim();
  }
}

BOOST_AUTO_TEST_CASE_TEMPLATE(checkReversed, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();
  CellsByDimReversed<ComplexType> cellsByDim(complex);

  BOOST_CHECK_EQUAL(std::distance(complex.iterators().allCells().begin(),
                                    complex.iterators().allCells().end()),
                    boost::distance(cellsByDim));

  typename ComplexType::Dim dim = complex.getDim();
  for (auto x : cellsByDim) {
    CAPD_DEBUG(x.getDim());
    BOOST_CHECK(dim >= x.getDim());
    dim = x.getDim();
  }
}

BOOST_AUTO_TEST_SUITE_END()
