/////////////////////////////////////////////////////////////////////////////
/// @file ChainMapFixtures.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-01-03
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_CHAINMAPFIXTURES_H
#define CAPD_FILE_CHAINMAPFIXTURES_H

#include <capd/complex/ChainMap.h>

#include "../apiRedHom/Fixtures.h"

#include <boost/mpl/push_back.hpp>
#include <boost/mpl/fold.hpp>
#include <boost/mpl/vector.hpp>
#include <boost/mpl/joint_view.hpp>

namespace capd
{
  namespace test
  {
    using namespace capd::apiRedHom;
    using namespace capd::complex;
    using namespace boost::assign;


    template<typename DomFixture, typename ImgFixture, typename Scalar = int, typename Int = int>
    struct ChainMapFixtures
    {
      typedef typename DomFixture::ComplexType DomComplex;
      typedef typename ImgFixture::ComplexType ImgComplex;
      typedef HomologyModel<DomComplex, Scalar, Int> DomHomologyModel;
      typedef HomologyModel<ImgComplex, Scalar, Int> ImgHomologyModel;

      DomFixture dom;
      ImgFixture img;

      DomFixture domOrg;
      ImgFixture imgOrg;

      DomHomologyModel domHomologyModel;
      ImgHomologyModel imgHomologyModel;


      ChainMapFixtures():
        domHomologyModel(dom.complex->template rawComplex<DomComplex>()),
        imgHomologyModel(img.complex->template rawComplex<ImgComplex>())
      {
        domHomologyModel();
        imgHomologyModel();
      }

      DomComplex& domComplexOrg() { return domOrg.complex->template rawComplex<DomComplex>(); }
      ImgComplex& imgComplexOrg() { return imgOrg.complex->template rawComplex<ImgComplex>(); }

      template<typename Function>
      ChainMap<DomHomologyModel, ImgHomologyModel, Function>
      makeChainMap(Function function)
      {
        return capd::complex::makeChainMap(domHomologyModel, imgHomologyModel, function);
      }

    };

    struct SimplicialCircleChainMap: ChainMapFixtures<SimplicialCircle, SimplicialCircle>
    {};

    struct SimplicialCircleToCubicalPointChainMap: ChainMapFixtures<SimplicialCircle, CubicalPoint<> >
    {};


    typedef boost::mpl::fold<ComplexFixtures,
                             boost::mpl::vector<>,
                             boost::mpl::push_back<boost::mpl::_1,
                                                   ChainMapFixtures<boost::mpl::_2, boost::mpl::_2> >
                             >::type AuthomorphismChainMapFixtures;

  }

}

#endif // CAPD_FILE_CHAINMAPFIXTURES_H
