/////////////////////////////////////////////////////////////////////////////
/// @file ChainTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-30
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <capd/complex/Chain.h>
#include <../apiRedHom/Fixtures.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/typeof/typeof.hpp>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd::test;
using namespace capd::complex;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(ChainTestSuite)

BOOST_AUTO_TEST_CASE( init )
{

}

BOOST_AUTO_TEST_CASE_TEMPLATE(op, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;

  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();

  BOOST_AUTO(allCells, complex.iterators(1).allCells());
  Chain<ComplexType, int> chainOrg(allCells.begin(), allCells.end());

  if (chainOrg.size() == 0) {
    return;
  }

  {
    Chain<ComplexType, int> chain(chainOrg);

    chain += typename Chain<ComplexType, int>::value_type(chain.begin()->first, - chain.begin()->second);
    BOOST_CHECK_EQUAL(chainOrg.size() - 1, chain.size());
  }

  {
    Chain<ComplexType, int> chain(chainOrg);

    chain.insert(typename Chain<ComplexType, int>::value_type(chain.begin()->first, 0));
    BOOST_CHECK_EQUAL(chainOrg.size() - 1, chain.size());
  }

  {
    Chain<ComplexType, int> chainA(chainOrg), chainB(chainOrg);
    chainA += chainOrg;
    chainB *= 2;
    BOOST_CHECK_EQUAL_COLLECTIONS(chainA.begin(), chainA.end(),
                                  chainB.begin(), chainB.end());
  }

}


BOOST_AUTO_TEST_CASE(boundary)
{
  typedef SimplicialCircle Fixture;
  Fixture fixture;
  typedef Fixture::ComplexType ComplexType;
  typedef Fixture::ComplexType::Cell Cell;

  ComplexType& complex = fixture.complex->rawComplex<ComplexType>();

  Chain<ComplexType, int> chain;

  chain.insert(fixture.ab.rawCell<Cell>(), 1);
  chain.insert(fixture.bc.rawCell<Cell>(), 1);
  chain.insert(fixture.ac.rawCell<Cell>(), -1);

  BOOST_CHECK_EQUAL(3, chain.size());

  Chain<ComplexType, int> bd = chain.boundary(complex);

  BOOST_CHECK_EQUAL(0, bd.size());
}

BOOST_AUTO_TEST_SUITE_END()
