/////////////////////////////////////////////////////////////////////////////
/// @file ComputeHomologyOrchestratorTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-05-26
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <../apiRedHom/Fixtures.h>

#include <TestTools.h>

#include <capd/complex/ComputeHomologyOrchestrator.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(ComputeHomologyOrchestratorTestSuite)

BOOST_AUTO_TEST_CASE_TEMPLATE(computeHomologyOverZ, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef ComputeHomologyOrchestrator<ComplexType, int, int> Alg;

  Alg alg(fixture.complex->template rawComplex<ComplexType>(), true, true, true, true);

  std::vector<typename Alg::Int> betti = alg();
  BOOST_CHECK_EQUAL_COLLECTIONS(fixture.betti.begin(), fixture.betti.end(), betti.begin(), betti.end());
}

BOOST_AUTO_TEST_CASE_TEMPLATE(computeHomologyOverZ__noMorse_noKMS, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef ComputeHomologyOrchestrator<ComplexType, int, int> Alg;

  Alg alg(fixture.complex->template rawComplex<ComplexType>(), true, true, false, false);

  std::vector<typename Alg::Int> betti = alg();
  BOOST_CHECK_EQUAL_COLLECTIONS(fixture.betti.begin(), fixture.betti.end(), betti.begin(), betti.end());
}

BOOST_AUTO_TEST_CASE_TEMPLATE(computeHomologyOverZ_morse_noKMS, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef ComputeHomologyOrchestrator<ComplexType, int, int> Alg;

  Alg alg(fixture.complex->template rawComplex<ComplexType>(), true, true, true, false);

  std::vector<typename Alg::Int> betti = alg();
  BOOST_CHECK_EQUAL_COLLECTIONS(fixture.betti.begin(), fixture.betti.end(), betti.begin(), betti.end());
}

BOOST_AUTO_TEST_CASE_TEMPLATE(computeHomologyOverZ_noMorse_KMS, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef ComputeHomologyOrchestrator<ComplexType, int, int> Alg;

  Alg alg(fixture.complex->template rawComplex<ComplexType>(), true, true, false, true);

  std::vector<typename Alg::Int> betti = alg();
  BOOST_CHECK_EQUAL_COLLECTIONS(fixture.betti.begin(), fixture.betti.end(), betti.begin(), betti.end());
}


BOOST_AUTO_TEST_SUITE_END()
