/////////////////////////////////////////////////////////////////////////////
/// @file ComputeHomologySignatureTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-09-20
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/complex/ComputeHomologySignature.h>
#include <capd/auxil/Logger.h>

#include <../apiRedHom/Fixtures.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <capd/rings/Z2.h>

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::complex;
using namespace capd::test;

BOOST_AUTO_TEST_SUITE(ComputeHomologySignatureTestSuite)

BOOST_AUTO_TEST_CASE_TEMPLATE(computeHomologyOverZ, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef ComputeHomologySignature<ComplexType, int, int> Alg;

  Alg alg(fixture.complex->template rawComplex<ComplexType>(), true);

  std::vector<typename Alg::Int> betti = alg().ranks();
  BOOST_CHECK_EQUAL_COLLECTIONS(fixture.betti.begin(), fixture.betti.end(), betti.begin(), betti.end());
}

BOOST_AUTO_TEST_CASE_TEMPLATE(computeHomologyOverZ_noKMS, Fixture, ComplexFixtures)
{
  Fixture fixture;

  if (fixture.cardinality > 100) {
    return;
  }

  typedef typename Fixture::ComplexType ComplexType;
  typedef ComputeHomologySignature<ComplexType, int, int> Alg;

  Alg alg(fixture.complex->template rawComplex<ComplexType>(), false);

  std::vector<typename Alg::Int> betti = alg().ranks();
  BOOST_CHECK_EQUAL_COLLECTIONS(fixture.betti.begin(), fixture.betti.end(), betti.begin(), betti.end());
}

BOOST_AUTO_TEST_CASE_TEMPLATE(computeHomologyOverZ2, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef ComputeHomologySignature<ComplexType, Z2, int> Alg;

  Alg alg(fixture.complex->template rawComplex<ComplexType>(), true);

  std::vector<typename Alg::Int> betti = alg().ranks();

  BOOST_CHECK_EQUAL_COLLECTIONS(fixture.bettiMod(2).begin(), fixture.bettiMod(2).end(), betti.begin(), betti.end());
}

BOOST_AUTO_TEST_SUITE_END()
