/////////////////////////////////////////////////////////////////////////////
/// @file CoreductionPairInclusionChainModelTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-01-01
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <capd/complex/CoreductionPairInclusionChainModel.h>

#include <TestTools.h>
#include <../apiRedHom/Fixtures.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/typeof/typeof.hpp>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd::test;
using namespace capd::complex;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(CoreductionPairInclusionChainModelTestSuite)

BOOST_AUTO_TEST_CASE( init )
{

}

BOOST_AUTO_TEST_CASE(circle)
{
  typedef SimplicialCircle Fixture;
  Fixture fixture;
  typedef Fixture::ComplexType ComplexType;
  typedef Fixture::ComplexType::Cell Cell;

  ComplexType& complex = fixture.complex->rawComplex<ComplexType>();

  std::vector<SReductionPair<ComplexType> > pairs =
    list_of
    (makeSReductionPair(complex, fixture.b.rawCell<Cell>(), fixture.ab.rawCell<Cell>()))
    (makeSReductionPair(complex, fixture.c.rawCell<Cell>(), fixture.bc.rawCell<Cell>()));

  BOOST_AUTO(chainModel, makeCoreductionPairInclusionChainModel<int>(complex, pairs));

  {
    Chain<ComplexType, int> chain(fixture.ac.rawCell<Cell>(), 1);
    chainModel(chain);
    BOOST_CHECK_EQUAL(3, chain.size());
    BOOST_CHECK_EQUAL(1, chain[fixture.ac.rawCell<Cell>()]);
    BOOST_CHECK_EQUAL(-1, chain[fixture.ab.rawCell<Cell>()]);
    BOOST_CHECK_EQUAL(-1, chain[fixture.bc.rawCell<Cell>()]);
    BOOST_CHECK(chain.boundary(complex).isZero());
  }

  {
    Chain<ComplexType, int> chain(fixture.ac.rawCell<Cell>(), -1);
    chainModel(chain);
    BOOST_CHECK_EQUAL(3, chain.size());
    BOOST_CHECK_EQUAL(-1, chain[fixture.ac.rawCell<Cell>()]);
    BOOST_CHECK_EQUAL(1, chain[fixture.ab.rawCell<Cell>()]);
    BOOST_CHECK_EQUAL(1, chain[fixture.bc.rawCell<Cell>()]);
    BOOST_CHECK(chain.boundary(complex).isZero());
  }

}


BOOST_AUTO_TEST_SUITE_END()
