/////////////////////////////////////////////////////////////////////////////
/// @file CoreductionPairProjectionChainModelTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-30
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <capd/complex/CoreductionPairProjectionChainModel.h>
#include <../apiRedHom/Fixtures.h>
#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/typeof/typeof.hpp>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd::test;
using namespace capd::complex;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(CoreductionPairProjectionChainModelTestSuite)

BOOST_AUTO_TEST_CASE(circle)
{
  typedef SimplicialCircle Fixture;
  Fixture fixture;
  typedef Fixture::ComplexType ComplexType;
  typedef Fixture::ComplexType::Cell Cell;

  ComplexType& complex = fixture.complex->rawComplex<ComplexType>();

  std::vector<SReductionPair<ComplexType> > pairs =
    list_of
    (makeSReductionPair(complex, fixture.a.rawCell<Cell>()))
    (makeSReductionPair(complex, fixture.b.rawCell<Cell>(), fixture.ab.rawCell<Cell>()))
    (makeSReductionPair(complex, fixture.c.rawCell<Cell>(), fixture.bc.rawCell<Cell>()));

  BOOST_AUTO(chainModel, makeCoreductionPairProjectionChainModel<int>(complex, pairs));

  {
    Chain<ComplexType, int> chain(fixture.a.rawCell<Cell>());

    chainModel(chain);
    BOOST_CHECK_EQUAL(1, chain.size());
    BOOST_CHECK_EQUAL(fixture.a.rawCell<Cell>().getId(), chain.begin()->first);
    BOOST_CHECK_EQUAL(1, chain.begin()->second);
  }

  {
    Chain<ComplexType, int> chain(fixture.b.rawCell<Cell>());

    chainModel(chain);
    BOOST_CHECK_EQUAL(1, chain.size());
    BOOST_CHECK_EQUAL(fixture.a.rawCell<Cell>().getId(), chain.begin()->first);
    BOOST_CHECK_EQUAL(1, chain.begin()->second);
  }

  {
    Chain<ComplexType, int> chain;
    chain.insert(fixture.c.rawCell<Cell>().getId(), 2);

    chainModel(chain);
    BOOST_CHECK_EQUAL(1, chain.size());
    BOOST_CHECK_EQUAL(fixture.a.rawCell<Cell>().getId(), chain.begin()->first);
    BOOST_CHECK_EQUAL(2, chain.begin()->second);
  }

  {
    BOOST_AUTO(allCells, complex.iterators(1).dimCells(1));
    Chain<ComplexType, int> chain(allCells.begin(), allCells.end());

    BOOST_CHECK_EQUAL(3, chain.size());
    chainModel(chain);
    BOOST_CHECK_EQUAL(1, chain.size());
    BOOST_CHECK_EQUAL(fixture.ac.rawCell<Cell>().getId(), chain.begin()->first);
    BOOST_CHECK_EQUAL(1, chain.begin()->second);
  }

}


BOOST_AUTO_TEST_CASE(triangle)
{
  typedef SimplicialTriangle Fixture;
  Fixture fixture;
  typedef Fixture::ComplexType ComplexType;
  typedef Fixture::ComplexType::Cell Cell;

  ComplexType& complex = fixture.complex->rawComplex<ComplexType>();

  std::vector<SReductionPair<ComplexType> > pairs =
    list_of
    (makeSReductionPair(complex, fixture.b.rawCell<Cell>(), fixture.ab.rawCell<Cell>()))
    (makeSReductionPair(complex, fixture.c.rawCell<Cell>(), fixture.bc.rawCell<Cell>()))
    (makeSReductionPair(complex, fixture.ac.rawCell<Cell>(), fixture.abc.rawCell<Cell>()));

  BOOST_AUTO(chainModel, makeCoreductionPairProjectionChainModel<int>(complex, pairs));


  Chain<ComplexType, int> chain(fixture.ac.rawCell<Cell>());
  chainModel(chain);
  BOOST_CHECK_EQUAL(0, chain.size());
}


BOOST_AUTO_TEST_SUITE_END()
