/////////////////////////////////////////////////////////////////////////////
/// @file CoreductionTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-05-28
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <../apiRedHom/Fixtures.h>
#include <capd/complex/Coreduction.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd::complex;
using namespace capd::complex;
using namespace capd::test;

BOOST_AUTO_TEST_SUITE(CoreductionTestSuite)

BOOST_AUTO_TEST_CASE_TEMPLATE(coreduction, Fixture, ComplexFixtures)
{
  typedef typename Fixture::ComplexType ComplexType;
  Fixture fixture;
  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();

  boost::shared_ptr<typename CoreductionFactory<ComplexType, int, int>::DefaultAlgorithm> algo =
    CoreductionFactory<ComplexType, int, int>::createDefault(complex);

  size_t reduced = (*algo)();

  BOOST_CHECK_EQUAL(fixture.coreductions, reduced);
}


BOOST_AUTO_TEST_CASE(coreduction_after_excision)
{
  typedef SimplicialTriangle Fixture;
  typedef Fixture::ComplexType ComplexType;
  Fixture fixture;
  ComplexType& complex = fixture.complex->rawComplex<ComplexType>();

  complex[fixture.a.id()].setColor(2);
  complex.setClosed(false);

  boost::shared_ptr<CoreductionFactory<ComplexType, int, int>::DefaultAlgorithm> algo =
    CoreductionFactory<ComplexType, int, int>::createDefault(complex);

  size_t reduced = (*algo)();

  BOOST_CHECK_EQUAL(fixture.coreductions - 1, reduced);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(doubleCoreduction, Fixture, ComplexFixtures)
{
  typedef typename Fixture::ComplexType ComplexType;
  Fixture fixture;
  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();

  {
    boost::shared_ptr<typename CoreductionFactory<ComplexType, int, int>::DefaultAlgorithm> algo =
      CoreductionFactory<ComplexType, int, int>::createDefault(complex);
    (*algo)();
  }

  boost::shared_ptr<typename CoreductionFactory<ComplexType, int, int>::DefaultAlgorithm> algo =
    CoreductionFactory<ComplexType, int, int>::createDefault(complex);
  size_t reduced = (*algo)();

  BOOST_CHECK_EQUAL(0, reduced);
}

BOOST_AUTO_TEST_SUITE_END()
