/////////////////////////////////////////////////////////////////////////////
/// @file CubicalComplexFromArrayTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-18
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <TestTools.h>
#include <capd/complex/CubicalComplexFromArray.h>
#include <capd/auxil/Logger.h>
//#include <../apiRedHom/Fixtures.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>

namespace capd
{
  namespace test
  {

  }
}

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(CubicalComplexFromArrayTestSuite)

BOOST_AUTO_TEST_CASE(simpleCubicalArray)
{
  std::vector<size_t> shape = {{2, 2}};
  boost::multi_array<size_t, 2> array(shape);

  std::fill(array.origin(), array.origin() + array.num_elements(), 1);
  array[0][0] = 2;
  array[1][1] = 2;

  auto complex = BuildCubicalComplexFromArray(array);

  BOOST_CHECK_EQUAL(9, boost::distance(complex->iterators().dimCells(0)));
  BOOST_CHECK_EQUAL(7, boost::distance(complex->iterators(1).dimCells(0)));
  BOOST_CHECK_EQUAL(2, boost::distance(complex->iterators(2).dimCells(0)));
}


BOOST_AUTO_TEST_CASE(from_view)
{
  typedef boost::multi_array<size_t, 2> array_type;
  typedef array_type::index_range range;
  std::vector<size_t> shape = {{10, 10}};
  array_type array(shape);

  std::fill(array.origin(), array.origin() + array.num_elements(), 2);

  {
    auto view = array[b::indices[2][range(2, 6)]];
    std::fill(view.begin(), view.end(), 1);
  }

  {
    auto view = array[b::indices[5][range(2, 6)]];
    std::fill(view.begin(), view.end(), 1);
  }

  {
    auto view = array[b::indices[range(2, 6)][2]];
    std::fill(view.begin(), view.end(), 1);
  }

  {
    auto view = array[b::indices[range(2, 6)][5]];
    std::fill(view.begin(), view.end(), 1);
  }

  auto complex = BuildCubicalComplexFromArray(array);

  BOOST_CHECK_EQUAL(12, boost::distance(complex->iterators(1).dimCells(2)));
}

BOOST_AUTO_TEST_SUITE_END()
