/////////////////////////////////////////////////////////////////////////////
/// @file CubicalComplexFullCubesBoundsTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-19
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <TestTools.h>
#include <capd/complex/CubicalComplexFullCubesBounds.h>
#include <capd/auxil/Logger.h>
//#include <../apiRedHom/Fixtures.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <boost/multi_array.hpp>
#include <capd/complex/CubicalComplexFromArray.h>
#include <capd/complex/FilteredComplex.h>

namespace capd
{
  namespace test
  {

  }
}

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(CubicalComplexFullCubesBoundsTestSuite)

BOOST_AUTO_TEST_CASE(cross3D)
{
  std::vector<size_t> shape = {{3, 3, 3}};
  boost::multi_array<size_t, 3> array(shape);
  std::fill(array.origin(), array.origin() + array.num_elements(), 2);
  array[0][1][1] = 1;
  array[2][1][1] = 1;
  array[1][0][1] = 1;
  array[1][2][1] = 1;
  array[1][1][0] = 1;
  array[1][1][2] = 1;

  auto complex = BuildCubicalComplexFromArray(array);
  auto bounds = makeCubicalComplexFullCubesBounds(*complex);

  for (auto cell : complex->iterators(1).allCells()) {
    BOOST_CHECK(bounds(cell));
  }

  {
    auto toRemove = (*complex)[(*complex)(0, 1, 1)];
    toRemove.setColor(2);
    BOOST_CHECK(bounds.adjust(*complex));
    BOOST_CHECK(! bounds(toRemove));
  }


  for (auto cell : complex->iterators(1).allCells()) {
    cell.setColor(2);
  }

  BOOST_CHECK(bounds.adjust(*complex));
  BOOST_CHECK(!bounds.adjust(*complex));

  for (auto cell : complex->iterators().allCells()) {
    BOOST_CHECK(! bounds(cell));
  }

}

BOOST_AUTO_TEST_CASE(cross3D_filteredComplex)
{
  std::vector<size_t> shape = {{3, 3, 3}};
  boost::multi_array<size_t, 3> array(shape);
  std::fill(array.origin(), array.origin() + array.num_elements(), 2);
  array[0][1][1] = 1;
  array[2][1][1] = 1;
  array[1][0][1] = 1;
  array[1][2][1] = 1;
  array[1][1][0] = 1;
  array[1][1][2] = 1;

  auto complex = BuildCubicalComplexFromArray(array);
  auto bounds = makeCubicalComplexFullCubesBounds(*complex);

  auto boundedComplex = makeFilteredComplex(*complex,
                                            bl::bind(bl::var(bounds),
                                                     bl::_1));

  BOOST_CHECK_EQUAL(boundedComplex.cardinality(), complex->cardinality());
  bounds.adjust(boundedComplex);
  BOOST_CHECK_EQUAL(boundedComplex.cardinality(), complex->cardinality());
}


BOOST_AUTO_TEST_SUITE_END()
