/////////////////////////////////////////////////////////////////////////////
/// @file ExcisionTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-05-23
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <capd/complex/Excision.h>

#include <TestTools.h>
#include "../apiRedHom/Fixtures.h"

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd::test;
using namespace capd::complex;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(ExcisionTestSuite)

BOOST_AUTO_TEST_CASE_TEMPLATE(remove_all, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  ComplexType& complex = fixture.rawComplex();
  Excision<ComplexType> excision(complex);

  Fixture subspace;
  excision(subspace.rawComplex());

  BOOST_CHECK_EQUAL(0, complex.cardinality());
}


BOOST_AUTO_TEST_CASE_TEMPLATE(remove_point, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;

  ComplexType& complex = fixture.rawComplex();
  Excision<ComplexType> excision(complex);

  const size_t cardinality = complex.cardinality();

  if (cardinality > 0) {
    excision(complex.iterators(1).dimCells(0).begin()->code());

    BOOST_CHECK_EQUAL(cardinality - 1, complex.cardinality());
  }
}



BOOST_AUTO_TEST_SUITE_END()
