/////////////////////////////////////////////////////////////////////////////
/// @file FilteredComplexTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-13
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/complex/FilteredComplex.h>
#include <capd/auxil/Logger.h>
#include <../apiRedHom/Fixtures.h>
#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <boost/range/distance.hpp>

namespace capd
{
  namespace test
  {

  }
}

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(FilteredComplexTestSuite)

BOOST_AUTO_TEST_CASE( init )
{

}

BOOST_AUTO_TEST_CASE_TEMPLATE(identity, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();
  auto filtered = makeFilteredComplex(complex, bl::constant(true));

  BOOST_CHECK_EQUAL(complex.size(), filtered.size());
  BOOST_CHECK_EQUAL(complex.cardinality(), filtered.cardinality());
  BOOST_CHECK_EQUAL(std::distance(complex.iterators().allCells().begin(),
                                    complex.iterators().allCells().end()),
                    boost::distance(filtered.iterators().allCells()));

  BOOST_CHECK_EQUAL(std::distance(complex.iterators(1).allCells().begin(),
                                    complex.iterators(1).allCells().end()),
                    boost::distance(filtered.iterators(1).allCells()));


  if (filtered.cardinality() <= 0) {
    return;
  }

  BOOST_CHECK_EQUAL(std::distance(complex.iterators().dimCells(0).begin(),
                                    complex.iterators().dimCells(0).end()),
                    boost::distance(filtered.iterators().dimCells(0)) );

  BOOST_CHECK_EQUAL(std::distance(complex.iterators(1).dimCells(0).begin(),
                                    complex.iterators(1).dimCells(0).end()),
                    boost::distance(filtered.iterators(1).dimCells(0)));

}


BOOST_AUTO_TEST_CASE_TEMPLATE(withoutFirst, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef typename ComplexType::ConstCell ConstCell;

  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();

  if (complex.cardinality() <= 0) {
    return;
  }

  auto firstCell = *(complex.iterators(1).allCells().begin());
  auto filtered = makeFilteredComplex(complex, bl::bind(&ConstCell::getId, bl::_1) != bl::constant(firstCell.getId()));

  BOOST_CHECK_EQUAL(std::distance(complex.iterators(1).dimCells(firstCell.getDim()).begin(),
                                    complex.iterators(1).dimCells(firstCell.getDim()).end()),
                    1 + boost::distance(filtered.iterators(1).dimCells(firstCell.getDim())));

  BOOST_CHECK_EQUAL(std::distance(complex.iterators(1).allCells().begin(),
                                    complex.iterators(1).allCells().end()),
                    1 + boost::distance(filtered.iterators(1).allCells()));

  BOOST_CHECK_EQUAL(complex.cardinality(), 1 + filtered.cardinality());
  BOOST_CHECK_EQUAL(complex.size(), filtered.size());

  for (auto bdCell : complex.iterators().bdCells(firstCell)) {
    BOOST_CHECK_EQUAL(std::distance(complex.iterators().cbdCells(bdCell).begin(),
                                    complex.iterators().cbdCells(bdCell).end()),
                      1 + boost::distance(filtered.iterators().cbdCells(filtered[bdCell.getId()])));
  }

  for (auto cbdCell : complex.iterators().cbdCells(firstCell)) {
    BOOST_CHECK_EQUAL(std::distance(complex.iterators().bdCells(cbdCell).begin(),
                                    complex.iterators().bdCells(cbdCell).end()),
                      1 + boost::distance(filtered.iterators().bdCells(filtered[cbdCell.getId()])));
  }

}


BOOST_AUTO_TEST_SUITE_END()
