/////////////////////////////////////////////////////////////////////////////
/// @file FundamentalGroupTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-04-12
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <TestTools.h>
#include <capd/complex/FundamentalGroup.h>


#include <capd/auxil/Logger.h>
#include <../apiRedHom/Fixtures.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/mpl/map.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <capd/complex/BuildGreedyDiscreteVectorField.h>

namespace capd
{
  namespace test
  {

  }
}

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(FundamentalGroupTestSuite)

// template<typename Fixture>
// std::vector<int> Legacy()
// {
//   typedef typename Fixture::ComplexType ComplexType;
//   typedef typename SelectorForComplex<ComplexType>::type ComplexSelector;

//   Fixture fixture;

//   typedef FundGroup<AKQReducedSComplexSupplier<HomologyTraits<ComplexSelector> > >  FundGroup;
//   typedef typename FundGroup::ComplexSupplier ComplexSupplier;
//   typedef typename FundGroup::ComplexSupplierPtr ComplexSupplierPtr;

//   FundGroup fundGroup(ComplexSupplierPtr(new ComplexSupplier(fixture.complex->template rawComplexPtr<ComplexType>())));
//   return fundGroup.HapInterfaceVector();
// }



std::vector<int> FundamentalGroupPresentation(int /*method*/, const ComplexFixture& fixture)
{
  if (fixture.bettiInDim(1) != 0) {
    return std::vector<int>({{1, 0}});
  } else {
    return std::vector<int>({{0, 0}});
  }
}

std::vector<int> FundamentalGroupPresentation(int method, const SimplicialRealProjectivePlane&)
{
  if (method == 0) {
    return std::vector<int>({{1, 1, 2, 1, -1, 1, -1}});
  } else {
    return std::vector<int>({{1, 1, 2, 1, 1, 1, 1}});
  }
}

std::vector<int> FundamentalGroupPresentation(int method, const CubicalTorus<capd::complex::DenseCubicalTag>&)
{
  if (method == 0) {
    return std::vector<int>({{2, 1, 4, 1, -1, 2, 1, 1, 1, 2, -1}});
  } else {
    return std::vector<int>({{2, 2, 4, 1, 1, 2, 1, 1, -1, 2, -1, 4, 1, 1, 2, 1, 1, -1, 2, -1}});
  }
}

std::vector<int> FundamentalGroupPresentation(int method, const CubicalTorus<capd::complex::OldCubicalTag>&)
{
  if (method == 0) {
    return std::vector<int>({{2, 1, 4, 1, -1, 2, -1, 1, 1, 2, 1}});
  } else {
    return std::vector<int>({{2, 2, 4, 2, -1, 1, -1, 2, 1, 1, 1, 4, 2, -1, 1, -1, 2, 1, 1, 1}});
  }
}

std::vector<int> FundamentalGroupPresentation(int method, const CubicalCircle2<capd::complex::OldCubicalTag>&)
{
  if (method == 0) {
    return std::vector<int>({{1, 0}});
  } else {
    return std::vector<int>({{2, 1, 2, 2, 1, 1, -1}});
  }
}

std::vector<int> FundamentalGroupPresentation(int method, const CubicalCircle2<capd::complex::DenseCubicalTag>&)
{
  if (method == 0) {
    return std::vector<int>({{1, 0}});
  } else {
    return std::vector<int>({{2, 1, 2, 1, 1, 2, -1}});
  }
}

template<typename T>
std::vector<int> FundamentalGroupPresentation(int /*method*/, const CubicalAntipodalPoints<T>&)
{
  return std::vector<int>();
}


BOOST_AUTO_TEST_CASE_TEMPLATE(fundamentalGroupCoreduction, Fixture, ComplexFixtures)
{
  Fixture fixture;
  if (FundamentalGroupPresentation(0, fixture).size() == 0) {
    return;
  }

  typedef typename Fixture::ComplexType ComplexType;
  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();

  typename BuildGreedyDiscreteVectorFieldViaCoreductions<ComplexType, int>::type algo(complex);
  auto& vf = algo();

  CAPD_DEBUG("criticalCells: " << vf.criticalCells());
  std::vector<int> fg = FundamentalGroupHapInterfaceVector(vf);

  BOOST_CHECK_EQUAL(FundamentalGroupPresentation(0, fixture), fg);
  //  BOOST_CHECK_EQUAL(fixture.hapInterfaceVector(), Legacy<Fixture>());
}

BOOST_AUTO_TEST_CASE_TEMPLATE(fundamentalGroupReduction, Fixture, ComplexFixtures)
{
  Fixture fixture;
  if (FundamentalGroupPresentation(1, fixture).size() == 0) {
    return;
  }

  typedef typename Fixture::ComplexType ComplexType;
  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();

  typename BuildGreedyDiscreteVectorFieldViaReductions<ComplexType, int>::type algo(complex);
  auto& vf = algo();

  std::vector<int> fg = FundamentalGroupHapInterfaceVector(vf);
  BOOST_CHECK_EQUAL(FundamentalGroupPresentation(1, fixture), fg);
}

BOOST_AUTO_TEST_SUITE_END()
