/////////////////////////////////////////////////////////////////////////////
/// @file GeometricBoundaryTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-10
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/complex/GeometricBoundary.h>
#include <capd/auxil/Logger.h>
#include <../apiRedHom/Fixtures.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <set>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(GeometricBoundaryTestSuite)

BOOST_AUTO_TEST_CASE_TEMPLATE(atLeastBoundary, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef typename ComplexType::Cell Cell;;

  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();

  auto geometricBoundary = GeometricBoundary<ComplexType>(complex);
  for (Cell cell : complex.iterators().allCells()) {
    auto gbd = geometricBoundary(cell);
    auto bdCells = complex.iterators().bdCells(cell);
    std::set<Cell> gbdUniq(gbd.begin(), gbd.end());

    BOOST_CHECK(std::distance(bdCells.begin(), bdCells.end()) + size_t(1) <= gbdUniq.size());
  }
}

BOOST_AUTO_TEST_CASE(gotoPoints)
{
  typedef SimplicialTriangle Fixture;
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef typename ComplexType::Cell Cell;;
  ComplexType& complex = fixture.complex->rawComplex<ComplexType>();

  auto geometricBoundary = GeometricBoundary<ComplexType>(complex);
  auto gbd = geometricBoundary(complex[fixture.abc.rawCell<Cell>().code()]);
  BOOST_CHECK_EQUAL(7, gbd.size());

}


BOOST_AUTO_TEST_SUITE_END()
