/////////////////////////////////////////////////////////////////////////////
/// @file HasseDiagramCellComplexTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-04-07
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <TestTools.h>
#include <../apiRedHom/Fixtures.h>

#include <capd/complex/HasseDiagramCellComplex.h>
#include <capd/auxil/Logger.h>
#include <capd/complex/BettiNumbers.hpp>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>

namespace capd
{
  namespace test
  {

  }
}

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(HasseDiagramCellComplexTestSuite)

BOOST_AUTO_TEST_CASE(disconnected)
{
  const size_t size = 10;
  HasseDiagram<int, int> diagram(size);
  diagram.set(0, 10);

  auto complex = BuildHasseDiagramCellComplex<size_t>(diagram, true);
  auto betti = ComputeBettiNumbers<int>(complex);

  BOOST_REQUIRE_EQUAL(11, betti.size());
  BOOST_CHECK_EQUAL(size - 1, betti[0]);
  BOOST_CHECK_EQUAL(1, betti[10]);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(convert, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  ComplexType& orgComplex = fixture.complex->template rawComplex<ComplexType>();
  auto complex = ConvertToHasseDiagramCellComplex<size_t>(orgComplex);

  for (auto cell : orgComplex.iterators().allCells()) {
    CAPD_DEBUG(cell.code() << " " << cell.getDim());
    BOOST_REQUIRE_EQUAL(cell.getDim(), complex[cell.getId()].getDim());
    BOOST_REQUIRE_EQUAL(cell.getColor(), complex[cell.getId()].getColor());
    BOOST_REQUIRE_EQUAL(boost::distance(orgComplex.iterators(1).bdCells(cell)),
                        boost::distance(complex.iterators(1).bdCells(complex[cell.getId()])));
  }
  BOOST_REQUIRE_EQUAL(fixture.cardinality, complex.cardinality());
  auto betti = ComputeBettiNumbers<int, size_t>(complex);

  BOOST_CHECK_EQUAL(fixture.betti, betti);
}

BOOST_AUTO_TEST_SUITE_END()
