/////////////////////////////////////////////////////////////////////////////
/// @file HomologyModelTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-01-01
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <capd/complex/HomologyModel.hpp>
#include <../apiRedHom/Fixtures.h>
#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <capd/homAux/Helpers.h>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd::test;
using namespace capd::complex;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(HomologyModelTestSuite)


template<typename Model, typename Fixture>
void CheckModelChains(Model& model, Fixture& fixture)
{
  typedef typename Model::Chain Chain;
  const size_t dim = model();

  // std::vector<int> bettis(dim);
  // model.evaluate(bl::var(bettis)[bl::_1]++);
  // homAux::Shrink(bettis);

  // BOOST_CHECK_EQUAL_COLLECTIONS(fixture.betti.begin(), fixture.betti.end(), bettis.begin(), bettis.end());


  for (size_t d = 0; d < dim; ++d) {
    BOOST_FOREACH(const Chain& chain, model.modelChains(d)) {
      BOOST_CHECK(!chain.isZero());
      BOOST_CHECK(chain.boundary(model.complex()).isZero());
    }
  }
}

template<typename Model, typename Fixture>
void CheckDomainChains(Model& model, Fixture& fixture)
{
  typedef typename Model::Chain Chain;
  Fixture fixtureOrg;
  const size_t dim = model();

  for (size_t d = 0; d < dim; ++d) {
    BOOST_FOREACH(const Chain& chain, model.domainChains(d)) {
      BOOST_CHECK(!chain.isZero());
      auto bd = chain.boundary(fixtureOrg.rawComplex());
      BOOST_CHECK(bd.isZero());
    }
  }
}

BOOST_AUTO_TEST_CASE_TEMPLATE(generators, Fixture, ComplexFixtures)
{
  Fixture fixture;
  if (fixture.cardinality > 100) {
    return;
  }

  typedef typename Fixture::ComplexType ComplexType;
  typedef HomologyModel<ComplexType, int, int> Model;

  if (fixture.rawComplex().cardinality() <= 0) {
    return;
  }

  Model model(fixture.rawComplex());
  CheckModelChains(model, fixture);
  CheckDomainChains(model, fixture);
}


BOOST_AUTO_TEST_CASE_TEMPLATE(generators_no_coreduction, Fixture, ComplexFixtures)
{
  Fixture fixture;

  if (fixture.cardinality > 100) {
    return;
  }

  typedef typename Fixture::ComplexType ComplexType;
  typedef HomologyModel<ComplexType, int, int> Model;

  ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();

  if (complex.cardinality() <= 0) {
    return;
  }

  Model alg(complex);
  alg.runCoreduction(false);

  CheckModelChains(alg, fixture);
}


// it is failing on cubical sphere

// BOOST_AUTO_TEST_CASE_TEMPLATE(generators_inclusion, Fixture, ComplexFixtures)
// {
//   Fixture fixture;
//   Fixture fixtureOrg;
//   typedef typename Fixture::ComplexType ComplexType;
//   typedef HomologyModel<ComplexType, int, int> Model;

//   ComplexType& complex = fixture.complex->template rawComplex<ComplexType>();
//   ComplexType& complexOrg = fixtureOrg.complex->template rawComplex<ComplexType>();

//   if (complex.cardinality() <= 0) {
//     return;
//   }

//   Model model(complex);
//   const size_t dim = model();


//   for (size_t d = 0; d < dim; ++d) {
//     BOOST_FOREACH(const typename Model::Chain& modelChain, model.modelChains(d)) {
//       BOOST_CHECK(modelChain.boundary(complex).isZero());
//       typename Model::Chain chain = modelChain;
//       model >> chain;
//       BOOST_CHECK_MESSAGE(chain.boundary(complexOrg).isZero(), "d = " << d << " chain = " );

//       model << chain;
//       BOOST_CHECK(chain.boundary(complex).isZero());
//       BOOST_CHECK_EQUAL_COLLECTIONS(modelChain.begin(), modelChain.end(),
// 				    chain.begin(), chain.end());

//     }
//   }

//   model.domainChains(0);
// }


BOOST_AUTO_TEST_SUITE_END()
