/////////////////////////////////////////////////////////////////////////////
/// @file HomologyTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-01-18
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <../apiRedHom/Fixtures.h>

#include <TestTools.h>

#include <capd/complex/Homology.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <capd/homAux/Helpers.h>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(HomologyTestSuite)

BOOST_AUTO_TEST_CASE_TEMPLATE(homologyOverZ, Fixture, ComplexFixtures)
{
  Fixture fixture;
  typedef typename Fixture::ComplexType ComplexType;
  typedef Homology<ComplexType, int, int> Alg;

  Alg alg(fixture.complex->template rawComplex<ComplexType>());

  typename Alg::Signature signature = alg();
  std::vector<typename Alg::Int> betti(signature.raw.size());

  for (size_t idx = 0; idx < signature.raw.size(); ++idx) {
    betti[idx] = signature.raw[idx].first;
  }

  homAux::Shrink(betti);
  BOOST_CHECK_EQUAL_COLLECTIONS(fixture.betti.begin(), fixture.betti.end(), betti.begin(), betti.end());

}


BOOST_AUTO_TEST_SUITE_END()
