/////////////////////////////////////////////////////////////////////////////
/// @file ImageCubCellComplexTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-09-17
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/complex/ImageCubCellComplex.h>
#include <capd/auxil/Logger.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <capd/vectalg/Matrix.hpp>

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(ImageCubCellComplexTestSuite)

BOOST_AUTO_TEST_CASE( init )
{
  typedef ImageCubCellComplex::Image Image;
  typedef ImageCubCellComplex::Complex Complex;

  double data[3][3] = {{2, 0, 0}, {0, 6, 0}, {0, 0, 12}};
  Image image(data);

  std::auto_ptr<Complex> complex(ImageCubCellComplex::Create(image));
  BOOST_CHECK_EQUAL(49, std::distance(complex->iterators().allCells().begin(),
                                      complex->iterators().allCells().end()));
}


BOOST_AUTO_TEST_CASE(coloredIterators)
{
  typedef ImageCubCellComplex::Image Image;
  typedef ImageCubCellComplex::Complex Complex;

  {
    double data[3][3] = {{0, 1, 0}, {0, 0, 0}, {0, 0, 0}};
    Image image(data);

    std::auto_ptr<Complex> complex(ImageCubCellComplex::Create(image));
    BOOST_CHECK_EQUAL(2, std::distance(complex->iterators(1).allCells().begin(),
                                       complex->iterators(1).allCells().end()));

  }

  {
    double data[3][4] = {{0,0,0, 0}, {0, 1, 1, 0}, {0, 0, 0, 0}};
    Image image(data);

    std::auto_ptr<Complex> complex(ImageCubCellComplex::Create(image));

    BOOST_CHECK_EQUAL(3, std::distance(complex->iterators(1).allCells().begin(),
                                       complex->iterators(1).allCells().end()));
  }

}


BOOST_AUTO_TEST_SUITE_END()
