/////////////////////////////////////////////////////////////////////////////
/// @file SimplicesReaderTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-11
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <capd/complex/SimplicesReader.h>
#include <capd/auxil/Logger.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>


namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(SimplicesReaderTestSuite)


BOOST_AUTO_TEST_CASE( oneCell )
{
  typedef SimplicesReader::CellCode CellCode;

  typedef std::vector<std::pair<std::string, CellCode> > TestCases;
  TestCases testCases =
    pair_list_of("0 1 2", list_of(0)(1)(2).operator CellCode())
    ("0, 1, 2", list_of(0)(1)(2).operator CellCode())
    (",0, 1, 2,", list_of(0)(1)(2).operator CellCode())
    ("(0, 1, 2)", list_of(0)(1)(2).operator CellCode())
    ("[0, 1, 2]", list_of(0)(1)(2).operator CellCode())
    ("(0, 1, 2,", list_of(0)(1)(2).operator CellCode())
    ("(0, 1, 2,)", list_of(0)(1)(2).operator CellCode())
    ("(0, 1, 2]\n\n", list_of(0)(1)(2).operator CellCode())
    ;


  for (TestCases::const_iterator it = testCases.begin(), end = testCases.end();
       it != end; ++it) {

    const CellCode& expected = it->second;
    std::stringstream input;
    input << it->first;

    SimplicesReader reader(input);
    CellCode cell;

    reader(bl::var(cell) = bl::_1);
    BOOST_CHECK_EQUAL_COLLECTIONS(expected.begin(), expected.end(), cell.begin(), cell.end());
  }
}


BOOST_AUTO_TEST_CASE( manyCells )
{
  typedef SimplicesReader::CellCode CellCode;

  const std::vector<CellCode> expected = list_of
    (list_of(0)(1)(2).operator CellCode())
    (list_of(2)(3)(4).operator CellCode())
    (list_of(1)(2)(3).operator CellCode());

  std::stringstream input;
  input << "0 1 2  " << std::endl << "   2; 3, 4\n\n" << "3, 2, 1";

  SimplicesReader reader(input);
  std::vector<CellCode> cells;

  reader(push_back(cells));
  BOOST_CHECK_EQUAL_COLLECTIONS(expected.begin(), expected.end(), cells.begin(), cells.end());

}


BOOST_AUTO_TEST_SUITE_END()
