/////////////////////////////////////////////////////////////////////////////
/// @file SqueezeIdsTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-01-19
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/complex/SqueezeIds.h>
#include "SqueezeIdsFixture.h"
#include <capd/auxil/Logger.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(SqueezeIdsTestSuite)

BOOST_AUTO_TEST_CASE_TEMPLATE(checkSize, Fixture, SqueezeIdsFixtures)
{
  Fixture fixture;
  SqueezeIds<int> squeezeIds(fixture.values.begin(), fixture.values.end());

  BOOST_CHECK_EQUAL(fixture.noOfDistincts, squeezeIds.size());
}

BOOST_AUTO_TEST_CASE_TEMPLATE(checkValues, Fixture, SqueezeIdsFixtures)
{
  Fixture fixture;
  SqueezeIds<int> squeezeIds(fixture.values.begin(), fixture.values.end());

  std::vector<int> newIds(fixture.values.size());
  for (size_t idx = 0; idx < fixture.values.size(); ++idx) {
    newIds[idx] = squeezeIds[fixture.values[idx]];
  }

  std::vector<size_t> count(fixture.noOfDistincts);

  for (size_t idx = 0; idx < newIds.size(); ++idx) {
    ++count[newIds[idx]];
  }

  for (size_t bucket = 0; bucket < fixture.noOfDistincts; ++bucket) {
    BOOST_CHECK(count[bucket] > 0);
  }

  for (size_t idx = 0; idx < fixture.values.size(); ++idx) {
    BOOST_CHECK_EQUAL(squeezeIds.reverse(newIds[idx]), fixture.values[idx]);
  }
}


BOOST_AUTO_TEST_SUITE_END()
