/////////////////////////////////////////////////////////////////////////////
/// @file ZigZagTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-19
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <TestTools.h>
#include <capd/complex/ZigZag.h>
#include <capd/auxil/Logger.h>
//#include <../apiRedHom/Fixtures.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <capd/complex/ComplexFunctors.h>

namespace capd
{
  namespace test
  {

  }
}

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::complex;

BOOST_AUTO_TEST_SUITE(ZigZagTestSuite)

BOOST_AUTO_TEST_CASE(zigzagCircle)
{
  const int DIM = 2;
  std::vector<size_t> shape = {{3, 3}};
  boost::multi_array<size_t, DIM> array(shape);

  std::fill(array.origin(), array.origin() + array.num_elements(), 1);
  array[1][1] = 2;
  auto complex = BuildCubicalComplexFromArray(array);

  DoZigZag<DIM>(*complex);
  BOOST_CHECK_EQUAL(4, b::distance(complex->iterators(1).dimCells(DIM)));
}

BOOST_AUTO_TEST_CASE(overgrowthCircle_bigToSmall)
{
  typedef boost::multi_array<size_t, 2> array_type;
  typedef array_type::index_range range;
  std::vector<size_t> shape = {{10, 10}};
  array_type array(shape);

  std::fill(array.origin(), array.origin() + array.num_elements(), 2);
  auto line = range(1, 8);
  {
    auto view = array[b::indices[1][line]];
    std::fill(view.begin(), view.end(), 1);
  }

  {
    auto view = array[b::indices[7][line]];
    std::fill(view.begin(), view.end(), 1);
  }

  {
    auto view = array[b::indices[line][1]];
    std::fill(view.begin(), view.end(), 1);
  }

  {
    auto view = array[b::indices[line][7]];
    std::fill(view.begin(), view.end(), 1);
  }

  auto complex = BuildCubicalComplexFromArray(array);
  BOOST_REQUIRE_EQUAL(100, boost::distance(complex->iterators().dimCells(2)));

  DoZigZag<2>(*complex);
  BOOST_CHECK_EQUAL(4, boost::distance(complex->iterators(1).dimCells(2)));
}


BOOST_AUTO_TEST_SUITE_END()
