/////////////////////////////////////////////////////////////////////////////
/// @file IteratedMultiValuedFunctorTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-10
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/homAux/IteratedMultiValuedFunctor.h>
#include <capd/auxil/Logger.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <boost/iterator/counting_iterator.hpp>

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::homAux;

BOOST_AUTO_TEST_SUITE(IteratedMultiValuedFunctorTestSuite)


struct ConstRangeGen
{
  typedef boost::iterator_range< boost::counting_iterator<int> > result_type;
  typedef int argument_type;
  result_type operator()(int)
  {
    return result_type(boost::make_counting_iterator(0), boost::make_counting_iterator(2));
  }

};

BOOST_AUTO_TEST_CASE(simpleStatic)
{
  {
    IteratedMultiValuedFunctor<1, ConstRangeGen> func;
    auto res = func(1);
    BOOST_CHECK_EQUAL(2, std::distance(res.begin(), res.end()));
  }

  {
    IteratedMultiValuedFunctor<2, ConstRangeGen> func;
    auto res = func(2);
    BOOST_CHECK_EQUAL(4, std::distance(res.begin(), res.end()));
  }

}

BOOST_AUTO_TEST_CASE(simpleDynamic)
{
  {
    IteratedMultiValuedFunctor<0, ConstRangeGen> func(1);
    auto res = func(1);
    BOOST_CHECK_EQUAL(2, std::distance(res.begin(), res.end()));
  }

  {
    IteratedMultiValuedFunctor<0, ConstRangeGen> func(2);
    auto res = func(2);
    BOOST_CHECK_EQUAL(4, std::distance(res.begin(), res.end()));
  }

  {
    typedef IteratedMultiValuedFunctor<0, ConstRangeGen> Iteration;
    BOOST_REQUIRE_THROW(Iteration(Iteration::MAX_DEPTH + 1),
                        std::runtime_error);
  }

}


BOOST_AUTO_TEST_SUITE_END()
