/////////////////////////////////////////////////////////////////////////////
/// @file MultiValuedTransformIteratorTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-03-10
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/homAux/MultiValuedTransformIterator.h>
#include <capd/auxil/Logger.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>
#include <boost/iterator/counting_iterator.hpp>
#include <boost/range/iterator_range_core.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::homAux;

BOOST_AUTO_TEST_SUITE(MultiValuedTransformIteratorTestSuite)

struct ZeroGen
{
  typedef boost::iterator_range< boost::counting_iterator<int> > result_type;

  result_type operator()(int)
  {
    return result_type(boost::make_counting_iterator(0), boost::make_counting_iterator(0));
  }

};

BOOST_AUTO_TEST_CASE(zero)
{
  MultiValuedTransformRange<ZeroGen, boost::counting_iterator<int> > transform(boost::make_counting_iterator(0),
                                                                               boost::make_counting_iterator(6), ZeroGen());
  BOOST_CHECK_EQUAL(0, std::distance(transform.begin(), transform.end()));
}


struct RangeGen
{
  typedef boost::iterator_range< boost::counting_iterator<int> > result_type;

  result_type operator()(int n) const
  {
    return result_type(boost::make_counting_iterator(0), boost::make_counting_iterator(n));
  }

};


BOOST_AUTO_TEST_CASE(sequence)
{
  MultiValuedTransformRange<RangeGen, boost::counting_iterator<int> > transform(boost::make_counting_iterator(0),
                                                                                boost::make_counting_iterator(6), RangeGen());
  BOOST_CHECK_EQUAL(15, std::distance(transform.begin(), transform.end()));
}


BOOST_AUTO_TEST_CASE(bind)
{
  auto transform = makeMultiValuedTransformRange(boost::make_counting_iterator(0),
                                                 boost::make_counting_iterator(6),
                                                 boost::bind(RangeGen(), _1));
  BOOST_CHECK_EQUAL(15, std::distance(transform.begin(), transform.end()));
}

BOOST_AUTO_TEST_CASE(lambda_bind)
{
  RangeGen gen;
  auto transform = makeMultiValuedTransformRange(boost::make_counting_iterator(0),
                                                 boost::make_counting_iterator(6),
                                                 boost::lambda::bind(boost::lambda::var(gen), boost::lambda::_1));
  BOOST_CHECK_EQUAL(15, std::distance(transform.begin(), transform.end()));
}


BOOST_AUTO_TEST_SUITE_END()
