/////////////////////////////////////////////////////////////////////////////
/// @file UniversalVariableTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2015-04-03
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2015 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <TestTools.h>
#include <capd/homAux/UniversalVariable.h>
#include <capd/auxil/Logger.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>

namespace capd
{
  namespace test
  {

  }
}

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::test;
using namespace capd::homAux;

BOOST_AUTO_TEST_SUITE(UniversalVariableTestSuite)

BOOST_AUTO_TEST_CASE( init )
{
  {
    UniversalVariable<int> uv(1);
    BOOST_CHECK_EQUAL(1, *uv);
  }

  {
    int x = 1;
    UniversalVariable<int> uv(x);
    BOOST_CHECK_EQUAL(1, *uv);
  }

  {
    int x = 1;
    UniversalVariable<int> uv(&x, true);
    BOOST_CHECK_EQUAL(1, *uv);
  }

  {
    UniversalVariable<int> uv(new int(1), false);
    BOOST_CHECK_EQUAL(1, *uv);
  }
}

BOOST_AUTO_TEST_CASE(change)
{
  {
    int x = 1;
    UniversalVariable<int> uv(x, true);
    *uv += 1;
    BOOST_CHECK_EQUAL(2, *uv);
    BOOST_CHECK_EQUAL(2, x);
  }

  {
    int x = 1;
    UniversalVariable<int> uv(x, false);
    *uv += 1;
    BOOST_CHECK_EQUAL(2, *uv);
    BOOST_CHECK_EQUAL(1, x);
  }

  {
    const int x = 1;
    UniversalVariable<int> uv(x);
    *uv += 1;
    BOOST_CHECK_EQUAL(2, *uv);
    BOOST_CHECK_EQUAL(1, x);
  }

  {
    int* x = new int(1);
    UniversalVariable<int> uv(x, false);
    ++(*x);
    *uv += 1;
    BOOST_CHECK_EQUAL(2, *uv);
    BOOST_CHECK_EQUAL(2, *x);
  }

  {
    int* x = new int(1);
    UniversalVariable<int> uv(x, true);
    ++(*x);
    *uv += 1;
    BOOST_CHECK_EQUAL(3, *uv);
    BOOST_CHECK_EQUAL(3, *x);
  }

}


struct X
{
  bool* x;

  X(bool& x): x(&x) {}

  ~X()
  {
    if (x) {
      *x = true;
    }
  }

  bool get() const { return *x; }
};

BOOST_AUTO_TEST_CASE(destructor)
{
  {
    bool destructed = false;
    {
      UniversalVariable<X> uv(new X(destructed), false);
    }
    BOOST_CHECK(destructed);
  }

  {
    bool destructed = false;
    {
      UniversalVariable<X> uv(new X(destructed), true);
    }
    BOOST_CHECK(!destructed);
  }

}

BOOST_AUTO_TEST_CASE(access)
{
  {
    bool destructed = false;
    UniversalVariable<X> uv(new X(destructed), false);

    BOOST_CHECK(!(*uv).get());
  }

  {
    bool destructed = false;
    UniversalVariable<X> uv(new X(destructed), false);

    BOOST_CHECK(!uv->get());
  }

  {
    bool destructed = false;
    const UniversalVariable<X> uv(new X(destructed), false);

    BOOST_CHECK(!uv->get());
    BOOST_CHECK(!(*uv).get());
  }


}

BOOST_AUTO_TEST_SUITE_END()
