/////////////////////////////////////////////////////////////////////////////
/// @file TestTools.h
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2013-12-30
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2013 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#ifndef CAPD_FILE_TESTTOOLS_H
#define CAPD_FILE_TESTTOOLS_H

#include <capd/complex/LoggerHelpers.h>
#include <boost/array.hpp>


#include <set>
#include <iostream>
#include <utility>
#include <vector>
#include <boost/range/iterator_range.hpp>

namespace boost {

  namespace test_tools {

  template<typename T>
  std::ostream& operator<<(std::ostream& s, const std::vector<T>& v);

  template<typename F, typename S>
  std::ostream& operator<<(std::ostream& s, const std::pair<F, S>& p);

  template<typename T, size_t N>
  std::ostream& operator<<(std::ostream& s, const boost::array<T,N>& v);


  template<typename F, typename S>
  std::ostream& operator<<(std::ostream& s, const std::pair<F, S>& p) {
    s << '<' << p.first << ',' << p.second << '>';
    return s;
  }

  template<typename T>
  std::ostream& operator<<(std::ostream& s, const std::vector<T>& v) {
    s << "{";
    for (typename std::vector<T>::const_iterator it = v.begin(), end = v.end();
         it != end; ++it) {
      s << *it << ", ";
    }
    s << "}";

    return s;
  }

  template<typename T, size_t N>
  std::ostream& operator<<(std::ostream& s, const boost::array<T,N>& v)
  {
    s << "{";
    for (typename boost::array<T, N>::const_iterator it = v.begin(), end = v.end();
         it != end; ++it) {
      s << *it << ", ";
    }
    s << "}";

    return s;
  }

  }
}

namespace boost
{
  using namespace test_tools;
}

namespace log4cxx
{
  namespace helpers
  {

    template<typename T, size_t N>
    std::ostream& operator<<(std::ostream& s, const boost::array<T,N>& v)
    {
      s << "{";
      for (typename boost::array<T, N>::const_iterator it = v.begin(), end = v.end();
           it != end; ++it) {
        s << *it << ", ";
      }
      s << "}";

      return s;
    }

    template<typename Iterator>
    std::ostream& operator<<(std::ostream& s, const boost::iterator_range<Iterator>& v)
    {
      s << "{";
      for (auto x : v) {
        s << x << ", ";
      }
      s << "}";

      return s;
    }


  }
}

namespace capd
{
namespace test
{

template<typename Iterator>
std::set<typename Iterator::value_type> Uniq(Iterator b, Iterator e)
{
  return std::set<typename Iterator::value_type>(b, e);
}


}
}

#endif // CAPD_FILE_TESTTOOLS_H
