/////////////////////////////////////////////////////////////////////////////
/// @file BuildFiltrationTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-30
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/persistence/BuildFiltration.h>
#include <capd/auxil/Logger.h>
#include "PointCloudFixtures.h"
#include <capd/persistence/PointCloudTypes.h>
#include <capd/persistence/PointCloudUtils.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <boost/scoped_ptr.hpp>
#include <boost/iterator/counting_iterator.hpp>
#include <boost/typeof/typeof.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <capd/persistence/PointCloudSimplicialCellSComplex.h>



namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::persistence;

BOOST_AUTO_TEST_SUITE(BuildFiltrationTestSuite)


BOOST_AUTO_TEST_CASE_TEMPLATE(all_euclidean, Fixture, capd::persistence::test::PointCloudFixtures)
{
  typedef EuclideanPointCloudType<0>::PointCloudType PointCloudType;
  Fixture fixture;
  const typename Fixture::Points& points = fixture.points();
  boost::scoped_ptr<PointCloudType> pc(makeSizeLimitedPointCloud<EuclideanPointCloudType<0> >(points.begin(), points.end(), 0));

  typedef PointCloudSimplicialCellSComplex PCComplex;
  PCComplex pccomplex(*pc, fixture.maxDim());
  pccomplex();

  typedef capd::persistence::BuildFiltration<PCComplex> BuildFiltration;
  BuildFiltration filtration(pccomplex);
  filtration();

  BOOST_AUTO(weightFunc, bl::bind<double>(&BuildFiltration::weight, bl::var(filtration), bl::_1));
  std::vector<double> weights(b::make_transform_iterator(b::make_counting_iterator((size_t)0), weightFunc),
                              b::make_transform_iterator(b::make_counting_iterator(filtration.size()), weightFunc));

  BOOST_CHECK(std::is_sorted(weights.begin(), weights.end()));


  for (size_t pos = 0; pos < filtration.size(); ++pos) {
    PCComplex::Cell cell = pccomplex[filtration.cellId(pos)];
    BOOST_CHECK_EQUAL(cell.color(), filtration.weight(pos));

    BOOST_FOREACH(size_t bdPos, filtration[pos]) {
      BOOST_CHECK(bdPos < filtration.size());

      PCComplex::Cell bdCell = pccomplex[filtration.cellId(bdPos)];
      BOOST_CHECK_EQUAL(bdCell.color(), filtration.weight(bdPos));

      BOOST_CHECK(bdPos < pos);
      BOOST_CHECK(filtration.weight(bdPos) <= filtration.weight(pos));

    }
  }
}


BOOST_AUTO_TEST_SUITE_END()
