/////////////////////////////////////////////////////////////////////////////
/// @file BuildVietorisRipsTest.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-25
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.

#include <capd/persistence/BuildVietorisRips.h>
#include <capd/persistence/PointCloudTypes.h>
#include <capd/persistence/PointCloudUtils.h>

#include <capd/auxil/Logger.h>
#include <capd/vectalg/Vector.hpp>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <boost/scoped_ptr.hpp>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
//using namespace boost::lambda;

using namespace capd;
using namespace capd::persistence;


BOOST_AUTO_TEST_SUITE(BuildVietorisRipsTestSuite)


BOOST_AUTO_TEST_CASE(traingleAndPoint)
{
  typedef EuclideanPointCloudType<0>::PointCloudType PointCloudType;
  typedef PointCloudType::Point Point;
  typedef PointCloudType::Scalar Scalar;

  std::vector<Point> points = list_of
    (Point((Scalar[]){0, 0}))
    (Point((Scalar[]){10, 10}))
    (Point((Scalar[]){0, 1}))
    (Point((Scalar[]){2, 0}));

  boost::scoped_ptr<PointCloudType> pc(makeSizeLimitedPointCloud<EuclideanPointCloudType<0> >(points.begin(), points.end(), 3));

  {
    std::vector<BuildVietorisRips::Simplex> simplices;
    BOOST_AUTO(push, push_back(simplices));

    BuildVietorisRips build(*pc, 3);
    build(b::bind<void>(b::ref(push), _1));

    BOOST_CHECK_EQUAL(8, simplices.size());
  }
}


BOOST_AUTO_TEST_CASE(ball)
{
  typedef EuclideanPointCloudType<0>::PointCloudType PointCloudType;
  typedef PointCloudType::Point Point;
  typedef PointCloudType::Scalar Scalar;

  std::vector<Point> points = list_of
    (Point((Scalar[]){0, 0, 0}))
    (Point((Scalar[]){0, 0, 100}))
    (Point((Scalar[]){0, 10000, 0}))
    (Point((Scalar[]){1, 0, 0}));


  boost::scoped_ptr<PointCloudType> pc(makeSizeLimitedPointCloud<EuclideanPointCloudType<0> >(points.begin(), points.end(), 0));

  {
    std::vector<BuildVietorisRips::Simplex> simplices;
    BOOST_AUTO(push, push_back(simplices));

    BuildVietorisRips build(*pc, 0);
    build(b::bind<void>(b::ref(push), _1));

    BOOST_CHECK_EQUAL(15, simplices.size());
  }
}


BOOST_AUTO_TEST_CASE(line)
{
  typedef EuclideanPointCloudType<0>::PointCloudType PointCloudType;
  typedef PointCloudType::Point Point;
  typedef PointCloudType::Scalar Scalar;

  std::vector<Point> points = list_of
    (Point((Scalar[]){0, 0}))
    (Point((Scalar[]){0, 1}))
    (Point((Scalar[]){0, 2}))
    (Point((Scalar[]){0, 3}))
    (Point((Scalar[]){0, 4}));

  boost::scoped_ptr<PointCloudType> pc(makeSizeLimitedPointCloud<EuclideanPointCloudType<0> >(points.begin(), points.end(), 0));

  {
    std::vector<BuildVietorisRips::Simplex> simplices;
    BOOST_AUTO(push, push_back(simplices));

    BuildVietorisRips build(*pc, 0);
    build(b::bind<void>(b::ref(push), _1));

    BOOST_CHECK_EQUAL(31, simplices.size());
  }
}


BOOST_AUTO_TEST_SUITE_END()
