/////////////////////////////////////////////////////////////////////////////
/// @file DistanceMatrixPersistentHomologyTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-10-21
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/persistence/DistanceMatrixPersistentHomology.h>
#include <capd/auxil/Logger.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <capd/vectalg/Matrix.hpp>

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::persistence;

BOOST_AUTO_TEST_SUITE(DistanceMatrixPersistentHomologyTestSuite)

BOOST_AUTO_TEST_CASE( init )
{

  double d[3][3] = { {0, 1, 1}, {1, 0, 1}, {1, 1, 0} };
  DistanceMatrixPersistentHomology::DistanceMatrix distance(d);
  DistanceMatrixPersistentHomology persistence(distance, 2, 0);
  persistence();

  BOOST_CHECK_EQUAL(2, persistence.pairs().size());
  BOOST_CHECK_EQUAL(1, persistence.cycles().size());

  BOOST_CHECK_EQUAL(2, persistence.pairs(0).size());
  BOOST_CHECK_EQUAL(1, persistence.cycles(0).size());

  BOOST_CHECK_EQUAL(0, persistence.pairs(1).size());
  BOOST_CHECK_EQUAL(0, persistence.cycles(1).size());
}


BOOST_AUTO_TEST_SUITE_END()
