/////////////////////////////////////////////////////////////////////////////
/// @file ImagePersistentHomologyTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-09-17
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/persistence/ImagePersistentHomology.h>
#include <capd/auxil/Logger.h>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <boost/typeof/typeof.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <capd/complex/ImageCubCellComplex.h>
#include <capd/vectalg/Matrix.hpp>

namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::persistence;

BOOST_AUTO_TEST_SUITE(ImagePersistentHomologyTestSuite)

BOOST_AUTO_TEST_CASE(circle)
{
  if (!ImagePersistentHomology::Enabled()) {
    return;
  }

  typedef capd::complex::ImageCubCellComplex::Image Image;
  typedef capd::complex::ImageCubCellComplex::Complex Complex;

  {
    double data[1][1] = {{1}};
    Image image(data);

    ImagePersistentHomology compute(image);
    compute();


    BOOST_CHECK_EQUAL(0, compute.signature().pairs().size());
    BOOST_CHECK_EQUAL(1, compute.signature().cycles().size());
  }

  {
    double data[3][3] = {{0,0,0}, {0, 1, 0}, {0, 1, 0}};
    Image image(data);

    ImagePersistentHomology compute(image);
    compute();

    BOOST_CHECK_EQUAL(0, compute.signature().pairs().size());
    BOOST_CHECK_EQUAL(0, compute.signature().pairs(0).size());
    BOOST_CHECK_EQUAL(0, compute.signature().pairs(1).size());
    BOOST_CHECK_EQUAL(0, compute.signature().pairs(2).size());
    BOOST_CHECK_EQUAL(1, compute.signature().cycles().size());
    BOOST_CHECK_EQUAL(1, compute.signature().cycles(0).size());
    BOOST_CHECK_EQUAL(0, compute.signature().cycles(1).size());
    BOOST_CHECK_EQUAL(0, compute.signature().cycles(2).size());

    BOOST_CHECK_EQUAL(1, compute.signature().diagram().size());
    BOOST_CHECK_EQUAL(1, compute.signature().diagram(0).size());
    BOOST_CHECK_EQUAL(0, compute.signature().diagram(1).size());
    BOOST_CHECK_EQUAL(0, compute.signature().diagram(2).size());
  }

  {
    double data[3][3] = {{0,0,0}, {0, 1, 0}, {0, 0, 0}};
    Image image(data);

    ImagePersistentHomology compute(image);
    compute();

    BOOST_CHECK_EQUAL(1, compute.signature().pairs().size());
    BOOST_CHECK_EQUAL(0, compute.signature().pairs(0).size());
    BOOST_CHECK_EQUAL(1, compute.signature().pairs(1).size());
    BOOST_CHECK_EQUAL(0, compute.signature().pairs(2).size());
    BOOST_CHECK_EQUAL(1, compute.signature().cycles().size());
    BOOST_CHECK_EQUAL(1, compute.signature().cycles(0).size());
    BOOST_CHECK_EQUAL(0, compute.signature().cycles(1).size());
    BOOST_CHECK_EQUAL(0, compute.signature().cycles(2).size());

    BOOST_CHECK_EQUAL(2, compute.signature().diagram().size());
    BOOST_CHECK_EQUAL(1, compute.signature().diagram(0).size());
    BOOST_CHECK_EQUAL(1, compute.signature().diagram(1).size());
    BOOST_CHECK_EQUAL(0, compute.signature().diagram(2).size());
  }

  {
    double data[3][4] = {{0,0,0, 0}, {0, 1, 1, 0}, {0, 0, 0, 0}};
    //double data[4][3] = {{0,0,0}, {0, 1, 0}, {0, 2, 0}, {0, 0, 0}};
    Image image(data);

    ImagePersistentHomology compute(image);
    compute();

    BOOST_CHECK_EQUAL(1, compute.signature().pairs().size());
    BOOST_CHECK_EQUAL(1, compute.signature().cycles().size());
  }

}


BOOST_AUTO_TEST_SUITE_END()
