/////////////////////////////////////////////////////////////////////////////
/// @file PointCloudFixtures
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-28
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#ifndef CAPD_FILE_CAPDREDHOM_PERSISTENCE_POINTCLOUDFIXTURES_H
#define CAPD_FILE_CAPDREDHOM_PERSISTENCE_POINTCLOUDFIXTURES_H

// #include
#include <vector>
#include <capd/vectalg/Vector.h>
#include <capd/vectalg/Vector.hpp>

#include <boost/mpl/list.hpp>

namespace capd
{
  namespace persistence
  {

    namespace test
    {

      using capd::vectalg::Vector;

      class PointCloudFixture
      {

      public:
        typedef std::vector<capd::vectalg::Vector<double, 0> > Points;
        explicit PointCloudFixture(size_t edges, size_t maxDim): _edges(edges), _maxDim(maxDim) {}

        const Points& points() const
        {
          return _points;
        }

        size_t size() const { return _points.size(); }
        size_t edges() const { return _edges; }
        size_t maxDim() const { return _maxDim; }

      protected:
        std::vector<capd::vectalg::Vector<double, 0> > _points;
        const size_t _edges;
        const size_t _maxDim;
      };

      struct  HouseExample :PointCloudFixture {  HouseExample(); };
      struct  Square :PointCloudFixture {  Square(); };
      struct  EquispacedCirclePoints :PointCloudFixture {  EquispacedCirclePoints(); };
      struct  RandomCirclePoints :PointCloudFixture {  RandomCirclePoints(); };
      struct  OctahedronVertices :PointCloudFixture {  OctahedronVertices(); };
      struct  TetrahedronVertices :PointCloudFixture {  TetrahedronVertices(); };
      struct  AnnulusVertices :PointCloudFixture {  AnnulusVertices(); };
      struct  IcosahedronVertices :PointCloudFixture {  IcosahedronVertices(); };
      struct  RandomSpherePoints :PointCloudFixture {  RandomSpherePoints(); };
      struct  GaussianPoints :PointCloudFixture {  GaussianPoints(); };
      struct  RandomTorusPoints :PointCloudFixture {  RandomTorusPoints(); };
      struct  RandomTrefoilKnotPoints :PointCloudFixture {  RandomTrefoilKnotPoints(); };
      struct  RandomSphereProductPoints :PointCloudFixture {  RandomSphereProductPoints(); };
      struct  RandomFigure8Points :PointCloudFixture {  RandomFigure8Points(); };
      struct  DisjointPatches :PointCloudFixture {  DisjointPatches(); };

      typedef boost::mpl::list<HouseExample, Square, EquispacedCirclePoints, RandomCirclePoints, OctahedronVertices, TetrahedronVertices, AnnulusVertices, IcosahedronVertices, RandomSpherePoints, GaussianPoints, RandomTorusPoints, RandomTrefoilKnotPoints, RandomSphereProductPoints, RandomFigure8Points, DisjointPatches> PointCloudFixtures;

    }

  }
}

#endif // CAPD_FILE_CAPDREDHOM_PERSISTENCE_POINTCLOUDFIXTURES_H
