/////////////////////////////////////////////////////////////////////////////
/// @file PointCloud.cpp
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-25
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2014 by the CAPD::RedHom Group.
//
// This file constitutes a part of the CAPD::RedHom library,
// distributed under the terms of the GNU General Public License.
// Consult  http://redhom.ii.edu.pl/ for details.


#include <capd/persistence/PointCloudTypes.h>
#include <capd/auxil/Logger.h>

#include <capd/vectalg/Vector.hpp>

#include <TestTools.h>
#include "PointCloudFixtures.h"
#include <capd/persistence/PointCloudUtils.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <boost/scoped_ptr.hpp>

namespace bl = boost::lambda;
namespace b = boost;

using namespace boost::assign;
using namespace boost::lambda;

using namespace capd;
using namespace capd::persistence;


BOOST_AUTO_TEST_SUITE(PointCloudTestSuite)


BOOST_AUTO_TEST_CASE(euclidean)
{
  typedef EuclideanPointCloudType<0>::PointCloudType PointCloudType;
  typedef PointCloudType::Point Point;
  typedef PointCloudType::Scalar Scalar;

  std::vector<Point> points = list_of
    (Point((Scalar[]){0, 0}))
    (Point((Scalar[]){10, 10}))
    (Point((Scalar[]){0, 1}))
    (Point((Scalar[]){2, 0}));

  boost::scoped_ptr<PointCloudType> pc(makeSizeLimitedPointCloud<EuclideanPointCloudType<0> >(points.begin(), points.end(), 3));
  std::vector<PointCloudType::Id> neighb;

  BOOST_CHECK_EQUAL(4, pc->size());
  BOOST_CHECK_EQUAL(3, pc->edges());

  pc->neighbours(0, neighb);
  BOOST_CHECK_EQUAL(2, neighb.size());

  pc->neighbours(1, neighb);
  BOOST_CHECK_EQUAL(0, neighb.size());

  BOOST_CHECK_EQUAL(2, pc->neighbours(2).size());
  BOOST_CHECK_EQUAL(2, pc->neighbours(3).size());
}


BOOST_AUTO_TEST_CASE_TEMPLATE(all_euclidean, Fixture, capd::persistence::test::PointCloudFixtures)
{
  typedef EuclideanPointCloudType<0>::PointCloudType PointCloudType;
  Fixture fixture;
  const typename Fixture::Points& points = fixture.points();

  boost::scoped_ptr<PointCloudType> pc(makeSizeLimitedPointCloud<EuclideanPointCloudType<0> >(points.begin(), points.end(), 0));
  std::vector<PointCloudType::Id> neighb;

  BOOST_CHECK_EQUAL(fixture.size(), pc->size());
  BOOST_CHECK_EQUAL(fixture.edges(), pc->edges());
}




BOOST_AUTO_TEST_SUITE_END()
