/////////////////////////////////////////////////////////////////////////////
/// @file PointCloudUtilsTest
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-07-01
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////


#include <capd/persistence/PointCloudUtils.h>
#include <capd/auxil/Logger.h>
#include <capd/vectalg/Vector.hpp>

#include <TestTools.h>

#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <boost/test/test_case_template.hpp>
#include <boost/mpl/list.hpp>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/assign.hpp>
#include <boost/foreach.hpp>

#include <map>
#include <vector>
#include <algorithm>
#include <utility>
#include <sstream>
#include <capd/persistence/PointCloudTypes.h>
#include <sstream>
#include <boost/scoped_ptr.hpp>


namespace bl = boost::lambda;
namespace b = boost;
namespace ba = boost::assign;

using namespace capd;
using namespace capd::persistence;

BOOST_AUTO_TEST_SUITE(PointCloudUtilsTestSuite)

BOOST_AUTO_TEST_CASE(readDynamicDim)
{
  typedef EuclideanPointCloudType<0> Traits;
  typedef Traits::PointCloudType PointCloudType;
  typedef PointCloudInputStreamIterator<Traits>::type IStreamIterator;
  std::stringstream stream("0.0 .1 .2\n1 2 3");

  boost::scoped_ptr<PointCloudType> pc(makeSizeLimitedPointCloud<Traits>(IStreamIterator(stream), IStreamIterator(), 1, 0));

  BOOST_CHECK_EQUAL(2, pc->size());
  BOOST_CHECK_EQUAL(1, pc->edges());
}

BOOST_AUTO_TEST_CASE(readFixedDim)
{
  typedef EuclideanPointCloudType<3> Traits;
  typedef Traits::PointCloudType PointCloudType;
  typedef PointCloudInputStreamIterator<Traits>::type IStreamIterator;
  std::stringstream stream("0.0 .1 .2\n1 2 3");

  boost::scoped_ptr<PointCloudType> pc(makeSizeLimitedPointCloud<Traits>(IStreamIterator(stream), IStreamIterator(), 1, 0));

  BOOST_CHECK_EQUAL(2, pc->size());
  BOOST_CHECK_EQUAL(1, pc->edges());
}


BOOST_AUTO_TEST_SUITE_END()
