#ifndef DANIEL_WILCZAK_HSET_H_
#define DANIEL_WILCZAK_HSET_H_

#include <vector>
#include "capd/capdlib.h"
#include "RosslerPoincareMap.h"

struct HSet{
  HSet(){}
  HSet(capd::DVector _x, capd::DMatrix _C, capd::DVector _r, int N=8) 
    : _x(_x), _C(_C), _r(_r), 
      _ix(4), _iC(4,4), _ir(4),
      N(N) 
    {
      capd::IVector r(4);
      for(unsigned i=0;i<3;++i){
        _ix[i] = _x[i];
        _ir[i] = capd::interval(-_r[i],_r[i]);
        r[i] = capd::interval(0.,_r[i]);
        for(unsigned j=0;j<3;++j)
          _iC[i][j] = _C[i][j];
          
      }
    }

  capd::DVector _x;
  capd::DMatrix _C;
  capd::DVector _r;
  capd::IVector _ix;
  capd::IMatrix _iC;
  capd::IVector _ir;
  int N;
  
  capd::IVector r() { return _ir; }
  capd::IMatrix C(RosslerPoincareMap& PM) { 
    _iC.column(3) = PM.iPoincareMap[0]->getVectorField()(this->x());
    _iC.column(3).normalize();
    return _iC; 
  }
  capd::IVector x() { return _ix; }
};

typedef std::vector<HSet*> HSetChain;

#endif
