/**
 * This file contains clock functions used to measure and report 
 * CPU time of computation. 
*/
#include <chrono>
#include <iostream>

#ifndef DANIEL_WILCZAK_TIC_TAC_H
#define DANIEL_WILCZAK_TIC_TAC_H

/**
 * This function is used to initialize clock and print a message
*/ 
std::chrono::time_point<std::chrono::system_clock> tic(const char message[]){
  auto start = std::chrono::system_clock::now();
  std::time_t t = std::chrono::system_clock::to_time_t(start);
  std::cout << "Start " << message << " at: " << std::ctime(&t) << std::endl;
  return start;
}

/**
 * This function is used to stop clock and print final information.
*/ 
void tac(std::chrono::time_point<std::chrono::system_clock>& start){
  auto end = std::chrono::system_clock::now();
  std::chrono::duration<double> elapsed_seconds = end-start;  
  auto t = std::chrono::system_clock::to_time_t(end);
  std::cout << "finished computation at " << std::ctime(&t)
            << "elapsed time: " << elapsed_seconds.count()/60. << " minutes\n";    
}

#endif
