/////////////////////////////////////////////////////////////////////////////
/// @file PointCloudFixtures
///
/// @author Mateusz Juda <mateusz.juda@{ii.uj.edu.pl,gmail.com}>
///
/// @date 2014-06-28
/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000-2014 by the CAPD Group.
//
// This file constitutes a part of the CAPD library (capdRedHom),
// distributed under the terms of the GNU General Public License.
// Consult http://capd.ii.uj.edu.pl and  http://redhom.ii.edu.pl/ for details.
/////////////////////////////////////////////////////////////////////////////

#include "PointCloudFixtures.h"
#include <boost/assign.hpp>

namespace capd
{
  namespace persistence
  {

    namespace test
    {
      HouseExample::HouseExample():
        PointCloudFixture(10, 3)
      {
        // HouseExample
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ -1.0, 0.0 }))
          (Vector<double, 0>((double[]){ 1.0, 0.0 }))
          (Vector<double, 0>((double[]){ 1.0, 2.0 }))
          (Vector<double, 0>((double[]){ -1.0, 2.0 }))
          (Vector<double, 0>((double[]){ 0.0, 3.0 })).operator Points();
      }

      Square::Square():
        PointCloudFixture(6, 3)
      {
        // Square
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ 0.0, 0.0 }))
          (Vector<double, 0>((double[]){ 0.0, 1.0 }))
          (Vector<double, 0>((double[]){ 1.0, 0.0 }))
          (Vector<double, 0>((double[]){ 1.0, 1.0 })).operator Points();
      }

      EquispacedCirclePoints::EquispacedCirclePoints():
        PointCloudFixture(4950, 1)
      {
        // EquispacedCirclePoints
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ 1.0, 0.0 }))
          (Vector<double, 0>((double[]){ 0.998026728428, 0.0627905195293 }))
          (Vector<double, 0>((double[]){ 0.992114701314, 0.125333233564 }))
          (Vector<double, 0>((double[]){ 0.982287250729, 0.187381314586 }))
          (Vector<double, 0>((double[]){ 0.968583161129, 0.248689887165 }))
          (Vector<double, 0>((double[]){ 0.951056516295, 0.309016994375 }))
          (Vector<double, 0>((double[]){ 0.929776485888, 0.368124552685 }))
          (Vector<double, 0>((double[]){ 0.904827052466, 0.425779291565 }))
          (Vector<double, 0>((double[]){ 0.876306680044, 0.481753674102 }))
          (Vector<double, 0>((double[]){ 0.844327925502, 0.535826794979 }))
          (Vector<double, 0>((double[]){ 0.809016994375, 0.587785252292 }))
          (Vector<double, 0>((double[]){ 0.770513242776, 0.637423989749 }))
          (Vector<double, 0>((double[]){ 0.728968627421, 0.684547105929 }))
          (Vector<double, 0>((double[]){ 0.684547105929, 0.728968627421 }))
          (Vector<double, 0>((double[]){ 0.637423989749, 0.770513242776 }))
          (Vector<double, 0>((double[]){ 0.587785252292, 0.809016994375 }))
          (Vector<double, 0>((double[]){ 0.535826794979, 0.844327925502 }))
          (Vector<double, 0>((double[]){ 0.481753674102, 0.876306680044 }))
          (Vector<double, 0>((double[]){ 0.425779291565, 0.904827052466 }))
          (Vector<double, 0>((double[]){ 0.368124552685, 0.929776485888 }))
          (Vector<double, 0>((double[]){ 0.309016994375, 0.951056516295 }))
          (Vector<double, 0>((double[]){ 0.248689887165, 0.968583161129 }))
          (Vector<double, 0>((double[]){ 0.187381314586, 0.982287250729 }))
          (Vector<double, 0>((double[]){ 0.125333233564, 0.992114701314 }))
          (Vector<double, 0>((double[]){ 0.0627905195293, 0.998026728428 }))
          (Vector<double, 0>((double[]){ 6.12323399574e-17, 1.0 }))
          (Vector<double, 0>((double[]){ -0.0627905195293, 0.998026728428 }))
          (Vector<double, 0>((double[]){ -0.125333233564, 0.992114701314 }))
          (Vector<double, 0>((double[]){ -0.187381314586, 0.982287250729 }))
          (Vector<double, 0>((double[]){ -0.248689887165, 0.968583161129 }))
          (Vector<double, 0>((double[]){ -0.309016994375, 0.951056516295 }))
          (Vector<double, 0>((double[]){ -0.368124552685, 0.929776485888 }))
          (Vector<double, 0>((double[]){ -0.425779291565, 0.904827052466 }))
          (Vector<double, 0>((double[]){ -0.481753674102, 0.876306680044 }))
          (Vector<double, 0>((double[]){ -0.535826794979, 0.844327925502 }))
          (Vector<double, 0>((double[]){ -0.587785252292, 0.809016994375 }))
          (Vector<double, 0>((double[]){ -0.637423989749, 0.770513242776 }))
          (Vector<double, 0>((double[]){ -0.684547105929, 0.728968627421 }))
          (Vector<double, 0>((double[]){ -0.728968627421, 0.684547105929 }))
          (Vector<double, 0>((double[]){ -0.770513242776, 0.637423989749 }))
          (Vector<double, 0>((double[]){ -0.809016994375, 0.587785252292 }))
          (Vector<double, 0>((double[]){ -0.844327925502, 0.535826794979 }))
          (Vector<double, 0>((double[]){ -0.876306680044, 0.481753674102 }))
          (Vector<double, 0>((double[]){ -0.904827052466, 0.425779291565 }))
          (Vector<double, 0>((double[]){ -0.929776485888, 0.368124552685 }))
          (Vector<double, 0>((double[]){ -0.951056516295, 0.309016994375 }))
          (Vector<double, 0>((double[]){ -0.968583161129, 0.248689887165 }))
          (Vector<double, 0>((double[]){ -0.982287250729, 0.187381314586 }))
          (Vector<double, 0>((double[]){ -0.992114701314, 0.125333233564 }))
          (Vector<double, 0>((double[]){ -0.998026728428, 0.0627905195293 }))
          (Vector<double, 0>((double[]){ -1.0, 1.22464679915e-16 }))
          (Vector<double, 0>((double[]){ -0.998026728428, -0.0627905195293 }))
          (Vector<double, 0>((double[]){ -0.992114701314, -0.125333233564 }))
          (Vector<double, 0>((double[]){ -0.982287250729, -0.187381314586 }))
          (Vector<double, 0>((double[]){ -0.968583161129, -0.248689887165 }))
          (Vector<double, 0>((double[]){ -0.951056516295, -0.309016994375 }))
          (Vector<double, 0>((double[]){ -0.929776485888, -0.368124552685 }))
          (Vector<double, 0>((double[]){ -0.904827052466, -0.425779291565 }))
          (Vector<double, 0>((double[]){ -0.876306680044, -0.481753674102 }))
          (Vector<double, 0>((double[]){ -0.844327925502, -0.535826794979 }))
          (Vector<double, 0>((double[]){ -0.809016994375, -0.587785252292 }))
          (Vector<double, 0>((double[]){ -0.770513242776, -0.637423989749 }))
          (Vector<double, 0>((double[]){ -0.728968627421, -0.684547105929 }))
          (Vector<double, 0>((double[]){ -0.684547105929, -0.728968627421 }))
          (Vector<double, 0>((double[]){ -0.637423989749, -0.770513242776 }))
          (Vector<double, 0>((double[]){ -0.587785252292, -0.809016994375 }))
          (Vector<double, 0>((double[]){ -0.535826794979, -0.844327925502 }))
          (Vector<double, 0>((double[]){ -0.481753674102, -0.876306680044 }))
          (Vector<double, 0>((double[]){ -0.425779291565, -0.904827052466 }))
          (Vector<double, 0>((double[]){ -0.368124552685, -0.929776485888 }))
          (Vector<double, 0>((double[]){ -0.309016994375, -0.951056516295 }))
          (Vector<double, 0>((double[]){ -0.248689887165, -0.968583161129 }))
          (Vector<double, 0>((double[]){ -0.187381314586, -0.982287250729 }))
          (Vector<double, 0>((double[]){ -0.125333233564, -0.992114701314 }))
          (Vector<double, 0>((double[]){ -0.0627905195293, -0.998026728428 }))
          (Vector<double, 0>((double[]){ -1.83697019872e-16, -1.0 }))
          (Vector<double, 0>((double[]){ 0.0627905195293, -0.998026728428 }))
          (Vector<double, 0>((double[]){ 0.125333233564, -0.992114701314 }))
          (Vector<double, 0>((double[]){ 0.187381314586, -0.982287250729 }))
          (Vector<double, 0>((double[]){ 0.248689887165, -0.968583161129 }))
          (Vector<double, 0>((double[]){ 0.309016994375, -0.951056516295 }))
          (Vector<double, 0>((double[]){ 0.368124552685, -0.929776485888 }))
          (Vector<double, 0>((double[]){ 0.425779291565, -0.904827052466 }))
          (Vector<double, 0>((double[]){ 0.481753674102, -0.876306680044 }))
          (Vector<double, 0>((double[]){ 0.535826794979, -0.844327925502 }))
          (Vector<double, 0>((double[]){ 0.587785252292, -0.809016994375 }))
          (Vector<double, 0>((double[]){ 0.637423989749, -0.770513242776 }))
          (Vector<double, 0>((double[]){ 0.684547105929, -0.728968627421 }))
          (Vector<double, 0>((double[]){ 0.728968627421, -0.684547105929 }))
          (Vector<double, 0>((double[]){ 0.770513242776, -0.637423989749 }))
          (Vector<double, 0>((double[]){ 0.809016994375, -0.587785252292 }))
          (Vector<double, 0>((double[]){ 0.844327925502, -0.535826794979 }))
          (Vector<double, 0>((double[]){ 0.876306680044, -0.481753674102 }))
          (Vector<double, 0>((double[]){ 0.904827052466, -0.425779291565 }))
          (Vector<double, 0>((double[]){ 0.929776485888, -0.368124552685 }))
          (Vector<double, 0>((double[]){ 0.951056516295, -0.309016994375 }))
          (Vector<double, 0>((double[]){ 0.968583161129, -0.248689887165 }))
          (Vector<double, 0>((double[]){ 0.982287250729, -0.187381314586 }))
          (Vector<double, 0>((double[]){ 0.992114701314, -0.125333233564 }))
          (Vector<double, 0>((double[]){ 0.998026728428, -0.0627905195293 })).operator Points();
      }

      RandomCirclePoints::RandomCirclePoints():
        PointCloudFixture(4950, 1)
      {
        // RandomCirclePoints
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ 0.998916959796, 0.0465285657654 }))
          (Vector<double, 0>((double[]){ 0.993085339654, 0.11739466838 }))
          (Vector<double, 0>((double[]){ 0.981300810871, 0.192480436884 }))
          (Vector<double, 0>((double[]){ 0.957142289757, 0.289618088453 }))
          (Vector<double, 0>((double[]){ 0.925061943232, 0.379816272931 }))
          (Vector<double, 0>((double[]){ 0.921706615274, 0.387887761293 }))
          (Vector<double, 0>((double[]){ 0.897936885717, 0.440124242992 }))
          (Vector<double, 0>((double[]){ 0.875794777528, 0.482683651738 }))
          (Vector<double, 0>((double[]){ 0.874278157881, 0.485425280195 }))
          (Vector<double, 0>((double[]){ 0.815711444165, 0.578459021762 }))
          (Vector<double, 0>((double[]){ 0.795333588315, 0.606171991515 }))
          (Vector<double, 0>((double[]){ 0.755880835386, 0.654709219957 }))
          (Vector<double, 0>((double[]){ 0.647235213605, 0.762290350372 }))
          (Vector<double, 0>((double[]){ 0.64438638411, 0.764700064061 }))
          (Vector<double, 0>((double[]){ 0.544300323409, 0.838890432617 }))
          (Vector<double, 0>((double[]){ 0.522616409412, 0.852567937828 }))
          (Vector<double, 0>((double[]){ 0.417593493932, 0.908633960308 }))
          (Vector<double, 0>((double[]){ 0.349991914695, 0.936752720652 }))
          (Vector<double, 0>((double[]){ 0.253579654188, 0.967314508824 }))
          (Vector<double, 0>((double[]){ 0.214522337227, 0.976719082864 }))
          (Vector<double, 0>((double[]){ 0.16959624187, 0.985513629913 }))
          (Vector<double, 0>((double[]){ 0.0952976148111, 0.995448825712 }))
          (Vector<double, 0>((double[]){ 0.00639870588577, 0.999979528072 }))
          (Vector<double, 0>((double[]){ 0.00204829746473, 0.999997902237 }))
          (Vector<double, 0>((double[]){ 0.00191188062137, 0.999998172355 }))
          (Vector<double, 0>((double[]){ -0.121282522777, 0.99261802808 }))
          (Vector<double, 0>((double[]){ -0.144662406637, 0.98948107011 }))
          (Vector<double, 0>((double[]){ -0.149197201444, 0.988807461077 }))
          (Vector<double, 0>((double[]){ -0.169155410075, 0.985589390792 }))
          (Vector<double, 0>((double[]){ -0.172257836998, 0.985051895888 }))
          (Vector<double, 0>((double[]){ -0.22150435195, 0.975159382905 }))
          (Vector<double, 0>((double[]){ -0.262376602506, 0.96496555299 }))
          (Vector<double, 0>((double[]){ -0.318753465545, 0.947837659203 }))
          (Vector<double, 0>((double[]){ -0.329026530395, 0.944320677681 }))
          (Vector<double, 0>((double[]){ -0.460010301183, 0.887913578455 }))
          (Vector<double, 0>((double[]){ -0.523346554686, 0.852119935043 }))
          (Vector<double, 0>((double[]){ -0.600131619669, 0.799901268329 }))
          (Vector<double, 0>((double[]){ -0.746833108284, 0.665011509954 }))
          (Vector<double, 0>((double[]){ -0.833900743084, 0.551914441452 }))
          (Vector<double, 0>((double[]){ -0.866279517246, 0.499559604051 }))
          (Vector<double, 0>((double[]){ -0.882292272479, 0.470701971447 }))
          (Vector<double, 0>((double[]){ -0.923397953607, 0.383844003828 }))
          (Vector<double, 0>((double[]){ -0.98234148129, 0.187096804188 }))
          (Vector<double, 0>((double[]){ -0.991119776246, 0.132972136684 }))
          (Vector<double, 0>((double[]){ -0.992481229107, 0.122396935704 }))
          (Vector<double, 0>((double[]){ -0.999360247357, 0.035764451652 }))
          (Vector<double, 0>((double[]){ -0.999664198535, 0.0259131273159 }))
          (Vector<double, 0>((double[]){ -0.999997011426, -0.00244481894397 }))
          (Vector<double, 0>((double[]){ -0.996045366009, -0.0888460964366 }))
          (Vector<double, 0>((double[]){ -0.995762207838, -0.0919653491326 }))
          (Vector<double, 0>((double[]){ -0.994459269244, -0.10512260373 }))
          (Vector<double, 0>((double[]){ -0.820194910063, -0.572084180438 }))
          (Vector<double, 0>((double[]){ -0.782521601496, -0.622623436109 }))
          (Vector<double, 0>((double[]){ -0.766834474511, -0.641844910163 }))
          (Vector<double, 0>((double[]){ -0.664398839044, -0.747378205915 }))
          (Vector<double, 0>((double[]){ -0.621853249524, -0.783133791926 }))
          (Vector<double, 0>((double[]){ -0.603328138213, -0.797493045513 }))
          (Vector<double, 0>((double[]){ -0.59081220502, -0.806809109021 }))
          (Vector<double, 0>((double[]){ -0.569646980377, -0.821889480251 }))
          (Vector<double, 0>((double[]){ -0.561717202327, -0.827329308444 }))
          (Vector<double, 0>((double[]){ -0.542137996822, -0.840289469411 }))
          (Vector<double, 0>((double[]){ -0.413778834856, -0.910377435916 }))
          (Vector<double, 0>((double[]){ -0.412270043531, -0.911061694512 }))
          (Vector<double, 0>((double[]){ -0.410802031083, -0.911724569845 }))
          (Vector<double, 0>((double[]){ -0.346454211406, -0.938066884289 }))
          (Vector<double, 0>((double[]){ -0.337178860497, -0.941440606748 }))
          (Vector<double, 0>((double[]){ -0.337031318399, -0.941493436206 }))
          (Vector<double, 0>((double[]){ -0.315047424617, -0.949075929651 }))
          (Vector<double, 0>((double[]){ -0.222481963002, -0.974936806228 }))
          (Vector<double, 0>((double[]){ -0.193609696477, -0.981078633663 }))
          (Vector<double, 0>((double[]){ -0.0962749260269, -0.995354780276 }))
          (Vector<double, 0>((double[]){ -0.0486084263868, -0.998817911776 }))
          (Vector<double, 0>((double[]){ -0.00738832446779, -0.999972705958 }))
          (Vector<double, 0>((double[]){ -0.00373513569874, -0.999993024356 }))
          (Vector<double, 0>((double[]){ 0.00097313148982, -0.999999526507 }))
          (Vector<double, 0>((double[]){ 0.167255735592, -0.985913545353 }))
          (Vector<double, 0>((double[]){ 0.171181834835, -0.985239452835 }))
          (Vector<double, 0>((double[]){ 0.207891308991, -0.978151932803 }))
          (Vector<double, 0>((double[]){ 0.217336099612, -0.976096829113 }))
          (Vector<double, 0>((double[]){ 0.217806942147, -0.975991872893 }))
          (Vector<double, 0>((double[]){ 0.224902592166, -0.974381251891 }))
          (Vector<double, 0>((double[]){ 0.234319187076, -0.972159718651 }))
          (Vector<double, 0>((double[]){ 0.311875526152, -0.950122968982 }))
          (Vector<double, 0>((double[]){ 0.398967620763, -0.916965014372 }))
          (Vector<double, 0>((double[]){ 0.739657983057, -0.672982962712 }))
          (Vector<double, 0>((double[]){ 0.759577573176, -0.650416720517 }))
          (Vector<double, 0>((double[]){ 0.784446711396, -0.620196224577 }))
          (Vector<double, 0>((double[]){ 0.809110628317, -0.587656354636 }))
          (Vector<double, 0>((double[]){ 0.815214792252, -0.579158736871 }))
          (Vector<double, 0>((double[]){ 0.828191437831, -0.560445307147 }))
          (Vector<double, 0>((double[]){ 0.849566140612, -0.527482106546 }))
          (Vector<double, 0>((double[]){ 0.890349709147, -0.455277273123 }))
          (Vector<double, 0>((double[]){ 0.900148028006, -0.435584122389 }))
          (Vector<double, 0>((double[]){ 0.920359382655, -0.391073658994 }))
          (Vector<double, 0>((double[]){ 0.951121754871, -0.308816138521 }))
          (Vector<double, 0>((double[]){ 0.96955323256, -0.244880642829 }))
          (Vector<double, 0>((double[]){ 0.981298041118, -0.192494557059 }))
          (Vector<double, 0>((double[]){ 0.99435196126, -0.106132827808 }))
          (Vector<double, 0>((double[]){ 0.995576501747, -0.0939543994094 }))
          (Vector<double, 0>((double[]){ 0.999959063667, -0.00904825891763 })).operator Points();
      }

      OctahedronVertices::OctahedronVertices():
        PointCloudFixture(15, 3)
      {
        // OctahedronVertices
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ 1.0, 0.0, 0.0 }))
          (Vector<double, 0>((double[]){ 0.0, -1.0, 0.0 }))
          (Vector<double, 0>((double[]){ -1.0, 0.0, 0.0 }))
          (Vector<double, 0>((double[]){ 0.0, 1.0, 0.0 }))
          (Vector<double, 0>((double[]){ 0.0, 0.0, 1.0 }))
          (Vector<double, 0>((double[]){ 0.0, 0.0, -1.0 })).operator Points();
      }

      TetrahedronVertices::TetrahedronVertices():
        PointCloudFixture(6, 3)
      {
        // TetrahedronVertices
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ 0.0, 0.0, 1.0 }))
          (Vector<double, 0>((double[]){ 0.943, 0.0, -0.333 }))
          (Vector<double, 0>((double[]){ -0.471, 0.816, -0.333 }))
          (Vector<double, 0>((double[]){ -0.471, -0.816, -0.333 })).operator Points();
      }

      AnnulusVertices::AnnulusVertices():
        PointCloudFixture(11175, 1)
      {
        // AnnulusVertices
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ 1.0, 0.0 }))
          (Vector<double, 0>((double[]){ 0.992114701314, 0.125333233564 }))
          (Vector<double, 0>((double[]){ 0.968583161129, 0.248689887165 }))
          (Vector<double, 0>((double[]){ 0.929776485888, 0.368124552685 }))
          (Vector<double, 0>((double[]){ 0.876306680044, 0.481753674102 }))
          (Vector<double, 0>((double[]){ 0.809016994375, 0.587785252292 }))
          (Vector<double, 0>((double[]){ 0.728968627421, 0.684547105929 }))
          (Vector<double, 0>((double[]){ 0.637423989749, 0.770513242776 }))
          (Vector<double, 0>((double[]){ 0.535826794979, 0.844327925502 }))
          (Vector<double, 0>((double[]){ 0.425779291565, 0.904827052466 }))
          (Vector<double, 0>((double[]){ 0.309016994375, 0.951056516295 }))
          (Vector<double, 0>((double[]){ 0.187381314586, 0.982287250729 }))
          (Vector<double, 0>((double[]){ 0.0627905195293, 0.998026728428 }))
          (Vector<double, 0>((double[]){ -0.0627905195293, 0.998026728428 }))
          (Vector<double, 0>((double[]){ -0.187381314586, 0.982287250729 }))
          (Vector<double, 0>((double[]){ -0.309016994375, 0.951056516295 }))
          (Vector<double, 0>((double[]){ -0.425779291565, 0.904827052466 }))
          (Vector<double, 0>((double[]){ -0.535826794979, 0.844327925502 }))
          (Vector<double, 0>((double[]){ -0.637423989749, 0.770513242776 }))
          (Vector<double, 0>((double[]){ -0.728968627421, 0.684547105929 }))
          (Vector<double, 0>((double[]){ -0.809016994375, 0.587785252292 }))
          (Vector<double, 0>((double[]){ -0.876306680044, 0.481753674102 }))
          (Vector<double, 0>((double[]){ -0.929776485888, 0.368124552685 }))
          (Vector<double, 0>((double[]){ -0.968583161129, 0.248689887165 }))
          (Vector<double, 0>((double[]){ -0.992114701314, 0.125333233564 }))
          (Vector<double, 0>((double[]){ -1.0, 1.22464679915e-16 }))
          (Vector<double, 0>((double[]){ -0.992114701314, -0.125333233564 }))
          (Vector<double, 0>((double[]){ -0.968583161129, -0.248689887165 }))
          (Vector<double, 0>((double[]){ -0.929776485888, -0.368124552685 }))
          (Vector<double, 0>((double[]){ -0.876306680044, -0.481753674102 }))
          (Vector<double, 0>((double[]){ -0.809016994375, -0.587785252292 }))
          (Vector<double, 0>((double[]){ -0.728968627421, -0.684547105929 }))
          (Vector<double, 0>((double[]){ -0.637423989749, -0.770513242776 }))
          (Vector<double, 0>((double[]){ -0.535826794979, -0.844327925502 }))
          (Vector<double, 0>((double[]){ -0.425779291565, -0.904827052466 }))
          (Vector<double, 0>((double[]){ -0.309016994375, -0.951056516295 }))
          (Vector<double, 0>((double[]){ -0.187381314586, -0.982287250729 }))
          (Vector<double, 0>((double[]){ -0.0627905195293, -0.998026728428 }))
          (Vector<double, 0>((double[]){ 0.0627905195293, -0.998026728428 }))
          (Vector<double, 0>((double[]){ 0.187381314586, -0.982287250729 }))
          (Vector<double, 0>((double[]){ 0.309016994375, -0.951056516295 }))
          (Vector<double, 0>((double[]){ 0.425779291565, -0.904827052466 }))
          (Vector<double, 0>((double[]){ 0.535826794979, -0.844327925502 }))
          (Vector<double, 0>((double[]){ 0.637423989749, -0.770513242776 }))
          (Vector<double, 0>((double[]){ 0.728968627421, -0.684547105929 }))
          (Vector<double, 0>((double[]){ 0.809016994375, -0.587785252292 }))
          (Vector<double, 0>((double[]){ 0.876306680044, -0.481753674102 }))
          (Vector<double, 0>((double[]){ 0.929776485888, -0.368124552685 }))
          (Vector<double, 0>((double[]){ 0.968583161129, -0.248689887165 }))
          (Vector<double, 0>((double[]){ 0.992114701314, -0.125333233564 }))
          (Vector<double, 0>((double[]){ 1.5, 0.0 }))
          (Vector<double, 0>((double[]){ 1.48817205197, 0.187999850346 }))
          (Vector<double, 0>((double[]){ 1.45287474169, 0.373034830747 }))
          (Vector<double, 0>((double[]){ 1.39466472883, 0.552186829027 }))
          (Vector<double, 0>((double[]){ 1.31446002007, 0.722630511153 }))
          (Vector<double, 0>((double[]){ 1.21352549156, 0.881677878439 }))
          (Vector<double, 0>((double[]){ 1.09345294113, 1.02682065889 }))
          (Vector<double, 0>((double[]){ 0.956135984623, 1.15576986416 }))
          (Vector<double, 0>((double[]){ 0.803740192468, 1.26649188825 }))
          (Vector<double, 0>((double[]){ 0.638668937348, 1.3572405787 }))
          (Vector<double, 0>((double[]){ 0.463525491562, 1.42658477444 }))
          (Vector<double, 0>((double[]){ 0.281071971879, 1.47343087609 }))
          (Vector<double, 0>((double[]){ 0.094185779294, 1.49704009264 }))
          (Vector<double, 0>((double[]){ -0.094185779294, 1.49704009264 }))
          (Vector<double, 0>((double[]){ -0.281071971879, 1.47343087609 }))
          (Vector<double, 0>((double[]){ -0.463525491562, 1.42658477444 }))
          (Vector<double, 0>((double[]){ -0.638668937348, 1.3572405787 }))
          (Vector<double, 0>((double[]){ -0.803740192468, 1.26649188825 }))
          (Vector<double, 0>((double[]){ -0.956135984623, 1.15576986416 }))
          (Vector<double, 0>((double[]){ -1.09345294113, 1.02682065889 }))
          (Vector<double, 0>((double[]){ -1.21352549156, 0.881677878439 }))
          (Vector<double, 0>((double[]){ -1.31446002007, 0.722630511153 }))
          (Vector<double, 0>((double[]){ -1.39466472883, 0.552186829027 }))
          (Vector<double, 0>((double[]){ -1.45287474169, 0.373034830747 }))
          (Vector<double, 0>((double[]){ -1.48817205197, 0.187999850346 }))
          (Vector<double, 0>((double[]){ -1.5, 1.83697019872e-16 }))
          (Vector<double, 0>((double[]){ -1.48817205197, -0.187999850346 }))
          (Vector<double, 0>((double[]){ -1.45287474169, -0.373034830747 }))
          (Vector<double, 0>((double[]){ -1.39466472883, -0.552186829027 }))
          (Vector<double, 0>((double[]){ -1.31446002007, -0.722630511153 }))
          (Vector<double, 0>((double[]){ -1.21352549156, -0.881677878439 }))
          (Vector<double, 0>((double[]){ -1.09345294113, -1.02682065889 }))
          (Vector<double, 0>((double[]){ -0.956135984623, -1.15576986416 }))
          (Vector<double, 0>((double[]){ -0.803740192468, -1.26649188825 }))
          (Vector<double, 0>((double[]){ -0.638668937348, -1.3572405787 }))
          (Vector<double, 0>((double[]){ -0.463525491562, -1.42658477444 }))
          (Vector<double, 0>((double[]){ -0.281071971879, -1.47343087609 }))
          (Vector<double, 0>((double[]){ -0.094185779294, -1.49704009264 }))
          (Vector<double, 0>((double[]){ 0.094185779294, -1.49704009264 }))
          (Vector<double, 0>((double[]){ 0.281071971879, -1.47343087609 }))
          (Vector<double, 0>((double[]){ 0.463525491562, -1.42658477444 }))
          (Vector<double, 0>((double[]){ 0.638668937348, -1.3572405787 }))
          (Vector<double, 0>((double[]){ 0.803740192468, -1.26649188825 }))
          (Vector<double, 0>((double[]){ 0.956135984623, -1.15576986416 }))
          (Vector<double, 0>((double[]){ 1.09345294113, -1.02682065889 }))
          (Vector<double, 0>((double[]){ 1.21352549156, -0.881677878439 }))
          (Vector<double, 0>((double[]){ 1.31446002007, -0.722630511153 }))
          (Vector<double, 0>((double[]){ 1.39466472883, -0.552186829027 }))
          (Vector<double, 0>((double[]){ 1.45287474169, -0.373034830747 }))
          (Vector<double, 0>((double[]){ 1.48817205197, -0.187999850346 }))
          (Vector<double, 0>((double[]){ 2.0, 0.0 }))
          (Vector<double, 0>((double[]){ 1.98422940263, 0.250666467129 }))
          (Vector<double, 0>((double[]){ 1.93716632226, 0.49737977433 }))
          (Vector<double, 0>((double[]){ 1.85955297178, 0.736249105369 }))
          (Vector<double, 0>((double[]){ 1.75261336009, 0.963507348203 }))
          (Vector<double, 0>((double[]){ 1.61803398875, 1.17557050458 }))
          (Vector<double, 0>((double[]){ 1.45793725484, 1.36909421186 }))
          (Vector<double, 0>((double[]){ 1.2748479795, 1.54102648555 }))
          (Vector<double, 0>((double[]){ 1.07165358996, 1.688655851 }))
          (Vector<double, 0>((double[]){ 0.85155858313, 1.80965410493 }))
          (Vector<double, 0>((double[]){ 0.61803398875, 1.90211303259 }))
          (Vector<double, 0>((double[]){ 0.374762629171, 1.96457450146 }))
          (Vector<double, 0>((double[]){ 0.125581039059, 1.99605345686 }))
          (Vector<double, 0>((double[]){ -0.125581039059, 1.99605345686 }))
          (Vector<double, 0>((double[]){ -0.374762629171, 1.96457450146 }))
          (Vector<double, 0>((double[]){ -0.61803398875, 1.90211303259 }))
          (Vector<double, 0>((double[]){ -0.85155858313, 1.80965410493 }))
          (Vector<double, 0>((double[]){ -1.07165358996, 1.688655851 }))
          (Vector<double, 0>((double[]){ -1.2748479795, 1.54102648555 }))
          (Vector<double, 0>((double[]){ -1.45793725484, 1.36909421186 }))
          (Vector<double, 0>((double[]){ -1.61803398875, 1.17557050458 }))
          (Vector<double, 0>((double[]){ -1.75261336009, 0.963507348203 }))
          (Vector<double, 0>((double[]){ -1.85955297178, 0.736249105369 }))
          (Vector<double, 0>((double[]){ -1.93716632226, 0.49737977433 }))
          (Vector<double, 0>((double[]){ -1.98422940263, 0.250666467129 }))
          (Vector<double, 0>((double[]){ -2.0, 2.44929359829e-16 }))
          (Vector<double, 0>((double[]){ -1.98422940263, -0.250666467129 }))
          (Vector<double, 0>((double[]){ -1.93716632226, -0.49737977433 }))
          (Vector<double, 0>((double[]){ -1.85955297178, -0.736249105369 }))
          (Vector<double, 0>((double[]){ -1.75261336009, -0.963507348203 }))
          (Vector<double, 0>((double[]){ -1.61803398875, -1.17557050458 }))
          (Vector<double, 0>((double[]){ -1.45793725484, -1.36909421186 }))
          (Vector<double, 0>((double[]){ -1.2748479795, -1.54102648555 }))
          (Vector<double, 0>((double[]){ -1.07165358996, -1.688655851 }))
          (Vector<double, 0>((double[]){ -0.85155858313, -1.80965410493 }))
          (Vector<double, 0>((double[]){ -0.61803398875, -1.90211303259 }))
          (Vector<double, 0>((double[]){ -0.374762629171, -1.96457450146 }))
          (Vector<double, 0>((double[]){ -0.125581039059, -1.99605345686 }))
          (Vector<double, 0>((double[]){ 0.125581039059, -1.99605345686 }))
          (Vector<double, 0>((double[]){ 0.374762629171, -1.96457450146 }))
          (Vector<double, 0>((double[]){ 0.61803398875, -1.90211303259 }))
          (Vector<double, 0>((double[]){ 0.85155858313, -1.80965410493 }))
          (Vector<double, 0>((double[]){ 1.07165358996, -1.688655851 }))
          (Vector<double, 0>((double[]){ 1.2748479795, -1.54102648555 }))
          (Vector<double, 0>((double[]){ 1.45793725484, -1.36909421186 }))
          (Vector<double, 0>((double[]){ 1.61803398875, -1.17557050458 }))
          (Vector<double, 0>((double[]){ 1.75261336009, -0.963507348203 }))
          (Vector<double, 0>((double[]){ 1.85955297178, -0.736249105369 }))
          (Vector<double, 0>((double[]){ 1.93716632226, -0.49737977433 }))
          (Vector<double, 0>((double[]){ 1.98422940263, -0.250666467129 })).operator Points();
      }

      IcosahedronVertices::IcosahedronVertices():
        PointCloudFixture(66, 3)
      {
        // IcosahedronVertices
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ 0.0, 0.0, 1.0 }))
          (Vector<double, 0>((double[]){ 0.894, 0.0, 0.447 }))
          (Vector<double, 0>((double[]){ 0.276, 0.851, 0.447 }))
          (Vector<double, 0>((double[]){ -0.724, 0.526, 0.447 }))
          (Vector<double, 0>((double[]){ -0.724, -0.526, 0.447 }))
          (Vector<double, 0>((double[]){ 0.276, -0.851, 0.447 }))
          (Vector<double, 0>((double[]){ 0.724, 0.526, -0.447 }))
          (Vector<double, 0>((double[]){ -0.276, 0.851, -0.447 }))
          (Vector<double, 0>((double[]){ -0.894, 0.0, -0.447 }))
          (Vector<double, 0>((double[]){ -0.276, -0.851, -0.447 }))
          (Vector<double, 0>((double[]){ 0.724, -0.526, -0.447 }))
          (Vector<double, 0>((double[]){ 0.0, 0.0, -1.0 })).operator Points();
      }

      RandomSpherePoints::RandomSpherePoints():
        PointCloudFixture(4950, 1)
      {
        // RandomSpherePoints
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ -0.0016693339737, 0.128610118291, 0.108214012956, 0.0317447577594, 0.120772799108, 0.0935749041882, 0.224419716307, -0.0484177183642, 0.0476641673827, 0.0462611714312, 0.180945542316, -0.07036586397, -0.0905780740767, -0.0448695120704, 0.0896848694498, -0.0230561263168, -0.0438521211402, 0.0699702173109, 0.14308860665, 0.0314202831138, -0.0101378101347, -0.0776386492064, 0.0279568439957, 0.0834913130425, 0.110027373117, -0.00855843556262, -0.172505263812, -0.0390205242194, 0.0783205247347, -0.0635497104527, 0.0100669031077, -0.0589135880266, -0.167222549786, -0.0224080828529, -0.0177585732935, 0.046282973511, 0.193755385707, -0.0552839792866, -0.0586299538263, 0.100283120162, -0.0105236296543, 0.21003167099, -0.00653816546464, -0.0271949463326, 0.00647106160276, -0.150943639904, 0.205330325434, -0.103938188642, -0.0543597424955, 0.117532883253, 0.127974913391, -0.0114902045514, -0.0551953756966, -0.10771562201, 0.0367599405367, -0.0631319270748, -0.0709774250011, 0.0530197459811, -0.0695144041081, 0.0646419593078, -0.0260194745755, 0.12619955489, 0.111994725258, 0.0626915593366, 0.118833652257, -0.162802669981, -0.0915352975034, -0.0383738687357, -0.00255087059791, 0.0155140237517, -0.0956289223284, 0.106216841776, -0.0143099088678, -0.0114399653444, -0.0789558432905, -0.261005985843, -0.139804788392, 0.0642436712766, -0.0527350823351, -0.153958680209, 0.000489882168619, -0.00999516426666, 0.156375243524, 0.00702863000217, 0.235904919663, 0.0536734356428, -0.0311418026171, 0.132229899987, -0.182761857317, -0.116792473519, 0.0877416110561, -0.0352025930125, 0.107978130868, 0.0327971886649, -0.0908780257047, -0.0883618380794, 0.161503093931, 0.0209089602104, -0.00102102412901, -0.0306554111295, -0.124806608532 }))
          (Vector<double, 0>((double[]){ -0.0262464166689, -0.0491950658448, 0.177827984423, -0.0335562052061, -0.0968393610334, 0.0543757903941, 0.0157346642028, -0.0498762720433, -0.159513306068, 0.0676397405802, -0.0731820020784, -0.0810374340569, 0.0856593560102, -0.129980496112, 0.092526048009, -0.0423174649214, -0.15524445369, -0.20499820836, 0.0520946630857, 0.0552122333856, 0.0734856665141, 0.130238851533, 0.161851437934, -0.135474722167, 0.112374943945, -0.071935446129, -0.00213018482016, 0.0362791292373, -0.0409007184018, -0.0919928404788, 0.133025069785, 0.225493083359, -0.0521431784351, 0.103230225614, 0.00440620898446, 0.187304002042, -0.0722265567761, 0.0568440603415, -0.0297484502441, 0.134852331191, 0.0729664989523, -0.0329722160026, 0.0226703801955, -0.119880930464, -0.00757767165987, 0.00594415215035, 0.0154573605292, -0.0837450806598, -0.159599869374, -0.0897569669438, 0.0796878307592, -0.125135596098, 0.018156287905, 0.0563760095978, -0.081322156502, 0.0211134555511, 0.0642623809252, 0.132340620918, -0.0848434547627, -0.0914447356497, -0.271890232373, 0.0429732377195, -0.0389984227962, -0.130397529259, 0.038394534115, -0.0360846833385, 0.113076623772, -0.027054635571, -0.0233572383468, -0.0632264818224, -0.149832420593, 0.0988310107634, 0.133155791527, 0.00372781031008, -0.196206941279, 0.0268655527345, 0.156252955961, 0.0144364941471, 0.139577086896, 0.0110949589415, 0.0960742342254, 0.0643198320905, -0.0919128979437, -0.0807791340054, -0.0552188161689, 0.0597704625642, -0.149806077228, -0.0591987509818, -0.0183648306417, -0.0344080540545, 0.117867275444, 0.164702470633, -0.0790370768467, -0.00742467322534, -0.106489880466, -0.0237959543857, 0.125472042611, 0.00123473535072, -0.0406265635318, -0.196919598599, -0.0403337279174 }))
          (Vector<double, 0>((double[]){ -0.114497870983, 0.0366619448384, -0.0845553588657, -0.103236344143, 0.156802277301, -0.0553421839085, 0.0460627694604, -0.140422058702, -0.165077406865, 0.0834653692248, 0.0324313449373, -0.0158080094282, -0.0675090182414, -0.0277432625274, 0.0843785112451, 0.118096550995, 0.131778304355, -0.0413767530328, -0.145063423121, -0.172765797393, 0.000730882789443, -0.0123287947911, -0.118719127548, -0.0599660375597, -0.05008711482, 0.0106036206013, -0.0417832705286, 0.111250257248, 0.107462274722, 0.11995399517, -0.0220852372473, -0.0568716567437, -0.0422623107358, -0.098482720783, 0.00656950345934, -0.0656626888991, 0.0706432431111, 0.0495738491242, -0.138945809732, 0.0382913550812, 0.0211812030888, -0.0305792292068, -0.273745969533, 0.0339275703577, -0.114425458028, -0.0435804901619, -0.106747066332, 0.121514009185, 0.114658853896, 0.0213679285656, 0.191182433249, -0.116155799524, 0.0980783324416, 0.121761135042, -0.084152127409, 0.0554463635678, -0.0759849780108, 0.0958906803735, 0.177794600319, 0.114627570839, 0.134290031325, 0.069800498003, -0.0138533460588, 0.0190016219156, -0.0494878300901, 0.00516019871315, 0.140654764354, 0.089895546943, -0.125989116045, 0.00624313595709, 0.023306585799, -0.147423260954, 0.025834174901, 0.0529654452168, -0.0362457485756, -0.167779331454, 0.0985943077839, -0.168974715748, -0.0349989389053, -0.0842702412959, -0.198384819154, 0.0344274389487, -0.0376793796965, 0.192752874399, -0.0118605325534, 0.00400543893671, -0.162649357432, 0.181242712518, -0.0222238913258, -0.13429525729, 0.0645715489015, -0.0956858224163, -0.0315242548979, 0.0700400602341, -0.039822874235, 0.041249385636, -0.0601211028724, -0.158131184289, -0.0125909028062, -0.0159691738067, 0.145931869086 }))
          (Vector<double, 0>((double[]){ -0.0560053739041, 0.0848460276553, 0.00566022085624, -0.148525862056, 0.326439110818, 0.0523680500294, -0.11995022103, -0.00826961940418, -0.000431304770482, -0.0237635684079, 0.0539166965598, 0.0297451262985, 0.000595704087655, 0.0479051343974, -0.0856141395334, 0.175900652574, 0.00959048237389, -0.0446098918435, 0.154641016849, 0.142992161925, -0.12251463158, -0.0288136617216, 0.0674972295779, 0.0225924008233, -0.0135443303914, 0.0020341150497, 0.0442866147297, 0.0376611896118, -0.0750064446011, 0.120839685481, -0.0938186892912, 0.20950402652, -0.0754573769046, 0.0424021247271, 0.0622581983654, 0.0455862734695, 0.0251563077701, -0.0244489061308, 0.153829455168, -0.154317474286, 0.0146020437924, 0.151393229682, -0.0249292848642, -0.00237309318769, 0.127477646274, 0.00989512888899, 0.0717326569467, -0.0652294419401, -0.0312087796237, -0.0202837436951, 0.0812518157123, -0.166924709755, -0.20085801723, -0.0681859891354, 0.00708038391453, 0.112486798884, -0.0785422382042, 0.0511584203498, 0.00754534497194, 0.0353544200162, -0.092044818894, -0.0628517543166, -0.0170757523908, -0.105273736675, 0.0342726930846, -0.0963677442576, 0.00449440033407, -0.0247159263216, -0.0122934021598, -0.0116265050488, -0.248434976989, -0.155394952329, 0.0314095033356, 0.0338565769245, 0.0195188189157, -0.00579437034324, 0.278085064424, 0.0172038553509, 0.0813058126712, 0.204067258912, 0.0818535202813, 0.138440423505, 0.0428210333031, 0.143806662252, 0.0554180634803, -0.111292751387, -0.0155088792493, 0.0538406493253, 0.0263904532137, -0.118494651102, -0.12591960448, -0.0923122617448, 0.0824903481435, 0.0405136828994, 0.0633489657288, -0.029113084989, -0.105118019875, -0.157670695566, -0.0319791054469, 0.0359315400749, -0.18070216747 }))
          (Vector<double, 0>((double[]){ 0.117924940486, -0.0543531212306, -0.0852538798154, -0.195297942363, -0.123858424108, -0.0576949771687, -0.212463548317, -0.137279786912, 0.0484692534681, -0.0936855627558, -0.0564464898401, 0.221431091897, -0.00932878145086, -0.0308519208483, 0.0220267834193, 0.0500019295422, 0.0889080359275, 0.0600935715478, 0.0586894914739, -0.14452992208, 0.00204658314651, -0.0830032402191, 0.0784734361851, 0.00662145404075, -0.0925398114925, -0.141857268841, -0.147347663356, 0.0950536813549, 0.094959274413, -0.033994994548, 0.0293973485612, 0.0808678893399, 0.0646647639164, -0.0203296833518, -0.0826576132442, -0.103615052386, -0.0518913468694, 0.00936770256315, -0.0756218269944, -0.00728234795427, 0.0487564280207, -0.0676634115112, 0.160094926888, -0.0610910434255, -0.0828920161866, 0.0174541382663, 0.0587713119932, -0.264420045139, -0.0811425693303, -0.0177585374239, 0.0366903332027, 0.243098760945, 0.0105700278454, -0.0785522823659, 0.0424621652531, 0.0389438698991, -0.112436403361, 0.00264256973424, 0.259478813635, -0.0379744973091, 0.0824088405589, -0.111010193833, 0.0218917319425, -0.0535952319795, -0.107916388155, 0.230220532725, 0.0216941920732, 0.0570155671236, -0.00141955974268, -0.0558899607668, 0.0690116916273, 0.110385195075, -0.117768871844, -0.0418599534355, -0.00501979720926, -0.056403046043, 0.105749490859, -0.049270572968, -0.0524849926903, 0.0664956298592, -0.0691744632841, 0.0638879781645, 0.0674927303289, 0.0485331683605, -0.0790322106312, -0.0749624635677, 0.110494377671, 0.0122598146742, -0.222979219163, 0.0562602963389, -0.186357270547, -0.0588104283587, 0.054618557197, 0.0798830425915, 0.0132063619164, -0.117077608008, 0.126694483473, 0.182547356697, 0.00411551305676, -0.0394404454099, 0.00351025544468 }))
          (Vector<double, 0>((double[]){ -0.0687046867339, 0.0822966717481, -0.0377407515803, 0.0156197872924, -0.0190343516785, -0.0953244255337, -0.100834833252, 0.0300739901651, 0.0578791407304, -0.233884065587, 0.0173793056541, 0.0656825034196, -0.0560381938679, 0.169393705393, 0.0846189132751, -0.010885465883, 0.108012637904, 0.0173507752828, 0.0835310896123, -0.114001909934, -0.144854272337, -0.187397678714, -0.056400765453, -0.005184424183, -6.91139350511e-05, 0.162841009365, -0.124444852701, 0.098589363226, 0.0434856208026, 0.105694863328, 0.134876404167, -0.0244382332571, -0.0674907927885, 0.0202857187835, 0.0535649785845, -0.0722577147441, -0.111935751668, 0.0377042421004, 0.0236669960932, 0.0169328440819, 0.0997451027488, -0.0136994747329, -0.0337436384174, 0.0224212100977, 0.102948464931, 0.065007964115, -0.0775966859765, 0.0872657402209, -0.0880639246213, -0.00906892579234, -0.0873670150611, 0.0667253067317, -0.0176610627696, -0.0637866169414, -0.1229080668, -0.0201490149705, -0.0927283140802, 0.0184452293319, 0.0177020914862, -0.0432615620709, -0.067245634784, 0.119059173494, -0.0374290785099, -0.0611073390606, 0.00354505970372, -0.0875599649802, 0.198709568062, 0.0592449431026, -0.0599554787564, -0.00717396657154, -0.0700166502228, -0.0491121566717, 0.160582955651, 0.0267315717021, -0.000748548773639, -0.0398107184452, 0.239563376262, -0.223649570956, 0.159478950061, -0.0601489023308, 0.174193930618, 0.00755772554712, -0.000295207331574, 0.106150509078, -0.032243599421, 0.212165414251, 0.0338317383561, -0.123273245964, 0.013714050687, -0.166738520232, -0.0822602239972, 0.0736866077462, -0.197980075175, -0.16479985422, 0.0611128713446, 0.0211044472608, -0.176489270768, -0.172091413332, 0.178486846928, 0.0557834293205, -0.100716759929 }))
          (Vector<double, 0>((double[]){ 0.0287734727944, 0.0354983710876, -0.0805604236793, -0.0759381276504, -0.054228302828, -0.0949708577527, -0.0106481725512, 0.0206358914892, -0.116099843847, -0.19517465089, -0.038122415441, -0.0657518486574, 0.0446694728115, -0.0431979398927, -0.0448326190718, -0.0284123201811, 0.00338195832297, -0.0576727116392, -0.113783613522, -0.0438972583727, -0.240519330594, 0.000294672533917, 0.10280231962, 0.0342233432829, -0.0444517355201, 0.103601467114, 0.0375126038941, -0.0395148575078, -0.05696046321, 0.156132763847, -0.0376284346102, -0.0184926046066, -0.0146186685221, 0.0915802677456, 0.000347125940728, -0.0634079248056, 0.20956819274, -0.0461434642053, 0.217892219371, 0.0619327763614, -0.0389602955227, 0.0335692027454, 0.0428203896465, -0.0720969532304, 0.0892536637197, 0.0130065451214, -0.0141411398034, -0.00408836131163, 0.0497375303425, 0.133622341211, 0.00697745684407, -0.0704809342996, 0.0239107180549, 0.168455348256, -0.143870558931, 0.0307792582027, 0.111728307661, -0.200364963681, -0.0250507045538, 0.273339884127, -0.0925913215198, -0.0877726238943, 0.0471184512938, -0.0485173377696, -0.0242822981764, -0.139818213761, -0.0673538600659, 0.00371940771769, -0.0369294132975, 0.0291020753141, -0.0721064117053, 0.119443864486, -0.0111039304262, -0.148943010787, -0.127124190004, -0.282576337773, 0.0536075115442, -0.0208024678182, 0.171974217775, -0.102855716342, -0.00236512573628, -0.0799555989194, 0.136061596887, -0.135178553652, -0.00940403914082, -0.0194560555036, 0.213769089077, 0.0555051165691, 0.0939654206954, -0.0225268734272, -0.0789594115114, -0.0615211919108, 0.0891023287486, 0.0441885611959, -0.0181491229395, 0.158665851786, -0.00688832153706, 0.242655740039, -0.0545895377751, -0.0781937679662, -0.0121309050467 }))
          (Vector<double, 0>((double[]){ 0.00678450791092, 0.00241496765495, -0.0299773167665, 0.00841477927241, -0.0638272402293, 0.0400805823351, 0.0242302684874, -0.136419705834, 0.188683353041, -0.159973324525, 0.018386591623, -0.0227838641059, -0.0529833941887, -0.0131818473994, 0.0583293359758, 0.0250237476994, 0.00924320448793, 0.04831906468, -0.163376142745, 0.0601168646389, 0.00902560368186, -0.099052058535, -0.128749701702, -0.0196043163561, -0.054827951775, -0.234921505108, -0.225935727024, 0.0103983261099, -0.100321505794, 0.0707944975097, -0.114366348088, -0.0635296530692, 0.126390330815, -0.196719799614, -0.0492359349328, 0.0959625699455, -0.127020387452, -0.000107737952, 0.0910314235285, -0.0792282950875, -0.154236538788, 0.0928524556384, -0.0723257198756, -0.111868036482, -0.010318806377, -0.103814680542, 0.0188869692416, -0.0170021882919, 0.113846696975, 0.130997738364, 0.00848790614336, -0.0874131544704, 0.00246985120423, 0.0812331061715, -0.0689899683809, -0.0430915837932, -0.107397033827, 0.0375123294326, -0.054541926233, -0.0445939909692, 0.100155062846, 0.0361275459131, -0.213659047706, 0.118578141726, 0.0112580279667, 0.00696701298598, -0.0691387510401, -0.0393104060184, 0.0778649436342, 0.0128510425535, 0.204050883915, 0.0251587929467, 0.0334309642854, 0.0912378897674, -0.0529001964325, -0.235686682683, -0.147953259152, 0.0488525523151, 0.0963268876703, -0.0487865686035, 0.165972060172, 0.0796604483627, -0.112861263528, 0.0462659295919, -0.0539334426853, 0.0606177059899, -0.0939176398111, 0.151127664818, -0.205040205705, -0.122827280778, -0.161998980811, -0.0888939581517, 0.0562981001601, 0.0265279173047, 0.0954230420437, -0.0750767572052, 0.0683158888245, 0.134994983403, -0.15218734639, 0.0275999674517, 0.0527542945187 }))
          (Vector<double, 0>((double[]){ 0.0909837883993, 0.024923405035, 0.00503592634874, -0.0303692299468, -0.103354429236, -0.14526061543, 0.0183568816119, -0.131043000058, 0.00293877472412, -0.0506858170299, 0.11882763357, 0.133894029677, -0.0830485820527, 0.0746616152287, -0.00909134978764, -0.0022083432064, -0.26734684953, -0.00155800853829, -0.0421517844353, -0.0448303553668, -0.150732941065, 0.0180978114296, -0.046611083895, -0.14842469873, 0.0750652506859, -0.12386502489, 0.0964882772471, 0.0853305955974, -0.141336499224, -0.0865688770909, -0.0708028566121, -0.091032402514, 0.00565440766706, 0.0812779817568, -0.0508376514776, -0.0145367317521, -0.0214760673879, -0.0767992916748, -0.0274604812151, 0.0487426159045, -0.176353931987, -0.0053043365597, 0.157149060171, -0.0845339597317, -0.13632717545, -0.111761037811, -0.0286627502157, 0.150675941398, 0.0227165146074, 0.169511624832, 0.0365826407715, 0.0368254380158, 0.066540799662, 0.130165529755, 0.035414739434, -0.00372525200537, -0.0256927463689, 0.0585125871969, -0.100774135657, 0.258879428857, 0.098224821545, -0.0861923821262, 0.180172520854, -0.0274195603164, 0.0921378565182, 0.0113636617347, -0.107470442543, -0.0706073159789, 0.00784665836081, -0.0319079550962, 0.0925076540739, -0.167568592512, -0.142019470699, -0.0686110397815, 0.035597692972, -0.179253617881, -0.0722391254393, -0.105115197339, -0.00972705913254, 0.0324574158232, 0.159365806378, 0.135687937698, -0.0657885917153, -0.14226438573, -0.100731632534, 0.0222948825261, -0.056753602613, -0.0576040208859, 0.013282422712, 0.149137531062, 0.240653041292, 0.0840530401651, -0.0202698203116, -0.134328988775, -0.104237323111, -0.104256373438, 0.0302770104326, 0.0156549677412, 0.0296689556981, 0.0339700758122, -0.123665376399 }))
          (Vector<double, 0>((double[]){ 0.087322905094, -0.071823932073, 0.196699906756, 0.0192301467301, -0.129095500849, -0.000964507713202, 0.0627933714713, -0.0665259503381, 0.00868580322714, 0.100480443149, 0.0284642770758, -0.102162865491, -0.102461234796, 0.0865002030399, 0.0930994764539, -0.0412617881989, -0.00552047161767, -0.0622274573189, 0.0205358755816, 0.0896785089759, -0.0156003128365, 0.0423113025514, -0.0611579900001, -0.114260492181, -0.0233680857405, 0.0525411522744, -0.0410310285119, 0.049813883486, 0.0699659975875, -0.0194122192225, -0.107421028905, 0.179466314444, -0.0320140195533, -0.134743102036, -0.000158715519024, -0.0495239238236, 0.0174204051673, 0.0515394940791, 0.159951330946, 0.0302413344076, -0.00295698023797, -0.0581459033261, 0.00959112446719, -0.0682421910528, -0.137759474022, -0.0449258704867, -0.0642879914025, -0.0132353963617, 0.109866895283, 0.276710969401, 0.128130880085, 0.121407713929, -0.0554258911047, -0.0224971048623, 0.0909235633693, -0.0033647758955, -0.0439070587278, 0.253786938961, -0.00338858606236, 0.0241956104292, -0.103748999535, 0.2307684726, -0.100040797929, -0.121831021214, -0.0457729977489, -0.0165153128537, -0.0971490977866, 0.0751234205601, 0.00400020737735, 0.280959885511, 0.136796233114, 0.0761013127527, 0.0855604364517, 0.19450366656, 0.0248785460323, -0.0427118826513, -0.208840926267, 0.117858594079, 0.010759519202, -0.0940069679176, 0.111714334969, -0.0765682121506, -0.0663869574936, 0.0307646846496, -0.0631218696149, 0.279217813132, 0.0060000403609, -0.0181823076551, 0.0731161830786, 0.0918177913429, -0.0437251155831, 0.0694686958091, 0.0997039007192, -0.100107355253, -0.0171770843067, -0.0310602999213, 0.0530384654721, -0.0225848492955, 0.132447663836, -0.100387714087, 0.00460936504736 }))
          (Vector<double, 0>((double[]){ 0.0138283135156, 0.0757101100564, -0.117475327796, 0.0374845656932, -0.080547110179, -0.111163737313, 0.0856623132471, 0.0191109111987, -0.0328451574144, 0.118188718325, 0.0481576324781, 0.0198609182244, 0.0316568097419, 0.169378447656, 0.0130297210992, 0.133650509713, 0.127934762961, -0.0944691632817, 0.0273754693106, 0.0561128590248, -0.0767809583931, -0.128617569486, -0.0315573678378, 0.209207127357, -0.0190222497051, -0.0492752485081, 0.187829357358, 0.0463622898895, 0.0323774639129, -0.0676746365053, -0.146952823019, -0.0680567003803, -0.0781392209178, -0.136285069374, 0.0615622165217, -0.112308709017, -0.0627315864472, -0.118783840538, 0.0075872972938, -0.129091211605, -0.0589203204787, -0.103859586573, 0.215348503144, -0.158474135358, 0.0379232457248, 0.0251411893796, 0.0481809612237, -0.0898079056166, 0.0364828540371, 0.0619573896266, -0.102075580693, 0.133996667838, -0.0421593632715, -0.0259712729627, 0.0913325856493, -0.0201349609292, -0.0800670748571, 0.051378384344, -0.112010383695, 0.0686049011266, 0.091615455668, -0.156797499894, 0.0645683885921, 0.0255842620917, -0.0157984116185, 0.0169075068526, -0.0419125236221, -0.116392297116, -0.185003165881, 0.0737936975378, 0.116606091174, 0.0377032962322, -0.0851845811712, -0.090661743527, -0.0511067633821, -0.0430525918468, 0.0474150523448, -0.210717727224, 0.0552433783196, -0.0266193810443, -0.162761396683, -0.00431420790185, 0.192046130183, -0.0419482176834, -0.268713882174, -0.00486574313972, -0.00646266727813, -0.0864839455142, -0.150146647512, 0.0567789953459, -0.0882340821698, -0.173970745842, 0.0595339324467, -0.0968856123597, -0.0528020603624, -0.189554167796, 0.0799984154374, -0.111034598334, -0.0283881049833, -0.0193599302802, -0.119612069831 }))
          (Vector<double, 0>((double[]){ -0.114532515859, 0.0298491114561, 0.137885801037, -0.0437946287805, 0.126792296859, -0.119220656446, 0.152920263344, -0.0575726630722, -0.132754035342, -0.00412269602305, -0.125073561962, -0.00982205244552, 0.108894478008, -0.0364951496289, 0.00413220696233, -0.231396828982, -0.0128035268082, -0.00604946357957, 0.196386796452, 0.280859018636, 0.0549705001469, -0.103921843599, -0.133375225391, 0.117732801083, 0.0994755476404, 0.0703487087775, -0.107180627294, -0.130898995667, 0.0058242893278, -0.147427436084, 0.150752469479, -0.111367372586, 0.0287546504188, -0.0308781239719, -0.196668612023, -0.141358230435, -0.0140197529646, -0.0944405827534, -0.0776239723906, 0.148198025333, -0.0317492876569, 0.0286738914481, 0.135240544631, 0.0231112432914, 0.0754653618546, -0.0635330317132, -0.049088144136, -0.239749682022, -0.174259542453, -0.0557042650674, -0.118384242523, 0.128426008923, -0.0325755623379, -0.0121324513102, -0.0143001435617, 0.0851810438941, -0.138539152546, -0.0230985108337, 0.152351853207, 0.0509017681922, 0.119623250902, -0.013641495428, 0.0615909910647, -0.0157657211691, 0.0166320955588, 0.136214769393, 0.0177718182505, 0.0199123964289, -0.043066877462, 0.0885701725984, -0.0643423842274, 0.031833321258, 0.13599866511, 0.105026864446, -0.118169211911, 0.0868464528961, -0.105090722388, -0.0432667136852, 0.0226886610339, 0.0500641571066, 0.0985225603265, 0.00462391431538, -0.0650027835886, -0.0859047176938, 0.15520838163, -0.0404111866936, -0.0472302141558, -0.0315274276183, 0.00116313958955, -0.0052165351531, -0.0427985935415, -0.0934954772892, -0.010560661475, 0.00933214907795, 0.0953726045383, 0.0109499642341, -0.117987185408, 0.0984325699503, -0.112385116297, -0.109313079299, 0.00830621936715 }))
          (Vector<double, 0>((double[]){ -0.0521766074991, -0.054786477608, -0.00520908314166, 0.0265683358313, -0.00719096000226, 0.0332643347776, 0.209335265177, -0.0704425484552, -0.0412787650334, 0.0284979881991, -0.0348914391763, -0.0593762220682, 0.130474948175, -0.0589872953704, 0.090615356949, -0.0917509499566, -0.0210279277036, 0.218067804824, 0.0970497632508, -0.117672603883, -0.116409703686, 0.00656059290021, -0.162126724932, -0.0433839228594, -0.0824433057509, -0.0816433687186, 0.0413396499463, -0.0385333381001, 0.076724009991, 0.0534272960224, 0.107210676085, -0.017092136652, 0.00996345856559, -0.159595396462, 0.0769062186917, 0.104850498632, -0.0526923872964, 0.0563995846603, -0.0341807070396, 0.0184739883518, 0.104428764493, -0.0436132855954, 0.0137667673145, 0.119516990649, 0.243938303025, 0.0631800548296, 0.0922401349508, -0.0238101414536, 0.0118587569045, -0.0622356906329, 0.157734253201, 0.0845636188029, 0.0408436932852, 0.0718838528895, 0.0385573823329, 0.115598929775, -0.0890644974132, 0.0747549130421, 0.0149243175028, -0.0137211514694, 0.0788203645518, 0.285678108745, 0.0469573430568, -0.124783577523, -0.100146385187, -0.126218080078, -0.120968870903, 0.0323210247785, 0.125147562268, 0.257237271478, -0.219805582866, -0.0713008690282, 0.0109307227503, 0.0969740780516, 0.156328363599, 0.0377036146761, 0.107464903406, 0.0168666346897, -0.0220709171753, -0.0328177477635, -0.0235424744448, -0.0511709209852, -0.0425120992708, 0.102598617015, 0.0893849029222, 0.0942839172608, 0.030988289905, 0.00943674512382, 0.0200263215024, 0.200027750155, -0.0922279370243, 0.0470092658849, 0.0554582787992, 0.128953694808, 0.178442593387, 0.135458251968, 0.0925662718796, -0.0418017057135, 0.0965349359804, -0.013691308958, -0.148358714053 }))
          (Vector<double, 0>((double[]){ 0.00574160158905, -0.0608886303526, -0.0897685376133, -0.0268399496877, 0.0411931513953, 0.163641206267, 0.0671192056226, -0.0581124231434, 0.148882784114, -0.16478168068, -0.0404849738738, 0.0755064912291, 0.051651345271, 0.0981888028669, 0.0872616221795, -0.192470075768, 0.0892833400025, 0.175347074081, 0.126643230839, -0.0931236769156, -0.260383382439, -0.1651021515, 0.00768762999096, 0.00717274257419, -0.0682314873121, -0.163926794578, -0.011169206935, -0.159602392219, 0.00361225096285, 0.0194824958847, 0.0676087932912, -0.133658037352, 0.0376951690525, -0.201886468899, -0.00654257041336, -0.00880154972781, -0.0368966716455, 0.0879078170239, -0.0671767212122, -0.0102299609476, -0.131168893326, 0.0320583085148, 0.121658052241, -0.0402249575889, -0.0841783984206, -0.00725040879, -0.00680115911852, 0.0512082538008, -0.0368533255102, -0.0522910789767, -0.00147413012719, -0.0244596831668, -0.0148171875039, 0.217481125962, 0.038105328107, -0.048659418991, -0.0397047441989, 0.055868939618, 0.00378344225553, -0.213102489111, -0.152199526214, -0.00359854613511, -0.0204227885125, -0.0868548823268, 0.0315378254819, -0.124343646437, -0.13915750804, -0.136451931769, 0.0107986067244, -0.0389613357201, -0.0223586950038, 0.00414265303714, -0.00191109412176, -0.0483298070773, -0.0825251441281, 0.110729078636, -0.096443675804, -0.0012990944491, 0.0640839684264, -0.136992570943, -0.0263311717132, 0.107817589936, 0.0599774493887, -0.118146335052, -0.175461020491, -0.137160562534, 0.105741260085, 0.0982268725714, 0.0708357553122, -0.0425380117962, 0.0884924463443, -0.111227961055, -0.0680405756212, 0.023039301159, 0.0335073445103, -0.280880886664, -0.0934875861098, 0.0226043201143, 0.0837350474372, 0.144995485342, 0.0348083607355 }))
          (Vector<double, 0>((double[]){ -0.148480012172, 0.0130318864972, -0.0394381837335, 0.100054211919, -0.15658237319, -0.00809049480707, 0.0813293916614, -0.0136176486727, -0.0334856763025, -0.104256368837, 0.043321403928, 0.00522642603264, 0.122128843958, -0.00512337665259, -0.100702009976, 0.140502852735, 0.0475438969309, -0.133718334258, 0.211595560843, 0.077512525006, 0.0621177516659, 0.0371525252888, -0.0318409717941, 0.123099736478, -0.113260381613, 0.145765138481, 0.0992609702507, -0.040668779465, -0.139238641847, -0.00569416512932, 0.0504247368954, 0.0144487857257, -0.0320806735859, 0.129402162805, 0.0440646584794, 0.0390766361719, -0.121612042647, 0.0763664253529, 0.007018598435, 0.140192985509, 0.0783952065869, 0.125453476425, 0.00942872521952, 0.0279817408675, -0.166253736003, -0.136556675594, -0.0100765712084, 0.0185829934876, -0.0547310562474, -0.0739993021638, -0.0555711073984, 0.046266805437, 0.193468707574, 0.0630194612421, -0.0232436160814, -0.256631917953, 0.0526702859048, 0.214828672486, -0.0756667381977, -0.0959925128038, -0.142917150401, -0.0364044881913, -0.123255030141, -0.0366739597424, -0.131034578272, -0.149383020227, 0.138179039306, 0.140717690254, -0.11983555177, 0.0218126907138, 0.149668903514, 0.00902646774627, -0.0779215994424, -0.190546669507, 0.0795095022223, -0.0269508579776, -0.0630075245918, 0.0769947140415, -0.0198559731981, -0.00982738753885, 0.00155771358077, 0.0663675388582, 0.12940109944, -0.108318663186, 0.0776305933914, -0.109137759668, 0.141917686039, -0.00559634913051, 0.0762871138285, 0.165168319607, 0.110396603919, -0.0941861240619, -0.0427266240089, -0.0153851049549, -0.0615667472449, -0.0950262580697, 0.148179564765, -0.00996889282236, -0.0373846350859, -0.0231677281144, -0.14757565353 }))
          (Vector<double, 0>((double[]){ -0.00717684711745, 0.156251498878, -0.0518086825318, 0.0979180032572, 0.172930191746, -0.0375591523293, 0.0777331446833, 0.0658656880654, -0.0382782293202, 0.00969121741646, 0.0714960653642, -0.104691615268, 0.155249571875, 0.135400544166, -0.0831578517155, 0.154616516081, 0.0931881456238, -0.0630426152487, 0.0622022111251, -0.0508954342594, 0.00504452571358, 0.0212791627862, -0.056203341115, -0.0940396718476, 0.0488843931071, 0.0887837812908, 0.299268859791, -0.055527979481, -0.104094824155, 0.0369833161974, 0.0377908663694, 0.0195885339754, -0.0760712492451, -0.0111513149136, -0.0998342521294, 0.0309360560506, -0.0563209689479, 0.0584699684898, 0.164366919863, 0.0439984163696, -0.043114330061, -0.168373059993, -0.208028270101, 0.0472812336244, 0.052979130144, -0.0675782744056, 0.0186691358679, -0.115906884064, 0.0105311126036, 0.00589100424871, -0.173708554093, 0.011960672821, 0.037381484746, -0.128690831658, 0.133626989869, -0.0178015216242, 0.0474153183032, -0.115110416296, 0.179914814937, 0.020679547952, -0.0361538729688, 0.102249197133, 0.101699322847, 0.163839386039, -0.0233196152202, 0.0554514838144, 0.130313495975, -0.00463205197226, -0.0497239050878, -0.0264207031868, -0.0853366489086, -0.0748118828544, 0.252391454172, -0.0564831440551, -0.0895031505049, -0.0690023114513, 0.0251168363408, 0.0728218561932, -0.134353059694, 0.0115508511188, -0.0830227359642, -0.0403098486625, 0.0357247269366, -0.0509835984965, -0.00178870371244, 0.191320059503, 0.00291270304139, 0.032908010521, -0.103741152392, 0.0874063165223, 0.152456131482, -0.060898621361, 0.214296026453, -0.115863934858, 0.141498281079, 0.0881034537016, 0.0377696134765, 0.0754853952471, 0.0961315140833, -0.157227985255, -0.070262964752 }))
          (Vector<double, 0>((double[]){ 0.0733820754533, 0.148813932211, 0.0337843636195, 0.245996398452, 0.0188310470863, -0.172255659777, 0.0910124116943, -0.0387562403854, 0.0960970678347, -0.0041026439131, -0.0997202214355, -0.0374459045093, 0.19595979281, -0.0862518576712, 0.14739439473, 0.0868608106365, -0.0247746888036, -0.0134846593427, -0.0997358246425, -0.00991770597498, 0.0553043653103, 0.0232161448856, -0.0801725880733, 0.142196768064, -0.0858874106043, 0.0299920538458, 0.0113378110174, 0.145746446078, 0.0372796538477, -0.0194300062255, 0.0589076455681, -0.104302300513, 0.0956869370152, -0.0068359037153, 0.119055705715, 0.0888698930226, -0.0671854319893, -0.26437916703, -0.0423025618719, -0.0415746597643, -0.00527162802781, -0.0451847012788, -0.022695465384, -0.246084379836, 0.184952321292, -0.0597335995386, 0.164553381851, 0.0561941587733, 0.123621013556, -0.0432127887286, -0.0480483102923, -0.0131794524135, 0.0500542493446, -0.0281518588159, 0.0112839162551, -0.0989329850217, -0.0151187130723, 0.0166931350262, -0.031778793279, -0.108475082151, -0.037729438273, -0.11400076564, -0.123764583917, 0.00727669265009, 0.106671575502, 0.0765597303128, -0.0361461746664, -0.0880481407257, 0.166930716668, -0.134358708582, -0.0237003940439, 0.135763832878, -0.0193712502329, -0.0437147756875, 0.0363491421527, 0.0312839533567, 0.0789498875648, -0.0721212937086, 0.0119032248869, -0.00876275102565, -0.00812739982822, -0.0189963889079, 0.0775723776553, -0.0471627074187, -0.173666420162, -0.106467363098, 0.132408401929, 0.198990830758, -0.148264606115, 0.106097926534, -0.0179006864338, 0.00127879388901, 0.0567793494241, 0.000251749284717, -0.139026798181, 0.164703874094, -0.00250418360296, 0.00275555633745, 0.194924793885, 0.187523757655, 0.0317914575841 }))
          (Vector<double, 0>((double[]){ 0.051314994598, 0.0065045763379, 0.114191037506, 0.0101957618997, -0.0286529494559, 0.0768346523727, 0.00862792081324, 0.0655926257816, -0.047553464289, 0.00498088222823, -0.0621417977735, -0.147384148188, -0.0239265397963, 0.194727006194, -0.141485686684, -0.0482906546513, 0.0557550187112, 0.066297508519, -0.0420360591634, 0.037622645313, -0.114053596399, 0.0782706927424, -0.106897602444, 0.036294700606, 0.0540178322971, -0.128460045249, 0.0793792099791, 0.0100167534459, -0.214590977818, 0.134097991854, -0.0605970242557, -0.192013182682, 0.0739881116159, 0.103447818144, 0.142697342254, 0.00472958948462, -0.13587287469, -0.066503940262, -0.0675176227925, 0.0163647406049, 0.00641415302834, 0.227161416433, 0.0880958866151, 0.00719493583692, -0.15382419197, -0.118489712092, 0.107410202382, -0.0322205732385, 0.101748334804, 0.0261284857948, 0.000984355195432, 0.0974628769292, 0.116901031876, 0.0404168074751, -0.08852387837, 0.0382571727644, 0.0435098419807, 0.232368939011, 0.11967852975, -0.0719017324991, 0.131922472507, 0.107967927512, -0.0739606398258, 0.0220357552085, 0.0793768419328, 0.0293284245185, -0.0233323761182, -0.0842821283654, -0.147800977889, -0.0186611181828, -0.0159141331375, -0.0280740721781, -0.0629727739816, 0.148765721914, -0.0197707693939, 0.204883118569, -0.0792084501845, -0.00518942388779, 0.0573614670319, 0.0957480553662, 0.044370725899, 0.0754360027781, 0.0608997534959, -0.11097620164, 0.120648363117, -0.0393652490912, -0.176923399909, -0.11261528847, -0.00736139510786, 0.0273189835792, 0.128531719991, -0.0706229965198, 0.255735175251, -0.0139748677799, 0.0819588278928, 0.0737642591976, 0.210001240748, 0.144398131485, 0.0852355325594, -0.0235046476658, -0.000207147526735 }))
          (Vector<double, 0>((double[]){ 0.0876607245428, -0.163902788311, -0.0424497140433, 0.0619554504668, 0.12278777637, 0.187647910209, -0.0653819510715, 0.0673149189434, -0.0338160367328, -0.0212517213944, 0.0792681573182, -0.151063838215, -0.0127240499647, -0.07776334655, -0.112854550831, -0.00396912170863, -0.0106224480038, 0.0669544243432, 0.0543310680157, 0.0538264732985, -0.0556761992969, -0.136600983015, -0.071836500026, 0.204007681985, -0.0770886747172, 0.00272509561376, 0.252724237048, 0.0138003630407, 0.0943838033243, -0.152618076962, 0.157127659827, 0.167913502467, -0.0734033314879, -0.0855663214997, -0.235651416145, 0.00145474165654, -0.0395285369289, 0.0542727230824, 0.0600422526476, 0.229895204798, 0.0687731054247, -0.0946028628969, 0.00833897861778, 0.108014490165, 0.0362179689279, 0.117521501154, -0.168057465924, 0.0657080532428, 0.120434592771, 0.0501600053685, -0.169936558234, 0.0350198941832, -0.0181088914263, 0.0208239429747, -0.0601327579026, -0.0721559332924, 0.134559145318, 0.0781711292671, -0.0595018623027, -0.0136865622225, -0.0235611446543, -0.130395368692, 0.00912421882725, -0.0621211342172, -0.0151835392596, -0.0552860142849, 0.0171344995277, -0.0458877305609, 0.111518696289, -0.0137364466052, -0.0630021982071, 0.104536462014, -0.0523023198241, -0.0661067525797, -0.149341105386, -0.0299985638332, -0.109721782865, 0.146930692242, 0.095916297323, -0.0277401020007, 0.0461507041204, 0.198012502435, -0.072420260031, -0.0600341663149, 0.0883522739308, -0.0210863376332, -0.0136474349424, 0.0535743551429, 0.00972381128489, -0.112378566365, 0.0641967887818, 0.0561478783707, 0.023846171013, 0.00373663148993, -0.160613079748, -0.0468812457233, -0.0157126698752, -0.181119082392, -0.110802777992, 0.206395428423, 0.0583487806488 }))
          (Vector<double, 0>((double[]){ -0.224674479381, 0.0139585327693, 0.0379728685283, -0.0793881028048, -0.0467451465479, 0.00114951798017, -0.000445338983741, -0.01905655229, -0.141095694877, -0.103025382313, 0.0332129940528, 0.0121696661836, 0.0676115166596, 0.0528200827336, -0.0649928347276, 0.135676162844, 0.213164907502, 0.0416304014732, -0.0322163404853, 0.138853729127, 0.0433079111406, -0.0244555135751, -0.0336284964383, -0.0961379396386, 0.0853485845273, -0.103652045641, 0.0914491914772, 0.0690963179432, -0.000947387413027, 0.012278674412, 0.0695348600023, 0.131880339767, -0.0463713350585, 0.215146343098, -0.241139407414, 0.100139005517, 0.0739513921614, -0.0163893498954, -0.202567629682, -0.195929528237, 0.0885118614852, 0.00162432300178, 0.0588860280749, -0.0210498650995, -0.0633346513014, -0.0736534882029, -0.0388405635208, -0.0207596698773, 0.198529798513, -0.221635489095, 0.00840090342578, 0.279954432239, -0.00881988292624, -0.0147154877604, 0.0828276862109, -0.0514383613893, -0.0834928400795, -0.165065826679, 0.0516885431714, 0.0550064022741, 0.0553059966204, 0.00432786805118, -0.112899293967, 0.035857899058, 0.0906363060704, -0.0249143916321, 0.00224245179188, 0.0721959889461, -0.0788669112339, -0.0095184187721, -0.0495691429207, 0.0585339912234, -0.0913416725943, 0.0737643983886, -0.00236748009948, -0.0687180619979, 0.230222996797, 0.0761603906934, -0.148763110868, -0.0083697704786, 0.00726474414383, -0.122153009739, 0.0994792325451, 0.0845369599592, 0.0381991045208, 0.0478394517968, 0.0479366678242, -0.0433849772185, 0.114167936447, 0.0601713424636, 0.00347883071814, 0.0338216982238, -0.0465743235891, -0.17531025987, 0.110101541917, 0.0126979860621, 0.125622713376, -0.0282146172377, -0.0568696619751, -0.18972704531, 0.0129258475396 }))
          (Vector<double, 0>((double[]){ 0.0996608512846, 0.0438904790643, -0.00727378979528, 0.11201605269, -0.190232451331, 0.0275955137507, 0.111125538998, -0.0473429909643, -0.142182941669, -0.0808308071326, 0.0697925027072, 0.0478742516474, 0.105473579151, 0.116200666031, -4.01313619586e-05, -0.120953766375, 0.115724666354, 0.0185915798339, 0.0905662729204, 0.11855318623, -0.231343945367, 0.00970188210568, -0.0760075774672, 0.185918013423, 0.0507369386141, -0.0216782570644, 0.0741349129778, -0.0796718150739, 0.0173781673235, 0.0252473136237, 0.0908141635017, 0.090193919824, 0.0183137622308, -0.0753704422143, 0.136462653453, 0.0793193045573, -0.0396637015705, 0.00974578750977, -0.0787378786511, 0.10318311246, 0.190190644649, 0.00295348180835, -0.0456625309152, 0.0404914311695, 0.047635776072, -0.164301133674, -0.13919559207, -0.118863701749, -0.0635435013856, 0.10982333361, 0.125713652208, -0.0921768237223, -0.235283953309, 0.0707173435834, 0.0385320854617, 0.00158048807964, -0.100199497764, 0.203999992356, 0.0763297435531, -0.0490308825569, -0.00118303138184, -0.207266377448, -0.10682304288, -0.0726631566355, 0.0300841553432, -0.113135768987, 0.0852783897641, 0.0606381369914, -0.0408496163696, -0.0530027561442, 0.101940282816, 0.0050909558576, -0.0997115054132, -0.206481077236, -0.0935211255826, 0.00991233434988, 0.0347127221361, -0.0160768743386, 0.0793268815989, 0.0216397045267, -0.0590155663332, 0.0375709105202, 0.0414560102297, 0.0607681822718, -0.0832499417459, -0.0539023486743, 0.113498793817, -0.0386926662235, 0.0037580052803, 0.0613592436615, 0.0218777185221, 0.176213287366, -0.129631224303, 0.174221788344, 0.0524894244309, 0.048105218406, -0.089482529911, 0.16257931135, 0.0488139838599, -0.107015168778, 0.187726757769 }))
          (Vector<double, 0>((double[]){ 0.0134604091132, -0.00750051608489, -0.166019307698, -0.029767488403, 0.257806882574, -0.0342495185054, 0.0933258920113, -0.018993736232, -0.0857720931704, 0.0664556731117, 0.133247121743, 0.162995040594, -0.127529436623, -0.00282056824477, -0.025055143886, 0.0114747513997, -0.00975506638323, 0.128305128496, 0.0311738752018, -0.174565883966, -0.0854049802038, 0.0188376269241, 0.116318763039, 0.00199563240069, -0.0674004083856, -0.140531339864, -0.128216323748, -0.0436655984353, 0.0242416670547, -0.211037837819, 0.0165744331744, -0.00952165473553, 0.0330398892906, -0.00609284369011, -0.0197463023352, 0.113632130477, 0.165289788613, -0.0521459079927, -0.150009780783, 0.0231863118039, -0.0398615895226, 0.146999074533, -0.000562004898483, 0.093284746831, -0.0105018483593, -0.0150822091772, 0.109287333153, 0.0161204366705, -0.0890539765548, 0.162681682156, 0.0691306695604, 0.0575904796659, -0.16369836493, -0.0587912442567, 0.166639257195, 0.0729674215448, -0.070236935378, 0.057154860913, 0.0429054752055, -0.18955795926, 0.098249627253, -0.0204284306582, -0.0927317205276, 0.180132939492, -0.0359872484331, 0.18004586971, 0.0616071567588, 0.0373922087561, 0.150044677348, -0.0232675139303, 0.106338647359, -0.121986282823, 0.190547950872, -0.0120174663833, -0.0643426988299, 0.0711213171005, -0.101376741191, 0.00889263501583, 0.117909545285, -0.0741943293075, 0.0745940696705, -0.0332786050711, 0.10010743555, -0.0198207765144, 0.161877018946, -0.00432618235561, 0.0349958236635, 0.0234166353357, 0.0487115100483, -0.0178739432575, 0.0735933572066, 0.227740911122, -0.0695427695509, 0.0483967338639, -0.0417164883902, 0.000926665733197, 0.122435926801, 0.0140385384966, -0.155331225419, -0.0832114166932, 0.159706592954 }))
          (Vector<double, 0>((double[]){ 0.148353784128, 0.0409043836712, 0.00589698510089, 0.000950939755405, 0.22191895199, 0.055691496179, 0.158233407067, 0.159590678245, 0.0653216216575, 0.0429284241167, 0.120394749309, -0.152819500533, 0.030089117722, 0.0189532473854, -0.0572752068599, -0.0529497474678, -0.00644836135698, -0.0399034304762, -0.0304488292306, -0.0170816758733, 0.0222947410793, 0.0788295768972, 0.0262313376861, 0.211560687308, 0.0820867222864, 0.15333176049, -0.0869093235502, -0.0882955842877, -0.0297483873409, 0.0298602693317, 0.21205843038, -0.105428494164, -0.176287268072, 0.166254525049, 0.027407272515, 0.198140676753, 0.082182034134, 0.118939409868, -0.0946356722005, -0.00394671490394, 0.0908906445958, -0.127317620907, -8.40680779141e-05, 0.0168037099425, -0.0168655344949, 0.0897881290157, -0.0377982077193, 0.00561849168273, -0.0242013819303, 0.00548479402035, -0.124411589853, -0.0387755444741, 0.091798897756, 0.1617673773, 0.0428501818989, 0.0517021190478, 0.115828339025, -0.153884535226, 0.143543427978, -0.0308566776115, -0.0735519009179, 0.0561962949682, -0.181994297985, 0.201039894301, 0.0509425362634, -0.0876506405615, 0.00269343044127, 0.0184943339731, 0.228104139883, 0.0468370675947, 0.0194745670213, -0.0609147031384, -0.157607776504, -0.0213974931236, -0.0248928008625, -0.0276534066995, 0.0196106223766, 0.0631182910276, -0.047259777252, -0.234168821512, -0.0245254595565, 0.0295039017834, 0.0968670947116, 0.119599940626, -0.0188263664527, -0.0194600829595, 0.00981504956734, 0.178978408244, 0.0113222891209, -0.0536179794946, -0.0250759244023, -0.117766699198, -0.061206683239, 0.126599274997, 0.0713840830378, 0.0801716643061, -0.112444282615, 0.129173910958, 0.0273499847773, 0.0866018355182, -0.0395927565855 }))
          (Vector<double, 0>((double[]){ -0.0106862558709, -0.0122683280731, 0.00253662117571, 0.12116921623, 0.261193901065, -0.0496144262018, -0.0515646147139, 0.147483536571, 0.0276646467383, 0.120559698593, 0.0328953126412, -0.043044496371, 0.218817069041, -0.13585745648, -0.0956609136936, 0.0144723384389, -0.0472378780848, 0.0325635524276, 0.03096661106, 0.00431879755732, 0.163680732773, -0.132521349822, -0.0176609020036, 0.182619551803, 0.0344254243535, 0.101504350374, 0.0104034203523, -0.0391706248865, 0.0668304177109, 0.120968835497, -0.0759327284644, -0.042528157294, -0.114168935445, 0.0336513979671, -0.0524312690725, 0.106433242879, 0.11329684821, -0.134267855688, -0.0779576945138, -0.0196272388146, -0.201116408353, 0.0604094053147, 0.128388302346, -0.133195272805, -0.101077513806, 0.0869234453772, 0.031098575249, -0.0312227608527, -0.0411161254722, 0.0651341107126, -0.0725326807235, 0.0115080849654, 0.0508112164844, -0.0802135768528, -0.193017565914, -0.106822723787, -0.0523939542551, 0.137996893706, -0.103483008534, 0.0335671899268, -0.0743739820965, -0.0272068960009, 0.0354798420608, -0.0162469882531, -0.0955909369197, -0.00301484761766, -0.0387031649104, -0.119064784067, 0.00398310829478, 0.155597507905, 0.0282659229326, 0.0973904157143, -0.00112807254335, -0.0924098528301, -0.127636955612, 0.0359793984122, -0.0113339774321, -0.111746431868, -0.0710180634105, 0.0862077719033, -0.0579922576138, 0.0862248171649, -0.153207471281, -0.00757506894045, 0.0104229501792, -0.0858687248068, -0.114357070033, -0.113785349698, -0.0410170158586, -0.00454792967304, -0.062691724002, 0.0437932404621, -0.115221678228, 0.0809710069898, 0.100187655656, -0.155432339658, 0.0541092868592, 0.221250478321, -0.154524333344, -0.0850564127799, 0.277380255081 }))
          (Vector<double, 0>((double[]){ 0.167730825833, 0.0314421516325, 0.207489201925, 0.0511871994372, -0.0854430081002, -0.0905072183079, -0.0182297838566, 0.111830694117, 0.140705815316, -0.0509161490631, -0.0676006423378, -0.0419691993803, 0.0712788958774, -0.0938532365279, -0.0380506151608, -0.0208872847229, -0.061780002047, 0.161941138904, -0.0197583407082, 0.0109519482806, -0.209125655036, -0.134961235064, 0.130011817669, 0.0748926038542, -0.106541169514, -0.107637126516, -0.0833195621197, 0.0800859312647, 0.00234825175631, -0.00744182113207, 0.0534239639342, 0.147201572537, 0.0880555648095, -0.00260864439292, 0.0401839577485, 0.00979412044805, -0.134915484084, -0.0999851137349, 0.263662119631, -0.0201728946475, 0.0408483433987, -0.12427566496, 0.0535873679084, -0.0624927530427, -0.0186377054153, -0.0579734868306, 0.0354359476573, 0.135066569448, -0.184724543208, -0.0153885909177, -0.0486565096087, 0.0168036516793, -0.0177105045961, -0.164703074024, -0.179854804978, -0.214985584703, 0.0237805612102, -0.13640265562, -0.00449522199139, -0.012927271992, -0.0968612803698, 0.113931224818, -0.108961049662, 0.0495289813741, -0.01430346246, 0.13195901118, -0.0165436946122, 0.0597068147712, 0.0108566703684, 0.0149140235668, 0.0470825067401, 0.172042502889, 0.100931758786, -0.0770449295366, 0.13704872274, -0.114580413484, 0.190135701019, -0.000202389800344, 0.158951283784, -0.00899019012231, -0.0325905915038, -0.111688028869, 0.115407962116, 0.0408306846456, -0.0972670053877, -0.0500079588415, -0.124327566949, 0.0348513353138, -0.202679712513, 0.105681044289, 0.0818997952459, 0.0321958110336, 0.118806301031, -0.0372997723768, 0.0135631496431, 0.0294095165765, 0.132039530603, -0.0235490277302, 0.0290735652173, -0.024048895369, -0.0804781558514 }))
          (Vector<double, 0>((double[]){ -0.10447916755, 0.0872541514392, 0.0270714166927, -0.0411089062407, 0.253138447444, 0.0687655130362, -0.0710229516521, -0.134627677311, 0.11946535276, -0.17890442114, 0.157834544526, -0.0716033322614, -0.157083549863, 0.0541286251123, -0.168281261854, 0.017276850307, -0.114753217626, 0.172905134247, -0.0148107954846, -0.0373194547446, -0.0521541494409, -0.118720156184, 0.0969089287667, 0.0328567685672, -0.204499877708, 0.0818589064902, 0.029100374317, -0.0739395115244, -0.0794863799323, -0.0646731251723, 0.0954155450199, -0.147497686506, 0.109598116774, -0.103591985903, -0.06317744883, -0.00248248977327, -0.203296744186, -0.151479192106, 0.206035155745, 0.0404350710325, -0.00884852435102, -0.0277550470293, -0.00581009656771, 0.0564879354152, -0.143434459081, -0.0170948851852, -0.127501735491, -0.0072129324129, 0.0805524166076, -0.0574701086027, -0.146878470611, -0.00570753890871, 0.0532182656289, 0.0371370196923, 0.0367284021378, -0.0949260984392, 0.0968044869025, -0.0116617981657, -0.0569277874254, 0.031301587838, -0.113448461868, -0.0508757237629, -0.15338987126, -0.0150425005313, 0.125089886573, -0.0392120023123, -0.112121275458, -0.0380494640479, -0.00268193810785, 0.168481880299, -0.116066366731, -0.197573717357, -0.0100824704717, -0.0575564131299, 0.101473686028, 0.141684434679, 0.157685592114, -0.0560342502267, 0.00106961007864, -0.00439847103862, -0.112712979146, 0.0387576755401, -0.0848551669273, 0.119174009434, -0.00851068153564, 0.04176726233, 0.0412718824599, 0.0299637304628, -0.0833491896554, 0.0662296526088, -0.070398345723, -0.0593347362619, 0.042461767055, -0.0901662778542, 0.0403055408564, -0.0675683610002, -0.0679611523281, -0.0409993291669, 0.199620443386, 0.122758539271, -0.00469877044627 }))
          (Vector<double, 0>((double[]){ 0.0107803481253, 0.0859347854453, -0.0833395711197, -0.124026179684, -0.056311219173, -0.124337572794, 0.0159230148859, -0.027353082889, 0.00354081042006, 0.151227100322, 0.00627615474774, 0.0402526109713, -0.0161558845806, 0.16234084926, 0.0106658849712, -0.158790343418, 0.0418053907896, 0.0750929815437, 0.074574018585, 0.0615057499021, 0.11430041816, -0.108142747693, 0.0318115021874, -0.0300813617524, -0.0766042555422, 0.00144598263549, 0.10327593391, -0.0789116598288, -0.0234227705741, -0.0890734187511, -0.104332789713, 0.08395215454, -0.0446063931308, -0.0754571863672, -0.0496323691611, -0.103066859587, -0.0564518365997, 0.0639282158599, -0.00177199314157, -0.172028243337, -0.0280077812062, -0.0339909114423, -0.124124088423, -0.129973843627, -0.166321885125, 0.0374258121663, 0.259313998851, -0.0469713134675, -0.133969777887, 0.0724543083751, 0.0943058020648, 0.0689520886611, -0.0496408079282, -0.163227702268, -0.145257473743, -0.0611053086977, -0.24437189965, -0.0484358211173, 0.159201539367, -0.207407105928, -0.105220569923, 0.0566536250115, 0.103564241732, -0.0336011219831, -0.0134013282203, -0.0365664113002, -0.137723963005, 0.113621636024, -0.103706256815, 0.048705067507, 0.0910224160524, -0.102123930888, 0.041096528056, -0.0491468034343, 0.132824791442, -0.00166315455589, -0.0961582680908, -0.155810093176, -0.0901670643399, 0.119656928273, 0.194079514093, -0.215211730316, 0.0184402133899, -0.0595967398798, -0.123243412714, -0.065172301653, 0.0510887712845, 0.0505173617838, -0.0104849499077, 0.0506778238446, 0.0122591556839, -0.00175224979402, -0.150184181257, 0.16948083936, -0.0325487244405, 0.0877418737582, -0.140768341854, -0.00603724152567, 0.0725828643005, -0.0123618465669, 0.0123477873275 }))
          (Vector<double, 0>((double[]){ -0.0835058868182, 0.15768348786, 0.200984363503, 0.123167783084, -0.141492996769, -0.0626274374094, -0.0173936649005, -0.105443485659, -0.00706731403165, -0.0737388894912, 0.0304190377707, -0.111336049298, -0.125412590259, -0.029851254224, 0.0182939100254, -0.15134225352, 0.0212148687852, -0.121410786067, -0.0875493995665, 0.0277313298881, -0.0179611163344, -0.106889179679, 0.00798863974842, -0.115151655711, 0.117019862211, -0.0527153308043, -0.0773886997932, -0.104434509363, 0.03863899271, 0.0608085636845, -0.0734358909148, -0.032803665989, 0.047466225733, 0.0332252486354, 0.116581573243, -0.049935145037, 0.129460893602, -0.0963133696941, -0.0905467613094, 0.0513813065943, 0.124561534539, 0.0738994608085, -0.0139742476481, -0.117358472523, 0.0999680051455, -0.0666054139593, -0.130868751271, 0.028764766171, -0.0836445538792, -0.00923387072146, -0.0793354361628, -0.0616243766142, 0.0386472562058, -0.140177287486, -0.10787203084, -0.0946369070583, 0.046203556584, -0.143917225186, -0.0910368051625, -0.102199878582, -0.177683753683, -0.0247436934865, -0.013703557414, 0.0954908936035, 0.106737962635, 0.0558437551481, 0.0899555743671, -0.00567863299953, -0.0991153121396, -0.164955087785, -0.268372656991, -0.143151041868, -0.172608412006, -0.0163838785312, -0.0663083172143, 0.111513992531, 0.207785558214, -0.0922127524076, 0.0358824324703, -0.119138229302, 0.0233235664707, 0.0179977275886, 0.00630167641632, -0.0628990139813, -0.176286410524, -0.0631944134138, 0.0519866969449, -0.128042589607, -0.121649882792, 0.0148473542343, 0.0618284717193, -0.153751729896, 0.0572446258642, -0.0173110659404, 0.013517609617, -0.00591997512214, -0.160454810185, 0.12361940556, 0.0112565560541, 0.133353357761, -0.169511876874 }))
          (Vector<double, 0>((double[]){ 0.0128701208458, -0.125531809866, -0.025116579403, 0.192381684254, -0.0191677317517, 0.0227240895947, 0.0752515487878, 0.0188411847957, 0.00139040296956, -0.029003047401, -0.133793595876, 0.191843218563, 0.00675949952615, -0.0699724793315, -0.0938888883343, -0.106574482036, 0.0666733748478, -0.0355752530794, -0.0911393849472, -0.0373078935188, 0.208847607902, -0.0497635753091, 0.103753155948, -0.101824448304, -0.0451134723698, -0.0182686543865, 0.00100249520809, 0.101903668423, 0.0881912063056, 0.082354913517, -0.098391950589, -0.0816175007622, -0.153281283379, 0.0529688612625, -0.116506384081, 0.0234009492322, 0.0780399685532, 0.0487356819824, 0.22252313641, 0.0491899916194, -0.0143674495557, 0.0383017706625, 0.0780582090407, -0.00713129529615, -0.00112382704278, -0.0293212226287, 0.109703073828, 0.0939777762086, -0.0536599368868, -0.0901485958306, 0.0202918637487, 0.0576481679831, -0.162559397249, -0.0110057316747, 0.064527173814, -0.114535580415, 0.0742356328736, -0.0330944213735, -0.0497944774938, -0.124625471785, -0.103852327619, 0.0356545731986, -0.0308277004629, 0.0491273864422, 0.0105681205094, -0.153718457635, 0.0539032673189, -0.112753556879, 0.0617670598248, 0.254567025601, 0.0310337457524, -0.0411405565211, 0.101723126998, -0.189241665854, -0.0130303485065, 0.193442070966, -0.0189070057056, -0.067938805874, -0.00937431288801, -0.0111561074193, -0.123371205542, 0.213645812493, -0.229054506613, 0.0867495289092, -0.0138882258615, 0.00806515681097, -0.1545851778, -0.0883197703362, -0.0862406685957, 0.0462966511031, 0.136345776903, -0.24417530917, 0.0823114568636, 0.0607815546766, 0.0274202142736, -0.0237787878905, -0.0827248998344, -0.0396992247906, 0.0340706801279, 0.148680094699, 0.141259165313 }))
          (Vector<double, 0>((double[]){ -0.0130755232052, 0.052985596544, 0.0206331235571, -0.0238581531118, -0.123714561043, -0.0478571357527, 0.0864435592123, -0.0915856911359, -0.0164797458098, -0.00142449089101, 0.0375378496261, 0.0588143413059, 0.0695463734854, 0.159207180557, -0.181873191689, 0.0594129550339, -0.148547553635, 0.081158862394, -0.0608351402052, 0.0793169453605, -0.0968357353851, -0.0894338123118, 0.0509942139314, 0.0459230680227, -0.209671533202, 0.152549528598, 0.271433638492, 0.0477906311768, -0.010369907832, -0.119237697256, -0.0724973401604, 0.0355058739035, -0.132452702885, 0.0316951868092, 0.225697037168, 0.121406158392, -0.0702067312251, -0.0274241524484, 0.0444847932287, -0.0662773632386, 0.282406065402, -0.0158214646146, 0.214056431513, 0.0678662100699, 0.0566113018052, -0.201654140767, -0.171306862785, 0.0292878978687, -0.0345696342527, -0.0112000634233, -0.0268519235285, -0.012791936409, 0.00312534787791, 0.101620997801, 0.122259273649, -0.0511236181777, -0.0115748497024, -0.04038522027, -0.0281944133225, 0.0902396034494, 0.00352366170188, 0.0852164896552, 0.0714390639042, -0.0251830127101, -0.0842455842655, -0.0809683461941, 0.0209593613384, -0.0577089331071, -0.0190265136414, 0.0824151618546, 0.00845239705857, -0.113882308006, 0.0902187359254, 0.0621432991339, 0.0582039637007, -0.168473781741, -0.0663961101798, -0.190987195575, 0.0974002586969, 0.149383042776, 0.125350883227, -0.0418705831755, -0.00882637684465, -0.0312717609043, 0.0233641405659, -0.000620177039125, -0.0818178116143, -0.0654621772392, -0.145727638329, -0.0828576022906, -0.0393385159874, 0.0569725626706, 0.0195769943778, 0.0283095189176, 0.132948933742, 0.16702052815, -0.0876818331076, -0.0969612884335, -0.0924978520253, -0.146265577647, -0.0213882798519 }))
          (Vector<double, 0>((double[]){ -0.00915098973956, -0.0430897175732, 0.0926176794498, 0.0811711463379, 0.0644452493813, -0.0643890285743, -0.158701186596, 0.00341377274039, 0.00530677580897, -0.048394958229, -0.0869096325592, -0.0587767282797, -0.0114050901415, 0.00426685059696, 0.197723740298, 0.048764236751, 0.00108044618106, -0.00686872569793, 0.0185526032294, 0.0168385774678, -0.231147142632, 0.0708707203554, -0.200578768493, -0.130073870515, -0.0316214648402, 0.0275119327369, 0.0685183987039, -0.0743943428241, -0.0194056223255, 0.0694975823184, -0.0317371967797, -0.154297441764, -0.120257907742, -0.0888686754451, 0.169331361441, 0.0787532513634, 0.0111774158161, -0.0826618351407, 0.119658973742, 0.0366756118261, -0.0132852376919, 0.0901872408034, -0.0129047529573, -0.0213336669365, 0.177771153773, -0.100652677125, -0.0987913135818, 0.0314534531547, 0.259177838088, 0.0636339154027, 0.00560191458397, -0.0107430364593, -0.05645321399, 0.00352036844828, 0.0139427941543, 0.00932304807534, 0.0513456208324, -0.219189282254, 0.0844382549629, -0.00238523215759, -0.21845768738, -0.0495818349161, -0.230572926635, 0.0743424516078, -0.0905226010421, -0.049349082883, -0.161590558921, -0.0464410456969, -0.0709182681523, 0.00195370079585, -0.00822585667907, 0.133233688939, -0.0635783598011, -0.0738674843413, 0.10370380617, -0.161822856881, -0.0183169150181, -0.181286807393, 0.094946630248, -0.157654139608, -0.0212942104065, -0.10431791928, -0.00207445620323, -0.13050536837, 0.034884214698, 0.0033571780993, 0.00138162144725, -0.147003476486, 0.0253797534951, -0.0411604937747, 0.170049660087, -0.208795859123, -0.11003342872, 0.0854323420498, 0.0305460366223, 0.110646814017, -0.107106733791, 0.0601560029527, 0.0513577640542, -0.0389594313003, -0.0122848758313 }))
          (Vector<double, 0>((double[]){ -0.0419906429492, 0.0729943852495, 0.0115657638846, 0.0788782969293, 0.124383926763, 0.0653515420332, 0.0429989498164, 0.212593927066, -0.14434107287, 0.0137788922815, -0.267487721795, 0.0841410542222, 0.212468473234, 0.203232625418, -0.0666262127495, -0.0840385474759, -0.0546638239835, 0.0297681019919, 0.0663284272111, -0.051367623341, -0.0263386653242, -0.00705239575673, 0.129379599413, 0.118333070653, 0.21746729294, 0.0521863321316, 0.0391321895584, 0.0299843729589, -0.0235764101731, 0.159955684792, -0.0866649686171, 0.0742143328889, 0.0542623542505, -0.316776936858, -0.0935629416773, -0.218387226008, -0.117987020975, 0.0336381036316, 0.0727153887145, 0.239230736855, -0.107799806497, 0.0366932038031, 0.0633749140438, 0.152226571691, 0.0054758558503, -0.129646282672, 0.0580424106024, -0.0203239559116, -0.0807550135477, -0.00956234988487, -0.0183763295243, -0.0018786711456, -0.0590549585901, -0.064910768159, -0.0610154183804, 0.094464604341, 0.0528766183787, -0.0760268883271, 0.0965351282935, 0.0970526701626, -0.0898863049981, -0.052902671146, 0.153118773538, 0.0435373117931, 0.028374894871, 0.109402253008, 0.064557138219, 0.149192714342, -0.0774239139113, 0.0221257795125, -0.0156590502931, 0.0666667393936, 0.0485993680879, -0.0243719664424, 0.0327397927411, 0.155806245993, -0.0328944137207, -0.0940610353446, 0.0858717206207, -0.0385913734752, -0.138082768529, -0.107056589457, -0.00423101827235, 0.0259409226281, -0.11949745529, 0.110592874749, 0.00212360267726, 0.00550745285654, 0.0155390493566, 0.0550203614995, 0.0408733252353, -0.157596749764, -0.0220683946747, -0.0492506425559, -0.056465502447, -0.0756632518134, -0.0111211609151, -0.0151066042443, 0.0142632676526, 0.0259689295493, -0.0342520077305 }))
          (Vector<double, 0>((double[]){ -0.0599721547356, 0.0880390297544, -0.00332753222687, 0.217447813943, 0.215530069887, -0.0337263759807, 0.156643099326, 0.11346649922, 0.114529438975, 0.0934679377292, -0.0505724792762, -0.00962026025857, -0.0680276857166, -0.104901321192, -0.0628861002985, -0.0412492941312, 0.0842029737581, 0.124529735848, -0.00925467085478, -0.0562456173252, -0.0568377344438, -0.187669328203, 0.0437770216672, -0.0981132040957, 0.0348628345733, 0.108965023661, 0.100361820175, -0.100557860507, -0.143677751243, -0.0528316321607, 0.0794867685372, -0.038258160703, 0.211125623673, 0.0683313576094, 0.00231111501201, 0.0349424287965, 0.0229590587436, 0.0715672798603, -0.237360306085, -0.117871395481, -0.108032917721, 0.087467073828, 0.129027639709, -0.0172792657865, 0.148165047029, 0.0744979100537, -0.0142569227376, -0.0853995568237, 0.0513077810463, -0.0826174698828, -0.080984912911, 0.131876345413, -0.149863075956, 0.127591933916, -0.103587617648, 0.0187066147761, -0.287409310348, -0.019406180512, -0.0829607976618, -0.114353335054, -0.0229443120211, 0.0219331843105, 0.012963695251, -0.0692936448366, 0.0486554405767, -0.033585105426, -0.0235405316812, 0.0613317589893, -0.00184683511595, -0.109051640411, 4.04311582939e-05, 0.0337446772402, -0.0162806431326, 0.131680211611, -0.0690533094134, 0.070907297726, -0.0235014159206, -0.155772238319, 0.116849394554, 0.0562457699896, 0.0982965823669, 0.02788080663, 0.119540980732, -0.0623686047288, 0.0421202970233, 0.053545724015, 0.120987751659, -0.0915315895502, 0.00451226492502, -0.172858520755, -0.0248480030237, -0.0583363459625, 0.0568957444725, -0.0651995598087, 0.0115140926317, 0.119259206692, -0.216423303969, 0.124278945869, 0.021399385172, -0.0980730264772, 0.0786636829034 }))
          (Vector<double, 0>((double[]){ -0.0663874110649, 0.0324877752611, -0.0973578992777, 0.185435367422, -0.0332315943449, 0.0820402002353, -0.244756386581, 0.0227799005365, 0.0156414174458, 0.0429204129739, 0.157936910827, -0.0293803123667, -0.000307181413473, -0.0195899341536, 0.0327658817511, 0.108007153172, -0.089211831721, 0.11357958336, -0.00741774749933, -0.0655852295902, 0.152749369369, 0.138217130174, 0.0410646938442, -0.0375260466965, 0.117302345363, 0.0463162867662, 0.00468748175311, -0.166906461264, 0.0696364835206, -0.0519637643701, -0.0840504729825, -0.0694728353412, -0.0270140182943, -0.0527379515952, 0.0444583813817, -0.129994047821, 0.11186243246, -0.0121431949669, -0.0798463904432, 0.00768645277736, -0.0117615605257, 0.053981876664, -0.0967738916371, 0.071533946387, -0.209400163535, 0.0769348514284, -0.210951651757, 0.129889032008, 0.0247923426762, 0.0976526527234, 0.100515912988, -0.071135442902, -0.0206392737912, -0.0630259847449, 0.130996061804, 0.0112620052867, 0.0631573989243, 0.0313657012758, 0.0559405138453, 0.0560356884753, -0.170235829295, 0.0242939105821, -0.0786893119904, 0.0743353331753, -0.18513848059, 0.0434799935912, -0.0670919670308, -0.00617407851892, -0.0755348470455, 0.026046592617, -0.0247049417301, -0.0416994113336, 0.000607025982691, 0.197673175533, -0.11858990028, -0.124947142612, -0.101289450299, -0.0844501689122, 0.0145717294666, -0.217946611739, 0.0393651547025, 0.0719623126383, 0.0210097392857, 0.279104524763, 0.064713338727, 0.00596317365489, 0.00205791666109, 0.134609923083, -0.082584540736, 0.0684141812987, 0.0195089467969, 0.149248292737, -0.0410512556454, -0.191622070995, -0.155848276644, 0.123140022802, -0.00885419279617, 0.0781396258073, 0.104180555318, 0.130727966394, -0.0298095698691 }))
          (Vector<double, 0>((double[]){ 0.146742790955, -0.184236682703, -0.108006044936, 0.131124611288, -0.0071326833439, -0.141330897985, 0.00882561820163, 0.071657162831, 0.102616430608, -0.0529704279304, -0.125721530133, 0.128064615601, -0.0148069533928, 0.138709328889, 0.0583833880052, 0.0135028507031, -0.04759029492, -0.0814606442234, 0.0419384850343, -0.0207235019949, -0.00547752091395, 0.0149937344287, 0.114504326256, 0.0923291945279, 0.0712325865077, -0.00070780631745, -0.118816162508, -0.104094651932, 0.217154954124, -0.0813004788326, 0.19815390113, -0.0673258331036, -0.0241059430368, 0.100099886904, 0.202179431116, -0.154023494484, 0.0470470660645, -0.0391022944724, -0.0254884273724, 0.0612916074198, -0.0980698662954, 0.0549317368745, 0.0123691357436, -0.162164099972, 0.0153644588748, -0.0292679172231, 0.00652392440347, 0.145378889374, -0.0136345139956, -0.117835883665, 0.0679017980954, -0.0272327264921, -0.0714025385141, 0.0153825318289, -0.00153431826517, -0.16573994603, -0.0395982627155, 0.215321628815, -0.17690615943, -0.01228009246, 0.0707508692877, 0.00623612696454, -0.137599813878, 0.156420789411, 0.171991566851, 0.0498004745189, 0.0312518240588, 0.230214306675, 0.165986055462, 0.0198809232361, -0.00807030698298, 0.2058338358, 0.0472897007254, 0.0791793685894, 0.0355549917504, -0.0532900937268, 0.00648388960525, 0.12160078387, -0.0579365065593, -0.081796802531, 0.0195548605441, -0.0400009756459, 0.00549318998731, -0.0794238003767, 0.116165160274, 0.0204685611052, 0.219520819043, 0.012042235116, 0.0345832807749, 0.0223453323921, -0.0277055222298, 0.0302737670766, -0.0639623182655, 0.0940791894912, -0.0381160379419, 0.0770614916953, 0.11057075199, 0.165349124587, 0.0643535858304, 0.00904622288915, 0.0673102430666 }))
          (Vector<double, 0>((double[]){ 0.113312174383, 0.0835572030383, 0.207703291849, -0.0887317700789, 0.0268872195098, 0.157880154579, -0.0123210695703, -0.0978998464249, 0.172193882293, 0.135624985829, -0.137431489166, -0.000946911720721, -0.117964622653, -0.0277393468718, -0.0520170583898, 0.130426226696, -0.0482665264982, 0.119609136056, -0.0248433751285, -0.0335113558841, 0.0994379608016, 0.115386838737, 0.0316339871438, -0.0913342353913, 0.0860774825352, -0.203130828097, -0.148484729887, -0.109532506587, 0.00782951459815, -0.0651710887543, -0.183321895903, -0.0509647207863, 0.0993346543542, -0.114075515643, -0.125818442219, 0.11291032174, -0.142461626313, -0.019645256281, -0.0113435811804, -0.0500979110308, -0.0276100531113, 0.0850015127532, 0.0174551480537, -0.143574005598, -0.104278363945, -0.0347421100007, -0.0793451133419, 0.088828637813, 0.00941554612147, 0.0715621922507, -0.0995570468838, -0.0533814178059, 0.0879687373718, 0.063160297939, -0.038797774184, -0.136594119494, -0.0151291973678, 0.237147074561, 0.00979936727326, -0.00326999186604, -0.054495231634, -0.0632661149301, -0.0451069553538, 0.0780865294032, 0.206929719443, 0.0599584818316, 0.0878846803343, 0.124292887875, -0.266293191724, -0.0559276999643, 0.143989615721, 0.0806734385081, -0.105924086037, -0.150393396732, 0.0195646117253, -0.14184746895, 0.0919438358176, 0.00438451052088, -0.156711822763, 0.102369267403, -0.0148729141638, 0.0318227720445, -0.0830052577762, 0.0896160923919, 0.0249833156795, -0.0765211839986, 0.0686234981235, 0.058404770625, 0.143616202956, 0.0549352128786, -0.0682658451428, 0.0204284255313, 0.0197446003504, 0.0890286020215, -0.093284868767, -0.0141321894626, 0.0747067663545, -0.0425776630015, -0.0472786833889, -0.0484374700866, 0.097009586661 }))
          (Vector<double, 0>((double[]){ 0.212843995694, 0.0759484937666, 0.0777362427573, 0.0831230904401, -0.0928539231922, -0.116135992016, 0.0675019911953, -0.0999954858092, -0.0094620098326, 0.0114970670481, -0.0243855097247, -0.0134487033711, 0.119764236864, -0.0168729246784, 0.0589234250844, 0.135247971995, -0.0763066348141, -0.0184162762613, 0.0230890674217, -0.0596566881424, -0.0117592614808, -0.185729159417, 0.126012414371, 0.0694898948457, -0.0458124296621, -0.0923662702792, 0.085813051331, -0.192828424942, -0.0595183161873, -0.15027337216, 0.0934085294912, 0.00096193830372, 0.00119868180628, 0.117566934314, 0.0654902568477, 0.118606468048, -0.108868485226, -0.117175684401, -0.0821771775354, 0.000382815230987, -0.0253805676819, 0.0186032963875, 0.0730740642251, 0.109510381262, -0.0882259736646, 0.0677803358621, -0.100691163352, 0.0648424574543, 0.0878372705682, -0.0606956154777, 0.0195485418105, -0.0679117345835, 0.0320985713977, -0.101734607802, 0.0366544270261, 0.0371341568472, 0.0914483554588, 0.0963777048358, 0.100524702944, -0.0124830198789, 0.0154721307833, 0.0704964659371, 0.0349462055845, -0.129455731797, -0.0501674221354, 0.24766698024, 0.150178838593, -0.0505381828415, 0.162193538693, -0.0396956575386, -0.0425141540742, 0.0328315365355, -0.152189786687, 0.232706857696, -0.0435393391263, -0.111800150003, 0.132354931252, 0.0707536594211, 0.059946208216, 0.1709065962, 0.0862592791912, 0.0159448921343, -0.0582087808121, 0.0834428445314, -0.052996474749, 0.023658840449, -0.00427127801055, -0.0155670966929, -0.138869005482, -0.0180976149295, -0.277229400919, -0.0976084073373, 0.00871348201081, 0.0430370426994, 0.0992777701547, -0.155042659356, 0.0988509388474, 0.26285692718, 0.0188153657204, -0.0739567292924, -0.0139012321544 }))
          (Vector<double, 0>((double[]){ 0.0537262059956, -0.0521303684718, 0.157139059036, -0.0701995391906, -0.0269480385482, 0.0210084011152, -0.0968710147923, -0.0421896876934, -0.151726350544, -0.100047654925, 0.0193525987347, 0.0342472429828, -0.0311265696193, -0.0386674720869, -0.0667526734961, -0.0709082084948, -0.0417667919855, 0.0771082969783, 0.0243302490784, 0.0894691422007, -0.0655927768211, 0.0772220258962, 0.0859550170976, -0.0480490211819, 0.0431895174648, 0.322886176252, -0.000494671349921, -0.0203853430466, -0.118647868293, -0.0995569866946, -0.168458636384, 0.0215696207737, 0.129624574376, -0.0906471253118, -0.0278339739669, -0.0984766592219, -0.183833432985, 0.0594940202846, 0.070400417186, 0.0683073465973, -0.0498115445265, 0.0465194079395, -0.117719010248, 0.0329285968932, -0.0315895248557, 0.152557279058, -0.0292394426328, -0.186790947941, -0.0260262346885, -0.155699151245, -0.0642539409953, 0.0171528030722, -0.0155666579623, -0.0753993862348, 0.0770155525292, 0.100758926752, -0.00507930800361, -0.135697849598, -0.169694615414, -0.0507545163835, 0.0529904783891, -0.0382572663226, 0.0376645662537, -0.000323768325846, -0.0834733647165, 0.102691282496, -0.00219291775672, -0.0620670375718, 0.195385350538, 0.0668580835685, 0.0557779334733, 0.110135817793, -0.0226156255302, 0.0818413540952, -0.271719779006, -0.0684020793027, -0.137429103568, -0.110773684108, 0.0881961761151, 0.133293314417, 0.0320265420283, -0.153183657399, -0.0656853222848, -0.100270710425, 0.210328857044, -0.0791884810121, 0.0498491572128, 0.113435670166, -0.0452306684381, 0.0586362254025, -0.106341309787, -0.0401934400711, 0.0439728316837, 0.0507068311177, -0.0505342701254, -0.216063504829, 0.022591199294, 0.181165799147, 0.0189694743818, 0.0730364527785, 0.0617548413994 }))
          (Vector<double, 0>((double[]){ 0.13600857056, -0.0255563675336, 0.0180730257401, 0.0592802169351, -0.0103363167656, 0.154750758397, -0.0169891007091, 0.272594534966, 0.125420662161, 0.0448570344996, -0.173291017533, -0.0496755845141, -0.0797033122068, -0.0366731341601, -0.103909427227, 0.0384331890497, -0.0529461524218, -0.147431781613, -0.120856720626, 0.106615796481, -0.0642049688686, -0.237033855194, -0.188644169223, 0.00811788998556, 0.0242091521496, 0.00101723710596, 0.0249811499704, -0.158121882173, -0.0423440119693, -0.0781857528837, -0.0231918523235, -0.119905642908, 0.144407392654, -0.0343098399042, 0.0380320247764, 0.179174240672, 0.0833122035497, -0.0941040347413, -0.0544205202633, 0.043767403968, 0.0659878935937, -0.00744806766099, -0.0211765036685, -0.0191007521978, 0.131072235638, 0.0624035867304, 0.0907018581337, 0.0628378094336, 0.062647478902, -0.0220862733258, 0.0446538840021, 0.0701935717595, -0.145183460118, 0.183918410584, -0.0278422452517, -0.0759605953988, -0.0273049455789, -0.0148408750177, -0.0180720749098, -0.106374226589, 0.0785890892302, 0.00685853797562, 0.0297418012887, 0.136473554179, -0.0565731861963, 0.124010358542, 0.0211844244132, -0.125667200128, 0.121535788135, 0.037952485372, -0.0347680132857, -0.108366864857, 0.000273069472667, -0.00900803119667, 0.0351330499291, 0.0334097724495, 0.0464323471363, -0.0825984506003, 0.0695475667745, -0.0560085841617, 0.0399263703835, 0.0085557762894, -0.0813215032492, 0.0309868768491, 0.199375896616, 0.155323307716, -0.187485625971, 0.0113273363236, -0.0218025982326, 0.0874973371163, -0.203579689468, -0.0268389875943, -0.163613425534, 0.0977705558334, 0.234907186927, 0.0553104844702, 0.0699163549986, -0.136841461679, 0.0765696658163, -0.100651955854, 0.102714487312 }))
          (Vector<double, 0>((double[]){ -0.00274354585256, -0.0346985218672, -0.0378560081663, -0.103240649118, 0.220610198199, -0.0978433334346, -0.00640075536343, -0.0416687660185, 0.00658134148336, -0.0775857987592, 0.137541481737, 0.160656561451, 0.0179608955824, 0.0164455448134, 0.12729134352, -0.0714892760244, 0.0345069615588, -0.110724865135, 0.0629296052203, -0.0401296770931, 0.24325532632, 0.193078949984, -0.0262340288046, -0.0964280950516, 0.22128859439, 0.075725604919, -0.0933984182656, 0.024656257533, 0.0130319464286, 0.110639366577, 0.0122271446194, -0.0246105979896, 0.0719053793716, -0.0798318267562, 0.0429920362746, 0.116041390686, 0.200058671026, -0.0542717123763, -0.153357340757, -0.0686808895558, -0.0876561267103, -0.0393261435664, 0.0786581332452, 0.0488818261183, -0.0628813503283, -0.0383007613536, -0.0760110961892, 0.0776854671966, -0.0392957411314, 0.190930292629, 0.0247626246713, 0.0411428839494, -0.0142597907761, 0.0856673371114, -0.140577569958, -0.112810047845, 0.0704306065142, -0.0359755350062, -0.173509774639, 0.0169016272759, 0.0810715296313, -0.00470512681437, -0.0412851786978, 0.024020471456, -0.110462851316, -0.00458208133731, -0.0729339678933, -0.0199325882106, 0.00711599246809, -0.0671479281679, 0.0201998873259, -0.094847807891, 0.200070253227, 0.240673205119, 0.0025747166338, 0.110061653954, -0.0882564934629, 0.112716442223, -0.114243598094, -0.0381644794733, 0.0705147398374, 0.0859835815343, 0.0685867810776, 0.02867008918, 0.0749086304652, -0.161500302967, -0.0905549282206, -0.179043024091, -0.0576569620413, -0.00935693718385, 0.125363411208, -0.120007129947, 0.0541831593854, 0.10832143307, 0.104846403118, 0.0130735115229, 0.0804410065392, 0.035511060268, -0.230286687035, 0.071615405053, -0.0168756222416 }))
          (Vector<double, 0>((double[]){ -0.085892754848, -0.127301774146, 0.0932562206831, 0.0372910574702, -0.0654285919962, 0.124899139827, -0.100946824227, -0.0302669566755, -0.0677910498294, -0.050674887893, -0.238423246475, -0.143534911102, -0.184908688352, -0.021038842662, -0.226300670066, 0.100554577017, 0.130480571599, -0.150776716801, 0.0682691264602, -0.0366583330461, -0.0646630459624, -0.0295863967752, 0.0791029139551, 0.0427299946235, -0.026744477328, 0.0275597791707, 0.137411311772, -0.0613787989076, 0.0879335784704, -0.0455829760857, -0.208600355281, 0.115091225358, 0.0721449439817, 0.09945373962, 0.0303580493952, -0.0503155647057, -0.053067949348, 0.0254500142889, 0.1305649337, -0.0189236143342, 0.0919643932144, 0.105204159987, 0.142677553504, -0.220217782954, 0.0254030246799, -0.0995246395974, -0.0281587547646, -0.0829798959461, -0.218825310465, -0.0623221468023, -0.0187604309928, -0.0278474054148, 0.0848096811847, 0.149126711923, 0.0932001319873, 0.0993791348746, 0.0394952364092, 0.0489850635873, 0.0189938877803, 0.0975315260248, -0.0890282906799, -0.180008975353, 0.0414576375428, -0.0178772350977, 0.128344687112, -0.0451246010428, 0.0375040523381, 0.0924755388921, -0.158243731856, 0.0255916881191, 0.0595679603015, 0.0743424120005, 0.0193779008907, -0.149469810791, 0.0548514426221, -0.0899441252725, -0.0253407605513, 0.0434616624504, 0.05105457233, -0.0594401376784, -0.149724114444, -0.0359139298255, -0.191252750244, 0.0476769496475, -0.11283200223, 0.136536701195, 0.0105108428482, -0.0689700897464, 0.0913739287026, 0.0275316872117, -0.0109895689762, 0.0197386665109, 0.111069355844, -0.107792022549, 0.0194136706481, -0.134709583576, 0.0285875466099, 0.134034947602, 0.0220688849287, 0.0651230530988, -0.148090266605 }))
          (Vector<double, 0>((double[]){ 0.0503137818673, -0.0075243331982, 0.0294673403469, -0.092651368928, 0.00917098234279, 0.121067552296, 0.0354916267655, -0.0380173391665, -0.156644584718, 0.094313744017, 0.0799906894779, 0.0784117327535, -0.244650740767, 0.113412825105, -0.00301383544839, -0.0545651675709, -0.0512769442052, 0.00594973217601, -0.117666075022, -0.0662614751953, -0.161981154543, 0.0160610107609, 0.00486734666604, 0.081729667918, 0.065461385729, 0.0303727228959, -0.242591569011, -0.0681628091039, 0.171132649122, 0.0125576899417, 0.0257792547954, 0.0561497363372, 0.104482411131, 0.0495506228625, 0.0782801953031, 0.22626163667, 0.0465043500076, 0.0198934857673, -0.0594922736177, -0.0301469514387, 0.094337487699, -0.147385849119, 0.042618834115, 0.122406930302, -0.0693192735532, 0.0582596344877, 0.15665529091, 0.0318776840735, -0.0865006889759, -0.0142935226361, 0.108669528246, 0.132565673638, -0.130587613786, 0.00757112420554, 0.0592597042052, -0.0370995193551, 0.118018908346, 0.00834006419555, 0.0583512202687, -0.0885373669275, -0.103786462601, 0.0541617195125, 0.0388384922428, -0.0349280644337, 0.0472807491104, -0.0526270861533, 0.195978008966, 0.101897110725, 0.00251408146804, -0.152218160809, 0.103586557821, 0.0552041955624, 0.00352512647027, -0.179283571221, -0.183118200123, 0.0321364231825, 0.0772611557188, -0.18719153069, -0.0624442638034, -0.0738339773279, -0.0414874018633, -0.0105130038994, 0.041637650871, 0.141176388715, 0.186187704653, 0.154360066727, 0.2012065441, 0.234205385771, 0.0475331866628, -0.0268577350359, 0.0469497295376, -0.0789867877937, -0.106934601842, -0.0143538484674, 0.0826996884218, 0.0119342394818, 0.0336248009368, -0.0405018924892, -0.02737776968, -0.0401288371589, -0.113176959426 }))
          (Vector<double, 0>((double[]){ -0.156754210729, 0.00985770649054, 0.00664592146611, -0.0211268408742, -0.0617014925873, -0.113823420402, 0.0843234231252, -0.101975916382, -0.270254093824, 0.0761474157955, 0.0372550159334, 0.0743009939088, 0.0342429124855, -0.0459006382766, 0.0970478564516, 0.0175641361305, 0.153889684315, -0.129958301027, -0.15108056768, -0.0421050540114, -0.142871307116, 0.0798258391452, -0.0356302736002, 0.265269083915, -0.242392954887, 0.00108833798503, -0.0954120412836, -0.0590504573249, 0.00236168725117, -0.0393686537656, -0.072228778995, -0.0514910160831, 0.0452311737124, 0.137436676928, 0.0837693558337, 0.0612426784315, -0.120163683566, -0.00475310293917, 0.173594669497, 0.168550902358, 0.0523481766292, -0.105582050227, 0.0580865143444, 0.034986152391, 0.0271222622521, 0.0172924199443, -0.0775272769873, -0.0904439900195, -0.0185770190178, -0.0423211899746, 0.0490193250587, 0.0637729012121, -0.0954217397334, 0.108781180319, 0.099660777273, 0.0561131558757, -0.0111601702656, 0.0754305361559, -0.0495454825597, -0.0142220722251, 0.0159722566988, 0.0767694279408, 0.022592887068, 0.143468107938, -0.109819731836, -0.202561167052, 0.0074511842145, -0.0979209863041, -0.167952157679, -0.0410129755279, 0.0404217288426, -0.128531763704, -0.0590222648523, -0.0577211535744, 0.204997294875, 0.0133937583648, 0.101186488072, -0.03787240614, -0.00974962349315, 0.106372261801, -0.120450226567, -0.0126269871937, 0.119454367054, -0.00831363514231, -0.141648041374, -0.20284126539, -0.0570080348544, -0.00590508420562, 0.0399380656032, -0.00867628303835, 0.00397401395504, 0.0566526832108, 0.1895532497, -0.0687898097524, 0.121210215975, -0.135310967839, 0.00209404782573, -0.0223903773188, -0.002999519965, -0.15033736548, 0.0513906281207 }))
          (Vector<double, 0>((double[]){ 0.0764272079922, -0.0414312918497, -0.159832056985, -0.0958242559009, -0.0416840434985, -0.0685988473598, 0.115015964332, -0.00752860351656, 0.191841750801, -0.0101063111649, 0.0862663725252, 0.0951581214347, -0.0087245111951, -0.0773283207456, -0.0991103179037, 0.0302360744096, 0.123041928142, -0.0316646263046, -0.00913708759444, -0.0340021311706, -0.0287325782442, 0.122990233298, -0.0132070132664, 0.271239135186, -0.0525658562598, -0.158376834574, 0.016286081077, -0.208715974162, -0.195432626346, -0.0612629570662, 0.0215645128233, 0.0223941467351, 0.00576142139993, 0.0821251460961, -0.119131647173, 0.227093012479, -0.00564677207498, -0.0469858709235, 0.0431561129171, -0.024859060078, 0.135002540584, 0.0147314529214, 0.00744448684638, -0.0908653099542, 0.0908501820465, 0.157319061998, -0.0617022125605, 0.0496576870115, 0.0779707517482, -0.0870391257324, -0.0450014430581, 0.0365833444956, -0.187762310263, 0.107413685613, -0.0232134990179, 0.0718893300444, -0.0229655629539, 0.0526123700648, 0.0020303152282, 0.0116568536991, -0.0241638073209, -0.0492417515437, 0.0852289714835, -0.0248361434035, 0.133961581144, 0.137217736851, -0.0634407108682, 0.123156933062, 0.0819005544689, 0.0400116507294, 0.0734577856039, 0.134990193851, 0.148914617406, -0.0752482855253, -0.00532971403592, -0.0969320106065, -0.0588389597666, -0.0977995178721, -0.177481745737, -0.0854980130269, 0.2056636548, 0.0478068009249, 0.0800093134819, 0.0127056958704, -0.00503972759489, -0.132134594481, 0.135324199867, 0.0363422416485, -0.180260436018, 0.0640266547646, 0.0568352952517, 0.192580209981, 0.0885215025386, -0.0178879911153, 0.117570247023, -0.00565146345897, 0.0136793026933, -0.156276877931, 0.0981720281357, 0.0754587508144, -0.121938729277 }))
          (Vector<double, 0>((double[]){ -0.0130548704136, -0.0903045442866, 0.0348792003416, 0.0099485967186, -0.0623903956917, -0.0459459163707, 0.0529132542788, -0.0736968152763, 0.0728675311441, -0.0515433418117, -0.127605177725, 0.095052237211, -0.00639716244193, 0.0107331370096, -0.122578275107, 0.0519231799127, 0.0235900229787, 0.0784246975073, -0.0473327458691, -0.153684369854, 0.0919517523816, -0.110803705315, -0.0176357974566, -0.0387095431975, -0.0733451211351, 0.0449122557198, 0.141408276783, -0.00599117101614, -0.113706592806, 0.144499575708, -0.0999476818851, 0.0503672298841, 0.0842935922587, -0.337814482857, 0.000112008011745, -0.0128670797714, 0.0048101196414, -0.0167233111719, -0.0184079232831, 0.0168853788704, -0.0200722524446, 0.103317604566, 0.228132278324, 0.0371070438786, 0.074174748825, -0.0189226243284, -0.00671657840623, 0.0198650623158, -0.0192241136737, 0.0668487649633, 0.00239202769005, -0.167858236266, 0.00154950690131, -0.1298334473, -0.23791168561, -0.0802210675429, -0.124011219247, 0.2540874027, -0.0563058591185, 0.155227290497, -0.0213497283492, 0.00169774225819, 0.00800932658279, 0.0612339497264, -0.0995386913689, -0.0392215210575, -0.0924256822483, 0.0832851083252, 0.0294522834493, -0.0614162723836, 0.0801603465865, 0.0375102289338, 0.129046616461, 0.11381835286, 0.293462269089, -0.151558971653, -0.0576739059495, -0.138460543582, -0.044691495104, 0.0160093758818, -0.0716573426567, -0.201111584324, 0.0491443934728, -0.00860896415927, 0.0397539807689, -0.137900478012, 0.0165342352038, -0.0203664765493, -0.0694335330136, 0.0471183290745, -0.0960437608796, 0.0261251344433, -0.145289018971, 0.0606837208609, -0.150548805838, -0.0729683402965, 0.0188759602453, -0.122959757785, 0.109676661377, 0.0749958461339, 0.0593429160165 }))
          (Vector<double, 0>((double[]){ -0.129165462883, -0.129870021969, -0.0968118212, -0.00154557632431, 0.0940404046218, -0.202765165643, 0.0379914436605, 0.0233948765305, -0.0779266235138, -0.0554025621516, 0.0666868091701, -0.0582892195566, -0.0114353499965, 0.0912201735183, 0.0506367906799, -0.00101204514282, 0.0860253829285, 0.117714771307, 0.224175907849, 0.0332003228269, 0.0113369779817, -0.0394460994021, 0.0249020311951, -0.00354298414602, -0.0756737562154, 0.0741465187386, -0.0601216873842, 0.0515751090928, 0.15767725341, -0.0369108540659, 0.0283059736335, -0.00529192927814, -0.0259124295593, -0.0594154181896, 0.0594358259755, -0.0135556022977, -0.00995919398731, 0.210784456259, 0.088403383575, 0.0871633861777, 0.0750940348616, 0.04846302403, 0.0435867256871, -0.077563652468, 0.0379318265406, 0.0369868802081, -0.0934567216905, 0.150026621711, -0.106481423467, -0.220851690055, 0.130277452036, 0.135798858533, -0.0243943572253, 0.162572117665, 0.0158770176364, 0.056839896662, -0.071219024402, -0.0350580896511, -0.14769270606, 0.161587057754, -0.21916355348, -0.0287231519284, 0.0245092477108, -0.0396073277911, 0.152339067351, 0.0430394396312, 0.0498651091418, -0.0288931464086, -0.128238143213, -0.00528828643417, 0.176827319115, 0.0422658480865, 0.0726475202071, 0.17938841556, 0.042842158431, -0.0667149817732, -0.272580910752, -0.072921137064, -0.256562090722, -0.0462557506969, 0.00523410221246, -0.0703972878406, -0.076366373786, -0.094882189802, 0.0727194278608, -0.157426150647, -0.132571981843, 0.0444312498534, -0.150510338886, -0.0163176807043, 0.0208200578456, 0.0100711237693, 0.0284488805731, 0.0575238323295, -0.0949938474675, -0.0316550493275, 0.0965274310227, 0.117012945372, -0.0519064228553, 0.00330154347301, -0.0809588259926 }))
          (Vector<double, 0>((double[]){ -0.018460762309, -0.000491618565891, -0.0335888695854, -0.00949359595843, 0.0459155775054, -0.0602092513795, -0.177614607769, -0.108039520606, -0.072371369131, 0.00481768437428, 0.10339929364, 0.0524143436168, 0.123303026043, 0.0548796997409, 0.0558759128687, 0.117638426491, 0.0999498361145, -0.0293474038493, -0.0829444903722, 0.0539117353688, 0.0375504953698, 0.00602328483087, -0.345333738231, 0.0165437396034, 0.046168198837, -0.038509951918, 0.100340978582, -0.123823159472, 0.10706257736, 0.147442456307, -0.0505231220977, 0.0356092204153, 0.0834120390692, 0.0193377984683, -0.00908039368808, -0.183906006819, -0.113628405397, -0.161678864974, -0.00408411983387, 0.0989435259472, 0.101736274827, -0.150029247639, 0.0122437224694, 0.107732523947, -0.0705886139869, 0.105963201665, 0.198546881619, -0.147127402022, -0.0425668397176, -0.0304895826933, -0.0482910288165, 0.0308310147798, -0.10756874494, 0.0423555517942, 0.0794174333426, 0.151148949998, 0.00342091574672, -0.0405824257542, -0.00485446566167, -0.137021685668, -0.00201337692489, 0.175377573752, -0.00269862469953, 0.0178407100738, -0.045432445297, -0.0526145762103, 0.122158612107, -0.0986790253002, 0.198083827386, -0.103592265631, 0.0779076895795, -0.146036895903, -0.180983626645, 0.0667023291337, -0.0103158576432, -0.0242633809536, -0.0577669346611, -0.067492476002, -0.0519662881987, -0.0713526997298, 0.0789256751122, 0.0871156655685, -0.107929828413, -0.0116745985432, -0.0659558950857, -0.0676220339308, 0.190568571967, -0.122310025813, 0.0939000002978, 0.0424848681656, 0.0803216893429, 0.0555109092902, -0.0928306501163, 0.201438898899, 0.0303981607294, 0.0175897346587, -0.0417149448761, -0.136927959052, 0.0829148949682, 0.0422477659648, -0.172950618694 }))
          (Vector<double, 0>((double[]){ -0.162645330222, 0.163291368533, 0.0298795815229, 0.0221489911475, -0.202468561376, 0.0420790521536, -0.0144086175599, -0.108690375807, -0.0432949503576, 0.141797997252, -0.00484336710433, 0.00295636975595, -0.0715789981668, -0.0709777301341, 0.0946099657514, -0.0609791987921, 0.0459737294284, 0.112197373345, 0.0527459760112, 0.0201587464365, -0.0461561115652, 0.0214318103832, -0.0263911087743, -0.0214953051389, -0.0666541443987, -0.0871642162306, -0.0170266258387, -0.111709771999, 0.15542235655, -0.0241413160348, 0.0575738685514, 0.0592977414268, -0.0199663918212, -0.0532635836899, -0.0179327833664, 0.0218167128328, 0.0308259616717, -0.0953620301651, -0.141039015419, 0.0152628874988, 0.176381829762, 0.0123778138671, -0.147483948376, -0.0590222067283, 0.098587409573, -0.0482767185247, -0.00479158308886, 0.0886992208161, -0.0829457323026, -0.195045369214, 0.0213696191127, 0.0635011443702, 0.175158929341, 0.0904780012087, -0.069616260323, -0.0615041075241, -0.0433963204239, 0.125846768208, 0.103591101669, -0.014777849054, -0.0506173718559, -0.182630503892, 0.0718793090506, 0.125337836655, -0.0338713914373, 0.0419273970923, 0.15620057444, 0.152170820524, 0.041230887546, -0.0485704781571, -0.114285053754, 0.0290997197928, -0.259445200885, 0.0279887663138, 0.0661946270832, -0.107384103872, 0.146191211002, 0.171950289665, -0.0292049260569, 0.0485316055655, 0.167306489319, 0.0921124969918, 0.0575504433678, 0.0845126002168, -0.114537628957, 0.116270681124, 0.14397377781, 0.185426291328, -0.0647413540711, 0.111977358081, -0.0156238812259, 0.100866876004, -0.156673645592, -0.0601833256648, -0.0704812602826, -0.0239357114949, 0.16338646916, 0.0727712830194, 0.11785573536, 0.130855715345, -0.12536379299 }))
          (Vector<double, 0>((double[]){ 0.00773130234778, -0.058798989933, -0.165544786265, 0.0807577757331, 0.0209905301387, -0.10377290337, 0.0532667886062, -0.00851909889536, -0.0131070876507, 0.0298818107432, -0.107320730558, 0.0217400925663, 0.110685492546, 0.200029026837, 0.0882379443615, -0.00919660311326, -0.251794236824, 0.0245013781758, -0.0541456001048, -0.0560735299425, 0.0122693737479, 0.114534626319, -0.0769435216843, 0.0673874367115, 0.259318276834, -0.0383588267212, 0.0686318824137, -0.194981547994, 0.0466017731707, 0.00111596206141, -0.0689260062562, -0.0528420920021, 0.0877484455786, -0.0707252602882, -0.0293231835188, 0.131941261139, 0.118544991202, 0.113568890185, -0.0346550588939, -0.0717501441092, -0.154342989847, -0.179937987623, -0.14013897032, 0.0283693959361, -0.0352677366891, -0.115021609459, -0.0528774481087, 0.0609553151241, 0.099529028284, 0.0829377169496, -0.00377025176731, -0.0366245444068, 0.00587976586647, 0.0515500516364, -0.0502512253309, -0.0568459902386, -0.0276271552966, 0.0531879709161, 0.00836685147304, -0.0594890924019, 0.0144558741532, 0.153220555013, -0.179259234075, 0.0126615520385, -0.0356345565944, 0.1163669404, 0.0541629763604, 0.04082491445, 0.0549783601904, -0.143690361659, -0.0576071652686, 0.0827165137334, -0.0309031426194, -0.183677916277, 0.0896948541113, -0.141905341458, 0.125911350624, -0.0442341693511, 0.0458567892176, -0.163511044338, -0.00228921192843, 0.113055154076, 0.139574489545, -0.134976904597, 0.10524591422, -0.0326692505347, -0.187100466574, -0.0289924727574, -0.0206319196199, -0.0889253725309, -0.106172031382, -0.00193865538025, 0.220887792098, -0.0336945760571, -0.149994595007, 0.00705090279819, 0.0709850447251, -0.0670223217916, -0.0530412034305, -0.131322995504, 0.160293342178 }))
          (Vector<double, 0>((double[]){ -0.100424151979, -0.0352101752503, 0.0279129030146, 0.167990668224, 0.105362093047, -0.00402209099017, -0.107692099534, 0.0313781644268, 0.0119435567759, 0.10801512706, -0.122123729395, 0.168780619844, -0.0328402742007, 0.114887630966, 0.108165866326, -0.0124016447877, 0.0267861775681, -0.0439093054939, 0.0149719542064, -0.11535806434, 0.0983369862672, -0.0830390246166, -0.0534873376982, 0.131973869632, 0.0912549520202, -0.223439001998, -0.00188979319181, -0.105451112463, 0.0528121563759, 0.0535274696882, -0.0631147067351, -0.0814617519775, 0.0168094980218, -0.147230350326, 0.0844366784759, -0.108323895169, 0.0476357422574, -0.045294638674, 0.0761310796137, -0.120341947757, -0.00820992221416, -0.0836488688759, -0.114455330127, 0.0584903943321, -0.0576058368061, 0.0520528422832, 0.139807533733, -0.0173130971759, -0.0705285567914, 0.15469040086, -0.0319107399113, -0.10765497187, 0.0152308735799, -0.071995293105, 0.0350771710175, 0.044826128799, 0.0631163152108, -0.0782573498065, 0.0499008671094, -0.121401384718, -0.138626029722, -0.0734906780766, 0.0829672509576, -0.116109329402, -0.0221712051086, 0.148676858775, 0.0230224009196, 0.0754936635578, -0.210081806378, -0.0447488455663, 0.0645765422508, 0.139601320033, 0.100061266816, -0.0642369389574, 0.163549315836, -0.011615555943, 0.0701308627211, -0.0385601232894, 0.051548642429, 0.07362862375, -0.0405172831431, -0.059935277156, 0.00982959650605, 0.157681925176, -0.0158210030966, -0.160393434645, -0.00662994689373, 0.025359892912, -0.0841919574572, 0.0761297218102, 0.203137079229, 0.0277130611515, -0.134297885553, -0.183965740759, 0.065339807543, 0.330779779626, 0.0193511107518, 0.0557795689156, -0.0990489786205, 0.00151861061344, 0.183935028755 }))
          (Vector<double, 0>((double[]){ 0.0747352043303, 0.0752947002934, 0.0492633373488, 0.0104082338622, -0.0351812743057, 0.057598892496, -0.203756013182, -0.215797679817, -0.0701651157662, -0.0551647290355, -0.180896620872, 0.0793709041802, 0.107220064879, 0.119658423255, 0.155717477619, 0.0794749483964, -0.0512183489864, -0.0367170598654, -0.157577091088, 0.179012884581, 0.0301081583446, -0.0405628490476, 0.104721879158, 0.143192277784, 0.0102462868785, 0.0721939658464, -0.0994353277967, -0.00118548540296, 0.0869614436983, 0.154775533752, 0.0573600728066, 0.159767592596, 0.122473320651, -0.0577233546575, 0.0374934511096, -0.00340860248121, 0.0210026056044, -0.0909020048928, -0.153797348321, -0.0158233066086, -0.00318659890415, -0.0610329246423, 0.137747277213, -0.087712561612, 0.236731933648, 0.134275485626, -0.11759413569, 0.112111399373, -0.0955445931559, -0.0790236599369, -0.0453537185919, 0.101707772612, 0.0571663651031, -0.0404522680104, -0.0210200227394, -0.00477503153068, -0.0849350626911, -0.142350267172, -0.192081739961, -0.0977174019256, 0.126072807199, -0.00639805404763, 0.0460801846452, -0.154895220971, -0.0780977580404, 0.024707729821, -0.117371053538, 0.0203097152809, 0.00864873830231, 0.00562119351537, -0.140325804741, -0.0696478495069, -0.000381460136582, 0.00272667081061, -0.0459156428001, 0.12489009626, -0.0804865844866, -0.0950796937078, 0.0469713856553, -0.0308902111876, 0.0921721757997, -0.130880821349, -0.122992626109, 0.135009373277, -0.0099532454561, -0.0425804850365, 0.063908710097, 0.0254740702205, 0.110502498572, 0.0355806119387, -0.230462723713, -0.0561506667207, -0.0568319361147, -0.00926576153377, -0.048830939397, 0.0239426094124, -0.00399093172393, -0.00429711415533, -0.0841339945286, 0.194478055197, -0.137587457585 }))
          (Vector<double, 0>((double[]){ 0.0312603695539, 0.150400714767, -0.0856036817392, 0.0475767896165, 0.0901571847938, 0.0518466504864, -0.0271622414283, -0.11415730718, -0.148058135226, -0.0172723107069, -0.0437708790167, -0.0703571746422, 0.0404659979826, 0.0316978895222, -0.0363295094798, 0.0326917304131, -0.151243031297, 0.0537331646939, -0.0588229208606, 0.0853190579142, 0.0902201072823, -0.0688975507524, 0.00442594330551, 0.048549654509, 0.00728219863493, 0.0577344595485, 0.0135113829372, -0.0692357930355, 0.00912700869348, -0.0474248885036, 0.0149105064313, -0.0974256859021, -0.24412252961, -0.0755587287055, 0.0487092592247, -0.00272724141343, 0.033243926234, 0.310989380982, 0.0161577627125, 0.108521009659, 0.150997446561, -0.0909778785862, -0.0552621259908, -0.173453730626, 0.0809486509184, 0.0658635969594, -0.0533307852844, 0.111550433531, 0.10354625039, -0.0272208446792, 0.0207893407117, 0.180704854975, -0.0754027975564, 0.0214454365842, -0.0447741046628, 0.113093284134, -0.0468025975608, 0.0242962675241, 0.0704103120817, -0.283090959808, -0.00859842033397, -0.0508299721907, -0.122858169439, 0.0233815610166, 0.0386726948001, -0.173471270462, -0.102107426621, -0.0618068551933, -0.134265796894, 0.016410309378, 0.18626613101, -0.087100558279, 0.135381126314, -0.0116005344241, 0.0342269664723, -0.18205625215, -0.0761536939703, 0.115008467845, -0.18463227659, -0.00475879031725, 0.0602980526375, 0.137778712626, 0.0589768402171, 0.0105072362569, -0.15301759667, 0.0489701255782, -0.100289574462, 0.0248222206728, -0.0367775929242, -0.0525252466752, 0.151250844509, -0.0702644655679, 0.146311056814, 0.162463469213, 0.0480426697036, 0.0599923242209, 0.0445915756252, 0.147393151492, 0.0506356131682, -0.0775504098652, 0.0581570381998 }))
          (Vector<double, 0>((double[]){ 0.0468030043774, 0.159455819245, 0.165319770427, 0.203417181536, 0.150930334454, 0.0591737918384, 0.046161305717, 0.0569267927059, 0.0548178648461, 0.125729300002, 0.0628547946428, -0.0224162333298, 0.125901856057, -0.188213218115, 0.130619803772, -0.0654704538465, 0.0230976931701, 0.0782231168522, 0.125333907953, 0.0212257142211, 0.0907640949621, 0.00542041922955, -0.0124212087768, 0.0720113072291, 0.138425529305, 0.0915973860217, 0.0176588975479, 0.0770435121207, 0.114977791024, 0.0768237148246, -0.153076645632, 0.161290972603, -0.095727816882, 0.0685848080572, -0.0691715052375, 0.0619308809814, -0.0273462056782, -0.1436431491, 0.192055503446, -0.0806502977895, 0.10535192621, -0.188431973526, 0.019336421154, 0.0974122934963, -0.0403810718453, 0.155013960826, -0.0781754684771, -0.0372195726083, -0.0766386975248, 0.018764726536, 0.0378732001364, 0.0739745041924, 0.137253417252, 0.0690428458322, 0.00113277200556, 0.0659125893036, -0.147324182119, -0.0866404302979, -0.085856040895, -0.048112238172, 0.000985940969649, -0.0763048353091, -0.053615469601, 0.0789096460088, -0.00197017853802, 0.0926151859558, 0.125212532379, 0.112105663039, 0.0219791338241, -0.0143034337413, 0.203677295066, -0.0607818178724, -0.07174599963, -0.0427511971469, 0.0399786802176, 0.0845706121534, 0.0776366494899, -0.0334217550646, -5.81141130392e-05, -0.0176420248092, 0.191254585356, -0.128835076318, 0.0868094446325, -0.162050717028, -0.00153988500653, 0.0213982737075, -0.224316507009, 0.0178418700555, 0.0883743345759, 0.102027203936, 0.061878056248, -0.0833556196915, 0.0523564860197, 0.0516575708324, 0.0146978340778, -0.0334589431051, -0.0960676251701, -0.0734794709237, -0.126005603783, 0.193969094261, 0.1331874964 }))
          (Vector<double, 0>((double[]){ 0.0962481793613, 0.0353351076593, -0.0124390778648, -0.205701628975, -0.161705706838, 0.0190245554164, -0.251955304161, 0.164230450115, -0.0799893704207, 0.0714649143687, -0.110062462303, 0.00568158702897, -0.0081994224233, 0.0641255104422, -0.0408520733844, -0.0550721314183, -0.068930786735, -0.0534644575985, -0.0135202520206, -0.123942521256, -0.10956313678, 0.0210818486355, 0.105244822669, -0.197836134764, 0.0389987796066, 0.0559021896604, -0.155608907453, 0.0104529512746, 0.156593666646, 0.142489558198, 0.0737912728382, 0.0987066701591, 0.0230018094637, -0.285134693481, 0.00814482341067, 0.0258330782812, -0.0237488241195, -0.267071139728, -0.112944466456, -0.0755923286899, -0.0697655569996, -0.071627635716, -0.05783757491, -0.136302035857, 0.00118307049993, 0.0573193658454, 0.00930951438451, 0.0955546039234, 0.0768399990035, 0.0895140989504, 0.0385488984478, -0.00158441683545, 0.082349867137, 0.0649663130135, 0.0440899283552, 0.152940427623, 0.0503119733145, -0.190135759249, 0.0719657309302, 0.101202909357, -0.00654252084081, -0.00907067482515, 0.0865327349298, 0.0422410269896, 0.0140976573185, -0.000988219112102, -0.0049509924465, 0.136989377153, -0.203171202048, -0.0178179286844, 0.00725661350917, -0.0717689621265, -0.00113163550752, 0.0584617917446, -0.0980784514618, 0.105897989164, 0.0651232987723, 0.0829343809076, 0.0738119912025, -0.178524197865, -0.070888328049, 0.129840312121, -0.00668838597682, 0.0200281598342, 0.0590214225237, -0.0589408413728, -0.00940276007134, -0.0209705466697, -0.0869465640269, 0.155866734427, -0.129701430846, 0.00184235811447, 0.00770333539112, 0.0200201506305, -0.0327694577265, 0.0508459836568, -0.0583913166522, 0.00211620685408, -0.0820038677153, -0.171400360008, -0.17179865892 }))
          (Vector<double, 0>((double[]){ 0.0129943095363, 0.0744408382322, 0.0234652393461, -0.153554001201, -0.0184912385092, 0.0686617755647, -0.0494575569822, 0.112512194747, 0.147551448398, 0.0948023001476, -0.114978159871, -0.090156309638, -0.0604795900948, 0.0628871045087, 0.0138144258442, -0.0314689800399, -0.0836159998755, -0.0405404949304, -0.137882090305, -0.144214358673, -0.0992083723034, 0.130902980213, -0.00463989593163, -0.012727259258, 0.161170332049, 0.165785917546, -0.0627561163229, -0.00758324241162, -0.0175344278871, -0.0614512393561, 0.0298659833335, 0.0238241629139, -0.0286352483656, 0.112218837057, 0.0215837108429, 0.0827601190199, 0.0532349376536, 0.0903665398739, -0.0134316827919, -0.0608268204272, -0.0102859148367, 0.105409739268, 0.081239356441, 0.0195856233368, 0.0283481234133, 0.190284965761, 0.0134312761244, -0.0490412139676, -0.175487468252, 0.224761500176, 0.197748520322, 0.0204025910377, 0.060614510806, 0.0334146846919, -0.0875938049495, -0.0697737959335, -0.00792892115946, 0.0116475930891, 0.0462441734139, 0.0669662850746, 0.0201093398462, 0.0238178062249, -0.065486375297, 0.18655335242, -0.0476842739549, 0.0176105498023, -0.00804958116693, 0.176588581124, 0.0503534305426, -0.108466932864, -0.108373785911, 0.121990982397, -0.147694947522, 0.00424589652886, 0.270607784515, -0.182720032307, -0.158034641571, 0.0535187904531, -0.10409308959, -0.0575544947118, 0.026136050547, -0.122123544415, -0.110050907517, -0.104453018513, -0.0739376622117, 0.223929916032, -0.0830225412379, 0.0224287459782, -0.193462740483, -0.014164971444, 0.08723952558, -0.0582944764308, 0.0468800205182, -0.0165434720094, -0.132165454421, 0.063460591991, 0.110836343809, -0.0400240461363, 0.0363552379181, -0.06567148096, 0.181910735881 }))
          (Vector<double, 0>((double[]){ -0.171668608329, 0.146270967936, -0.013848165114, -0.0275785219426, 0.111188972963, -0.0634699847233, 0.0687204701059, 0.0301333450371, -0.00353224201176, 0.0747333690383, 0.0130830923434, -0.0617960197685, 0.0431716445497, 0.0311953551861, 0.0132433140712, -0.0708256315794, -0.00801031234485, -0.0265020863354, -0.112461776825, -0.0564624266442, -0.0393898151197, 0.136058254405, -0.13843712604, -0.022518401074, 0.03453892283, 0.111312103248, 0.0931977260551, 0.0418671500468, 0.159759068552, -0.0163125563765, -0.0794537567284, -0.00443836195291, -0.0177980610608, -0.0332116522324, 0.0390902124099, -0.061588796092, 0.0775456965681, -0.160067395924, 0.0191508096503, 0.03272346204, -0.0659949073353, -0.104224298405, 0.011902357498, -0.238624201445, 0.133556747715, -0.0158478429587, -0.0129660206301, -0.035062190245, -0.22936837766, 0.146226055899, -0.165404989758, -0.0748141284783, -0.0250450654608, -0.0862227889886, 0.0165182273615, 0.108272859169, -0.0223931852163, 0.0139731797658, -0.239198156082, 0.00877568578244, 0.0150883669822, 0.0279514394961, 0.198617742629, 0.0107687330384, 0.0237425863285, -0.0615866301705, -0.0145097286695, 0.0943044271035, 0.206665540355, -0.10060453877, 0.0351007272053, -0.0821696297621, 0.111260240474, -0.12781471716, -0.167627403596, -0.136893407265, 0.0794722077203, -0.0371113952679, -0.0400226261078, -0.222681575581, 0.0770821289205, 0.000686897401266, 0.0258077368963, -0.00225611861682, 0.0295755489901, -0.066580214614, -0.0384316199259, 0.0831700581818, -0.0178708188918, 0.180795408094, 0.0694264801803, 0.0628742468095, 0.158005771505, -0.0118617407477, -0.0145732786601, 0.194446737463, 0.0567471794953, 0.0355359959585, 0.150348576702, -0.259583383278, 0.102405018626 }))
          (Vector<double, 0>((double[]){ 0.00336582730504, 0.131933249254, -0.023919087189, -0.116586762432, -0.101369367301, 0.105686452189, -0.00818195180144, -0.0766797138034, 0.129131076803, 0.0795630425733, -0.0188052251896, 0.0943603692886, -0.071266190408, 0.112754613722, 0.0109989111828, 0.102657454609, -0.0181442203496, 0.151018015681, -0.14610679432, 0.170959325922, 0.0779535258699, 0.0567043320194, 0.220875973418, 0.135121718874, 0.0972262697565, 0.271768698853, -0.158783111451, -0.103380546379, 0.0477772124104, 0.170417796713, 0.0657206352023, -0.090997780082, -0.0163818504651, -0.106729599701, 0.0361480813717, 0.13420311451, 0.0241968058741, 0.0496635544515, -0.0404146981583, 0.0715247234333, -0.195273182799, 0.00529965819295, -0.176807709469, -0.0240258951823, -0.0361865165658, -0.0299018478349, -0.0537293446957, 0.0124945652036, -0.0515725159396, 0.161748640144, 0.160146199905, 0.116031373256, -0.0218240277845, -0.059892097178, 0.00319952553697, -0.0530126598225, 0.0645208640607, 0.155703594335, 0.0846822228592, 0.00335392698393, 0.0234205385267, 0.17298501662, -0.00583084849099, 0.0197706174935, 0.099299402781, -0.00177328379975, -0.255792122995, -0.0379376193146, 0.0781204940853, -0.0231244370439, -0.046010123752, -0.0287019142057, 0.0867569994302, 0.143187010786, 0.0644943173471, 0.0462092044969, 0.129218070299, -0.114434243968, -0.0192623095306, 0.0040885965405, 0.0210048293115, 0.0136292693134, -0.104199075162, -0.167666498639, -0.069954404096, 0.0543906562913, -0.000198112215906, -0.0361261608682, -0.0733046986178, 0.141319943022, 0.0465374998793, 0.0620448851347, 0.0780071526131, 0.112396814862, 0.12853666886, 0.129764337534, 0.0168254845318, 0.112036802283, 2.91574033272e-05, -0.111225390098, -0.0398843014471 }))
          (Vector<double, 0>((double[]){ -0.0199849369451, 0.0758235547106, -0.0762687074748, -0.0287240921256, 0.0931562040141, -0.0038629054821, 0.0154166800462, -0.00832146025122, 0.21149077996, 0.0401746937264, 0.019084458995, 0.0609088879501, -0.250532513284, -0.0782817586381, 0.0820729046353, -0.0328009662232, -0.020591583767, 0.0377860959017, -0.0761858577632, 0.0327483619924, -0.218406098746, -0.0137535174556, -0.0841942626798, -0.0516594221803, 0.0481868626062, -0.0075718265921, 0.106013450689, -0.0669756055724, -0.0349154413379, -0.0612597511094, 0.203078733691, 0.153823824064, -0.0159473137262, 0.0036414256278, 0.0385286646805, -0.0832828772185, 0.0709585922447, 0.216183465332, -0.0703838535652, 0.193458354274, -0.196135510476, -0.100701388991, -0.127203535709, -0.022733871107, 0.0199395784798, 0.0115332591567, -0.0678444343761, 0.153021944624, -0.0480935318606, -0.0588113569391, 0.196907790742, -0.139854578345, -0.0469981781436, -0.0512569234567, -0.11267974475, -0.0837283205939, 0.086154028805, -0.0621946115655, -0.0234604183465, 0.111275885845, -0.0485148968789, 0.0508875467974, -0.0503550471509, -0.159590489793, -0.119857930049, -0.163890157097, -0.0037007321172, -0.0428110869362, 0.0782553748632, 0.0202959349091, 0.125462860349, 0.0638906073155, 0.0237143080204, 0.144913808787, 0.100622831362, 0.0398544031935, -0.0499687991313, 0.0443411633207, -0.0510784596277, 0.142508225964, 0.124328004361, 0.186837595942, -0.110560394001, -0.0446965252871, -0.230321621404, 0.0502300626913, 0.0709399798108, 0.0401271747651, -0.160555061111, -0.0508612142776, 0.0781201385047, 0.0425059126337, -0.0358265164096, -0.0600645966051, 0.0947187711322, 0.169252359923, 0.0164898776408, -0.00384432209702, 0.106596698147, -0.0673896378474, -0.0485317961116 }))
          (Vector<double, 0>((double[]){ -0.0565383262678, 0.0379860236438, -0.0455954602947, 0.0020631202676, -0.00408628045657, -0.048032561848, -0.0649306035204, -0.00339307992603, -0.00179938826821, 0.128311887986, 0.097150115939, -0.143085471752, 0.0694539231937, 0.00840617996119, -0.0633851547437, 0.103629289097, 0.237644666098, -0.0926785183158, -0.0610725588376, -0.195741787845, -0.108217315847, -0.00701139637395, -0.0624452928413, 0.0356663327382, 0.0408310167488, -0.132429236152, 0.0284916307628, 0.0514981978826, 0.0726699174645, -0.00837575851464, -0.0354942008343, 0.10323358439, -0.0639030206291, 0.0371158254546, -0.0334369883978, -0.0375469884946, 0.0334001838652, 0.171573466241, 0.163226623995, -0.0580495141883, -0.140117773512, -0.0204197574387, 0.196778347826, -0.0343804754148, 0.170484559356, -0.0170638819669, -0.0525213236286, 0.0285270068724, 0.0248052366304, 0.154061897363, -0.111576021618, -0.059345713909, 0.0344254674109, 0.0870759093022, 0.0187364363085, 0.0374179189841, 0.13572898145, 0.00151119482215, -0.182695889688, -0.247517485605, 0.107114246551, 0.061756499945, 0.034089853344, 0.0910338815018, -0.0366077569469, 0.0961092133723, 0.0410216116395, 0.0616283975571, -0.0315755130604, -0.097200990463, -0.0704169307436, 0.15454417454, -0.160202596342, 0.0841997218913, -0.001665092337, -0.23157531581, 0.0147711957444, 0.0771174021322, 0.00969615871071, -0.052359648233, 0.16367466892, 0.0449542737639, 0.235278913696, -0.107270540031, -0.220414649748, 0.0349783705188, 0.0325139890649, -0.0539194445468, -0.0108920220518, -0.00292126544941, 0.0768160097074, 0.0589549029451, -0.0366028157592, 0.106637329872, -0.0849151082801, -0.0758111019639, -0.158882107691, -0.114866114285, -0.0650057547661, 0.136723861957, 0.0996680114653 }))
          (Vector<double, 0>((double[]){ 0.230701615112, 0.112386703072, -0.0684196145937, 0.0257560796208, 0.0966518240261, 0.102932083437, 0.0128563516874, -0.0221607809119, -0.0548327795063, -0.0628577192909, 0.00964918795766, 0.121151950929, 0.0928980340365, 0.0506480899283, -0.0555904703324, 0.0475327236605, 0.12610195891, -0.12435928511, 0.191470916325, 0.0269549752135, -0.0502777952447, -0.124023250771, -0.0953276439425, 0.0201510951851, 0.0591795916618, 0.148223058958, 0.0184355866192, -0.127452032011, -0.0326059321456, 0.12598638996, 0.00740061042184, -0.101044792751, -0.027459349797, 0.102096963814, -0.0166487643691, -0.0476722692446, 0.118244776063, -0.101794807185, 0.00612027582174, 0.0286056482822, -0.163978140067, -0.0834179654339, -0.0152784947571, -0.147562230022, 0.00260209168459, 0.0402237675734, -0.100709636927, 0.0664223602023, -0.081689811866, 0.119025680593, -0.0400953738476, 0.0915575806008, 0.127718287228, 0.0530367106271, 0.161988638807, -0.115340686278, 0.0176971688807, -0.0504905042693, 0.0544366453111, -0.0879254185977, -0.0461803061233, -0.0882401452346, 0.124903045912, 0.0764127002015, -0.260084472489, -0.0328361115138, -0.0903827235109, -0.192230986507, -0.0928315688302, -0.0477937689356, -0.0485347383065, -0.150397635579, 0.0985329208513, -0.0316543534338, 0.116069141218, 0.104826801986, -0.0510884980575, -0.131766037025, 0.126405937384, 0.133314477384, 0.0743737142515, 0.0415806123658, 0.0460639496741, -0.137900533907, -0.0122023390049, -0.0373514669597, 0.184333359341, 0.033562200474, 0.0796180696452, 0.0299555925366, -0.00151558938299, 0.20792426547, 0.13613134147, 0.130324946215, -0.0284521198724, 0.181495411995, -0.0683343378205, -0.0600786170121, 0.0630261742779, -0.0641806903081, 0.139182301861 }))
          (Vector<double, 0>((double[]){ 0.0936815550241, 0.302947283288, -0.00457330001, 0.213189419693, -0.0919830897829, -0.0553168174148, -0.0376991458152, -0.0407784371914, 0.202169522446, -0.149631479455, 0.106065645874, 0.0178040610988, -0.228334498336, -0.0594437949758, -0.0814128459072, -0.159047596341, 0.0253726357686, 0.12864422057, 0.220724294308, 0.178859949797, -0.0896070048349, 0.00555787396358, -0.153505114367, 0.135680989563, -0.0664145823255, -0.0583260160729, -0.103113001467, -0.00726468282088, -0.0916293695797, 0.0966789808413, -0.159073149847, 0.0230207057852, -0.0931800391133, 0.122378479341, -0.00618878358469, 0.0116831722069, 0.0265449423751, 0.133515049982, 0.0218555212538, 0.101884649527, 0.0525399644159, -0.0664423760778, -0.0461856107724, -0.0458374197992, 0.0938924630787, -0.029141272178, 0.0107672631662, -0.105132963279, -0.113383797095, 0.0597689828167, -0.0666015583028, -0.0680046693549, -0.0235103237945, -0.0305826536143, -0.0541595942305, -0.00323631310809, 0.135072079044, -0.0371756005087, 0.0668900313598, -0.139873641006, -0.0534910887822, 0.0100052324599, -0.094837787139, -0.106937461345, 0.0921838333002, -0.120485935036, -0.124063760671, 0.0549773250258, 0.159503676862, 0.0239323727287, 0.0426529218358, 0.0697938949599, -0.0668037222224, -0.0558553980579, 0.133809277646, 0.0563260063396, -0.0890374730496, 0.0394566794101, 0.0922681275297, 0.0691607341817, 0.0305899566958, 0.0511671062878, -0.0674354519688, 0.0776247327644, -0.125056401905, -0.0496825892204, 0.00500853494739, 0.0203823117793, -0.0249240963788, 0.100698790519, 0.0778399468451, -0.0519148230716, 0.183959821435, 0.120557208085, -0.128702212805, -0.0601497827193, -0.0414957967394, 0.176263078025, -0.0708324585087, -0.0185848708802, -0.0041206818761 }))
          (Vector<double, 0>((double[]){ 0.136647080104, -0.0623818249341, -0.0230183666008, -0.0728315335688, -0.183520808136, 0.294833451035, 0.0178688584062, 0.0125299802746, 0.110749811257, -0.00489663590831, 0.0955871349452, 0.201411391283, -0.000961812618521, -0.0870927272387, -0.036397371471, -0.0291687935895, 0.313231575465, -0.0272104709089, 0.0981752662281, -0.048668851047, 0.0125589464083, -0.0374143692224, 0.0256487159635, -0.0819228447531, -0.00528683279817, -0.000393834775584, 0.0450289154285, 0.0773449680448, -0.0166829402105, 0.101475017815, 0.0654968090559, 0.0373580203172, 0.0545789353555, -0.0322461637176, 0.00693615602656, 0.133673095478, 0.0604961343057, -0.107058791209, -0.0182833259723, -0.0372397750105, -0.0723869234477, 0.146906194219, -0.056288392068, -0.0241999869132, -0.158038871088, -0.0477816480498, -0.0368087225178, -0.0811585853729, 0.115384276213, -0.056704657947, -0.030439773575, -0.140684970014, 0.085654859168, 0.0835706803597, 0.118783263479, -0.101354516978, -0.136664478394, -0.0463372409126, -0.114544183169, 0.128269593331, -0.12848395274, 0.0440932376461, -0.0150689802249, -0.0156841727917, 0.0299123813558, -0.017102149474, 0.0401076235514, 0.088992819796, 0.149323044141, -0.104944067976, -0.0977353194687, -0.191505381566, -0.0623249574547, 0.0758962312221, 0.0446436591875, -0.0584071225397, -0.0191389784874, -0.0692554561957, 0.0271287547769, -0.195457978571, 0.0246829935818, -0.0992127956325, 0.106245900848, 0.0963498253155, -0.11132284438, -0.00825288212197, -0.0714573875513, -0.134694553174, -0.0120834631111, -0.00754032077967, 0.151650700626, 0.252271613483, -0.132148527781, 0.109859679723, 0.147018701258, -0.0198610018812, 0.125489662564, -0.0468856422321, -0.0739926430909, -0.043837933148, -0.0706088912238 }))
          (Vector<double, 0>((double[]){ 0.113276127003, 0.0707625333731, 0.0546272928666, 0.0275088915683, 0.137303354148, -0.0951418922795, 0.0613534388876, 0.160477314673, 0.134073445479, -0.132110951193, 0.0238423355614, -0.119032972034, -0.120489536087, -0.0808369525845, -0.150982294566, -0.0756342492775, 0.0553021369097, 0.064843525007, 0.0766685751402, 0.0680908595271, -0.0471544899418, 0.229245412639, 0.00537729596747, 0.0603747404719, -0.0197932064401, -0.0558940109936, 0.0370079918101, -0.0217730696183, 0.015132373013, -0.014534993066, -0.00465875452759, -0.0110117688135, -0.0933901469636, -0.0708414963394, -0.123008417903, 0.0491484246983, -0.0520978082943, -0.111857186784, -0.0862049242843, 0.0814394527265, 0.0899465432554, -0.106722354155, -0.0410836054239, -0.130473069249, 0.0859506062294, -0.144274737852, -0.074930472292, 0.412258429279, 0.012091609224, -0.0467902492009, 0.128367611625, 0.219281630614, 0.0702335933153, 0.151344956639, 0.119546561813, 0.0491568518848, -0.105798928596, 0.00175238010213, 0.0834222599868, -0.0398050455959, -0.0097562875688, 0.105616951876, 0.168206059679, 0.0418878492035, -0.149022937309, 0.129518266305, -0.0604867166582, 0.0566770164683, -0.0242429157393, 0.0740781926808, -0.142138296504, -0.0504944132122, -0.0451678091949, -0.0491685088909, -0.0633806865958, 0.108344415148, -0.0109611632709, 0.0344875276372, 0.0162744583146, -0.0890870745286, 0.00946979671259, 0.0543604516492, 0.117018831431, 0.01094782654, 0.0727490537263, 0.00335742643629, 0.0109524000986, 0.130798930285, 0.0853822397343, 0.0141154385655, -0.0973052301778, -0.158891130452, 0.0518841006234, -0.0395146033106, 0.12765578976, -0.110777692966, 0.047910390734, 0.006646197462, -0.167064899289, 0.0534060158246, 0.110202365765 }))
          (Vector<double, 0>((double[]){ -0.0740479241531, 0.0470768774762, -0.00319321698183, 0.0325189030917, 0.117432182456, 0.0693177300453, 0.110552457307, 0.0608747668235, 0.0177482471161, 0.0950986423361, 0.00162804467862, 0.00806718646049, -0.0905574378061, -0.323085947434, -0.0433320193516, 0.0520682449094, -0.0216887654465, -0.00334325461965, 0.0376341226423, -0.226882700022, -0.105804390435, -0.0335351512473, -0.111488134763, -0.102154909001, 0.0913581544057, 0.0330585459155, -0.0514684954943, 0.0153251117783, -0.0751513513163, -0.0159974981287, -0.0397344714, -0.129379159271, 0.106539913907, -0.046954683479, 0.0358466168799, 0.121047211688, -0.0446142059453, -0.0690247058216, -0.0527013864835, 0.202600976125, -0.067555427967, 0.0517109463902, -0.0491069541111, 0.108955066652, -0.0198181910161, 0.055296536043, -0.19163167744, -0.0359026635446, -0.116662525806, 0.00643098943123, -0.0595315251553, -0.108031618624, 0.0227713638242, -0.0140560356609, -0.0501190466801, -0.0470968480828, -0.0628756782326, 0.0737712965461, 0.0440499555955, 0.0274637404042, 0.112766137824, 0.00170173549087, 0.17439048073, -0.0971404170421, 0.168211070879, -0.01232024929, 0.0868560068533, -0.0410773290063, 0.133790491416, -0.0220293479013, -0.0753358677067, -0.0624887478391, 0.2275245623, 0.241985701054, -0.12494372954, -0.0713710877955, -0.0642412294012, -0.145604760031, 0.149256457644, -0.15590839133, 0.0600266349121, 0.0962377484739, 0.035024899988, 0.0188917343634, -0.0257193941824, 0.0742293799946, -0.106834082307, 0.0299210718103, 0.175963834089, 0.00346462047814, -0.0923669357662, 0.0977504241889, 0.0797631762948, -0.0182538416937, 0.165201154473, 0.143105318724, 0.146581843949, 0.00912202051619, -0.106249280089, 0.152720260842, -0.0419286434806 }))
          (Vector<double, 0>((double[]){ -0.0661009813012, -0.0457574255852, 0.109504011886, -0.0109604675829, 0.0219460362685, -0.0443321948504, -0.00248766943468, -0.0626797989497, -0.0465294367975, 0.0403920187675, -0.111192116615, -0.0591820752897, -0.0344873705686, 0.0147303819398, 0.0662417322885, 0.0234093900569, 0.0524085166842, 0.088762836945, -0.0343464479507, -0.141706507978, -0.0609606810226, -0.0965526340961, 0.0366301437011, -0.0398317951224, -0.0433213720064, -0.0928666915664, -0.11195879658, -0.201523707091, 0.0726125075268, -0.052111835886, 0.0739710564937, -0.104038558846, -0.21360917528, -0.0233687058185, -0.220779593128, 0.0144945626188, 0.160735032975, -0.114763727048, 0.0187793125829, -0.122959462789, 0.269282513373, -0.0975185888097, -0.0557091307922, -0.021577067173, -0.0463403861262, -0.00835552316862, 0.130548060433, -0.0367161262733, -0.113142070113, -0.072079097483, 0.124138456488, -0.136411306233, -0.114563478164, 0.0475722991167, -0.126716861748, 0.0743401452624, -0.032804325541, 0.134257079299, 0.144595202347, 0.0200014939681, 0.0871224430162, -0.0741358914162, -0.0229908539645, -0.0117251674856, 0.0831332681183, -0.0492289505671, -0.0154088027512, -0.0150643121848, 0.127736021531, -0.151570953868, -0.0177115423585, 0.0867688133781, -0.00962335579271, 0.0497160517923, -0.0938564338486, 0.172307347061, 0.026394992497, 0.1170103463, 0.189726784192, -0.0264276417753, 0.251620814813, -0.0137061934945, -0.00962885515189, 0.0334453497002, -0.101884962082, -0.0366611361804, -0.0568405034217, 0.0244295083697, -0.0233707501003, -0.0774634456902, 0.0155737603562, 0.109974061809, -0.0232692181224, -0.0440905672405, -0.182028771797, -0.111046021591, -0.035683416199, -0.282386003446, -0.0810200955881, -0.0513492464113, 0.167052164396 }))
          (Vector<double, 0>((double[]){ -0.158233476839, 0.0792593378603, -0.0213610399963, -0.0935556565743, -0.0553255929905, 0.0106331014329, -0.00235999032146, -0.0288268517559, -0.104318567116, -0.0598171383964, -0.0292173075906, -0.00893642779273, -0.093913085641, 0.159053284014, -0.228843161741, 0.0232887090192, 0.155236983293, 0.00859771836721, 0.0707257438263, 0.0206102566957, 0.0458852572203, 0.0226022419688, -0.216302387129, 0.203446790875, -0.0879967458844, 0.236530832178, 0.0320274472674, 0.0275038941952, -0.0401491067626, 0.00676672211576, 0.075561207626, 0.0635733030873, -0.116028308012, -0.00158988471948, 0.102450610267, 0.0222332824811, 0.111982548117, -0.00978021036063, -0.0878438025947, 0.0768180036756, 0.13505820075, 0.0751927214644, 0.0379021920657, -0.0129229078489, -0.0184274136116, 0.116414237086, 0.0485201484353, 0.0333698854631, 0.060885399748, 0.0559060017074, -0.116473775977, -0.189289895076, 0.0872911830585, 0.234545197901, -0.093075111948, 0.182988531795, -0.0557396128776, 0.0571711570981, 0.0677587199663, -0.0472142946112, 0.0140680647607, 0.0136225134372, 0.0194336827416, -0.0716064024143, -0.0660263583978, -0.142176693472, 0.0224569774399, -0.0880713530356, 0.0447702354939, -0.0413019058967, 0.0362263829515, 0.0146287693444, 0.120448111794, -0.216803133968, -0.0797624294261, -0.00969217274842, -0.0357133981933, 0.209120470203, 0.0755083369936, 0.0732305804719, 0.0295831725778, -0.0377471549516, -0.145885251115, 0.194898455856, 0.111311487412, 0.0631747391999, -0.0465392402781, -0.00479959345995, -0.158481731008, 0.0076154675778, 0.0612309892323, 0.154810141963, -0.00762838561853, 0.166922550877, -0.193946892255, 0.0341418579034, -0.0882269414621, 0.0166878917379, 0.059899251804, -0.0920229065008, 0.012049584065 }))
          (Vector<double, 0>((double[]){ -0.117235716098, -0.0671784472087, 0.14684548309, 0.000493790237528, 0.0719611355399, -0.0074526843114, -0.0846814970699, -0.027953726386, 0.046394873351, 0.0642974945311, -0.0473102595308, 0.00539462160081, -0.0754920358156, -0.172279519542, -0.00547288275913, -0.00451230292752, -0.15221688856, 0.137230962252, 0.0484072874827, 0.1007448197, -0.112588768857, 0.10079626435, 0.0526898449374, 0.0488171025562, -0.105009421555, -0.0391035001695, -0.0651063074036, 0.114395652481, -0.00260845333034, -0.0218284595588, 0.00472701457566, 0.0491821029678, 0.0448756045747, -0.163449068094, 0.0623255562545, 0.0735151328752, 0.097555179646, -0.0452125560045, -0.137141826422, 0.0185501565758, -0.00231335221039, 0.00569481643519, -0.267673995812, -0.0492643149991, 0.121026347516, -0.0450349892169, 0.0356250597219, -0.0136112694765, 0.077299834264, 0.0507998907169, 0.165645602979, -0.0237771458902, 0.063771029133, 0.05559265659, -0.140062799488, 0.0448254867116, 0.149154346313, -0.059260503976, 0.141181179404, 0.000743019248107, -0.000244961545208, 0.074478559867, 0.0832711070004, 0.0435364515561, 0.161455106473, -0.0388068234816, -0.0192625234172, -0.182148732342, 0.00398275189001, -0.0627352224107, 0.0442044598839, -0.150115928231, 0.344210113023, 0.027547508689, -0.028669746416, -0.0830699983288, 0.147841874801, 0.122752138001, -0.0837513952148, -0.155480113843, -0.101394316084, 0.0216392273432, 0.01391431943, 0.167043970273, 0.0388970468481, -0.0592823978768, -0.065956750776, 0.108224650772, 0.0643343007097, 0.0867011949814, 0.0177404161686, 0.0440159421185, 0.0252175958959, -0.103443661443, -0.269741497103, -0.149240603344, 0.140232333771, 0.0340364365961, 0.104095436113, -0.0150579023636, -0.0677156790127 }))
          (Vector<double, 0>((double[]){ -0.122727535447, 0.0785917509872, 0.11686529354, 0.0870806070548, 0.004920876119, 0.0175047275994, 0.0311296920033, -0.0359490599735, 0.0688941129131, 0.0763007150992, 0.108984287572, 0.07792068116, -0.0554065710988, -0.0364473255826, -0.066975537735, -0.127793888815, -0.103643085707, -0.0933394459919, -0.079992828389, 0.0570736353027, -0.059400021221, -0.000132308778243, 0.140206311476, -0.0567877336838, -0.198293473343, -0.121609527536, 0.0221531528054, -0.00441880075421, 0.193852666691, -0.0882253640283, -0.14110921156, -0.0087458083527, -0.103584422892, 0.00302771675966, 0.0989622031785, -0.0489329652782, 0.0263736104743, 0.0719026174834, -0.19256443099, -0.0101867946599, -0.275997915261, 0.00719880773806, 0.0609922345606, 0.0158310343169, 0.0939852354177, 0.072124823183, -0.0411626992137, 0.0830399090287, -0.078670341111, 0.0603770305411, -0.127804602119, -0.106548602908, 0.089666330362, 0.221089349319, -0.0621850508208, -0.0772382694141, -0.159058708086, -0.116524809918, 0.126586127992, 0.144511756722, -0.00712726941442, -0.0363369433295, -0.200788147807, -0.0186820023645, 0.0207609901267, 0.0283671180306, 0.127298426523, -0.207205241301, -0.115581522039, 0.0239327501923, -0.0978013007477, 0.0599773483376, 0.0471160982603, 0.100614550759, -0.0540531918425, -0.00654582765134, -0.0598963372178, -0.137581681906, 0.11506194672, 0.0724825247599, -0.0862953337302, 0.183386543301, -0.0689127903523, 0.0271402553476, -0.0827383365224, -0.0665885300876, 0.00706399855962, -0.0538923357131, -0.118221894149, 0.0769623349079, -0.0123214905756, 0.16141714615, 0.112469833202, 0.0159362581908, -0.00186585156731, 0.133500927923, 0.0961422470478, 0.0996985200211, -0.174003120133, 0.00693446689008, -0.0622985630178 }))
          (Vector<double, 0>((double[]){ 0.00186712844521, -0.036361062769, -0.0504916162693, 0.225085124357, 0.177862461953, -0.174715045025, -0.103553071743, -0.0727130945338, -0.0697840458918, 0.107246948106, -0.0753678084755, 0.122936906867, -0.161514707895, 0.0141110592238, -0.0471979703497, 0.10461894024, -0.0387209748017, 0.0642039600883, 0.10313506665, 0.0641950590632, 0.0148187976376, -0.00784086141223, 0.149206578647, -0.127183142417, -0.266935594833, -0.0495179374157, -0.00233188776955, 0.106331430923, 0.106637934467, -0.0167173303361, -0.0750617550732, -0.155904709538, -0.0660231320924, -0.000618152432275, 0.0338155642588, 0.329578623055, -0.194883620726, -0.0803758852637, -0.127572816745, 0.111802038542, 0.000518751487515, -0.0633183323962, -0.0212222410427, 0.0204401034626, 0.0172615235438, 0.198913081187, 0.163630223481, 0.111255201535, -0.0616920889191, -0.0241623903292, 0.0333653962449, 0.0248428149553, 0.064574142088, 0.061538779791, -0.0378716850081, 0.0340690083816, 0.0632574726552, -0.177143604435, -0.00682176107592, 0.0676220906255, 0.025228792021, -0.0406918043365, 0.0401274087377, -0.14550954455, 0.00715216468764, 0.0314381117282, 0.140526099765, -0.0312566548664, 0.159550148941, 0.0984820568418, -0.0845355886188, -0.0444954414659, 0.0740629897557, -0.108467053117, -0.0512297504737, -0.0827426608791, 0.0107492094535, -0.0431037268267, -0.0304514225509, 0.12742689594, 0.0541613482342, -0.0117687350828, -0.0591956274217, -0.168215548099, 0.073375135189, 0.0874651483215, -0.0327269433652, -0.0340530002549, 0.0490822639658, 0.00340580626599, -0.0692557053097, 0.14440045706, 0.0877485284591, 0.0322830008618, -0.099931126863, -0.0489025999259, -0.021932674684, -0.0747281393506, 0.0152653459725, -0.144179325829, 0.116872942275 }))
          (Vector<double, 0>((double[]){ -0.116100233672, 0.136527588512, 0.116901555106, -0.0051463832705, -0.0585036282668, -0.102721394498, 0.140992600988, -0.0513252172378, 0.160558261709, 0.154971745254, 0.011189652104, 0.138172713212, 0.119851005122, -0.143247550577, 0.0328743626039, 0.188674725371, -0.0488683323125, 0.166753897658, -0.098797662749, 0.107696737689, 0.0503101384346, 0.142163091459, -0.0665806094295, -0.0747028921076, 0.0262532458238, 0.0353116130459, -0.00701395945429, 0.00206151590457, -0.0118529368427, -0.0566603623518, -0.0236155094388, 0.20814744849, -0.13061398822, -0.00512222133321, -0.0949401268653, -0.0705594528966, 0.0900646189733, 0.0107588831404, 0.0793146583659, -0.0662598291487, -0.0445498698048, -0.189005174587, 0.178823199611, -0.0234704824295, -0.0245256082524, 0.0445695889969, -0.130021701485, -0.134147701363, 0.0314537839684, 0.0612533470805, 0.0933670467072, 0.154412299516, 0.0591815750135, 0.10202002692, -0.0839567170107, -0.124994789858, 0.158590705807, -0.126037248394, -0.0688865443077, -0.0252615194934, -0.0469116200819, -0.124655240807, -0.0991220416424, 0.129594233165, 0.0306260789609, 0.11606656507, -0.0391975489783, -0.0911591822721, 0.0373705555, 0.00687501284806, 0.00132461085144, -0.0185024430034, -0.0561518183386, 0.14708946719, -0.0717242470293, -0.0618478276371, 0.119077997997, -0.0772102176841, 0.0327636820917, 0.115025089358, 0.0882040883169, -0.0887068337537, 0.0451065426077, -0.0361631906143, 0.0071776836823, 0.0580440823588, 0.0618176682203, -0.108753754931, -0.15648957511, -0.129321783266, -0.0560614137769, 0.103853690302, 0.030044192321, -0.134587160877, -0.148615805901, -0.00336871329949, 0.133839478005, 0.0345780168036, -0.131618042432, -0.0518497130359, 0.231370356588 }))
          (Vector<double, 0>((double[]){ -0.0089962446889, -0.0434441413654, -0.140275408869, 0.0986542087829, -0.0540475063829, -0.0850607357616, 0.121014138823, -0.129199208105, -0.0375954177857, -0.139280000282, -0.0193052696304, 0.159466337677, -0.0669491041833, 0.0734384067716, 0.013511452753, 0.0867290170038, -0.0120071818633, 0.0441663468838, -0.00649596328404, 0.109800896327, -0.0994192350683, -0.0349919049209, -0.0356263237149, 0.0620695432415, 0.215887562537, 0.133915342394, 0.189556804411, 0.046833702202, 0.0353968439653, 0.179122854841, 0.098873708943, 0.00869175028927, 0.0174012818904, 0.0160116371112, -0.145278840532, 0.107815320157, 0.00680236237121, -0.00885715535646, -0.0134608255165, -0.0630995877987, 0.0236308015326, 0.0966292700594, -0.132308352436, 0.0670693735337, -0.0124640205275, -0.103992712717, 0.210453087911, 0.0274892050263, 0.00639129777023, -0.0665697263534, 0.0498696989719, 0.0487096764405, 0.00165925455189, -0.163230973601, 0.0127257119242, 0.203229547961, 0.183513500835, 0.183684525868, -0.0534879427707, -0.227078875813, -0.0923263376146, 0.0333167985192, 0.075018032235, -0.106173777121, -0.0904567831398, -0.0180401484802, 0.151763150229, 0.102007178836, -0.00961627891633, -0.0220694686491, -0.127894610239, -0.0268226066184, -0.0650468939305, 0.0375208230953, -0.0865479441778, 0.0629998849473, 0.006616861356, -0.0259624562985, 0.0606410114375, 0.183613807481, 0.0415527529598, 0.00290561894429, -0.0299597281575, -0.213014937155, 0.170967069434, 0.000957572196081, -0.0847459842456, -0.00269413808273, 0.0260666345537, -0.0955800551785, -0.0703970637128, 0.0226691539062, -0.065859470364, -0.116478466733, -0.203073385553, -0.0401756657503, -0.124102320704, 0.0154936191711, 0.0580943150377, 0.143666609553, -0.142850121443 }))
          (Vector<double, 0>((double[]){ -0.0505713121081, 0.060769367333, -0.0165336829656, 0.0284047599444, 0.119470871285, 0.0190097653568, -0.00251220017497, 0.0507953487486, -0.00154583313506, 0.10120235078, -0.105553959935, -0.0346556860334, -0.0126398636426, -0.137871765936, -0.00895602378781, 0.126035327257, 0.121692017234, 0.0617902720333, 0.128941583793, 0.140322928156, 0.0620031585283, 0.149676959728, 0.0985950666379, 0.0224357298161, -0.0925245892381, -0.0155927170886, 0.0857670659498, 0.0155093614989, -0.0728659088567, -0.0217349697935, 0.0438755363597, -0.074377362932, 0.0214234211819, 0.113427580102, -0.100106328393, -0.125719304732, -0.0345140529578, -0.244299089264, -0.0604857009244, 0.0340570927015, 0.0248575466842, 0.0127426734044, -0.012076515252, 0.00768348580363, 0.0934202836207, 0.0404651424874, -0.120592683833, 0.166931749807, -0.0386103538421, -0.0917104513244, 0.0350514712105, -0.0142349262241, 0.105320295872, 0.0696724931874, -0.185934613113, -0.0515242781213, -0.102467688879, 0.227953952003, 0.112347249165, 0.130507593783, 0.0108921670291, 0.159048518667, -0.176047900031, 0.190393818694, -0.034350870714, 0.0646645988819, 0.00646168408683, 0.0249363497119, 0.0950480814789, 0.00992850214187, 0.103604324928, -0.0282233092241, -0.0480182991833, -0.134564219156, -0.205481121947, -0.100010055958, -0.0605373691168, 0.0804287356622, 0.102146143407, -0.111466438365, 0.114398475361, -0.105985501121, -0.155323052147, 0.111740826669, -0.101172861268, -0.0832860638838, -0.131326174204, 0.172311462259, -0.052990084762, -0.215203655559, 0.0344871712228, -0.0524391989762, 0.0828434917133, 0.207202014744, 0.0229840724637, 0.0581297107439, 0.0961577530429, -0.0790952255804, 0.0751752886033, 0.0570787476203, -0.0160231204915 }))
          (Vector<double, 0>((double[]){ -0.14515229288, 0.166896701281, 0.000859643197665, 0.0782266099269, -0.0267908194502, -0.066718854471, -0.0270699156461, -0.0254758300486, -0.19450669878, 0.0983499966917, 0.0996431344696, -0.0239241683514, 0.192894810475, -0.106890817204, -0.124792086937, 0.0657555842366, -0.147719661855, 0.0839625168572, -0.0454802213369, -0.0855319030061, 0.0303102222507, 0.029221879703, -0.030647021176, -0.0160133594384, -0.0702322999665, -0.17112862864, -0.00303915455002, 0.189480912211, -0.255692737145, 0.00695748764283, 0.025916569652, 0.047130244783, 0.0452978484244, 0.0692344640439, -0.0769643259553, 0.0510669651598, 0.0282436388524, 0.0392433561905, -0.0955050994167, -0.0669002902817, 0.133808032693, 0.101809062985, -0.00660796585137, -0.0250516674713, 0.169851350996, 0.12502782014, 0.229142469779, -0.035960824437, 0.100561059092, 0.108119951379, -0.00484024326643, 0.0391300917627, 0.0437800917346, 0.0986103600487, -0.0621716481288, 0.0258168286408, -0.157345199386, -0.0319087971731, 0.211627782097, 0.0711578556404, 0.0472703042234, -0.0332901127512, 0.00172875786474, -0.190934427344, 0.0782299487481, 0.0491736424807, -0.0731692955149, -0.0439633113036, -0.0703170028347, -0.0880658373427, 0.0406819086185, 0.0297873326981, -0.0290907410641, -0.0387196702635, 0.101523240008, -0.030473672941, -0.0642444256824, -0.0577576263498, 0.081696291982, -0.211135244978, -0.12068132479, -0.0528604034754, -0.165366723539, 0.205574551076, 0.115591834583, 0.0683019531365, 0.0632942716907, 0.0197448236647, -0.0635821883478, 0.134789281736, -0.00693799851669, 0.0296432210719, -0.123599612337, -0.0857238773069, 0.0814274214616, -0.148199739473, -0.0260776468184, 0.119639405551, -0.0394210782325, -0.120058336843, 0.0251936672151 }))
          (Vector<double, 0>((double[]){ 0.0103562913798, -0.0593374415877, -0.0355473331409, -0.086989764673, 0.0380485063047, -0.0636008349128, 0.105611361965, -0.0855838047481, -0.0456585595319, -0.0505490913502, -0.176931869365, -0.0124432717798, 0.0341832134185, 0.00877618404082, -0.0368771894794, 0.116291036484, -0.212819700848, 0.029251212281, -0.087574351566, -0.0934341137136, -0.0193821406297, -0.152089015651, 0.0989169337936, 0.131917929246, -0.114175040721, 0.127869231454, -0.022827232461, 0.147809968032, 0.186971845895, -0.0820088376748, -0.0492424001923, -0.064776031063, 0.0608031775432, 0.0585992831682, -0.138829471999, 0.0232072987401, -0.0549629063632, 0.0332248503713, 0.140681913874, -0.00879637664167, -0.0784784182449, -0.00962777215557, 0.120654958814, 0.202424530426, -0.0895414797211, -0.0817854304832, -0.211200710742, 0.186490878097, 0.0240149578195, -0.0231283496716, 0.0735638373192, -0.0144665683895, -0.0248964356104, -0.125056977116, 0.104353323248, -0.177125042795, 0.0272867280588, 0.0934056346816, -0.115738205903, 0.0370862842984, 0.12679600449, 0.0895900903944, -0.0813738553206, 0.0244821652074, 0.0299582861341, -0.0278215992083, -0.0163110540148, -0.0103208246203, 0.0519800598752, -0.172970771531, -0.0610705625683, 0.0838088783551, -0.086743212426, 0.0427836345033, 0.0664985629229, -0.0376106281274, 0.166681376519, -0.219587179448, 0.0446611004684, 0.191261678932, -0.0264963686267, -0.0693129563005, 0.054528524815, -0.0663374941186, -0.056075029212, -0.0527477896701, 0.264145609816, -0.00114339821095, 0.0680186664317, 0.113993796148, 0.098266367838, -0.112451981278, 0.129127323099, 0.0680659731439, -0.118300464374, 0.00634030189125, 0.108905413912, -0.0847293199891, 0.0679974493056, 0.0410737381635, -0.0776244332794 }))
          (Vector<double, 0>((double[]){ 0.04408000297, -0.128349430991, -0.034062622654, 0.0408526776246, 0.0344268805681, 0.148804939192, -0.185900035964, 0.0691397830708, -0.0239791607097, 0.0912992823413, 0.0149051432496, -0.0404257357763, 0.121834654297, -0.024956653427, -0.118690138558, 0.0128236684828, -0.0134324946062, -0.0981049152196, -0.123974072332, -0.0739776221079, 0.0252647707844, 0.0147763045875, -0.0583303211946, -0.0882575479613, 0.0513069404799, 0.0592510382244, 0.221965501419, -0.0898086925498, -0.0611598049653, 0.0158947073217, -0.00699044115416, -0.0368797951676, -0.0502409339086, 0.00813471911335, 0.119907511835, -0.0478500525242, -0.156735303275, -0.00746117557138, -0.137021947738, 0.0381622697215, 0.139878544352, 0.00136922398408, -0.122828328175, 0.0807569540106, 0.0479275303497, 0.0602470335038, -0.0421701004836, 0.18066244567, -0.0150326649029, 0.024142140034, 0.00684370204627, 0.118074269901, -0.039597656104, 0.0251875349611, 0.169669663154, 0.0629763221763, -0.0592357841065, 0.10694095579, 0.0584841226846, -0.109684979397, -0.0461682774233, -0.280139511753, 0.0420506692097, 0.0563643334664, 0.0624292037014, -0.0222832182324, 0.143502847121, 0.0147295657191, 0.100951723243, -0.0622404783789, 0.160634858509, 0.0838597045253, -0.0643572687254, -0.0305009040034, -0.0682022691, -0.0619007798202, 0.000670479220745, 0.0268422933997, -0.0390042067746, 0.0701346818508, -0.107533787035, 0.0709029691714, 0.15245891474, -0.194672450325, 0.12479611793, 0.204102897266, -0.00193902102082, 0.014962922556, -0.0120662141606, 0.101846956679, 0.0634739630546, -0.0622013078676, -0.106728581212, -0.124093495103, 0.0999575656403, 0.0479214379982, -0.173778661121, 0.283860242629, 0.22646768735, -0.0609715924948, 0.0374439014177 }))
          (Vector<double, 0>((double[]){ -0.171818638922, -0.308955013307, 0.0166483384156, -0.0944842848086, 0.118716984697, 0.0237800771204, -0.0868940294426, 0.141496484303, -0.0519083662284, 0.0725891652394, -0.0862297159048, 0.136389019975, 0.037853010738, 0.0164493128202, -0.174986566898, -0.00541410697042, -0.0610959508202, -0.124790284691, 0.129518335555, 0.106891040126, 0.0713577825085, -0.0797925532926, -0.00746805941248, -0.0407103849968, 0.090552431679, 0.0160468006375, -0.140621854065, 0.0126688007208, 0.0156908471039, 0.0782455133418, -0.086619290788, 0.0216487749348, -0.212680958129, -0.00711659059701, 0.1809935263, 0.113159349607, -0.0461473694787, -0.166094921052, 0.195696117746, 0.0536190667322, 0.0368963637826, 0.0126384778111, 0.198222857291, -0.0456728425551, 0.0253939146181, 0.122260195474, -0.0981854180492, 0.0657362146368, -0.157863237186, -0.0872252572357, -0.0201734375065, -0.209504408041, 0.101494866086, 0.0465257807137, -0.0376837488539, -0.0542773487224, 0.0104018905548, -0.0111980104164, -0.065035804276, -0.0825302121547, -0.074707423267, 0.121217658102, -0.134568214205, -0.0415098705182, 0.0347618731911, 0.00302055942232, -0.0458087096604, 0.0946415481477, -0.0562047452571, -0.0730595422254, 0.142911612148, -0.0114338535727, -0.0336602859504, 0.134605840371, -0.114470376395, -0.113606115803, -0.00597721043205, -0.0480844243269, 0.0207716407701, 0.107940553755, -0.0106782991407, -0.0249296098516, -0.0372319297706, -0.0674371244271, 0.0622417245032, 0.115354846073, -0.01320917792, 0.150932708683, -0.066112702584, 0.0308914070335, -0.144208558016, 0.121745924084, 0.113558262094, -0.0704365398268, 0.0971597672378, -0.0294597344268, 0.0314383650086, 0.00732627426849, -0.149591927426, -0.104731612759, 0.152564332875 }))
          (Vector<double, 0>((double[]){ 0.150112898272, 0.174393410943, 0.0634206619286, -0.00945663960596, -0.0289998206203, -8.03316523635e-05, 0.129527514552, -0.0832040400362, -0.0421760025271, -0.0295218475036, 0.0752061047355, 0.125832442098, -0.00894654685524, -0.00749536926531, -0.103702453667, 0.0174527936448, -0.0119727229219, 0.0676665146968, 0.140643217385, -0.042172309396, 0.0049085056627, 0.071479434914, -0.00451791824692, -0.0233441264769, -0.220102856257, -0.140255040824, -0.0888524146364, -0.0391808947467, -0.0983637060906, 0.0548106375324, 0.000875006533386, 0.062113450064, 0.00124271254087, 0.161179197969, 0.00815104075293, 0.227413754589, -0.0956507405457, 0.0355188056299, -0.107650583307, -0.0187319536292, 0.0497000716422, -0.0393583934488, 0.101360727703, 0.0841324664065, -0.00338146870657, 0.146047520329, -0.131907658715, 0.0432574738211, 0.00931176597688, 0.203828076617, -0.0335899374836, -0.236266532389, -0.0254544014613, 0.0503719270233, 0.10356959407, -0.00729419355038, 0.127394007553, 0.0461719166308, -0.0563351671918, -0.117236185234, 0.105084301247, 0.218607951247, -0.0186755186616, -0.000157444788543, 0.107790463656, 0.0502590505547, -0.055323067798, 0.0839816334935, 0.0797293414637, -0.00203982210827, -0.0333607102766, 0.103525508253, -0.137593088491, -0.00608251373014, -0.105173393844, -0.149595179284, 0.0220351448922, -0.0361835940706, 0.0147394837339, -0.0290071032403, -0.149944594891, -0.0586439670304, 0.199810373938, 0.0421192076531, 0.161657601941, 0.0834078569132, 0.006905968321, 0.140380103789, -0.0221530467817, 0.054447180602, -0.177534381594, -0.160397045084, -0.0190259094512, 0.113609985057, 0.0446718565652, 0.0254248954149, 0.0382248217917, 0.263326953273, 0.00970648062858, 0.144472655758, -0.0365581946215 }))
          (Vector<double, 0>((double[]){ -0.151500501818, -0.0462792227965, 0.0566941497307, -0.0918575788435, -0.2100370451, -0.0339676821787, -0.175257672086, -0.0569284528754, 0.0793591668191, 0.0466999204311, 0.0163057773663, -0.00261696726328, 0.114007086963, 0.0022557906126, 0.0493464932109, -0.0528330125268, 0.0892937116313, 0.0025954137354, -0.015634679146, 0.0638428783374, -0.060208317528, 0.0190952976473, 0.120413894369, 0.167495741275, -0.0569172075075, -0.0645084002106, -0.066309303861, 0.078907625338, -0.179657434835, 0.0763363211404, 0.0212012839656, 0.0730323934381, -0.0503823792493, 0.00757832210989, 0.0538582179672, 0.0119233375408, 0.0443864640757, 0.145110733732, -0.0512160911568, -0.0930202146156, 0.0982454064866, 0.135063859289, -0.0920448763512, 0.298400921346, 0.106460407792, 0.00297936003239, -0.0984211283906, -0.00439905021229, 0.0806221819302, 0.00224039063361, -0.194939182643, 0.0287619928757, -0.0264352421145, -0.10260943931, -0.0927586775677, 0.187871897962, 0.130240640362, -0.113523109087, -0.197727945958, -0.0555370439715, 0.0950351355421, -0.0819490958971, -0.0397667894608, 0.0544694868063, 0.0739581868631, 0.00371755443697, -0.109285123852, -0.133909869872, -0.00286678368587, 0.14887063626, -0.0374451919546, -0.0171434247683, -0.100772309585, 0.021561279179, -0.109382859732, -0.0635723446515, 0.17719641645, -0.0545366769282, -0.0745552724945, 0.0820673090816, -0.107826846471, 0.0475937157792, -0.0985092670171, 0.121110302577, -0.0766550715446, 0.173247259767, 0.00459401182721, 0.0743140951539, 0.0928799657348, 0.0743383990337, -0.106655981747, 0.0915635707005, 0.0774232806185, 0.0665536474223, -0.0627756184589, -0.00718236209672, -0.265996282856, 0.0577132025232, -0.0647023064094, 0.0161575164392, 0.134266719268 }))
          (Vector<double, 0>((double[]){ 0.0460613102249, -0.150099551446, -0.133454159729, -0.158874329495, 0.0164666190197, -0.0976090377145, -0.0469698714844, 0.0955635674233, 0.116686463238, 0.0620325726537, 0.0642052539359, 0.110941704493, 0.0718380613838, -0.0308502306035, -0.079117552939, 0.1097006232, 0.0273296903314, -0.142795509198, -0.0445448358242, 0.0673185528984, 0.00342187116492, 0.0122950708046, 0.105315563975, 0.00663689611106, 0.194010905415, 0.0369150344008, 0.112442823045, -0.050322648556, -0.142473573348, 0.0990795278379, 0.0279729681338, 0.0538076756278, -0.110610199814, -0.0238028706543, -0.0308871010014, -0.179867824207, 0.0136702334963, 0.131636438515, -0.109753096986, 0.0230540281463, 0.0913102978996, 0.114437098526, -0.0251050534291, -0.161507306619, -0.130730095158, 0.0365740274296, -0.0884870137252, -0.00577790258688, 0.024060993561, -0.122287641398, 0.0540934394738, -0.0672595646351, -0.0126636636306, 0.126391272297, -0.0107263140651, 0.0646672975433, 0.0230877329544, 0.124956867477, 0.142589187957, 0.0127987002099, 0.150377939054, 0.0395489564823, -0.200615999575, 0.0189632820921, -0.0116565119301, 0.24613352813, 0.0695967005369, 0.0551617389582, 0.0523560697083, -0.0361173722401, -0.0696541050589, -0.095941557986, -0.0822517981349, -0.0554926624899, 0.0877740555277, 0.0901717633174, 0.165182576605, 0.221236360061, -0.134966357406, 0.0782500116093, -0.085966451911, 0.0972717069097, -0.158523065686, -0.214547994986, -0.0470881441778, -0.178330677193, 0.0489324394737, -0.0366485732446, -0.0204400544613, 0.00870240783759, -0.12364993928, 0.0161514385275, -0.000274956241608, 0.0416183814395, 0.0814212165241, 0.0730130580997, 0.162532749738, -0.0834008267617, -0.0809557442092, -0.0651505862784, -0.133500028693 }))
          (Vector<double, 0>((double[]){ 0.165218731682, -0.150964446058, -0.036432252519, 0.159803986206, -0.13194987596, -0.0138079000967, -0.0287214906825, 0.129471545078, 0.0424060484667, -0.011528276635, -0.104039100443, 0.0881396271268, 0.0748449257805, -0.00935974388982, -0.105439278159, 0.0965583149738, 0.141536383889, -0.0555650495588, 0.0934256748405, -0.0813897300307, 0.152375727315, -0.159622383129, 0.0564137483423, 0.0240417220668, -0.0199544183458, -0.0102613676508, 0.142213796569, 0.129771790249, 0.0170873810686, -0.0338534981555, 0.0710643973127, -0.0599530852995, 0.284650289894, 0.16065474793, 0.0346401262322, -0.058167366332, -0.165961584148, 0.0701219910139, -0.0264132366168, -0.000946022873905, -0.128084279859, 0.0518895039942, 0.177016247378, 0.0561335294205, 0.000198086660312, -0.101690340581, -0.0672432271656, 0.0261230915678, 0.163776677494, 0.0196142111936, -0.0776267606486, -0.0342871733764, -0.0391433589412, -0.0341999136316, 0.129359629143, -0.131424890135, 0.0420514801417, -0.0808349196702, -0.143890892477, -0.143422241907, -0.0322823158776, -0.163109578192, 0.0320211770309, -0.0778504144024, 0.123695530081, 0.090670007341, -0.0292391678046, 0.0532444100901, 0.0999508732093, -0.0188114972051, -0.0472433782849, 0.146077634558, -0.131488876332, 0.0417921392364, -0.048063074167, 0.0694451221111, 0.0414095929582, 0.0498971814143, -0.167499995306, 0.0531687464764, 0.0820931410989, 0.0501526545641, -0.119395132731, -0.117893138058, 0.114119396564, 0.0972302442757, 0.0960131968183, -0.133425142788, 0.0717694528719, -0.164506790481, 0.184472637768, -0.0788857461116, 0.0279347072868, 0.181156328879, 0.0115985185407, -0.00839168198907, 0.00824571544665, -0.0373503438638, -0.0238396595164, -0.0921049189473, 0.0861105926533 }))
          (Vector<double, 0>((double[]){ 0.0159384850054, 0.0732446303696, 0.00424961094527, 0.0849587539466, 0.0105597245193, 0.205366917387, 0.0577548525575, -0.0930437902166, -0.271473924764, 0.0798217995697, 0.017598853411, -0.0454959419305, -0.162451886925, 0.0140570940976, 0.0542764031818, 0.111440930886, -0.0425819431048, -0.165114837544, 0.304239354428, -0.0341788215226, -0.149559288575, 0.0601386619886, -0.268919589765, 0.115075828115, 0.166351521045, -0.00012800192929, 0.077250579265, 0.00986697356784, -0.0124641983263, -0.00695245706429, -0.163474660719, -0.0921775990127, 0.101277411051, -0.00438975298481, 0.0786802355192, -0.23350950467, 0.0165219902039, 0.0170995175748, 0.000490362425511, 0.0334133289718, 0.0406313341306, 0.129034550953, -0.0736777733035, 0.08847688345, -0.120566215273, 0.0626883284387, 0.0944946343767, -0.0206159691861, 0.024740040346, -0.102632865287, -0.0020511118149, 0.0281969466208, 0.126693212246, -0.0874506279419, -0.11909708057, 0.0590488070155, -0.0881535641751, 0.13345041013, 0.0269119134639, 0.00328812400003, 0.0609065746467, 0.00252451442144, 0.186040932587, -0.0797894430552, 0.0591308352553, -0.120784481742, -0.0810609175585, 0.182186078347, -0.0629103206856, 0.0103820618101, -0.102393114251, 0.0220420401331, 0.0286713074398, -0.190859911737, 0.092963209379, 0.00242632986825, 0.0844069046529, 9.43420187142e-05, 0.0115413851423, -0.0726677563515, 0.002556064336, 0.0087838152931, -0.00646685556863, 0.0669605240152, -0.113833005603, 0.0746397508386, -0.00135907571349, 0.082799230206, -0.0239050293985, -0.132896830355, -0.0649803286124, -0.0589587927792, 0.0679717359859, -0.0999858749957, -0.15304304083, -0.0850070259781, -0.000567909591025, 0.114595953288, -0.0103742966758, 0.0530826677097, 0.00161111135793 }))
          (Vector<double, 0>((double[]){ 0.0734472471472, 0.057141016188, 0.0516594385045, -0.0847309394674, 0.104596051739, -0.0888004846018, -0.102755190088, -7.96002509741e-05, -0.0663743837333, -0.115123581482, 0.0322998507175, 0.0410485271838, -0.191167525628, 0.0833186291441, 0.187244651487, 0.0664662503267, -0.00129360604517, -0.060127354356, 0.109499466975, 0.0625078250798, -0.140279611845, 0.00247581538311, 0.0333288600034, 0.0467849132395, 0.0809094900482, -0.0500748252301, -0.112601468901, -0.0504995502111, -0.17228952997, 0.0281578719156, -0.0731582166099, 0.0866338533145, 0.029364925809, 0.110513820929, 0.0397041101704, 0.0125413350812, 0.025874047385, 0.189529382979, -0.0179658317237, 0.0692786004174, -0.0698354968714, -0.108042206118, -0.0756690431931, 0.0334125095761, 0.0678382309741, 0.109552198294, 0.0969408587903, -0.00758484940917, 0.0247051787246, 0.00591716286943, -0.0115727721096, 0.0290574192947, -0.182203730939, -0.19269988781, -0.00577449532477, -0.0464394264743, -0.0166598724349, -0.00597245559586, 0.0519610875388, 0.108517448074, -0.178407566353, 0.0758492008602, -0.00711533310058, 0.0847949098296, -0.0528881501872, -0.106627873555, 0.0120741035138, -0.118323098352, -0.162108773283, -0.0302029891251, 0.19577050518, 0.0791952608898, -0.0870987766808, 0.0710453406783, 0.0937064355581, -0.0562717257371, -0.0612853033085, 0.184371240469, -0.108054034091, -0.210451223649, -0.0632785461316, -0.00856646519175, 0.162195380801, -0.0313365418101, -0.0559609965967, 0.0772378144239, -0.0969083037474, -0.197577851348, 0.0457432055765, -0.00827412820876, -0.0575070984055, -0.0280926870383, 0.227701549826, 0.107891607516, 0.0131016201648, -0.0607071928557, 0.131508753462, -0.0650973052303, -0.170504713886, 0.0157047871185, 0.259162877813 }))
          (Vector<double, 0>((double[]){ -0.147928124252, -0.0297602163431, -0.0852925857205, 0.126714781117, -0.265016949721, 0.0827533282424, -0.0346295163914, -0.140665342616, -0.0291867826284, 0.171063594983, 0.111639629532, 0.0488545280549, 0.0757574158283, -0.0255611210645, -0.0449601096236, 0.0962854666281, 0.101110088184, 0.161433918599, -0.0132251041742, 0.0420913883461, -0.0667393100617, -0.214947782693, 0.0347318443318, -0.0138622621687, -0.0866712002497, 0.0500528643745, -0.00496925515476, -0.0982315911455, -0.0310976356255, -0.0494965730945, -0.0563068244152, -0.0876336695245, -0.155595845295, 0.134474495311, -0.0207766414177, -0.16645275918, 0.0292199531631, -0.0656676147939, -0.054714322533, -0.0581296352638, 0.0336887483028, 0.0320019381492, 0.074893155381, -0.143367242391, 0.0111071057151, 0.00998528133053, 0.190476291993, 0.100880726487, 0.00738306873607, 0.0575476598214, -0.0975756503191, -0.152200749585, -0.105025496022, 0.0124533410921, 0.00114753911683, -0.0779960379915, -0.190450011171, -0.051443075935, 0.0811032399161, 0.108663229008, 0.0578715983145, 0.0876023276544, -0.0472343379246, -0.0433832980987, -0.0683817442001, 0.128667422022, -0.0800176920902, -0.0211351988734, 0.22770713083, 0.0488750801699, -0.0526070667441, -0.0985258946513, 0.0194335042781, -0.0390540410671, 0.0127506327806, 0.0571707250557, 0.112919232249, -0.121261206316, -0.0545379286076, -0.112650826888, 0.00757181450923, -0.145455271553, -0.0582694984551, -0.129298582511, 0.0243405264709, 0.0133418727108, 0.0845753419484, 0.0822797830577, 0.0217263064071, 0.0739934492618, 0.0629273698106, -0.2064172808, -0.00718573279552, 0.170184248127, 0.0867684109723, 0.091558284271, 0.178175034256, -0.113402226919, 0.178202144764, -0.105541932274, -0.0446065849747 }))
          (Vector<double, 0>((double[]){ 0.00295478245509, -0.000881403120641, 0.0157725468239, -0.164758535532, -0.0399646421049, 0.0805484861752, -0.132547247158, 0.0631503386642, -0.0900259343633, 0.00546447204393, 0.163326055401, 0.0352005189453, -0.114766016832, -0.0994749672097, 0.135634730334, -0.112011343495, -0.145899728747, 0.0066165283775, 0.0449460986368, 0.0770426624223, -0.0275317737153, -0.0233153319538, 0.105648789618, 0.0649647175927, -0.192432877448, -0.0214975784945, 0.0752899165674, -0.0154128544002, 0.237010084122, -0.0879980079796, 0.00616720714718, -0.0440696271038, 0.0147819640282, 0.0794859641108, 0.101141052754, -0.177422863172, -0.153075440495, -0.185365302293, 0.117063596896, -0.186191529449, 0.00552041945138, -0.0867445104833, -0.066568825929, -0.0992160274834, 0.119471529315, -0.106118004617, -0.0797468500385, -0.144993759594, 0.0168690557361, -0.0233095544968, 0.00914838486278, 0.106128011716, 0.124591456336, 0.0770741466105, 0.0381502134745, -0.0592507159085, -0.0723455101122, -0.0522952907935, -0.0406166369589, -0.161153250955, -0.0562093622599, 0.0327695501186, 0.0775136064196, -0.119639403747, -0.0371365621255, -0.00320135477421, 0.0294675651062, 0.10595738957, 0.125397773342, -0.122403379273, -0.0252299669686, -0.00230044566884, 0.0632805166601, 0.108043956578, 0.00116911071197, 0.218554783838, 0.0776473554225, 0.0214633674043, -0.0935642881753, -0.0683312744843, -0.155192786081, -0.0383130351165, 0.0945501184439, 0.122700247615, -0.110944300926, 0.00886675277287, -0.0536366938262, 0.0130709437972, 0.0842370438003, -0.13391935574, 0.0568617361898, 0.0938536088238, 0.0113586104411, -0.0693238677664, -0.253591012037, 0.0218794260685, 0.0395892893421, -0.0564839787634, 0.148746799726, 0.19188416787, -0.0778008355133 }))
          (Vector<double, 0>((double[]){ -0.0096899275558, 0.0472423753162, 0.225176512587, 0.0531001068778, -0.0558323758805, -0.0641805368462, -8.29458008438e-05, 0.00434475500792, -0.0247001355065, 0.099387863817, 0.0142596781367, -0.116186432087, 0.10163134916, 0.0778228629361, -0.054825804661, 0.132769801234, 0.0745641007004, -0.0734313957992, -0.12218566613, 0.0346486643686, -0.0929011674536, -0.105884634411, -9.26196130391e-05, 0.000418999173752, 0.151610280355, 0.0457572415133, -0.117398839572, -0.0956286113686, 0.0821563402548, -0.0946103167392, 0.0059697456568, 0.0443887586237, 0.101216430203, -0.0921428085591, -0.172408748378, 0.0827756304516, -0.0325074059675, -0.00619586274604, -0.0212787820776, -0.031363538086, -0.168289401543, 0.0900057464557, 0.08175295515, 0.0726342216385, 0.104923359064, -0.0145935765012, -0.146861423621, -0.211757678529, -0.00865407375485, 0.0110909158298, -0.0516688910851, 0.20388287926, 0.0816509273372, 0.0368436078861, 0.0490098665176, -0.0621124108691, 0.00280721905063, -0.0631478678114, 0.0062405464894, 0.0484479731265, 0.0765730104326, 0.0187114943003, 0.0285336101128, -0.0886172283429, 0.0095090034369, -0.0256342382959, 0.0599014335495, 0.0608318674612, -0.136640521051, -0.100078882638, -0.100579873886, 0.000638737790151, -0.0653811417392, 0.180198306271, 0.171055408936, -0.137309037231, 0.109976515521, -0.0962416939578, -0.0303160770023, 0.0602463588432, 0.214067844276, 0.0636752526523, -0.313121954392, -0.0322885933389, -0.186892415942, 0.0385130282414, -0.0909470226543, -0.156174284646, -0.0438596117569, 0.0463375071684, -0.176971400808, 0.0348409678828, 0.1316776437, -0.0614080840406, 0.113588476534, -0.0502423926494, 0.193987585087, -0.00210812446871, -0.0187415275452, -0.0774142273478, 0.080263312601 }))
          (Vector<double, 0>((double[]){ 0.00264205846252, -0.00716688301442, -0.0672004879536, -0.158833285491, -0.0218127119391, -0.0119016003074, -0.124408150907, -0.0334969038734, 0.0144057584291, 0.229816265092, 0.148447910003, -0.0840761737799, -0.0156679997969, 0.168970712268, 0.114285476951, 0.0580614073552, 0.108992319259, 0.088713912307, -0.0336349451389, 0.0279236574841, -0.0934935538383, 0.0719643737987, -0.0561424257157, -0.155519099285, -0.104720412361, 0.20059205066, 0.0491814796729, 0.0135224141849, -0.202587500312, -0.0689320294241, 0.0516463936424, -0.111641360599, -0.0095560281004, 0.022717574955, -0.188424798692, -0.192426762047, -0.151342736592, -0.07412220273, 0.199909245517, 0.00145478772744, 0.119703145053, 0.0341092983568, -0.0179756527592, -0.0279248805449, -0.0198779972703, 0.0831927297256, -0.00929879286251, 0.0228060058253, 0.0815364996062, 0.0815727667384, 0.0303765785148, 0.0973664632621, -0.193220338499, 0.0773362736384, 0.0112111950035, -0.044815626796, -0.195109092759, 0.0735260775949, -0.137865754313, -0.0481826806263, -0.028344294371, -0.0681772251313, 0.118707366757, 0.141598096735, -0.0454093597272, -0.109099829069, -0.00825240263716, 0.0236563418584, 0.19612341032, -0.110547929392, 0.0038965470961, 0.116566938726, 0.0166493374765, 0.0494844237003, -0.0520677735991, 0.0878578049302, -0.0249910286417, -0.135494695359, -0.0284776109287, -0.0187444795049, 0.0691637829668, -0.171106470443, 0.114756889253, -0.0429514371089, -0.0881754674227, -0.0230465171016, -0.0468051783594, -0.120259471512, 0.010336972096, 0.166925908558, 0.110983820895, -0.0798257730901, 0.0145148447602, -0.0554020756689, 0.18304010335, -0.00790028861961, -0.097748497684, 0.0806643057133, 0.0993068188902, 0.0580836847634, -0.0968666765058 }))
          (Vector<double, 0>((double[]){ -0.0548779969459, 0.015050984803, 0.0347346477728, 0.0330934945174, 0.0906185065607, 0.0872836586319, 0.038310733741, 0.0410460377316, -0.183271817081, 0.0348287485019, 0.0468866819159, -0.267829350637, -0.0489043324328, -0.0940568757557, -0.0606293057118, 0.115000628047, -0.097501439297, -0.0548165348972, -0.0203885064126, -0.0517139296281, 0.16116583291, 0.0744863388278, -0.116998498689, -0.163195220871, 0.0446363902215, -0.00702941220276, 0.115618064573, 0.075997401677, -0.0747940361063, 0.255582857363, -0.00438591692719, 0.0845905375414, -0.228666792903, -0.0423261923403, -0.0222363670068, -0.149740934158, 0.11584244816, -0.0205831282515, -0.0247278969943, -0.0379680377703, 0.0643077939979, 0.102648837415, -0.0203957766785, 0.01702527762, 0.0285137990741, 0.0950944696755, -0.111003656018, 0.0868044974282, 0.054507629099, -0.0513344992018, 0.0496224661464, 0.101774496489, -0.0218598730845, 0.109187712689, 0.123638143296, 0.0724172981824, 0.151177790348, -0.085335385462, -0.020986864886, 0.0334651702684, 0.000615390636023, -0.0016058625161, 0.00883618853482, -0.0494089982339, 0.0290647937051, 0.154528328233, -0.0427301542039, 0.037852790428, 0.133770952474, -0.0675567127229, 0.0399220482155, 0.12236606928, -0.182775647295, -0.0815884480571, -0.0120446067785, -0.0691571188378, -0.102879489277, 0.04526264577, -0.00786746728405, -0.16746898313, -0.0517768433835, -0.176830062564, -0.0859422427578, 0.223325454671, -0.117084072624, 0.0905122321777, 0.0821194538574, 0.037244510673, 0.0293470082656, -0.0456729597612, -0.121035860452, -0.0612147535629, -0.00012969680631, 0.138713323264, -0.230498974812, -0.058496461752, -0.00790484631453, 0.0592779769995, 0.0235748394637, 0.227580046403, -0.0357607413078 }))
          (Vector<double, 0>((double[]){ 0.0899868823902, -0.191449363529, 0.102299963694, 0.0221044145866, 0.0312882052846, -0.0204561318935, -0.161160408861, -0.0272302938773, 0.0582121515951, 0.00302136656324, -0.0645107191195, -0.157283989224, 0.115211659954, 0.116680752156, 0.0658722389788, 0.0297542042764, -0.103444035801, 0.00938925574252, -0.122718365071, -0.00558889453914, 0.00703919861161, 0.0154691463966, 0.133198418909, 0.101672221135, 0.080727525042, 0.143167320457, -0.0739545585543, -0.0137130975896, -0.0289284450034, -0.1378252732, -0.00641762236597, 0.0426874213785, 0.0284990899766, 0.0828890852092, -0.0620977280553, 0.0228467628492, -0.150980431345, -0.0997606550658, 0.0511435111616, 0.0931681622178, -0.00224037749127, -0.0286805988778, 0.0277243509734, 0.0509084308344, 0.120362875135, 0.0522003257099, -0.107095630229, -0.0999117131522, -0.105164966755, 0.034581280255, 0.274630925852, 0.00356177611646, -0.081467042975, -0.0500020822179, 0.276186626224, 0.160260541024, 0.134691020687, -0.0111981753893, 0.110175750528, 0.184394128773, 0.0181811627619, -0.125634472608, -0.218854481609, 0.0597665816894, -0.012007911743, -0.0578504381796, -0.102264487638, -0.0737725210578, 0.0859018923866, 0.0220807993187, 0.116242717406, -0.0826318695744, 0.0490211751634, 0.00519298587303, 0.0237223235542, 0.066033562377, 0.0663941877524, 0.110017380129, 0.0712323518917, -0.0237333979209, 0.00241951010965, 0.0216116197042, -0.0955694719975, 0.00660504484838, 0.176338680498, -0.0054829823921, -0.157541670603, 0.0623332011898, -0.0322120571242, -0.123072044811, 0.203828602281, 0.0891451805812, 0.0229821980434, 0.0630963265929, 0.196215477884, 0.0712899758664, -0.0858402828899, 0.179783009991, -0.0416470367588, -0.0605122196889, 0.089357421235 }))
          (Vector<double, 0>((double[]){ -0.159714604756, 0.00788124851469, -0.0185771676522, -0.107754131063, 0.0253840042259, 0.121304930875, 0.0215452869734, -0.0129105292419, 0.155319957718, 0.0992622610693, -0.0426943243702, -0.0181211648823, 0.0706215948225, 0.166491002676, 0.179541163623, -0.011364579459, 0.159092987212, -0.0890385180435, -0.152777547665, -0.103058531667, -0.0571479698139, -0.0810615671836, 0.0604329848199, 0.0730721844357, 0.0340787100495, 0.0547382485196, 0.016729967984, 0.0569086348642, 0.0562979826212, 0.0471375142224, 0.0138689125094, -0.00970956426664, -0.0262369484143, 0.0655145334459, -0.0682877658508, -0.0986130116552, 0.08301652438, 0.0659922919967, 0.025853785419, -0.0237275288296, -0.0552700260991, 0.0218828115777, -0.00298949323293, -0.171837167821, -0.035542013561, 0.0930324632201, -0.0116603190234, -0.105664608226, 0.102496573328, -0.00538260766853, -0.133463316041, 0.0579794660077, -0.115724563236, 0.122094705663, 0.043682697094, -0.0202886893101, 0.0329463869704, -0.00339806894404, 0.149884153214, -0.23828638469, -0.00385591597765, 0.0980047651021, -0.0356301844692, 0.242917987845, -0.270301999365, 0.00811633144575, 0.0344091358268, -0.00335943739497, -0.0454653849592, -0.048901714106, -0.211238362808, 0.189218548022, -0.03712583946, -0.204505750225, -0.0959112075019, -0.113734439694, -0.0527461269107, 0.0410909546701, -0.0624739449807, -0.110155775746, 0.0403649874971, 0.055527278932, 0.0750982135515, -0.0686869166799, -0.110980262982, -0.010222393304, -0.10501692646, 0.078150998478, -0.046876079375, -0.0768940549921, -0.0745017243379, 0.0796591060115, -0.0333309215833, 0.277415897506, -0.036656934936, -0.00790855708691, 0.145288226652, 0.142914696729, -0.0974939990712, 0.0227530726264, 0.0804826023572 }))
          (Vector<double, 0>((double[]){ 0.010120329825, 0.00574014969588, 0.0836556618678, 0.0303820619147, 0.0260640142277, -0.00386002728162, 0.136725201744, -0.0706477409314, -0.106674221105, -0.0257661688523, 0.0976673877122, 0.114498017704, 0.141634634431, 0.0537956091128, -0.00838411609654, 0.163435425609, 0.0726975780203, 0.0152747608137, 0.00592268649393, -0.0428830082491, -0.0364946978519, 0.117391852686, 0.129763938357, -0.123364284977, 0.137271894886, 0.00721779216136, 0.0681676973052, -0.193071261687, 0.0740383850569, 0.0226050110958, -0.133491821389, 0.113200185528, -0.0878398196058, -0.322406822131, -0.00825136129701, 0.0373506813112, 0.182285361606, 0.0265990156941, -0.0316423278824, 0.191210419674, 0.012261945725, 0.0430411381674, -0.10412500396, 0.171777033598, 0.0592008449834, -0.0567015124525, -0.177454888761, 0.102166445044, 0.152841595528, 0.0374719664778, -0.0405197516376, 0.0569943674473, 0.0264888811247, 0.032491182598, -0.0219923648365, 0.00214526901984, -0.0751773449895, -0.0768756822112, 0.103010223335, 0.0813728608869, -0.0945379141999, -0.00912180003343, 0.200107315083, -0.126528099488, 0.0536272274566, 0.129693398502, 0.0164511435594, -0.0304261828283, -0.119160514249, -0.00354073528288, -0.0359364462576, 0.0119639328863, 0.168718000504, 0.0782588720748, 0.0393181477736, 0.0716164061791, 0.00966430931169, 0.0601367225133, 0.105107449126, -0.104605219545, -0.025576655975, 0.0403268226536, 0.025477750006, -0.0357556386803, 0.0296462605581, 0.0259058530558, 0.217978927008, -0.046973095924, 0.123303016539, -0.0756724045158, 0.0121002940694, 0.0258767433168, 0.122041002959, 0.0561293587014, 0.0992088435153, -0.00700978010005, -0.190681242573, -0.206850210658, 0.0702096412356, -0.146717470233, -0.12437871891 }))
          (Vector<double, 0>((double[]){ 0.135025113892, 0.0414337834852, 0.0824090573834, -0.0730819464825, -0.201959754145, -0.00275358172003, -0.16125998065, -0.0408744021147, -0.00189537287034, -0.12110590657, -0.12916091911, 0.100945323548, -0.0176394604284, -0.0243576061103, -0.263334323174, -0.176109349501, -0.0376307044336, 0.0767302066476, 0.00229556949614, -0.0587653061467, -0.0213390882521, -0.0552717001434, -0.0455039686804, -0.135946995868, 0.00200063430561, -0.169570835342, -0.150074923316, -0.0411957442295, 0.0755845572491, 0.0610557591347, 0.0181352872395, 0.0127939178227, -0.105116866531, 0.100555193128, -0.0929494609785, -0.0312781737449, -0.135359482441, 0.104053192426, 0.0178091155825, 0.101257160805, -0.0544564744079, -0.0851162694772, -0.0960456528487, 0.0111387208197, 0.102350058229, 0.0821481513774, -0.066914225347, 0.00602490449519, 0.0309900140473, -0.151210487417, -0.14501477791, -0.121138291216, 0.00161338472711, 0.0173792546906, -0.00211424178314, 0.000801080344174, -0.152163359515, 0.0639974425962, -0.176680232675, 0.0652545227406, 0.00815380701149, -0.04502112767, -0.0429675103932, 0.0566456615366, 0.0397286553192, 0.077916255411, 0.04278765787, 0.0656114836306, 0.0633440328825, 0.0790063244197, -0.0150677040538, 0.00157934007674, 0.130310328619, -0.160590048969, -0.0979206582782, -0.0332968117138, 0.116111564635, 0.0270899648709, -0.093626248671, 0.00638649980779, 0.0740880643105, 0.117816484788, -0.0176226366631, 0.121270775731, -0.125055630157, 0.0747967406909, 0.0744565489688, 0.0748710929143, 0.0315165395689, -0.199468593692, 0.0501921139013, 0.0310254543294, -0.119344479095, 0.0448519296294, -0.052189076397, -0.328358557223, 0.00176435393688, -0.178479705089, 0.170014956103, 0.103193812575, -0.0860192838897 }))
          (Vector<double, 0>((double[]){ -0.0568561443584, 0.0701144030892, 0.0985126776611, -0.0842583051103, 0.0561341831605, -0.077783623446, -0.268042103132, 0.121686706765, -0.15294660231, -0.0230271411958, 0.00851321362345, -0.140292554283, -0.00744954541556, -0.146882760028, 0.081231746395, 0.0206101970572, 0.0862735477663, 0.0143282235143, 0.0808705339727, 0.0756777736118, -0.0265333458866, -0.0362552972862, -0.000325177101128, 0.00817098645815, -0.0221302050959, 0.146737406939, -0.0553155423365, -0.0801789809419, -0.0678439016039, -0.229952679064, 0.088106932553, -0.011981662412, 0.100436216528, 0.0176252816412, -0.151917509489, 0.0366788600931, 0.163826717615, -0.0411396898247, 0.0239999984647, -0.0303057171455, 0.09809801756, -0.00229649087307, 0.0331246195084, 0.0937761648675, 0.0372999862633, 0.0551761627784, 0.103133622555, 0.225846515534, 0.0824378752009, -0.11748961495, 0.117280222627, -0.149421342428, 0.10599392491, 0.0437413799052, -0.0231545772833, 0.120411950128, -0.0769059028982, 0.0302745954391, -0.0461381281028, -0.0787045715334, -0.0689763433343, -0.00997358800792, -0.00352265207383, -0.0105084857658, 0.0515155986316, -0.0146076106045, -0.0432216401442, 0.129734453801, -0.107513044035, -0.188073411991, 0.00627326768424, -0.0843955409944, 0.100714816834, -0.0091235579953, 0.0855950910048, -0.0694749576435, 0.186176526503, -0.218387151092, 0.0611416628756, -0.14746064101, 0.0423791439481, -0.034310501456, -0.0233057536772, 0.0762340475271, -0.167800231217, 0.0723876468467, 0.0656431539075, 0.0173223774879, -0.0852477655435, 0.0238025996761, 0.113411118648, -0.0160684672901, -0.283133871691, -0.115371555294, 0.101906956525, 0.0811247410976, -0.0236253801169, -0.0679616344383, 0.0908129956814, 0.0656273088752, 0.173722556974 }))
          (Vector<double, 0>((double[]){ -0.302709476714, -0.097698749002, -0.00504441406015, 0.0870874919097, -0.0562809199746, 0.157456220383, 0.189757017035, -0.163911963674, 0.176085770091, -0.219115689531, -0.0899165253804, -0.152961374718, 0.0529451332054, -0.0398053673127, -0.124324078984, 0.0223686315123, -0.0012327901926, -0.0598302951889, -0.0628423632373, 0.0674141671319, 0.131313240498, 0.0537076749748, 0.100980262802, -0.129932840165, 0.0737558434425, 0.0568435355877, -0.19730990333, -0.0354543153738, 0.0463932081489, 0.0104012497894, 0.0136808491845, 0.0611285511934, 0.00743580741039, -0.0223877722427, -0.0350889333296, -0.0789106160286, 0.119909676178, 0.00930355876003, 0.0698935801994, 0.0330794131362, -0.0250372636181, 0.116759987374, -0.019475236744, 0.114030831746, 0.00682881782807, -0.00208592690161, -0.203779980812, -0.012489463762, -0.204150116113, 0.108509131637, 0.0281925243722, -0.104870647172, -0.0430511985982, -0.00435667460664, -0.136172814508, -0.0554838423504, -0.0424165469893, 0.0406027028533, 0.226341518909, 0.0763967455976, 0.0712295081403, 0.0765616113778, -0.159653570516, 0.0537404412981, 0.0319624405468, 0.043740853129, 0.0161558108279, 0.102281400951, 0.00565731202865, -0.166318909156, -0.0358080921733, 0.0735535397504, 0.039561083483, -0.0597930981369, -0.0734529686447, 0.0152527584364, 0.0217258300787, 0.00908073387079, -0.0115102858481, 0.0432044003665, -0.0697912920167, 0.0625241718344, 0.0180490753373, -0.097659662608, -0.0755697491021, 0.0868066787315, 0.0757925634172, -0.189603264351, -0.0949203711509, -0.198370276219, -0.108979258421, -0.0481488496624, 0.0297100620083, 0.114325205674, 0.148835370572, 0.12854345766, 0.0104904600294, 0.0201478722149, 0.0731572589559, 0.111569082488, 0.0189483439336 }))
          (Vector<double, 0>((double[]){ 0.0772830792263, -0.227663235246, 0.094680952564, 0.181132873361, -0.0336166304652, 0.00355513002425, 0.121513387249, -0.0389165236444, 0.0731021212759, 0.0311464754806, 0.0476194661664, 0.0266791123555, -0.0244543460304, 0.0420210782205, -0.074985828118, -0.140687301505, 0.0278785894258, 0.018560352724, 0.0339633830141, -0.0601964111687, 0.0620854728029, -0.0133899176048, -0.0398778302768, 0.0854774564644, 0.0511269223968, 0.0357706601574, -0.136690144953, 0.12918161964, -0.0137098559178, -0.087713321902, -0.0364996638042, 0.0325268395686, -0.0303041620929, -0.109880338162, -0.0876654447612, 0.0282837801565, -0.117908795277, 0.0936413568131, 0.024188564571, -0.21100931703, 0.182063352879, 0.143725901199, -0.0196189826166, 0.0197211589283, 0.168464242427, -0.147650371167, -0.106237396009, 0.0848857433376, -0.0189316493759, -0.0245194381498, 0.0953003675113, 0.0356116117117, -0.0338001685656, -0.0135718895997, 0.158021500085, 0.0734503756474, 0.113504292595, 0.182644984435, -0.243919883874, 0.0454707148763, 0.0530373499421, -0.0513864746521, 0.0455326612922, -0.0517577728266, -0.0662138577689, 0.0581275680392, -0.14344740628, -0.185517719967, -0.0104535236673, 0.18196842649, -0.0902015136515, 0.190105136217, -0.0392062294809, 0.114937435431, -0.0977129905607, -0.0805094033333, 0.0624066575415, -0.00271111585283, 0.036764358443, -0.0127048094776, -0.0593583932555, -0.0603559346105, -0.146489588769, 0.0671923580169, -0.00681274636636, 0.0167006183979, 0.108418841459, -0.0361153556951, 0.0312574456481, 0.155241152702, -0.020470008663, 0.0277369148786, -0.0311695741806, -0.0971684507145, 0.2732609193, 0.1176338031, -0.143675232723, 0.173778945102, 0.0136253777335, -0.0529572813033, 0.0452222243813 }))
          (Vector<double, 0>((double[]){ 0.103291922032, 0.208178265377, 0.0668397653328, 0.169318386973, 0.00291838928233, -0.0297740650831, -0.148388129936, -0.00742978974136, -0.133736726195, -0.145849978508, -0.0929069084509, 0.0116402433761, 0.0523405270949, -0.066951965986, 0.0229210488897, -0.0979874761263, 0.0993129307194, 0.0580490779716, -0.142377725907, 0.0383913742084, 0.0845548646352, 0.202259961498, -0.0809619496029, 0.142568180928, -0.113750804259, 0.0471300173749, 0.0338300020108, 0.0213461862093, -0.105418793021, -0.00335263969198, -0.0618825171677, -0.0414750818401, -0.0458951904727, -0.0574796486333, 0.0678696458645, 0.31111456037, -0.14727757991, -0.0329844819137, -0.0249027038292, -0.034178686903, 0.17642368279, -0.0113183999096, 0.0157547022033, 0.00966134558183, -0.150580014655, -0.0265952186154, 0.0836014010276, -0.175370643525, -0.157448160252, 0.118254027669, -0.129282253176, 0.0162089694831, 0.0611383879612, -0.0906318263796, 0.122057357339, -0.0466867066636, -0.0542574901023, 0.070078800393, -0.019518596493, 0.0632612758748, 0.0516045661485, -0.0738633886336, -0.225084144685, -0.0976711349899, -0.0917091851596, -0.125736141512, -0.109210144568, -0.0138366771251, 0.0973481513122, 0.0659676137021, 0.0589940169742, 0.0161295587462, -0.0386982530991, -0.00851125657746, 0.0482906181333, 0.123060043759, -0.0926719836338, -0.0544127274091, -0.0165078251176, -0.149007810947, -0.00125384191698, -0.0717522794757, 0.110611103461, 0.0345502323189, -0.0233212301269, -0.0582686251042, -0.161671355111, 0.0629841756143, -0.026505448179, -0.108729916761, -0.0472591569303, 0.0911894444116, 0.0328223290501, 0.00915666113558, 0.0448360450804, -0.246463806829, 0.056425687317, -0.145144974391, -0.0622182213275, -0.104079182693, 0.0792549538352 }))
          (Vector<double, 0>((double[]){ 0.11243560066, 0.0296988082355, -0.102304095483, 0.0961306159207, -0.0307480265321, 0.139487584608, -0.112963595829, 0.0296571528434, -0.0135583618202, -0.00152854727098, 0.0264850842627, 0.0349221572385, -0.0345661289039, -0.00385870768229, 0.0297713014259, 0.0110163495504, -0.00336691231268, -0.0811240565331, -0.0994079991884, -0.145740268076, -0.121301583135, -0.131356220341, 0.108563794615, -0.107496927123, -0.111535562244, -0.0516630061896, 0.0812105668511, 0.0720549995213, -0.0789100663927, 0.0540387175469, -0.0577044130465, 0.031519091348, -0.254211985863, -0.0476728975953, -0.126289022015, 0.0115623008386, 0.0184712307758, -0.0146220745982, 0.0473758657607, -0.0427095928481, 0.00991946850291, -0.234097989764, -0.0922065114934, -0.100567269845, -0.209360435383, -0.0657919727174, 0.0188457428447, 0.0708329642802, 0.0904628504485, -0.139702539202, -0.218748550281, -0.0585114267311, -0.338007359236, -0.0792806286958, -0.0180061458016, -0.0671191730438, -0.0133582812414, 0.0452581026839, 0.0757556692994, -0.0403440134036, -0.139808726135, -0.0333982044342, -0.0484570543977, -0.118464476302, -0.0431739074851, -0.0770609217269, 0.0283371584539, -0.0047246598107, 0.0641368503614, -0.0932989781934, 0.0732119943207, -0.0880456140058, 0.00624621239487, -0.0684093701631, -0.134116212106, -0.119545912536, -0.054731302684, 0.0467324409153, 0.116975113937, -0.0261255862589, 0.115538909694, -0.115440582831, -0.132979206147, -0.0706794446001, -0.0544791648952, -0.0368686625247, -0.0905961397513, -0.163992274268, 0.203378325427, 0.0588242332566, -0.0317067105381, -0.19481179063, -0.0188382303033, 0.0436875827279, 0.063248909207, 0.117021974392, 0.0369729273012, 0.0773814078142, 0.0828502043347, 0.0644722382981, 0.165487234839 }))
          (Vector<double, 0>((double[]){ -0.0868033393222, -0.134744702107, 0.00172741548755, -0.0205139503055, 0.015031426344, -0.137801492766, 0.112442539556, 0.0603126675891, 0.078497965693, 0.205336504697, 0.0467778558002, 0.00615655048395, -0.0396406364731, -0.11612513546, -0.0751386908589, 0.0899679657705, 0.0802065252394, 0.018576782184, -0.0278061753637, 0.087072391727, -0.0090110350588, -0.0291999785301, -0.0830226467226, -0.0700659842343, 0.139634179203, 0.01527362282, -0.0627157784064, -0.130521993809, -0.157818021676, -0.0604710757393, -0.197947958109, 0.0574420249294, 0.149497292791, 0.102893295263, 0.227126511418, -0.0335281675306, -0.0812856498996, 0.0420265139809, -0.0581150801961, 0.0322716999296, 0.0668742195978, -0.0122338188423, 0.125094174257, 0.0348194599394, 0.135131642635, -0.106243808654, 0.0426147461513, 0.0166986260266, 0.103975271444, -0.0789947618107, 0.13390048969, -0.0377305410375, -0.0601577839002, -0.0765299437499, 0.103125210128, -0.235433694255, 0.0500295760458, -0.0437168558992, -0.149522106943, 0.0694267524414, -0.0878433113749, 0.258893602543, 0.0355522778794, -0.0625998002609, 0.0810155430676, 0.0040590452366, -0.152756476776, -0.161221609686, 0.145413449299, -0.0783682466741, -0.00760888761027, 0.150670885387, -0.0407786456455, -0.110773319217, -0.0857552396065, -0.126762730738, -0.0905963683827, -0.0435544327159, -0.0905698033209, -0.0766975847739, 0.117837561366, 0.0301691423479, 0.0489624769074, 0.103277450282, -0.095444532886, -0.201426887385, -0.0474304026188, -0.0795847703071, 0.0940849144109, 0.00320840034435, 0.0184819657845, -0.0217585363151, 0.124000181008, -0.0957627714876, -0.128246584155, -0.139991889447, -0.0272704180757, 0.0398767940565, 0.0750254178169, -0.0945210510094, -0.0600878820853 }))
          (Vector<double, 0>((double[]){ -0.14299947593, -0.0531380704625, 0.183858486936, 0.0390634981424, 0.0293176728571, -0.0899726068627, 0.102154429952, -0.0304981808241, 0.0320916748192, -0.197048697973, 0.15919028566, -0.0986274001207, 0.201086730316, -0.148628625362, -0.0854431773033, -0.0211810875452, 0.171026370833, -0.115573492107, -0.0619724381192, 0.0845342648299, 0.062426931592, -0.150376128118, 0.0422194695378, 0.0147999367574, 0.0243353659984, -0.0587557123487, -0.0588037707173, 0.125054160344, -0.0379167896752, 0.0173336293159, 0.00935491098641, 0.0150272714078, -0.00120110671327, 0.174026377134, -0.0326996297318, -0.0639983895843, -0.020159766595, 0.0507779371656, -0.0446345536119, 0.0699510733257, -0.154463398914, -0.0783282896237, 0.0482434574368, 0.0947444978727, -0.0988910774542, -0.0846132042869, -0.0280397130003, 0.0602214401901, 0.0520558111746, -0.0297230529271, 0.0657863879917, 0.142466448554, -0.0442019141142, -0.134076902267, 0.14183997673, 0.132212309625, 0.0749673921541, -0.0104403840783, 0.0882663118488, -0.0592158588485, -0.0239949177943, 0.0209532366572, -0.0636923947553, -0.00067196750139, -0.0633664842843, 0.14951375876, -0.111846330628, 0.164892569648, 0.0188334529601, -0.0669827108545, -0.116289269993, 0.0606559829706, 0.143830645594, -0.282841226328, 0.0897385886672, -0.0622334827825, 0.138320457659, 0.143361401172, -0.0677942961506, -0.2266102155, -0.122758050884, 0.0512014484115, 0.0874387358404, 0.0142705742582, -0.0853465076978, -0.103595343795, 0.105906789564, -0.041710726091, -0.100861666897, 0.187471328268, -0.0450843079634, -0.134342577461, -0.0220330018418, -0.105254348481, 0.0956657895521, -0.0587174346591, 0.00649517206012, -0.0244678080898, -0.0459917064125, 0.0558661817808, -0.0667028421675 }))
          (Vector<double, 0>((double[]){ -0.0790981834538, -0.174970170283, 0.0797405206351, -0.101382138674, 0.0484115057373, 0.0259240611522, -0.292686946338, -0.0222523987628, 0.111100107422, 0.142863313828, -0.190300001754, -0.022266995195, 0.0797956173918, 0.0537840148263, -0.0971756411017, -0.0309796031082, -0.0260814956833, 0.00979143998708, 0.0960990983437, -0.0987484326939, -0.197877358026, -0.0395430644233, -0.0763237321786, 0.0531442770288, -0.000717296799269, -0.0114420157026, -0.016312460282, -0.120905579768, -0.0103321166071, 0.117663402554, 0.106465126124, -0.0849443744351, 0.0241651304224, -0.0525085352427, -0.0543629095571, -0.0657693449212, 0.0813448962948, -0.0797939978169, 0.0811765905535, 0.134756273078, 0.0751710571531, -0.113068610094, 0.256359432446, -0.0212512779621, -0.141570757107, 0.0907596193295, -0.0482936922356, -0.123382920046, 0.058696646585, -0.0938964150479, 0.0535947730362, -0.00988356781165, -0.00074897644524, 0.0494476720859, 0.0547283944593, 0.0389902793623, -0.00125045139063, -0.0687442508288, 0.163018479003, 0.124698552021, 0.0189568704449, 0.193424392181, 0.0864876045892, -0.238714829802, 0.107530891301, 0.0850460818941, 0.00068024413096, -0.162830356076, -0.0428388936943, -0.0891017030054, 0.0772670815469, 0.0220499338005, -0.0853453446393, 0.142574184157, 0.0676623525283, 0.0709439480911, -0.0865468049145, -0.0292634883071, 0.0876887683036, 0.0832612329546, -0.134029656197, -0.140451196972, 0.211484319522, -0.122662610722, -0.0512203677085, -0.0622228592374, 0.0056910858359, -0.104484735373, 0.0978085188751, -0.0090650440018, -0.0750315154834, 0.0314059433301, -0.0974394749362, -0.000143401404324, -0.048279434253, 0.0637445737434, 0.037620906781, 0.111465485106, -0.0777478257689, -0.0717441663818, -0.0873937019499 }))
          (Vector<double, 0>((double[]){ 0.133761205939, 0.0104823992536, -0.16428971365, -0.128359646126, -0.122386741641, 0.0138385163257, -0.130110457099, -0.0244437590447, -0.104251460574, 0.0660914002276, 0.00256918290397, 0.0993699433382, -0.00493433623378, 0.124038886201, -0.110279420847, 0.0360677511799, -0.162613899994, 0.141796040725, 0.041001388476, 0.0125376861377, 0.00731326260846, 0.0335679575058, -0.155562076251, -0.0770398770423, 0.0732432741341, 0.0672732221082, 0.137882997046, 0.103375136862, 0.0314293155393, 0.0573423693741, 0.0192521880944, 0.00778258026313, 0.0520370986163, -0.0448794095876, -0.0311093544691, -0.0559703664073, -0.187292408342, -0.0735802316897, 0.119722210424, -0.119811070058, -0.145568035493, 0.000765536045384, 0.0198008484606, -0.0833231207176, -0.0586803617522, 0.025860567232, 0.0832930098392, 0.031059360524, -0.00226177975242, 0.0528803228676, 0.00559868251242, -0.0266895277563, 0.0279262395336, 0.0183242821209, -0.136550141106, -0.0372323042611, 0.121546431619, -0.132734888563, 0.0436988703222, -0.0212074729584, -0.097261502148, -0.000235625976013, -0.0928355343924, 0.124118198992, 0.113963419804, -0.203024535116, -0.000966753222677, -0.0243444101617, -0.150018686295, -0.0556481366518, 0.00581216257513, -0.0718916987142, 0.242618900516, -0.0529743727092, -0.00876724918099, 0.0107233307207, 0.159537927551, 0.15107563294, 0.0110146371052, 0.00939085826104, -0.0516356105047, 0.159406133773, 0.0716798149718, -0.047130329904, -0.0927925718911, 0.104197936055, -0.140892254089, -0.0941627033906, 0.0221352852071, 0.00880501430514, -0.122295598708, 0.12356559269, 0.0238273125936, -0.125254700185, -0.0268614094155, 0.0766544189254, -0.196420437209, 0.121085545965, 0.049569781979, -0.349386757504, -0.0733823954684 })).operator Points();
      }

      GaussianPoints::GaussianPoints():
        PointCloudFixture(4950, 1)
      {
        // GaussianPoints
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ 0.17795691155, 0.93409727292, 0.0427775162829, -0.405914639696, -0.491920399229, -1.47989706996, 0.386534612203, 1.59626650505, -0.299933125253, 0.0103707492759 }))
          (Vector<double, 0>((double[]){ -0.305256095069, -0.776302727613, 0.160157543428, 0.588696232535, -1.01501806872, 0.463764377357, -0.449579276435, 0.851227744381, -0.830825281089, 1.32973303531 }))
          (Vector<double, 0>((double[]){ -0.170184102832, 1.07637162916, -0.735245739802, -0.902682887583, 0.139312801515, 1.20603883199, 0.896945621171, -1.44112562761, -0.464589740976, -0.161441667607 }))
          (Vector<double, 0>((double[]){ -0.360028435448, 0.752313689985, -0.245879873971, 0.946743663511, -0.587234159412, 0.350869369782, 2.17545014922, 0.516172957301, -0.969474285218, 1.67656709185 }))
          (Vector<double, 0>((double[]){ 0.425598018884, 1.02856726248, 0.903447891997, 0.776616243755, -0.292061435303, -0.203289706465, 1.19294991426, -0.615763427832, 1.46174985232, -0.798954218294 }))
          (Vector<double, 0>((double[]){ -1.07732240793, 2.2027163093, -0.02492255026, -2.05830709361, -1.9143171032, -0.209835860548, 0.70479228601, -0.590695070343, -0.0322725523627, 0.864706751789 }))
          (Vector<double, 0>((double[]){ -0.947053634783, -0.132662827354, 0.260081503327, -2.02804348459, -0.00474669655984, -0.0545020140601, 1.15227626366, -0.0700056274997, 1.35187905107, -0.85112800352 }))
          (Vector<double, 0>((double[]){ -2.37396334685, 0.882445533802, 2.09405235021, -0.885867078625, 0.151186008443, -0.587819740723, -0.00152731959854, 0.387305966311, -0.11236320059, -0.869549631929 }))
          (Vector<double, 0>((double[]){ 0.0677450913588, 0.385284027061, 0.701504068602, 1.75054969866, -0.336603836775, 1.0862537878, 1.61787318086, -1.8444901784, 1.34620656128, 1.0196474694 }))
          (Vector<double, 0>((double[]){ -0.881214438713, -0.228287384775, -0.692101077913, -0.594912850295, 0.449669959502, -0.65667407833, -0.331887154293, 0.131448831551, -0.91163523961, -0.941666224209 }))
          (Vector<double, 0>((double[]){ 1.48778610382, -1.46599710607, 1.05871063074, -1.66776775643, 0.0654681123657, 2.30151806631, 1.41015735954, 1.31582873446, 0.493031475766, 0.388032637337 }))
          (Vector<double, 0>((double[]){ 1.79324447077, -0.793697770728, -0.67133521671, -0.118405176007, 0.614990710842, 0.9061401962, 0.147356390387, 1.04815756767, 0.469626240638, -1.81333814689 }))
          (Vector<double, 0>((double[]){ 0.440336374104, -0.755727131558, 0.857409510985, 1.93502873701, -0.524419303403, -1.64662746152, 1.58632820183, -0.667912059074, 0.410517222039, -1.40857181582 }))
          (Vector<double, 0>((double[]){ -0.29091646724, 0.76820973663, -1.14267352465, 0.615782017328, 0.915769023545, 1.37325326112, 1.6338111743, -0.0222697733738, 0.0692291871342, 0.239453890631 }))
          (Vector<double, 0>((double[]){ -0.0105760809605, -0.434976932059, 1.2354228227, 0.249510254505, -2.0871879389, -0.0021774575129, 0.177840138093, -2.02292038063, -0.258614904821, -0.672738738451 }))
          (Vector<double, 0>((double[]){ -0.30158416991, 0.904111010727, 0.112253463522, 0.400454304267, 0.895257311084, 0.134814133946, -0.413143324792, 1.55637957526, 0.354996489723, 1.16777120394 }))
          (Vector<double, 0>((double[]){ -0.0651261856159, 0.256038403598, 1.52393919727, 0.98310743513, 1.15557661788, 0.493066905648, 0.560937796965, 0.364715373456, -0.175680518214, -0.443188423117 }))
          (Vector<double, 0>((double[]){ -1.57024248084, 1.05990643419, 0.42901187138, -0.366747888251, -0.729343164328, -0.166247621663, 1.15052557099, -0.358234963717, 2.37055984041, 0.536610431116 }))
          (Vector<double, 0>((double[]){ -0.193992155608, 1.71647825497, 0.0375712126451, 1.39850332149, 0.18267475677, -0.63019649877, -1.92591406289, -0.39432492356, -0.725781503123, 0.995078394847 }))
          (Vector<double, 0>((double[]){ 1.88441178619, 1.36147244876, 1.57989913019, 1.22537646907, -0.963796876818, 0.181788123389, 0.476575869843, 1.02730285878, 0.49539648426, 0.048153482182 }))
          (Vector<double, 0>((double[]){ 0.770993981449, -0.0520163961869, -0.294419505023, 0.0324625444924, 0.10712815362, -1.48063071656, 1.58732560248, 0.409724825254, -0.673819250217, 1.41903897864 }))
          (Vector<double, 0>((double[]){ 0.197589059577, -0.0903400643838, 0.357791516243, -0.0388665522254, 1.72310790392, -0.145770586546, 0.368089874892, 1.10377648187, -0.474289280654, -1.00265571672 }))
          (Vector<double, 0>((double[]){ 0.740282103742, 0.577827585006, -0.0882010367525, 0.242212199209, 0.575799694641, -1.20178710477, -0.084849298175, 1.01821852733, -0.119529015547, 1.54622588019 }))
          (Vector<double, 0>((double[]){ 0.18795379405, 1.64002401225, 1.50516107044, -0.852116824563, 0.436388988094, -1.18686616842, 0.27165638763, 0.160034502802, -1.2313996009, 1.35192896649 }))
          (Vector<double, 0>((double[]){ -1.37560005515, -0.311877763316, 1.49435002138, 0.177292480645, -1.05739874943, 0.667187913739, -0.180461152623, 2.56759100154, 0.964386415888, 0.592415564208 }))
          (Vector<double, 0>((double[]){ 0.0445881051012, -1.71692282885, 0.533247152073, 1.66224650692, -0.630813912884, 0.661755143469, -0.11544951538, -1.02719364954, 1.20109482218, 1.03450805616 }))
          (Vector<double, 0>((double[]){ 0.33641623994, -2.482042817, -0.495429745018, 1.00453640079, 0.262898084753, -0.995867805924, 0.733039549909, 2.59633310575, -0.508281517283, 0.0642808233122 }))
          (Vector<double, 0>((double[]){ 1.81158308903, 3.31560168414, -0.425244176064, 1.91588374667, -0.400825795444, -0.161920129728, -0.170204769697, 2.23857462865, -1.77375184771, 0.698100299831 }))
          (Vector<double, 0>((double[]){ 0.481591797177, 0.617673974749, -1.09205528708, 0.907251953451, 1.76651394534, 0.172313487966, -0.377742969445, -0.36740399083, -0.21077967009, 2.37854321556 }))
          (Vector<double, 0>((double[]){ -0.81249870809, -1.28938901752, -0.64009801562, 1.25947673404, -0.00627503343221, -0.240258601398, -0.580250963109, 0.371935851634, 1.12976530843, 0.117589940802 }))
          (Vector<double, 0>((double[]){ 0.0220082305845, 0.0779606643895, 0.482414596221, 1.53099485566, 0.309585476917, 0.633542192863, -0.0848966984881, 1.0280738143, -0.275130681322, 0.277727139014 }))
          (Vector<double, 0>((double[]){ 1.9064718765, 0.317709045795, 0.354561487539, 0.240523732468, 2.25888518607, -1.12633480757, -0.159755313408, 0.633693473047, -0.689212840552, -0.13192739291 }))
          (Vector<double, 0>((double[]){ -0.851239065339, 1.54478094857, 1.09277916118, 0.0510998881472, 1.46953047533, -1.3391164343, 0.305515878041, 0.461518760296, -1.31977404329, -0.562516596474 }))
          (Vector<double, 0>((double[]){ 2.25781406412, -1.01007085566, 0.671375723151, -1.09161127286, -1.41778313044, -0.757454512165, -0.860322095355, -0.881555174306, 0.0956567418832, -0.886109975831 }))
          (Vector<double, 0>((double[]){ -0.376758133233, -0.278679187735, -0.556269565909, -0.670843039304, 0.617166158735, -0.671554937513, -1.32708440937, -0.428194977942, -0.464215322766, 1.52509313176 }))
          (Vector<double, 0>((double[]){ -0.147621598687, 0.138742636564, -0.486592602219, 0.247185206922, 0.337727421227, 0.232208269191, 0.292704486824, -0.352556231484, -0.505713491738, 0.265664274838 }))
          (Vector<double, 0>((double[]){ -0.36140063543, -0.234141297438, -2.14919079474, -1.22256241808, 0.394510282795, 0.024305831394, 1.02113943092, 0.710028733972, -3.76327972712, -0.160033574591 }))
          (Vector<double, 0>((double[]){ 1.00003562908, -1.25980691041, -0.0987088313254, -0.273682768397, 1.88405945257, -0.405602354142, -1.97288497296, 1.51111779344, -1.52560501154, 1.58998742987 }))
          (Vector<double, 0>((double[]){ 0.308861011395, 0.0792433748051, 0.414335472056, 0.00335718521365, -0.084086806926, 0.357308970839, -0.0432559618892, 0.602037535831, 0.365775767655, -2.83595821799 }))
          (Vector<double, 0>((double[]){ -2.66264081547, 0.608484242622, -1.39883376375, 1.26410911423, 0.0955992741206, -0.144236559549, -0.690907178984, 0.342742328456, 0.303768504068, -0.866282033067 }))
          (Vector<double, 0>((double[]){ -1.20562815688, -0.236780316323, 0.225628400056, 0.159484104368, -0.347996909738, -0.157612157277, -1.28124900349, -2.2737171514, 0.66954090538, 0.435653940962 }))
          (Vector<double, 0>((double[]){ 0.365607703733, 1.55579260568, 0.959324906613, -1.22622114249, -0.557592873555, -1.88678074083, 0.673439387365, -0.99011488019, -0.61150049326, 1.49067199936 }))
          (Vector<double, 0>((double[]){ -0.913017211923, -0.888354425931, 1.82617807459, -0.180502434069, 0.0329450075155, 1.80618498576, 1.67386344652, 0.0385046154929, 1.09498269158, -0.811876131038 }))
          (Vector<double, 0>((double[]){ -1.35366626828, 0.682727650223, -0.536561288095, 0.410931369021, -0.239634785094, 1.22130517286, -0.0687752448944, 0.715832166916, -2.03248104932, -1.01165577727 }))
          (Vector<double, 0>((double[]){ 0.105887210514, 0.338909064424, -1.38405471747, 0.822893949486, -0.419517916482, 0.406792873729, -2.27722086315, -0.691429238906, 0.749255919618, -1.25611766555 }))
          (Vector<double, 0>((double[]){ -0.293660448887, -1.26762273979, -0.642259233987, 0.0641663163346, 0.634385761906, 0.258149545994, -0.283896467051, 0.332252052188, -1.32235636806, 0.441190797458 }))
          (Vector<double, 0>((double[]){ 1.46542733726, -0.102804481182, -0.91998813283, -0.614352467529, -0.546285305884, -0.750598651062, 1.35982259136, 1.15574157007, -1.37115890719, 1.15004633411 }))
          (Vector<double, 0>((double[]){ -0.148274098986, 1.77323939184, -0.0948542819121, 0.0492254915998, 1.61365924245, -0.135205491773, 1.13807454715, 1.29911919011, -1.19895360929, -0.780373571044 }))
          (Vector<double, 0>((double[]){ 0.499463975651, -1.21168403574, 0.747810396039, 0.115714537725, 1.6317259227, -0.850548405555, 1.11563711653, -1.02648962403, -1.74745689739, -0.102122516746 }))
          (Vector<double, 0>((double[]){ 1.11376526508, -0.114210460215, 1.15150495492, 0.592587546827, -0.203998903038, 0.453830782584, 0.305320308614, 0.354906214323, -1.86786708395, 1.29604238939 }))
          (Vector<double, 0>((double[]){ 0.257423097406, -0.104512038147, -0.691687419753, -0.375452355685, -1.22778858117, -0.544673133543, -0.330045703231, -1.01350819531, -1.38885993479, 0.445956201569 }))
          (Vector<double, 0>((double[]){ -0.925405530091, 0.438347672861, 1.35899988955, 0.5114835209, 2.09646899544, 0.581497165447, 0.365971272093, -1.32213716567, -0.0374926698324, -0.47033674889 }))
          (Vector<double, 0>((double[]){ -0.153715748356, -0.839005869679, -0.558169435106, -1.0413079597, 0.0506019060382, -1.67645465443, 0.868000714349, 0.136039981177, 0.850736371775, 1.28989960089 }))
          (Vector<double, 0>((double[]){ -0.0836979356228, 1.08272348143, 0.289393123752, -1.09079975819, -1.01116291291, 1.53904445102, 0.0737900144466, 0.0785060024082, 0.255239906517, -0.889223218189 }))
          (Vector<double, 0>((double[]){ -0.88524616446, 1.83815395491, 0.22358108109, -0.717615884044, 1.71502206759, 0.442581663136, -0.261511404903, 0.477497210443, 0.115974306554, 0.281707440382 }))
          (Vector<double, 0>((double[]){ 0.380467717439, 0.291826894763, 0.924446079428, -0.136780675325, 0.571831008456, 0.335643309031, -0.0233865083109, -0.0673727755491, -0.639434104932, 0.408822956786 }))
          (Vector<double, 0>((double[]){ -0.430738136461, 1.14070406255, 1.29406302682, -0.947836274133, -0.895658612183, -1.64164946827, -1.11010096132, -0.877469777155, 1.08831064387, 0.533624837457 }))
          (Vector<double, 0>((double[]){ 0.18683732882, 0.452463560997, 0.523658345063, -1.12838041571, 1.38457439417, -0.665937020675, -0.389489804674, -0.225247225369, 1.85341519245, -0.00987517613344 }))
          (Vector<double, 0>((double[]){ 2.29676175379, 0.0111999943954, -0.548358372985, -1.01234463452, 2.26969177969, 1.34100017135, -0.63408221332, -1.42205940994, -1.16263991146, -0.828715490317 }))
          (Vector<double, 0>((double[]){ 0.00914999274225, 2.03846800469, -3.13124254848, -0.560860977347, 0.161887122956, -0.107427978802, 0.0983985297568, 0.819434985353, 0.954984612128, -0.0110732316248 }))
          (Vector<double, 0>((double[]){ 0.172848036463, -0.180956381133, 0.143831327841, 1.16605802849, -0.889594186873, -2.60874129881, -0.366389348492, -2.28452326532, 2.70630678887, 1.20585190972 }))
          (Vector<double, 0>((double[]){ 0.753844836065, 0.00501687357927, -0.743636407979, -0.530615122609, -1.64676120137, 0.293256549702, -0.700673109337, -0.952443020435, -1.75799455787, 0.598707703761 }))
          (Vector<double, 0>((double[]){ -0.649143914526, 0.589328906134, 0.0937418237311, 0.22922597584, -0.955696402613, -1.4190239616, -1.95594336641, 0.772747067469, -1.32413940753, -1.30776368403 }))
          (Vector<double, 0>((double[]){ 1.794725739, 0.311994126264, -0.823572206945, 0.556893874905, -1.53577930103, -0.624227842864, -0.310306087178, -0.348113289426, 0.678818843321, -0.298832645052 }))
          (Vector<double, 0>((double[]){ -0.695820768693, -0.359340311308, -0.530919423836, -1.49080382544, -2.5059339103, 1.96616180935, 0.350656235696, -1.76757545416, 0.473585777961, 0.56970668584 }))
          (Vector<double, 0>((double[]){ -2.49615304128, 1.69486847392, 0.9902142437, -0.63065836496, 0.965458809665, -0.321558188689, 1.19191092533, 0.671834778066, 2.56544481443, 0.231224842288 }))
          (Vector<double, 0>((double[]){ 1.36669632344, -1.01653389048, 0.241627287905, 0.32976728693, -0.590112720808, -0.229607790161, -1.24841246215, -1.05646187804, -0.437571796953, -1.35908910567 }))
          (Vector<double, 0>((double[]){ 0.658239064732, 1.01803366188, 0.987246309255, 1.40706165093, 0.0172082287225, -0.557976234213, -0.395845666499, 0.672762399711, 0.332091835871, -1.70654610896 }))
          (Vector<double, 0>((double[]){ 1.50897411308, -0.0207237962884, -1.1437479617, 0.724337853176, 0.448361538312, 0.323039580143, -0.697134861961, -0.651072557653, 0.010243937222, 2.08207765537 }))
          (Vector<double, 0>((double[]){ -0.964518944063, 0.682486710228, 2.16619607782, 0.388968022687, 1.38350118047, 0.971344899448, 0.0075292532301, -0.827534830985, 0.0615974165493, -1.1000006697 }))
          (Vector<double, 0>((double[]){ 0.174122063199, 1.83268513454, -0.577422739707, 1.05282365327, 0.534522839717, 1.65442425265, -1.88757241281, -1.97251533813, 1.01294257218, -1.28059707613 }))
          (Vector<double, 0>((double[]){ 1.07532071746, 0.964113134193, -1.20474915129, -0.250856436743, 0.539076022147, -0.385645182694, -1.23029634756, 0.566659947898, 1.39604501855, 0.150696517754 }))
          (Vector<double, 0>((double[]){ -0.379625281249, 0.237685254661, -1.54124419276, -0.209494205796, -1.55569394366, 0.320430121696, 0.835346515206, 0.746997673298, -0.286919145547, 1.3416742575 }))
          (Vector<double, 0>((double[]){ 0.524055347832, -0.261649255033, -1.14337903423, 0.694338701487, -2.19391869582, 0.356604772405, -1.18586280121, 1.42227096354, 1.55624085118, -1.31736679638 }))
          (Vector<double, 0>((double[]){ 2.55056861958, 0.957799848698, 0.897253599964, 0.59395117402, -0.328888746953, 0.907150165475, -1.98683267584, 1.18178125891, -0.410290948272, -0.335973605521 }))
          (Vector<double, 0>((double[]){ 0.175707054108, -0.45404727467, -1.28442335261, -0.84657953774, -0.521303100611, -0.264594741625, -0.782956141819, 0.347189376082, -0.399348932492, -1.78147913133 }))
          (Vector<double, 0>((double[]){ -0.275574414029, 1.12716983518, -0.956741446548, -1.52822518163, -0.51954823992, -1.06089574244, -1.02856408458, -0.951936004026, 1.80059578229, 0.350375878164 }))
          (Vector<double, 0>((double[]){ -1.02740870387, 0.291330368454, 0.120978612116, -1.49736463835, -0.0386925775086, -0.847069759796, -0.54273954296, 2.69542410971, -0.0876933938393, 0.543661394994 }))
          (Vector<double, 0>((double[]){ 1.27374935033, -0.639848200262, 0.109116868033, 0.879311211724, 0.814712869178, 1.64144426802, -0.082224932436, -0.259371836843, -0.093735990741, -0.533454808675 }))
          (Vector<double, 0>((double[]){ -0.441404579023, 0.345174628539, 0.786346991276, 1.85395727507, 0.256843373884, 1.36968852896, -0.792762310074, -1.22529260998, -0.375820890137, 0.285229749285 }))
          (Vector<double, 0>((double[]){ 0.191597462885, 1.88833208518, -0.108897913021, 2.3547479242, -1.17225703903, -0.621656012709, -2.08432673675, -1.10370263435, -0.866253638826, 0.282813336278 }))
          (Vector<double, 0>((double[]){ 1.14890733866, -0.36582091293, 0.998998832979, 1.02502168676, -0.950680368917, 1.25475382392, 0.001016082512, 1.35555290521, 0.599700836981, 0.499294706548 }))
          (Vector<double, 0>((double[]){ 0.590082194349, -0.933148481433, -0.226914395474, 1.20929826558, 1.09754148344, -0.623473003426, 0.156394899559, -0.872577187503, -1.99856910827, 2.10748736738 }))
          (Vector<double, 0>((double[]){ 0.851169401587, 0.986962163556, 0.471155842082, -0.987684466746, -0.373515483648, -1.2063359668, 0.884392293316, 0.636720166601, 1.58197369818, 0.0308507835601 }))
          (Vector<double, 0>((double[]){ -0.546687669697, -0.63604490946, 0.80582178167, -1.89909340606, -0.471261956186, 1.48229208637, -1.90138083472, 0.0225594594167, 0.497990496788, 0.0576367931955 }))
          (Vector<double, 0>((double[]){ -2.25299328776, 0.619782081318, 1.72973474168, 0.564570905422, -0.561963413563, 0.344682850105, -0.427846219049, -0.890415708145, 1.55771131902, -0.504331624792 }))
          (Vector<double, 0>((double[]){ -0.399569895712, 1.60582903605, -1.02534497719, 0.107868648853, -1.4032741996, -0.157876879458, 1.94978762639, 0.0233288083042, -1.53485144978, -1.39858356257 }))
          (Vector<double, 0>((double[]){ 0.507648334167, 0.866156626236, -2.00334276687, 0.97322175454, 0.217798214175, 0.0340620184767, -0.448180754775, 0.0903858703995, 1.6932232186, -0.9436586537 }))
          (Vector<double, 0>((double[]){ -0.0854059139265, -0.478390750723, -1.62914103888, 1.76943985339, 2.20932471915, -1.83843668552, -0.40651544211, -1.5965863426, 1.13806450894, -0.147888842786 }))
          (Vector<double, 0>((double[]){ -1.22553761895, 0.433632747939, -0.555824368751, -1.22433717431, 0.272346555702, -0.0689049458126, -1.64300555513, 0.225097822535, -0.238189072112, 0.776087552331 }))
          (Vector<double, 0>((double[]){ 1.37236680547, -0.875564745509, -0.261243675047, 0.209054275382, -0.0915798811579, 1.15797913283, 0.687279936209, -1.87510366568, 2.75905155962, -1.3252894206 }))
          (Vector<double, 0>((double[]){ -0.29359736753, 0.634124648784, -0.597379643681, 0.730165389383, -0.334089358738, -0.46631610594, 0.229703827012, 0.125721921276, 0.292824946817, -0.490916785556 }))
          (Vector<double, 0>((double[]){ -0.538150439517, 1.32478702707, 0.961231623539, 1.56196832038, -0.167752591958, 2.21641869233, 2.80180001332, 1.25143268327, 0.698750788788, 1.05335128245 }))
          (Vector<double, 0>((double[]){ -0.229171782518, -0.00317206715246, -1.38911271207, -0.81603348008, 0.609536159307, -1.04471582976, 1.44217096779, 0.101760783682, 0.580968024308, 0.753219141185 }))
          (Vector<double, 0>((double[]){ 1.15777041256, 1.62115499862, 0.949245334038, 0.124638801211, 1.62512213668, -0.143784000693, 0.969389107809, 1.17407716933, -0.740197400857, -1.58514257005 }))
          (Vector<double, 0>((double[]){ -0.826617232132, 0.553450110137, -0.693669492184, 1.10656309141, 0.0642421031963, -1.08204839989, 0.323583069527, -0.119742165459, -1.61690340005, 0.643061151837 }))
          (Vector<double, 0>((double[]){ 0.422501583018, 2.52934048914, -0.988149410793, -0.704221915177, -0.330721562997, 0.215582285442, 1.88978747818, -0.798289945007, -0.5105424061, -0.564292311662 }))
          (Vector<double, 0>((double[]){ -1.06196364776, -1.7094881008, -0.397833628619, 0.889580983523, 1.91259608095, 1.05989990152, 1.0880025134, -2.11344089521, -0.0671775757884, -1.09527205101 }))
          (Vector<double, 0>((double[]){ 0.879792245881, 0.225133915622, -0.122278204504, -0.0201003321201, 0.122510514938, 1.46720233546, -0.641318813309, -0.905137715182, 1.30561497057, 0.132170055055 }))
          (Vector<double, 0>((double[]){ 0.0372619666781, -0.0714929187192, -0.128029900107, -0.445845999876, 0.16144023346, -1.89150449719, 0.938569440281, 0.0243727893461, 0.47584121197, 0.891861042793 })).operator Points();
      }

      RandomTorusPoints::RandomTorusPoints():
        PointCloudFixture(19900, 2)
      {
        // RandomTorusPoints
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ -51.3349413327, 5.02056266052, -9.87441306572 }))
          (Vector<double, 0>((double[]){ 42.0387695908, 1.37489872768, 6.0808058722 }))
          (Vector<double, 0>((double[]){ 40.9233976609, -17.7247481773, 8.41471576375 }))
          (Vector<double, 0>((double[]){ -53.86570321, 4.94394873751, 9.12439680465 }))
          (Vector<double, 0>((double[]){ 24.5404448703, 31.641345676, 0.921835866663 }))
          (Vector<double, 0>((double[]){ 34.5310028937, -23.1536621613, 5.3869936622 }))
          (Vector<double, 0>((double[]){ 3.27910574945, 42.3617569722, -6.60129359408 }))
          (Vector<double, 0>((double[]){ 59.3884305175, -7.77239127847, 1.4462016614 }))
          (Vector<double, 0>((double[]){ 31.2019628379, -49.8587455469, 4.71782652459 }))
          (Vector<double, 0>((double[]){ 45.0667797768, 39.5788867229, -0.644841128997 }))
          (Vector<double, 0>((double[]){ -21.6033884163, 34.7563721783, -4.196722295 }))
          (Vector<double, 0>((double[]){ -40.5796653733, -0.786033652606, 3.3764854769 }))
          (Vector<double, 0>((double[]){ 37.5965371414, -45.7976061642, 3.79231165756 }))
          (Vector<double, 0>((double[]){ 47.0545695705, -10.9144018375, -9.85509468144 }))
          (Vector<double, 0>((double[]){ 36.7679349336, 28.7632263964, 9.43347911911 }))
          (Vector<double, 0>((double[]){ 31.2569337559, -35.2614455815, -9.57653079945 }))
          (Vector<double, 0>((double[]){ 11.7501534059, 58.6004158072, 2.14683246813 }))
          (Vector<double, 0>((double[]){ -54.1799265624, 25.7321904412, 0.630865659166 }))
          (Vector<double, 0>((double[]){ 3.5628641557, -55.7690642472, -8.08660437272 }))
          (Vector<double, 0>((double[]){ 18.0942634635, -45.9537784987, 9.98124131476 }))
          (Vector<double, 0>((double[]){ -28.589742745, 28.4664847417, -2.60380622641 }))
          (Vector<double, 0>((double[]){ 57.6739502349, 2.15972925613, -6.36305223893 }))
          (Vector<double, 0>((double[]){ -9.94788395406, 38.7454114, 0.204498078058 }))
          (Vector<double, 0>((double[]){ 39.4715748615, -9.22126398029, -3.22525416122 }))
          (Vector<double, 0>((double[]){ -28.673167653, 34.5470728686, 8.59937094842 }))
          (Vector<double, 0>((double[]){ 16.925526589, -54.6330533817, 6.94514224173 }))
          (Vector<double, 0>((double[]){ -36.1682619139, 47.7705550444, -1.27784304788 }))
          (Vector<double, 0>((double[]){ -47.6043745237, -24.8242920501, 9.29501159237 }))
          (Vector<double, 0>((double[]){ 32.6531633052, 32.1120029638, -9.07407757362 }))
          (Vector<double, 0>((double[]){ 38.4963390771, -43.007008022, -6.35651502179 }))
          (Vector<double, 0>((double[]){ 41.1868019027, -25.6699326797, 9.89157488194 }))
          (Vector<double, 0>((double[]){ -24.2400537075, 39.4192735972, -9.28068491524 }))
          (Vector<double, 0>((double[]){ 1.58147415917, -40.2245176778, 2.24645500544 }))
          (Vector<double, 0>((double[]){ -56.3892055919, 20.3512310302, 1.00598386236 }))
          (Vector<double, 0>((double[]){ -22.180265624, -41.5453547291, -9.56888417736 }))
          (Vector<double, 0>((double[]){ 14.021171233, 40.9593827937, -7.41708295322 }))
          (Vector<double, 0>((double[]){ -36.3289881676, 34.9680744945, 9.99101479716 }))
          (Vector<double, 0>((double[]){ 51.1916458054, -29.1281696691, 4.56250601225 }))
          (Vector<double, 0>((double[]){ -16.3758412068, 37.7609742991, -4.67288730241 }))
          (Vector<double, 0>((double[]){ 36.9323730946, 23.2818142815, 7.73189792447 }))
          (Vector<double, 0>((double[]){ -35.5293154034, 24.7115523531, 7.40377780245 }))
          (Vector<double, 0>((double[]){ 34.6509902928, 27.0258685292, -7.95780955917 }))
          (Vector<double, 0>((double[]){ -2.67099635635, -44.3882800205, 8.3308625346 }))
          (Vector<double, 0>((double[]){ -29.4754297632, -51.9401836, 2.34616108086 }))
          (Vector<double, 0>((double[]){ 19.7611548861, 35.8057291215, 4.1391924241 }))
          (Vector<double, 0>((double[]){ 40.7504895546, 15.0306271054, 7.54248795695 }))
          (Vector<double, 0>((double[]){ 48.8627319419, -33.5634755072, 3.72674588241 }))
          (Vector<double, 0>((double[]){ -41.5444496319, 13.297711321, -7.70099057731 }))
          (Vector<double, 0>((double[]){ 45.5071130967, 24.1417920023, 9.88467930472 }))
          (Vector<double, 0>((double[]){ -53.7401771076, -21.5504590855, -6.13085786585 }))
          (Vector<double, 0>((double[]){ 1.10969633891, 49.9860015118, 9.99999985849 }))
          (Vector<double, 0>((double[]){ 50.6811264569, 8.50093233236, -9.90304623204 }))
          (Vector<double, 0>((double[]){ -23.713392219, 36.2450506786, -7.43545504562 }))
          (Vector<double, 0>((double[]){ -14.6517612287, -39.2575276952, 5.86788338972 }))
          (Vector<double, 0>((double[]){ -41.3312685894, 35.617069062, 8.89953585771 }))
          (Vector<double, 0>((double[]){ 39.0133759586, 19.1117452809, 7.55029587968 }))
          (Vector<double, 0>((double[]){ 25.3072640506, -53.8297313644, -3.17700908057 }))
          (Vector<double, 0>((double[]){ -0.354519438845, -44.1910690075, -8.1408132054 }))
          (Vector<double, 0>((double[]){ -35.6505855518, -18.949015159, 2.70792611328 }))
          (Vector<double, 0>((double[]){ -29.3120138163, -49.5710356477, -6.51218502796 }))
          (Vector<double, 0>((double[]){ -26.1056027793, 54.0013932726, -0.625181951273 }))
          (Vector<double, 0>((double[]){ -47.7302345286, 13.0820261917, 9.98701468534 }))
          (Vector<double, 0>((double[]){ 43.1133510051, -16.171303113, 9.18526775576 }))
          (Vector<double, 0>((double[]){ 58.304856715, 6.17925051989, 5.04967042594 }))
          (Vector<double, 0>((double[]){ 14.4709976662, 38.6206377546, -4.82808051661 }))
          (Vector<double, 0>((double[]){ -20.868902327, -38.7247709565, 7.99248056691 }))
          (Vector<double, 0>((double[]){ 39.9561805432, -14.1812982629, 6.49710071912 }))
          (Vector<double, 0>((double[]){ 20.3820977315, 37.7556237527, -7.04796948339 }))
          (Vector<double, 0>((double[]){ 29.0565533509, -28.9592240484, 4.40690189241 }))
          (Vector<double, 0>((double[]){ -9.76303163894, 57.8756416731, 4.94227255812 }))
          (Vector<double, 0>((double[]){ -36.7968906063, -16.6777075287, -2.79987637779 }))
          (Vector<double, 0>((double[]){ 47.6022231746, 31.6929497582, 6.95264487247 }))
          (Vector<double, 0>((double[]){ 22.8013999169, -42.7452687744, -9.87859245851 }))
          (Vector<double, 0>((double[]){ 49.3793481821, 23.7415323363, 8.77796831462 }))
          (Vector<double, 0>((double[]){ -59.3669484694, -6.07485317305, -2.52123591412 }))
          (Vector<double, 0>((double[]){ -49.5468850378, 7.79663309496, 9.99877424935 }))
          (Vector<double, 0>((double[]){ 7.4728796147, -44.648955833, -8.81062596848 }))
          (Vector<double, 0>((double[]){ 33.2915661492, 23.661012001, -4.01921729615 }))
          (Vector<double, 0>((double[]){ 2.96068162322, 55.1372322338, 8.53149540245 }))
          (Vector<double, 0>((double[]){ 38.9748840022, 23.134412388, -8.83927131912 }))
          (Vector<double, 0>((double[]){ 58.5840785358, -5.40532439234, 4.68824351705 }))
          (Vector<double, 0>((double[]){ -38.1259127458, 46.3216267332, 0.346769191689 }))
          (Vector<double, 0>((double[]){ 11.9523509163, -53.5467572108, 8.73708142771 }))
          (Vector<double, 0>((double[]){ 38.1411545811, -44.3840152635, -5.23405380874 }))
          (Vector<double, 0>((double[]){ -18.3933902345, 51.5299072373, -8.81906649608 }))
          (Vector<double, 0>((double[]){ -33.1818379168, 24.455027652, 4.78648048781 }))
          (Vector<double, 0>((double[]){ -44.0117057257, 37.3280248962, 6.36869144638 }))
          (Vector<double, 0>((double[]){ -33.8505158239, 42.2807140791, 9.09275646257 }))
          (Vector<double, 0>((double[]){ -50.0287242978, -26.9069769114, 7.32706686205 }))
          (Vector<double, 0>((double[]){ 8.21373352132, 55.6823352699, 7.77819164076 }))
          (Vector<double, 0>((double[]){ -34.6825448165, -48.8725510686, -1.19507020053 }))
          (Vector<double, 0>((double[]){ 52.4785008061, 3.20815912719, 9.66239078923 }))
          (Vector<double, 0>((double[]){ -22.4933611345, 40.5292167485, 9.31111597674 }))
          (Vector<double, 0>((double[]){ -15.8439196255, -44.7302702564, 9.67031123536 }))
          (Vector<double, 0>((double[]){ 53.2318264367, -15.0051648482, -8.4760611269 }))
          (Vector<double, 0>((double[]){ -51.5368128763, 22.1214501751, -7.93639713174 }))
          (Vector<double, 0>((double[]){ -28.1258659901, -33.5187790323, -7.81091373253 }))
          (Vector<double, 0>((double[]){ 40.0676779967, -38.8006142857, -8.16355256983 }))
          (Vector<double, 0>((double[]){ 26.1636077432, 53.9475500283, 0.92361071104 }))
          (Vector<double, 0>((double[]){ 44.0477381595, 15.2731692221, -9.41164802533 }))
          (Vector<double, 0>((double[]){ -9.1134687046, -44.7973195978, -9.03538632456 }))
          (Vector<double, 0>((double[]){ -7.11859924034, 39.4402346877, 1.24263135025 }))
          (Vector<double, 0>((double[]){ -23.5859206728, 32.6381032271, -2.30115895598 }))
          (Vector<double, 0>((double[]){ -13.7552409555, 45.872199064, 9.77489036227 }))
          (Vector<double, 0>((double[]){ -37.8432615849, -13.4884510712, 1.86394714523 }))
          (Vector<double, 0>((double[]){ -55.7636299368, -20.36679392, -3.50250065531 }))
          (Vector<double, 0>((double[]){ 9.81010882748, 47.3321152382, 9.86092957989 }))
          (Vector<double, 0>((double[]){ -20.0989702609, -56.1585056597, 2.63407325595 }))
          (Vector<double, 0>((double[]){ -11.507611565, -49.8744664239, 9.9295599903 }))
          (Vector<double, 0>((double[]){ -34.4828451179, -21.1203076774, -2.92318641647 }))
          (Vector<double, 0>((double[]){ -38.7798522481, 35.8179922989, -9.60285209089 }))
          (Vector<double, 0>((double[]){ 20.6394823832, 50.2232311666, -9.02885373523 }))
          (Vector<double, 0>((double[]){ 37.3185838042, 39.581995984, -8.97974757149 }))
          (Vector<double, 0>((double[]){ -57.5667726464, -9.92386689153, -5.40118140498 }))
          (Vector<double, 0>((double[]){ -51.5622516346, -29.447091589, 3.47061971256 }))
          (Vector<double, 0>((double[]){ 40.0957224054, -27.7229785275, 9.92113779378 }))
          (Vector<double, 0>((double[]){ -0.911847762004, -42.9438971809, -7.09562696137 }))
          (Vector<double, 0>((double[]){ 20.5228885857, -55.0080490432, -4.90965331183 }))
          (Vector<double, 0>((double[]){ -57.1036321677, 14.5126796707, 4.52242518512 }))
          (Vector<double, 0>((double[]){ -38.9900448629, -45.5021643534, -1.24493506041 }))
          (Vector<double, 0>((double[]){ 22.763535603, 45.2526242625, 9.97849358875 }))
          (Vector<double, 0>((double[]){ -54.9738115561, -15.3279875239, 7.07141493014 }))
          (Vector<double, 0>((double[]){ -18.8506145921, -52.4692615694, 8.17960102003 }))
          (Vector<double, 0>((double[]){ 15.9695424375, 41.3523020755, -8.23632585931 }))
          (Vector<double, 0>((double[]){ 54.1388526469, -0.798707485813, -9.10060972811 }))
          (Vector<double, 0>((double[]){ 29.1397197561, 31.462663822, -7.02567144674 }))
          (Vector<double, 0>((double[]){ -19.4801511029, -34.9970734727, -1.03164959936 }))
          (Vector<double, 0>((double[]){ -21.095443055, -53.1519135783, 6.95509426557 }))
          (Vector<double, 0>((double[]){ 44.410213782, -5.47397390723, -8.50873961227 }))
          (Vector<double, 0>((double[]){ 52.0602077544, 0.310349455763, -9.78528137504 }))
          (Vector<double, 0>((double[]){ -29.4045678359, 27.5873973184, -2.50905706326 }))
          (Vector<double, 0>((double[]){ -24.1589255887, 36.6805209056, -7.94065935682 }))
          (Vector<double, 0>((double[]){ 59.3987378192, 7.51664213401, 1.59209967508 }))
          (Vector<double, 0>((double[]){ 52.5845577367, -2.30896143259, -9.64653222508 }))
          (Vector<double, 0>((double[]){ 29.6707219356, 33.7476033505, 8.62304168616 }))
          (Vector<double, 0>((double[]){ -30.8736992208, 28.6375333096, 6.14456497424 }))
          (Vector<double, 0>((double[]){ -17.2224930584, -36.3534350178, 2.11716587185 }))
          (Vector<double, 0>((double[]){ 5.23065142971, -48.4752971379, 9.92240712167 }))
          (Vector<double, 0>((double[]){ 59.8213062095, 4.14360650187, -0.840198111082 }))
          (Vector<double, 0>((double[]){ 27.3141949923, -31.5893631197, -5.6668644478 }))
          (Vector<double, 0>((double[]){ -11.0994440356, 48.6724308736, -9.99969556885 }))
          (Vector<double, 0>((double[]){ 29.6557789495, 32.87772287, 8.20010567701 }))
          (Vector<double, 0>((double[]){ -12.3338646707, -38.4697858432, 2.79527029014 }))
          (Vector<double, 0>((double[]){ -44.3497930727, -7.62002227563, -8.66005442552 }))
          (Vector<double, 0>((double[]){ 49.5556194192, -24.5541947129, -8.47670864505 }))
          (Vector<double, 0>((double[]){ -49.2875084826, -6.7801644779, -9.99691624863 }))
          (Vector<double, 0>((double[]){ 40.548280404, -15.7452237397, 7.59762219476 }))
          (Vector<double, 0>((double[]){ -58.2575191891, -14.0456764128, 1.20786519035 }))
          (Vector<double, 0>((double[]){ -19.7910012662, 56.2821078829, -2.58404877724 }))
          (Vector<double, 0>((double[]){ -8.45537154601, -53.8853921539, -8.9075994059 }))
          (Vector<double, 0>((double[]){ 5.25617050906, 49.9462790911, 9.99753354176 }))
          (Vector<double, 0>((double[]){ 29.0038896552, -49.8551471522, 6.40679594161 }))
          (Vector<double, 0>((double[]){ 40.9840302515, -6.66920656904, -5.30494251676 }))
          (Vector<double, 0>((double[]){ 22.5494756704, -38.7052385707, 8.53849639617 }))
          (Vector<double, 0>((double[]){ -40.4020500252, -13.6577790225, 6.77861273438 }))
          (Vector<double, 0>((double[]){ -49.6409494258, -33.5598146256, -1.25721286036 }))
          (Vector<double, 0>((double[]){ 59.7996374255, 4.46315236488, 0.824402931065 }))
          (Vector<double, 0>((double[]){ 26.4570779381, -30.4371605228, -2.54252596819 }))
          (Vector<double, 0>((double[]){ 57.4079089878, 12.5457898895, -4.81804995868 }))
          (Vector<double, 0>((double[]){ -19.6525760164, -46.1947709449, -9.99797168732 }))
          (Vector<double, 0>((double[]){ -36.9434490957, 45.4395497388, -5.16554990108 }))
          (Vector<double, 0>((double[]){ -4.77115155271, 42.0937313087, 6.45602486974 }))
          (Vector<double, 0>((double[]){ 0.276523800929, 42.7697935237, 6.90919965018 }))
          (Vector<double, 0>((double[]){ 5.95074456617, -45.5128016558, 9.12093589653 }))
          (Vector<double, 0>((double[]){ 10.2326707462, 38.6710766333, 0.199811568522 }))
          (Vector<double, 0>((double[]){ -56.041309067, -19.9363318108, 3.17730872588 }))
          (Vector<double, 0>((double[]){ -48.3812589368, -35.0136150459, 2.34213969241 }))
          (Vector<double, 0>((double[]){ 57.9401656536, 4.91227796952, 5.79738080714 }))
          (Vector<double, 0>((double[]){ 34.0251408577, -48.2191725804, -4.32728640701 }))
          (Vector<double, 0>((double[]){ 33.2386741827, 49.1052963983, 3.68306209816 }))
          (Vector<double, 0>((double[]){ -47.2929465231, -14.6999740547, -9.98870585066 }))
          (Vector<double, 0>((double[]){ -53.8398144131, 11.7375521557, -8.59912915098 }))
          (Vector<double, 0>((double[]){ 10.8505817439, -50.5126460571, 9.86043037811 }))
          (Vector<double, 0>((double[]){ -2.98169442515, 47.4905168521, -9.70376616909 }))
          (Vector<double, 0>((double[]){ 46.4325310189, -14.6710304514, 9.91450477861 }))
          (Vector<double, 0>((double[]){ 44.1284437154, 38.745599963, -4.88742657466 }))
          (Vector<double, 0>((double[]){ 18.209943857, -44.8242387816, -9.86822980106 }))
          (Vector<double, 0>((double[]){ 10.4322014831, -49.187394623, 9.99603671712 }))
          (Vector<double, 0>((double[]){ -27.8033846972, -28.7645024237, -0.32582945583 }))
          (Vector<double, 0>((double[]){ -58.2205014659, -11.5776464415, -3.5186762668 }))
          (Vector<double, 0>((double[]){ -29.7730340893, -26.7274461885, -0.444255349039 }))
          (Vector<double, 0>((double[]){ 43.7305065079, -22.97696689, 9.98194615697 }))
          (Vector<double, 0>((double[]){ 21.7359048655, 51.7794408142, 7.88014460738 }))
          (Vector<double, 0>((double[]){ -2.45284922073, 55.1916537341, 8.51340695995 }))
          (Vector<double, 0>((double[]){ -5.73060178424, -56.527974493, 7.31566039798 }))
          (Vector<double, 0>((double[]){ -48.0660861689, 5.37790696454, -9.86559998949 }))
          (Vector<double, 0>((double[]){ -13.6500017646, 47.3973208129, 9.97710552122 }))
          (Vector<double, 0>((double[]){ 4.45577102366, -58.3665595722, -5.20864800619 }))
          (Vector<double, 0>((double[]){ 41.1592546796, -36.1312486307, -8.79003118075 }))
          (Vector<double, 0>((double[]){ 6.58288039272, 57.5554262314, -6.09135676063 }))
          (Vector<double, 0>((double[]){ -56.9615276034, -12.1125050715, 5.67299841906 }))
          (Vector<double, 0>((double[]){ 58.3329975857, -1.85262927057, 5.48362173821 }))
          (Vector<double, 0>((double[]){ -37.7180238623, 45.4412371398, 4.24233630401 }))
          (Vector<double, 0>((double[]){ -37.334589775, 14.9884424238, 2.13649449704 }))
          (Vector<double, 0>((double[]){ -27.3689253684, 29.2260779507, 0.896326872051 }))
          (Vector<double, 0>((double[]){ 41.7867164572, 10.2055553274, 7.15601892987 }))
          (Vector<double, 0>((double[]){ 30.44805455, 30.0434955061, -6.91364841189 }))
          (Vector<double, 0>((double[]){ 33.4207139888, -27.2732678281, -7.27290675858 }))
          (Vector<double, 0>((double[]){ 58.844730953, -2.61581519906, 4.55405271156 }))
          (Vector<double, 0>((double[]){ 32.8643875382, -46.3937135435, 7.28111044697 })).operator Points();
      }

      RandomTrefoilKnotPoints::RandomTrefoilKnotPoints():
        PointCloudFixture(19900, 1)
      {
        // RandomTrefoilKnotPoints
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ -1.05243414452, 2.77814445048, -0.239855176738 }))
          (Vector<double, 0>((double[]){ -0.0476310024366, -1.25285353382, -0.665675454057 }))
          (Vector<double, 0>((double[]){ 0.919125465629, 1.89779823634, -0.994079419442 }))
          (Vector<double, 0>((double[]){ 0.220484089647, 1.45264197061, 0.847546833567 }))
          (Vector<double, 0>((double[]){ 1.03783905409, 0.27154675606, -0.374506287725 }))
          (Vector<double, 0>((double[]){ 2.64716141026, -1.04369161953, -0.534007282824 }))
          (Vector<double, 0>((double[]){ 1.9532589937, -1.59061525885, -0.854784290573 }))
          (Vector<double, 0>((double[]){ 1.1299737323, -0.71494546366, 0.748758016305 }))
          (Vector<double, 0>((double[]){ -0.930666915544, 0.710250934207, -0.558840913645 }))
          (Vector<double, 0>((double[]){ -1.77313151294, 0.250807760273, -0.977869007662 }))
          (Vector<double, 0>((double[]){ 0.0808582370674, 2.67309375297, -0.738442535151 }))
          (Vector<double, 0>((double[]){ -2.14475128616, 0.225648341754, 0.987663883929 }))
          (Vector<double, 0>((double[]){ 1.04882099695, 0.318172426099, -0.427574101086 }))
          (Vector<double, 0>((double[]){ -1.91775011576, -2.25960776242, -0.266941761311 }))
          (Vector<double, 0>((double[]){ 1.01056022928, -0.133467772851, 0.195698419457 }))
          (Vector<double, 0>((double[]){ -0.409745310772, -0.926305689087, -0.160004588582 }))
          (Vector<double, 0>((double[]){ 0.968823367112, 1.80106384377, -0.998982304291 }))
          (Vector<double, 0>((double[]){ 1.54886992208, 1.71523329095, 0.950388508778 }))
          (Vector<double, 0>((double[]){ 1.14708107619, 0.900991036557, -0.840779693931 }))
          (Vector<double, 0>((double[]){ -2.24537917733, 1.72621485619, 0.554425612327 }))
          (Vector<double, 0>((double[]){ 2.66032700477, 1.02614382541, 0.524565638808 }))
          (Vector<double, 0>((double[]){ -0.327120221618, 0.994942472285, 0.304033344694 }))
          (Vector<double, 0>((double[]){ -1.20647116231, 0.61182212289, -0.76226725427 }))
          (Vector<double, 0>((double[]){ 1.12510919003, -1.29930417497, 0.959631162563 }))
          (Vector<double, 0>((double[]){ -0.515633832787, -0.857231153836, 0.0268931030123 }))
          (Vector<double, 0>((double[]){ -0.20781096933, 2.77403366099, -0.623520959058 }))
          (Vector<double, 0>((double[]){ -1.84021990787, 2.34032250899, 0.212476259385 }))
          (Vector<double, 0>((double[]){ 1.14483283507, 1.74370972953, 0.996299897568 }))
          (Vector<double, 0>((double[]){ -2.32834018265, 0.729896127157, 0.897966005797 }))
          (Vector<double, 0>((double[]){ 2.42950608803, 1.28110228097, 0.66529132248 }))
          (Vector<double, 0>((double[]){ 1.13718458002, 1.21125163753, -0.94093792087 }))
          (Vector<double, 0>((double[]){ -0.228251955173, -1.08696584311, -0.457271074359 }))
          (Vector<double, 0>((double[]){ -1.36970492122, 2.66602977933, -0.0734371904591 }))
          (Vector<double, 0>((double[]){ 0.168723165291, 1.41888860162, 0.82087006743 }))
          (Vector<double, 0>((double[]){ 0.361470393681, -1.53366845774, -0.9055171127 }))
          (Vector<double, 0>((double[]){ -1.79083158668, -2.38663672484, -0.179223358427 }))
          (Vector<double, 0>((double[]){ 1.00026862868, -0.02074552512, 0.031100455435 }))
          (Vector<double, 0>((double[]){ 0.791117084128, -1.69456887062, -0.991532559408 }))
          (Vector<double, 0>((double[]){ -1.08560692071, -2.76961904167, 0.223156261962 }))
          (Vector<double, 0>((double[]){ -0.705260786643, 2.82698508697, -0.406547598545 }))
          (Vector<double, 0>((double[]){ 2.99966653991, 0.0338110058819, 0.0169058608786 }))
          (Vector<double, 0>((double[]){ 1.11990245054, -0.647807690155, 0.707980737601 }))
          (Vector<double, 0>((double[]){ -0.747473088595, -0.765052981153, 0.366519624582 }))
          (Vector<double, 0>((double[]){ 2.97975559636, -0.262702689142, -0.131520556754 }))
          (Vector<double, 0>((double[]){ -0.225517197359, -2.77844887039, 0.616204651659 }))
          (Vector<double, 0>((double[]){ 2.99285528963, -0.156354534691, -0.0782127591158 }))
          (Vector<double, 0>((double[]){ -1.55318827811, -2.56607018801, -0.0310349205597 }))
          (Vector<double, 0>((double[]){ -1.22330641419, 0.604627962262, -0.772159162704 }))
          (Vector<double, 0>((double[]){ -2.01892916497, -2.13612399365, -0.343275627835 }))
          (Vector<double, 0>((double[]){ -1.7107350242, -2.45451450583, -0.127294748581 }))
          (Vector<double, 0>((double[]){ 1.10233992317, 0.551990234447, -0.641432538814 }))
          (Vector<double, 0>((double[]){ -0.441362894362, -0.903306850983, -0.103473882579 }))
          (Vector<double, 0>((double[]){ 2.99137474661, -0.171756481845, -0.0859253157428 }))
          (Vector<double, 0>((double[]){ 0.220459411961, -2.60272637656, 0.790821772513 }))
          (Vector<double, 0>((double[]){ 1.11163595224, -0.600300054662, 0.676292045473 }))
          (Vector<double, 0>((double[]){ 0.716891496767, 1.67515692106, 0.984050342836 }))
          (Vector<double, 0>((double[]){ -1.43974055258, -0.495866416332, 0.878762003381 }))
          (Vector<double, 0>((double[]){ -2.1068353457, -2.00618702078, -0.416323284534 }))
          (Vector<double, 0>((double[]){ -2.25958788409, -0.482950168896, -0.950533220089 }))
          (Vector<double, 0>((double[]){ -1.57324553745, -0.410975246426, 0.927444348621 }))
          (Vector<double, 0>((double[]){ -0.306834647309, 1.01326170797, 0.337572640612 }))
          (Vector<double, 0>((double[]){ -0.299963015873, 1.01955962342, 0.348711259263 }))
          (Vector<double, 0>((double[]){ 1.804859633, 1.64855756087, 0.895811319631 }))
          (Vector<double, 0>((double[]){ 2.99851104085, 0.0714342838053, 0.0357205190531 }))
          (Vector<double, 0>((double[]){ 1.15343925994, -1.7440400047, -0.995854897513 }))
          (Vector<double, 0>((double[]){ -2.23936052751, 0.428913885285, 0.959980636937 }))
          (Vector<double, 0>((double[]){ -0.678151173851, 0.787468219099, -0.277340062792 }))
          (Vector<double, 0>((double[]){ 0.0464076098274, 1.33002820046, 0.7431161845 }))
          (Vector<double, 0>((double[]){ -2.16279281583, -0.259756488087, -0.983969713199 }))
          (Vector<double, 0>((double[]){ -0.508641884252, -0.861107338001, 0.0149127208386 }))
          (Vector<double, 0>((double[]){ 1.00348682391, -0.0753409973145, 0.112171347314 }))
          (Vector<double, 0>((double[]){ 0.00909301138631, 1.30031717736, 0.714484735078 }))
          (Vector<double, 0>((double[]){ -2.33872011626, -1.38883158766, -0.693961673518 }))
          (Vector<double, 0>((double[]){ 1.00577865031, 0.0975475710191, -0.144519222718 }))
          (Vector<double, 0>((double[]){ 2.5564135325, 1.15348793978, 0.593816351992 }))
          (Vector<double, 0>((double[]){ -1.51824624992, -0.447819570361, 0.908866635005 }))
          (Vector<double, 0>((double[]){ 1.86426835524, 1.62717268053, 0.880251452695 }))
          (Vector<double, 0>((double[]){ 1.148813186, -0.950811998438, 0.860912050444 }))
          (Vector<double, 0>((double[]){ -0.0646462887827, 2.73099351966, -0.6815639604 }))
          (Vector<double, 0>((double[]){ 0.934197955458, -1.86991876678, 0.995915305434 }))
          (Vector<double, 0>((double[]){ 0.601169952925, 1.63819522691, 0.966945882379 }))
          (Vector<double, 0>((double[]){ 1.93761428852, 1.59744833955, 0.859454100357 }))
          (Vector<double, 0>((double[]){ -2.2334009072, 1.75628584809, 0.540668759413 }))
          (Vector<double, 0>((double[]){ -1.26075893792, 0.588022590758, -0.79328132667 }))
          (Vector<double, 0>((double[]){ 1.75634202224, 1.66430687938, 0.907690915417 }))
          (Vector<double, 0>((double[]){ 1.00969431448, 0.127598349403, -0.187444936359 }))
          (Vector<double, 0>((double[]){ 2.02922956842, 1.55485471723, 0.83089298625 }))
          (Vector<double, 0>((double[]){ 0.910156912542, 1.9138582749, -0.992863733991 }))
          (Vector<double, 0>((double[]){ -2.36424190259, -1.11157974244, -0.790456788655 }))
          (Vector<double, 0>((double[]){ 0.325943236115, -2.53872901536, 0.828785015659 }))
          (Vector<double, 0>((double[]){ -1.12535668294, 2.7584673262, -0.202948422598 }))
          (Vector<double, 0>((double[]){ -0.818043491849, -0.743862856284, 0.447426865807 }))
          (Vector<double, 0>((double[]){ 1.06245442359, -0.374799071083, 0.487048249326 }))
          (Vector<double, 0>((double[]){ 1.13373868466, 0.744192410658, -0.765165052217 }))
          (Vector<double, 0>((double[]){ 0.469822877083, 2.43394406575, -0.87788354007 }))
          (Vector<double, 0>((double[]){ -0.254738194798, 2.78531039247, -0.604065035405 }))
          (Vector<double, 0>((double[]){ 1.11647096643, 0.627448069118, -0.694702011322 }))
          (Vector<double, 0>((double[]){ -0.0895923006174, -1.21605043126, -0.624963856073 }))
          (Vector<double, 0>((double[]){ 0.21471871732, -2.60592957299, 0.788713755327 }))
          (Vector<double, 0>((double[]){ 1.14870018352, -1.04373118988, 0.894063081559 }))
          (Vector<double, 0>((double[]){ 0.749309067765, -2.15244539912, 0.960250024177 }))
          (Vector<double, 0>((double[]){ -1.59042701945, 2.54205746348, 0.0531718779914 }))
          (Vector<double, 0>((double[]){ 1.02900837348, 0.232107353594, -0.326668889913 }))
          (Vector<double, 0>((double[]){ 1.1063548789, 1.39846673896, -0.976211564039 }))
          (Vector<double, 0>((double[]){ 0.652015176144, 2.26492455829, -0.934140210431 }))
          (Vector<double, 0>((double[]){ 1.0894399857, 0.490853160633, -0.593156306345 }))
          (Vector<double, 0>((double[]){ 0.983800380617, 1.76885506044, -0.999711154883 }))
          (Vector<double, 0>((double[]){ 0.829304649738, 2.043909694, -0.978605541177 }))
          (Vector<double, 0>((double[]){ 0.542533826823, 2.37201384802, -0.901265183634 }))
          (Vector<double, 0>((double[]){ 1.11818504502, -1.33944674723, 0.966897993787 }))
          (Vector<double, 0>((double[]){ 2.77703766952, 0.846368653122, 0.429325796828 }))
          (Vector<double, 0>((double[]){ -0.0637720014966, 2.73068945252, -0.681912229056 }))
          (Vector<double, 0>((double[]){ 2.42420546328, 1.28589392176, 0.668025530964 }))
          (Vector<double, 0>((double[]){ -0.800648243164, 0.749013824522, -0.42834485401 }))
          (Vector<double, 0>((double[]){ -1.50363215064, -0.457155351385, 0.903585291936 }))
          (Vector<double, 0>((double[]){ 1.1492594101, -1.00867143535, 0.882198542257 }))
          (Vector<double, 0>((double[]){ 1.29721291291, -1.74356789277, -0.984886993397 }))
          (Vector<double, 0>((double[]){ -2.24882283446, 1.7172584817, 0.558477690551 }))
          (Vector<double, 0>((double[]){ -2.14148604099, -0.219656621303, -0.988269207857 }))
          (Vector<double, 0>((double[]){ 2.20126506034, -1.45662653326, -0.768733030595 }))
          (Vector<double, 0>((double[]){ -0.608669521251, -0.813313726095, 0.177356005769 }))
          (Vector<double, 0>((double[]){ -0.985905761765, -0.692916600645, 0.606673910407 }))
          (Vector<double, 0>((double[]){ -0.205241050294, -1.10884251005, -0.488930185603 }))
          (Vector<double, 0>((double[]){ 2.85721891631, 0.685424359763, 0.345868719885 }))
          (Vector<double, 0>((double[]){ 2.04260602577, 1.54810111065, 0.826471855707 }))
          (Vector<double, 0>((double[]){ -1.06117361294, -0.667681280213, 0.665665188833 }))
          (Vector<double, 0>((double[]){ 0.315805057845, 1.50911361509, 0.888850708834 }))
          (Vector<double, 0>((double[]){ -2.05017317944, 0.0702801965914, 0.998679307586 }))
          (Vector<double, 0>((double[]){ 2.87611192158, -0.64023880859, -0.322672320786 }))
          (Vector<double, 0>((double[]){ -2.14133036913, -0.219372288221, -0.988297602355 }))
          (Vector<double, 0>((double[]){ 1.10188509417, -0.549732974798, 0.639735098228 }))
          (Vector<double, 0>((double[]){ 0.995815418977, 1.73154178001, 0.999996798838 }))
          (Vector<double, 0>((double[]){ 2.19675420242, -1.45953602892, -0.770518082014 }))
          (Vector<double, 0>((double[]){ -2.01872865219, -0.025423679671, -0.999821591882 }))
          (Vector<double, 0>((double[]){ 0.991130506256, -1.7309603321, -0.999985600978 }))
          (Vector<double, 0>((double[]){ -1.99817127819, -0.00243409389042, 0.999998330597 }))
          (Vector<double, 0>((double[]){ -0.985614892553, 0.693010203341, -0.606432566119 }))
          (Vector<double, 0>((double[]){ -0.47349723634, -0.882015932575, -0.046361898551 }))
          (Vector<double, 0>((double[]){ 1.13130708754, 0.724965119396, -0.754467819204 }))
          (Vector<double, 0>((double[]){ -0.412120414562, -0.924508872398, -0.15576246269 }))
          (Vector<double, 0>((double[]){ -2.33722976123, -1.39736887407, -0.690741608018 }))
          (Vector<double, 0>((double[]){ -0.526855628138, -2.82461812708, 0.487123082309 }))
          (Vector<double, 0>((double[]){ 0.72260502181, 2.18521131816, -0.953438400202 }))
          (Vector<double, 0>((double[]){ 2.57248506432, 1.13532801282, 0.583829298119 }))
          (Vector<double, 0>((double[]){ 2.96702081023, 0.334692112922, 0.167697746628 }))
          (Vector<double, 0>((double[]){ 0.643165921441, 2.27427853796, -0.931604679282 }))
          (Vector<double, 0>((double[]){ 1.14447322562, 0.856158091116, -0.821140926723 }))
          (Vector<double, 0>((double[]){ -0.99130508776, -0.691174110622, 0.611134432151 }))
          (Vector<double, 0>((double[]){ -1.23695882997, 2.72151538482, -0.144986855409 }))
          (Vector<double, 0>((double[]){ 2.70800924176, 0.958447687718, 0.488402768429 }))
          (Vector<double, 0>((double[]){ 0.660026053213, -2.25634074371, 0.936414760247 }))
          (Vector<double, 0>((double[]){ 0.387059342752, 2.49687255508, -0.850054353834 }))
          (Vector<double, 0>((double[]){ 1.09489151041, -1.44795477664, 0.982797348012 }))
          (Vector<double, 0>((double[]){ 1.36608232827, -1.73932628594, -0.977343512274 }))
          (Vector<double, 0>((double[]){ -0.936956042707, 0.708317941364, -0.564495925758 }))
          (Vector<double, 0>((double[]){ 2.45144547626, -1.26084721592, -0.653777863827 }))
          (Vector<double, 0>((double[]){ 2.99279316847, 0.15703141229, 0.0785516617368 }))
          (Vector<double, 0>((double[]){ -1.07380055835, -0.663228298364, 0.67492002482 }))
          (Vector<double, 0>((double[]){ 1.00005626035, 0.00948902434647, -0.0142318281982 }))
          (Vector<double, 0>((double[]){ -0.864622088168, 0.730106128706, -0.495947706701 }))
          (Vector<double, 0>((double[]){ 0.861118740489, 1.70990618344, 0.996338044803 }))
          (Vector<double, 0>((double[]){ 1.11545254075, -1.35399526682, 0.969343683652 }))
          (Vector<double, 0>((double[]){ 1.21697326927, -1.74522531296, -0.991821066801 }))
          (Vector<double, 0>((double[]){ 0.595965371808, -2.32201355421, 0.917700129748 }))
          (Vector<double, 0>((double[]){ -2.02305369031, 2.1305630621, 0.346542586954 }))
          (Vector<double, 0>((double[]){ -2.21591831776, 0.372039222865, 0.969032596961 }))
          (Vector<double, 0>((double[]){ -0.740299057237, -0.767272777192, 0.357755965415 }))
          (Vector<double, 0>((double[]){ 1.0500247667, 0.323196798458, -0.433069026381 }))
          (Vector<double, 0>((double[]){ -0.627708410361, 2.82821009572, -0.441967075822 }))
          (Vector<double, 0>((double[]){ 0.849878677492, -2.01304811893, 0.982719928121 }))
          (Vector<double, 0>((double[]){ -2.328673775, -0.731540587771, -0.897568269233 }))
          (Vector<double, 0>((double[]){ -2.26385475621, -0.495059757324, -0.948307652026 }))
          (Vector<double, 0>((double[]){ -1.82442087714, 2.35553859819, 0.201726794502 }))
          (Vector<double, 0>((double[]){ 1.51598747878, 1.72096484552, 0.955972741083 }))
          (Vector<double, 0>((double[]){ 1.66570101481, 1.68975552919, 0.927941028104 }))
          (Vector<double, 0>((double[]){ -0.204475576892, 1.10956950448, 0.489956224751 }))
          (Vector<double, 0>((double[]){ -1.84519068718, -0.180785739689, 0.989288416696 }))
          (Vector<double, 0>((double[]){ -1.89174383948, -2.28786658227, -0.248333896793 }))
          (Vector<double, 0>((double[]){ 1.32446149144, -1.74220206847, -0.982076082128 }))
          (Vector<double, 0>((double[]){ -0.910155704564, -0.716495374414, 0.540006118391 }))
          (Vector<double, 0>((double[]){ 1.83109782163, 1.63940245114, 0.889078377868 }))
          (Vector<double, 0>((double[]){ -1.15330654903, 0.633501469365, -0.729333805056 }))
          (Vector<double, 0>((double[]){ 2.99967272549, -0.033495975322, -0.0167483356856 }))
          (Vector<double, 0>((double[]){ 1.1297694228, 0.713439383148, -0.747891616328 }))
          (Vector<double, 0>((double[]){ 2.50937179993, -1.20388024891, -0.621753512437 }))
          (Vector<double, 0>((double[]){ -2.2716141182, -1.65385388506, -0.586586312453 }))
          (Vector<double, 0>((double[]){ 0.153113275457, 1.40826775343, 0.812161191294 }))
          (Vector<double, 0>((double[]){ 1.00001890499, -0.00550007309815, 0.00824977691615 }))
          (Vector<double, 0>((double[]){ 1.65584867739, 1.69221774557, 0.929991847995 }))
          (Vector<double, 0>((double[]){ 0.345921290577, -2.52545287663, 0.835800088445 }))
          (Vector<double, 0>((double[]){ -0.0562087545387, -1.24544267336, -0.6576889933 }))
          (Vector<double, 0>((double[]){ -2.36000936691, -1.21267945869, -0.75706096041 }))
          (Vector<double, 0>((double[]){ -2.34139707715, -1.37283319234, -0.699954914792 }))
          (Vector<double, 0>((double[]){ 2.92740900372, 0.493744350507, 0.2480191902 }))
          (Vector<double, 0>((double[]){ -0.939307653088, -0.707592764007, 0.56659622114 }))
          (Vector<double, 0>((double[]){ -1.84991878916, -0.175917611352, 0.989904548956 }))
          (Vector<double, 0>((double[]){ -0.805613564631, 2.82022997815, -0.359778526545 }))
          (Vector<double, 0>((double[]){ 1.11523167742, -0.620330959747, 0.689954648997 }))
          (Vector<double, 0>((double[]){ 1.05302204624, -1.5925141123, 0.995867046172 }))
          (Vector<double, 0>((double[]){ -1.86603814908, -2.31459789014, -0.230284977147 })).operator Points();
      }

      RandomSphereProductPoints::RandomSphereProductPoints():
        PointCloudFixture(4950, 1)
      {
        // RandomSphereProductPoints
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ -0.91391106572, -0.141347235894, -0.380509556858, 0.881686385231, 0.388354816523, -0.267973234822 }))
          (Vector<double, 0>((double[]){ 0.990611509907, 0.00303536380334, -0.136673417333, 0.808896512488, -0.586635045029, -0.0393161039315 }))
          (Vector<double, 0>((double[]){ 0.0884538434976, 0.27280307851, 0.957994988466, 0.784303895112, -0.411104595705, -0.464607803962 }))
          (Vector<double, 0>((double[]){ 0.429305878069, 0.731689144769, 0.529459590982, 0.946240945426, 0.319620134426, -0.0497095852863 }))
          (Vector<double, 0>((double[]){ -0.0498644497485, -0.978610420087, 0.199587530543, 0.290184628762, 0.595322955589, -0.749255270104 }))
          (Vector<double, 0>((double[]){ -0.881032192427, 0.147199918617, -0.449571417982, -0.294084044304, 0.245147846397, -0.923805774117 }))
          (Vector<double, 0>((double[]){ 0.651002505371, 0.74032266664, -0.167684487235, 0.633044032413, -0.187794366826, -0.750991696902 }))
          (Vector<double, 0>((double[]){ -0.629903939652, 0.168106279213, 0.75826202971, 0.299751530807, 0.913144500906, 0.276253760597 }))
          (Vector<double, 0>((double[]){ -0.495566819091, 0.31434051571, -0.809693502505, -0.597609918773, -0.67938048061, 0.425798717177 }))
          (Vector<double, 0>((double[]){ 0.227093020513, 0.516481376086, -0.825636571495, -0.868099330401, -0.269818921467, 0.416654895777 }))
          (Vector<double, 0>((double[]){ 0.0412043045974, 0.548143756646, -0.835368557783, 0.66137181495, 0.00781982275642, -0.750017448305 }))
          (Vector<double, 0>((double[]){ 0.729582427143, 0.261504045376, -0.631921764348, -0.433928711482, -0.784940508873, 0.442237799019 }))
          (Vector<double, 0>((double[]){ -0.229551158366, 0.785901583025, -0.574164582233, -0.41144990639, 0.214731570398, 0.885776115735 }))
          (Vector<double, 0>((double[]){ 0.465834411135, -0.819809176976, -0.333033353808, 0.93304922943, -0.193003404614, 0.30359318383 }))
          (Vector<double, 0>((double[]){ -0.895306671214, 0.426887831882, 0.127250711078, -0.788646683992, 0.530449441142, -0.310901589286 }))
          (Vector<double, 0>((double[]){ -0.661028600478, -0.745754694322, -0.0830128017064, 0.249010431248, -0.966711761328, -0.0588402552679 }))
          (Vector<double, 0>((double[]){ 0.744991369972, 0.374103499769, 0.552299221553, 0.617244702467, -0.718178435714, 0.321292252865 }))
          (Vector<double, 0>((double[]){ 0.804227031335, 0.593683439053, 0.0275473458671, 0.659590543185, 0.714840351993, 0.232257586536 }))
          (Vector<double, 0>((double[]){ -0.873200284124, 0.166399193554, -0.458074854353, -0.560031480525, 0.029020025643, -0.827962909152 }))
          (Vector<double, 0>((double[]){ 0.119950083778, -0.69671069368, 0.707252562183, 0.0496490274239, 0.172576793633, -0.983743983146 }))
          (Vector<double, 0>((double[]){ -0.440354788009, -0.621373396798, 0.64806077063, -0.553163469291, 0.0985962212268, 0.827217602207 }))
          (Vector<double, 0>((double[]){ 0.391875160577, -0.91204712312, -0.12084661241, 0.343515938445, 0.805145135401, -0.483464694651 }))
          (Vector<double, 0>((double[]){ -0.878904700967, 0.476148853153, 0.0284393435097, -0.375280521878, 0.0236320798598, 0.926609979819 }))
          (Vector<double, 0>((double[]){ 0.371779279399, -0.0647107180096, -0.926063005623, -0.581859869074, -0.672138098482, 0.457896788948 }))
          (Vector<double, 0>((double[]){ 0.333501910635, -0.648841901002, -0.683944926954, 0.346477741669, 0.287853355937, 0.892800996865 }))
          (Vector<double, 0>((double[]){ -0.118455157233, -0.387377681438, 0.914279447242, 0.195237108849, 0.432605901896, 0.88019009593 }))
          (Vector<double, 0>((double[]){ -0.203787888402, 0.33127798769, 0.921262932833, -0.170213460078, -0.759832707449, 0.62744054276 }))
          (Vector<double, 0>((double[]){ -0.186972211131, 0.375353095405, 0.907827872471, 0.90519903736, -0.423142263903, -0.0395642169442 }))
          (Vector<double, 0>((double[]){ 0.0157538019146, 0.556328316753, 0.830813229134, -0.748900117051, 0.644671401421, -0.15345161736 }))
          (Vector<double, 0>((double[]){ -0.789486310037, 0.177683869092, 0.587486007432, -0.704459353328, 0.585369594602, -0.401347053339 }))
          (Vector<double, 0>((double[]){ -0.429515899368, 0.541908419738, 0.722392799526, 0.291158397458, 0.7075477363, -0.643896721879 }))
          (Vector<double, 0>((double[]){ -0.974066930134, -0.0695537833965, 0.21530417282, 0.560837740774, 0.80421528685, -0.196720108072 }))
          (Vector<double, 0>((double[]){ 0.734572540929, -0.644990685445, -0.210689814187, 0.324037287271, -0.817162121601, -0.476703160761 }))
          (Vector<double, 0>((double[]){ -0.857701714323, -0.139384963661, 0.494893525066, -0.151511095776, 0.646562693087, 0.747663742443 }))
          (Vector<double, 0>((double[]){ 0.796539508853, -0.570417546912, 0.200371237987, 0.0482153819194, -0.973852752174, 0.222004716234 }))
          (Vector<double, 0>((double[]){ 0.916876172824, 0.222728758959, 0.331255163945, -0.192534215767, -0.738541274744, -0.64613261894 }))
          (Vector<double, 0>((double[]){ -0.0823420618905, -0.97897049093, -0.186645553744, 0.949428847626, -0.248890196762, 0.191411946469 }))
          (Vector<double, 0>((double[]){ -0.0419574585335, 0.461690600453, 0.886048170884, -0.0699015100922, 0.673888685305, 0.735518741233 }))
          (Vector<double, 0>((double[]){ 0.412561641669, 0.909390182573, -0.0529375827022, -0.895374983398, 0.112461012963, -0.43087835832 }))
          (Vector<double, 0>((double[]){ 0.450371942246, 0.874044510705, 0.182239696403, -0.362120681847, -0.929933540422, 0.0639704789444 }))
          (Vector<double, 0>((double[]){ -0.612265865732, 0.34870804106, 0.709600741093, 0.166072781647, 0.375886025696, -0.911663055565 }))
          (Vector<double, 0>((double[]){ 0.271922766823, 0.7425280528, -0.612135687318, 0.738656393225, 0.298733624931, 0.604272251625 }))
          (Vector<double, 0>((double[]){ -0.015330463284, 0.900398580874, 0.434795782473, -0.158171447069, -0.987410869245, 0.00125244920035 }))
          (Vector<double, 0>((double[]){ 0.285169684017, 0.519849249895, 0.805254623521, -0.277011949991, 0.846977381264, 0.453755104863 }))
          (Vector<double, 0>((double[]){ -0.880646791102, 0.246222041757, 0.404766519707, 0.799815646915, -0.0189642887677, -0.599946069828 }))
          (Vector<double, 0>((double[]){ 0.260953912161, -0.430449949161, -0.864069382049, 0.164331626909, 0.986287288989, 0.0152479499061 }))
          (Vector<double, 0>((double[]){ 0.433704296216, -0.673513751891, 0.598564791361, -0.329605422381, 0.0373262814925, -0.94338063063 }))
          (Vector<double, 0>((double[]){ 0.288361123097, -0.890519545851, -0.351884641814, -0.523308233477, -0.851661079846, -0.0286687608809 }))
          (Vector<double, 0>((double[]){ -0.873045930329, -0.0362508499896, 0.48628867909, 0.555064562858, 0.148805858212, 0.818388750913 }))
          (Vector<double, 0>((double[]){ -0.973900170015, -0.132631320269, 0.184193897098, -0.848089056872, 0.523567387945, -0.0813765438851 }))
          (Vector<double, 0>((double[]){ 0.273498738988, 0.800707963433, 0.532977670327, -0.895875235686, -0.443012941239, -0.0338688053865 }))
          (Vector<double, 0>((double[]){ -0.913380896292, 0.403357140227, -0.0551212818834, 0.663104698339, -0.6576771534, -0.357425406114 }))
          (Vector<double, 0>((double[]){ 0.446320172227, 0.648632557008, 0.616501508394, 0.3886874356, 0.197485365812, 0.899956447667 }))
          (Vector<double, 0>((double[]){ -0.269302892463, -0.264118781229, -0.926130240038, 0.496870560803, -0.118618378875, 0.859679781081 }))
          (Vector<double, 0>((double[]){ 0.296668266602, 0.916141731244, -0.269577943952, 0.442331558115, -0.3031174274, -0.844075007273 }))
          (Vector<double, 0>((double[]){ 0.925991455641, 0.374354489683, 0.0489748929059, -0.454401402909, 0.110035818762, -0.883974820696 }))
          (Vector<double, 0>((double[]){ 0.0200084310575, 0.932770188948, 0.359915875304, -0.228199983346, 0.538688119761, 0.811011638159 }))
          (Vector<double, 0>((double[]){ -0.364446858078, -0.284947043248, 0.886557200738, 0.863792004363, 0.473702019058, 0.17166761587 }))
          (Vector<double, 0>((double[]){ -0.915570084839, 0.0159794355134, 0.401840860774, 0.970863891012, -0.220173185, 0.0945889725943 }))
          (Vector<double, 0>((double[]){ -0.318791429983, 0.67698186337, -0.663375897088, -0.974586934914, -0.0630610225887, 0.214950258723 }))
          (Vector<double, 0>((double[]){ 0.848947397585, -0.528302347836, -0.0135994633791, 0.478114045487, -0.497218818088, 0.72400304312 }))
          (Vector<double, 0>((double[]){ -0.237892726513, -0.141469854068, -0.960933572658, 0.51621625722, 0.723429394622, 0.458443766212 }))
          (Vector<double, 0>((double[]){ 0.605151733343, 0.126673469204, 0.785967691341, -0.421996063675, -0.361765465936, -0.831291206437 }))
          (Vector<double, 0>((double[]){ 0.0363480170529, 0.0904347198846, 0.995238857308, -0.682161806592, -0.70110481672, -0.207622989089 }))
          (Vector<double, 0>((double[]){ 0.84178204894, -0.51049341105, -0.175497747439, -0.30994037764, -0.01839925446, -0.950577945117 }))
          (Vector<double, 0>((double[]){ 0.8273429664, -0.512691174265, 0.229458876009, -0.189812461762, 0.351418769203, 0.91677482405 }))
          (Vector<double, 0>((double[]){ 0.920622314618, -0.376745326128, 0.102554927081, 0.586336426264, -0.388130866211, 0.711030256691 }))
          (Vector<double, 0>((double[]){ 0.555945824396, 0.046748719497, 0.829902884416, 0.216955958268, -0.497839190657, -0.839694142184 }))
          (Vector<double, 0>((double[]){ 0.967911543835, -0.251121661309, -0.00922792138832, -0.422623876309, -0.126129410123, -0.897485616083 }))
          (Vector<double, 0>((double[]){ -0.756523297583, 0.104386365754, 0.645581897871, -0.376228624076, -0.863535723917, -0.335794693147 }))
          (Vector<double, 0>((double[]){ -0.856657967627, -0.0182855012704, -0.515560633626, 0.483049520071, -0.824816814943, -0.293837340967 }))
          (Vector<double, 0>((double[]){ -0.121154134349, 0.949406798142, 0.289738515512, 0.0265249164819, 0.421763754149, -0.906317695122 }))
          (Vector<double, 0>((double[]){ 0.615201633931, -0.0490989970713, -0.786839397905, 0.465984087427, 0.0746627745208, -0.881637283902 }))
          (Vector<double, 0>((double[]){ 0.72321021683, -0.00673435930015, -0.690595127899, -0.204017663125, 0.881069289894, -0.426724383577 }))
          (Vector<double, 0>((double[]){ -0.9084489844, -0.00394099060893, -0.417977166047, 0.329639971597, -0.403708717076, 0.853438199803 }))
          (Vector<double, 0>((double[]){ 0.104984106451, -0.813673621511, -0.57176356569, 0.195884953011, 0.139790896222, 0.970611967017 }))
          (Vector<double, 0>((double[]){ -0.616601635651, 0.52350916849, 0.587997086234, 0.111137043074, -0.316147196494, -0.94217806587 }))
          (Vector<double, 0>((double[]){ -0.778321116933, -0.234722927908, 0.582341296879, -0.322915671979, 0.538995068511, 0.777952302466 }))
          (Vector<double, 0>((double[]){ 0.952225121683, -0.154672175113, 0.263332177831, -0.111593606845, 0.328109671159, -0.938025005319 }))
          (Vector<double, 0>((double[]){ -0.694484491995, -0.446803069164, 0.563966583908, -0.53796480364, 0.834600598435, 0.118472406641 }))
          (Vector<double, 0>((double[]){ -0.628349496855, -0.608122021242, -0.485143810722, -0.767170863156, 0.0150567271672, 0.641266061546 }))
          (Vector<double, 0>((double[]){ -0.858055265786, 0.021755993715, -0.513096323894, -0.224812119299, 0.931873986248, -0.284728616003 }))
          (Vector<double, 0>((double[]){ 0.0300911279613, -0.978780799924, 0.202688602833, 0.165850364643, -0.597697209706, 0.784379820022 }))
          (Vector<double, 0>((double[]){ 0.0861640197473, 0.355531584508, 0.930684186025, -0.375483781045, 0.817184564822, -0.437288597139 }))
          (Vector<double, 0>((double[]){ -0.631500361127, 0.518765265809, 0.576272412046, 0.500340064122, -0.656096850992, 0.564974992678 }))
          (Vector<double, 0>((double[]){ 0.83510618007, 0.412928247677, -0.36343903241, 0.834650021258, 0.262444006165, 0.484233916245 }))
          (Vector<double, 0>((double[]){ -0.950512976998, -0.0225135909132, -0.309868066737, -0.63625512986, 0.760465602148, 0.129890252434 }))
          (Vector<double, 0>((double[]){ 0.610205210512, 0.371735114969, -0.699616041385, 0.49490802688, -0.796945578728, -0.346329019108 }))
          (Vector<double, 0>((double[]){ 0.0945087951818, 0.379311175675, 0.920429855905, -0.491596745383, 0.168405567412, -0.854384108463 }))
          (Vector<double, 0>((double[]){ 0.242396925569, -0.884007321863, -0.399718382573, -0.956717113356, -0.0804459321851, -0.279679847336 }))
          (Vector<double, 0>((double[]){ 0.21175463722, 0.851025574429, 0.480536622208, 0.29526144253, 0.853442588611, -0.429483909476 }))
          (Vector<double, 0>((double[]){ -0.872855815503, -0.456415498054, -0.172648829938, -0.185878180365, 0.0127862383816, 0.982489600033 }))
          (Vector<double, 0>((double[]){ -0.766946060919, -0.526290935911, -0.367166978933, -0.691764825401, -0.45476651861, -0.560935682488 }))
          (Vector<double, 0>((double[]){ -0.64528960602, 0.694834684829, 0.317499740348, -0.753230060776, -0.558767840334, -0.347020137964 }))
          (Vector<double, 0>((double[]){ -0.0271244805831, -0.941541274939, 0.335803945984, 0.579847585423, 0.535897245257, -0.613670041803 }))
          (Vector<double, 0>((double[]){ -0.31557058217, -0.242393903422, 0.917420516041, -0.363498792183, -0.716412927505, -0.595500751792 }))
          (Vector<double, 0>((double[]){ 0.292764769736, -0.711791534482, -0.638468167602, 0.929990201605, -0.333628944923, -0.154304737543 }))
          (Vector<double, 0>((double[]){ 0.180037407994, 0.30407975929, 0.935479573114, -0.335222540586, 0.56674126772, -0.752615561722 }))
          (Vector<double, 0>((double[]){ 0.47996586657, 0.838342461047, 0.258485366961, -0.841324702697, -0.53900186034, -0.0406169814439 }))
          (Vector<double, 0>((double[]){ -0.450662957184, -0.751453045429, -0.481893369468, 0.514313868179, -0.299993987265, 0.803420719551 })).operator Points();
      }

      RandomFigure8Points::RandomFigure8Points():
        PointCloudFixture(4950, 1)
      {
        // RandomFigure8Points
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ -0.997672864118, 0.931817478753 }))
          (Vector<double, 0>((double[]){ -0.12212533931, 1.99251468578 }))
          (Vector<double, 0>((double[]){ 0.99971217211, 1.02399110117 }))
          (Vector<double, 0>((double[]){ -0.530230062096, 1.84785380889 }))
          (Vector<double, 0>((double[]){ -0.616618525108, 0.21273791245 }))
          (Vector<double, 0>((double[]){ 0.953112207174, 0.697382881289 }))
          (Vector<double, 0>((double[]){ 0.891348386278, 0.546681067812 }))
          (Vector<double, 0>((double[]){ 0.513304388735, 1.8582066211 }))
          (Vector<double, 0>((double[]){ 0.904207423986, 1.42709358975 }))
          (Vector<double, 0>((double[]){ 0.632986434066, 0.225837113852 }))
          (Vector<double, 0>((double[]){ -0.428847623364, 0.0966231595094 }))
          (Vector<double, 0>((double[]){ -0.31372435727, 1.94951410082 }))
          (Vector<double, 0>((double[]){ -0.324163757095, 1.94600098234 }))
          (Vector<double, 0>((double[]){ -0.438597627591, 0.101316451095 }))
          (Vector<double, 0>((double[]){ 0.749024733521, 0.337457964674 }))
          (Vector<double, 0>((double[]){ 0.275704231251, 0.0387574828013 }))
          (Vector<double, 0>((double[]){ 0.905687684531, 0.5760544633 }))
          (Vector<double, 0>((double[]){ 0.975251439322, 0.778901311402 }))
          (Vector<double, 0>((double[]){ -0.599637968099, 0.199728604026 }))
          (Vector<double, 0>((double[]){ 0.999386525605, 0.964977543775 }))
          (Vector<double, 0>((double[]){ 0.591869580676, 0.193966254137 }))
          (Vector<double, 0>((double[]){ -0.719822311982, 0.305841632499 }))
          (Vector<double, 0>((double[]){ 0.58532531734, 1.81079854026 }))
          (Vector<double, 0>((double[]){ 0.986883746935, 0.838567444313 }))
          (Vector<double, 0>((double[]){ 0.949582928611, 0.686484032797 }))
          (Vector<double, 0>((double[]){ 0.374768322698, 0.0728815047135 }))
          (Vector<double, 0>((double[]){ 0.865785453544, 1.50041537589 }))
          (Vector<double, 0>((double[]){ 0.910471799064, 0.586428841541 }))
          (Vector<double, 0>((double[]){ 0.838923497478, 0.455750640442 }))
          (Vector<double, 0>((double[]){ 0.223950643888, 0.0253995130814 }))
          (Vector<double, 0>((double[]){ -0.991144291389, 0.867210717121 }))
          (Vector<double, 0>((double[]){ 0.931408672911, 0.636024885431 }))
          (Vector<double, 0>((double[]){ -0.619860048398, 1.78471238068 }))
          (Vector<double, 0>((double[]){ -0.381970441256, 1.92417454088 }))
          (Vector<double, 0>((double[]){ 0.650517508303, 0.240508741728 }))
          (Vector<double, 0>((double[]){ -0.99088896496, 1.13468132432 }))
          (Vector<double, 0>((double[]){ -0.633409846395, 0.226183505933 }))
          (Vector<double, 0>((double[]){ -0.452665025397, 0.108319353814 }))
          (Vector<double, 0>((double[]){ 0.911912384382, 1.41038494516 }))
          (Vector<double, 0>((double[]){ 0.570502074212, 0.178703839459 }))
          (Vector<double, 0>((double[]){ 0.133574533623, 1.99103877016 }))
          (Vector<double, 0>((double[]){ -0.554528675525, 0.167835384061 }))
          (Vector<double, 0>((double[]){ 0.957155404548, 0.710425257412 }))
          (Vector<double, 0>((double[]){ -0.381690163155, 1.92429033282 }))
          (Vector<double, 0>((double[]){ 0.763907006895, 1.64532633978 }))
          (Vector<double, 0>((double[]){ -0.137121889117, 1.99055418202 }))
          (Vector<double, 0>((double[]){ -0.956239864455, 0.707416128901 }))
          (Vector<double, 0>((double[]){ -0.517554630696, 0.144349835362 }))
          (Vector<double, 0>((double[]){ 0.667962507832, 1.74419492617 }))
          (Vector<double, 0>((double[]){ -0.638049809802, 1.76999509103 }))
          (Vector<double, 0>((double[]){ 0.901386042303, -1.43301640008 }))
          (Vector<double, 0>((double[]){ -0.754373375418, -1.65644558835 }))
          (Vector<double, 0>((double[]){ 0.419177580773, -0.0920957342443 }))
          (Vector<double, 0>((double[]){ 0.538578180674, -0.1574244584 }))
          (Vector<double, 0>((double[]){ -0.426602350282, -1.90443929301 }))
          (Vector<double, 0>((double[]){ 0.14132323618, -0.0100364941496 }))
          (Vector<double, 0>((double[]){ -0.0593939637615, -0.00176537974848 }))
          (Vector<double, 0>((double[]){ 0.136324719168, -0.00933579304397 }))
          (Vector<double, 0>((double[]){ -0.837736138971, -0.453924765749 }))
          (Vector<double, 0>((double[]){ -0.0870270879839, -0.00379405444604 }))
          (Vector<double, 0>((double[]){ -0.974380401032, -1.22490627845 }))
          (Vector<double, 0>((double[]){ -0.933272379544, -0.640830589303 }))
          (Vector<double, 0>((double[]){ -0.574748738006, -0.181670061552 }))
          (Vector<double, 0>((double[]){ 0.998999647689, -0.955281950881 }))
          (Vector<double, 0>((double[]){ 0.829949522828, -1.55783849774 }))
          (Vector<double, 0>((double[]){ -0.9349752341, -1.35471299894 }))
          (Vector<double, 0>((double[]){ 0.547824174629, -0.163406506305 }))
          (Vector<double, 0>((double[]){ 0.98653843685, -0.836470453383 }))
          (Vector<double, 0>((double[]){ 0.906728905419, -0.578286006781 }))
          (Vector<double, 0>((double[]){ 0.746973294569, -0.335145957972 }))
          (Vector<double, 0>((double[]){ 0.22662738509, -1.97398153387 }))
          (Vector<double, 0>((double[]){ 0.778183494341, -0.371963019293 }))
          (Vector<double, 0>((double[]){ 0.0966124391626, -1.99532207682 }))
          (Vector<double, 0>((double[]){ -0.994049281073, -1.10893129393 }))
          (Vector<double, 0>((double[]){ 0.644626272284, -1.7644978542 }))
          (Vector<double, 0>((double[]){ -0.412289263387, -0.0889470030258 }))
          (Vector<double, 0>((double[]){ 0.361315020421, -1.9324438085 }))
          (Vector<double, 0>((double[]){ -0.150038564756, -1.98868014498 }))
          (Vector<double, 0>((double[]){ 0.458237907406, -1.88882957884 }))
          (Vector<double, 0>((double[]){ 0.993458415296, -0.885805529571 }))
          (Vector<double, 0>((double[]){ 0.0632380479365, -1.99799847159 }))
          (Vector<double, 0>((double[]){ 0.750067251252, -0.338638435801 }))
          (Vector<double, 0>((double[]){ 0.507782248491, -1.86148545439 }))
          (Vector<double, 0>((double[]){ -0.941237862702, -0.662255590991 }))
          (Vector<double, 0>((double[]){ -0.946069383268, -0.676035924765 }))
          (Vector<double, 0>((double[]){ -0.756827723743, -0.346385590295 }))
          (Vector<double, 0>((double[]){ 0.46958513557, -1.88288719577 }))
          (Vector<double, 0>((double[]){ -0.732154361334, -0.31886125409 }))
          (Vector<double, 0>((double[]){ -0.868853729061, -1.49506888157 }))
          (Vector<double, 0>((double[]){ -0.258676139586, -1.96596410638 }))
          (Vector<double, 0>((double[]){ -0.272640451709, -0.037884006945 }))
          (Vector<double, 0>((double[]){ -0.999142379591, -1.04140658529 }))
          (Vector<double, 0>((double[]){ 0.140555189877, -1.9900728451 }))
          (Vector<double, 0>((double[]){ 0.9530304165, -1.30287460314 }))
          (Vector<double, 0>((double[]){ 0.701451027558, -0.287282344868 }))
          (Vector<double, 0>((double[]){ 0.221998368818, -1.97504703694 }))
          (Vector<double, 0>((double[]){ -0.919744418409, -0.60748222358 }))
          (Vector<double, 0>((double[]){ -0.994728855336, -0.897459742721 }))
          (Vector<double, 0>((double[]){ -0.828266765338, -1.56033397669 }))
          (Vector<double, 0>((double[]){ 0.132233772693, -0.00878144218375 })).operator Points();
      }

      DisjointPatches::DisjointPatches():
        PointCloudFixture(4950, 1)
      {
        // DisjointPatches
        _points = boost::assign::list_of(Vector<double, 0>((double[]){ -0.644859810546, -0.890205594159 }))
          (Vector<double, 0>((double[]){ -0.0946211968507, 1.63253784297 }))
          (Vector<double, 0>((double[]){ 1.35453077443, -0.658332458579 }))
          (Vector<double, 0>((double[]){ 0.551959684332, 2.74929321121 }))
          (Vector<double, 0>((double[]){ -0.492923683969, 0.272362697438 }))
          (Vector<double, 0>((double[]){ -0.686797382501, 1.91150755054 }))
          (Vector<double, 0>((double[]){ 1.42831022454, -0.362450210446 }))
          (Vector<double, 0>((double[]){ -0.461277270385, 2.49847509777 }))
          (Vector<double, 0>((double[]){ -0.0590085288055, -1.79876204443 }))
          (Vector<double, 0>((double[]){ -0.149664460258, 1.65084408539 }))
          (Vector<double, 0>((double[]){ -1.1543396139, 0.760008397433 }))
          (Vector<double, 0>((double[]){ 2.66562911593, 2.52845948756 }))
          (Vector<double, 0>((double[]){ -0.105761119934, 0.752929107285 }))
          (Vector<double, 0>((double[]){ -0.0567596891974, 1.02730578059 }))
          (Vector<double, 0>((double[]){ -0.522435463903, -1.57673171736 }))
          (Vector<double, 0>((double[]){ 0.524910033565, 2.90351754771 }))
          (Vector<double, 0>((double[]){ -0.867078446664, 0.153904506636 }))
          (Vector<double, 0>((double[]){ -1.12365431832, -0.201951727589 }))
          (Vector<double, 0>((double[]){ -0.717340226531, 0.528833047824 }))
          (Vector<double, 0>((double[]){ -0.789048049935, 3.31925077897 }))
          (Vector<double, 0>((double[]){ -0.0111472241384, 0.309822725594 }))
          (Vector<double, 0>((double[]){ 2.03433429047, 2.13106328207 }))
          (Vector<double, 0>((double[]){ -0.630132977784, 0.996570621863 }))
          (Vector<double, 0>((double[]){ 1.04790276329, 1.4396377738 }))
          (Vector<double, 0>((double[]){ 0.327137560506, -0.237531405385 }))
          (Vector<double, 0>((double[]){ 0.0610326957599, 1.50640198389 }))
          (Vector<double, 0>((double[]){ 0.79432967688, 0.625284116745 }))
          (Vector<double, 0>((double[]){ -0.88671704608, 2.17552730332 }))
          (Vector<double, 0>((double[]){ 1.18113605888, -1.03001303994 }))
          (Vector<double, 0>((double[]){ 0.151638288701, 1.31254704487 }))
          (Vector<double, 0>((double[]){ -0.563846735117, -1.07574823907 }))
          (Vector<double, 0>((double[]){ -0.906275920888, 2.38180414197 }))
          (Vector<double, 0>((double[]){ 1.19118016789, -0.768178809532 }))
          (Vector<double, 0>((double[]){ -0.758757714588, 1.49228314988 }))
          (Vector<double, 0>((double[]){ -0.479489902831, 0.490854332139 }))
          (Vector<double, 0>((double[]){ 0.221995104218, 2.68262914661 }))
          (Vector<double, 0>((double[]){ 0.378948895086, 2.25350515799 }))
          (Vector<double, 0>((double[]){ -0.511609888732, 3.37345371815 }))
          (Vector<double, 0>((double[]){ 0.227027671684, 0.760318675055 }))
          (Vector<double, 0>((double[]){ 1.54589595665, 1.36623129261 }))
          (Vector<double, 0>((double[]){ -1.81765639116, -1.25940448672 }))
          (Vector<double, 0>((double[]){ 1.39060396265, 3.30816330495 }))
          (Vector<double, 0>((double[]){ 0.751272056773, 0.53644323055 }))
          (Vector<double, 0>((double[]){ 2.38132252433, 2.55479665312 }))
          (Vector<double, 0>((double[]){ 0.443758601681, 0.0262587808125 }))
          (Vector<double, 0>((double[]){ -0.530995771216, 0.827979725273 }))
          (Vector<double, 0>((double[]){ 0.674560151454, -0.677021663055 }))
          (Vector<double, 0>((double[]){ -1.26052275215, 0.208273917198 }))
          (Vector<double, 0>((double[]){ -1.36771496235, -1.31390903405 }))
          (Vector<double, 0>((double[]){ -0.536808181307, 2.73149535858 }))
          (Vector<double, 0>((double[]){ -0.260324451693, -0.381963409993 }))
          (Vector<double, 0>((double[]){ 1.15685182016, 2.98278676912 }))
          (Vector<double, 0>((double[]){ -1.57756720919, 0.85499304956 }))
          (Vector<double, 0>((double[]){ -1.55199512772, 1.83078786928 }))
          (Vector<double, 0>((double[]){ 1.12488300744, -0.364017700432 }))
          (Vector<double, 0>((double[]){ 0.300747491311, 2.4902186098 }))
          (Vector<double, 0>((double[]){ -1.06280413075, -0.525630330974 }))
          (Vector<double, 0>((double[]){ -1.04916788057, 1.25152012209 }))
          (Vector<double, 0>((double[]){ -0.0938394884106, 0.501931679248 }))
          (Vector<double, 0>((double[]){ -0.0294095921617, 3.24133393985 }))
          (Vector<double, 0>((double[]){ -1.77897447111, 1.22831439477 }))
          (Vector<double, 0>((double[]){ 0.341574100684, 1.73936248245 }))
          (Vector<double, 0>((double[]){ -0.269239458761, -0.8333717653 }))
          (Vector<double, 0>((double[]){ -0.15598308661, 0.86477444943 }))
          (Vector<double, 0>((double[]){ 1.05633589062, -0.859236665551 }))
          (Vector<double, 0>((double[]){ 0.194508245566, 0.561921524434 }))
          (Vector<double, 0>((double[]){ 0.385236413177, 0.204413809215 }))
          (Vector<double, 0>((double[]){ 0.941810804346, 1.31405760452 }))
          (Vector<double, 0>((double[]){ 0.569967505207, -1.16744067449 }))
          (Vector<double, 0>((double[]){ 2.07073493957, 3.95487570054 }))
          (Vector<double, 0>((double[]){ -0.885327710508, -0.866008733403 }))
          (Vector<double, 0>((double[]){ 0.547723578449, 1.71739719524 }))
          (Vector<double, 0>((double[]){ 0.663603729033, 0.0829678114406 }))
          (Vector<double, 0>((double[]){ 1.21129280013, 1.72060534798 }))
          (Vector<double, 0>((double[]){ 0.362073962081, -0.747447484945 }))
          (Vector<double, 0>((double[]){ 1.704221995, 2.5660101505 }))
          (Vector<double, 0>((double[]){ 0.601945549954, 1.04645173428 }))
          (Vector<double, 0>((double[]){ 0.841680001536, 1.77433564872 }))
          (Vector<double, 0>((double[]){ 0.766895771272, -1.09310654651 }))
          (Vector<double, 0>((double[]){ -0.938283001022, 2.64075930468 }))
          (Vector<double, 0>((double[]){ 1.37421650821, -0.0884833216355 }))
          (Vector<double, 0>((double[]){ -0.870080333191, 1.09163109877 }))
          (Vector<double, 0>((double[]){ 0.153254168029, -0.0742498753145 }))
          (Vector<double, 0>((double[]){ -0.862544599711, 2.49206014435 }))
          (Vector<double, 0>((double[]){ -0.912087408735, 0.124674144565 }))
          (Vector<double, 0>((double[]){ 0.796100878611, 2.77818710981 }))
          (Vector<double, 0>((double[]){ 0.0436451024323, 0.421699170186 }))
          (Vector<double, 0>((double[]){ 0.907424859054, 2.47416500243 }))
          (Vector<double, 0>((double[]){ -0.111324456536, -0.592280300276 }))
          (Vector<double, 0>((double[]){ -0.822866163039, 3.52851911018 }))
          (Vector<double, 0>((double[]){ 0.62001149669, 0.0991518650202 }))
          (Vector<double, 0>((double[]){ -0.854237746236, 3.54311163796 }))
          (Vector<double, 0>((double[]){ -0.476107281397, -0.899403107723 }))
          (Vector<double, 0>((double[]){ 1.88874741345, 1.89023082672 }))
          (Vector<double, 0>((double[]){ 0.665830968824, -0.0433007868608 }))
          (Vector<double, 0>((double[]){ 1.00783214662, 4.03083398237 }))
          (Vector<double, 0>((double[]){ -1.95245211188, 0.328503305109 }))
          (Vector<double, 0>((double[]){ -0.356658177247, 3.57131776301 }))
          (Vector<double, 0>((double[]){ -0.578042709973, 0.0141742797922 }))
          (Vector<double, 0>((double[]){ 0.202262764033, 1.87062315462 })).operator Points();
      }


    }

  }
}
